Attribute VB_Name = "CircScapeFunctions"
Option Explicit
Const c_sModuleFileName As String = "D:\arcGIS_stuff\consultation\brost_climate_change\code\CircScapeFunctions.bas"


Public Function ExportRastersToTable(pDocParams As esriSystem.IVariantArray, _
    pParams As esriSystem.IVariantArray) As String
  On Error GoTo ErrorHandler
  
  Dim pApp As IApplication
  Dim pThisExt As TopoCorridor.clsExtension
  Dim pMxDoc As IMxDocument
  
13:   Set pApp = pDocParams.Element(0)
14:   Set pMxDoc = pDocParams.Element(1)
15:   Set pThisExt = pDocParams.Element(2)

  ' RETURN ARRAY CONTAINS THE FOLLOWING:
  ' 0) IRasterBand:  TPI SLOPE CLASSIFICATION RASTER BAND
  ' 1) String: Name of TPI SLOPE CLASSIFICATION Band Layer
  ' 2) IVariantArray:  ADDITIONAL RASTER LAYERS; COULD BE EMPTY
  ' 3) IVariantArray:  ADDITIONAL FEATURE LAYERS; COULD BE EMPTY
  ' 4) Boolean:  WHETHER TO CLIP OUTPUT TO POLYGON LAYER
  ' 5) IFeatureLayer:  POLYGON LAYER TO CLIP TO, COULD BE NOTHING
  ' 6) String:  cbxFormat Text
  ' 7) IWorkspace: WORKSPACE, EITHER TextFileWorkspace OR ShapefileWorkspace
  ' 8) Boolean:  WHETHER ALL DATASETS ARE MISSING A SPATIAL REFERENCE
  ' 9) Boolean:  Whether to add combined raster to map.
  
  Dim pTemplateBand As IRasterBand
  Dim strTemplateName As String
  Dim pTemplateRaster As IRaster
  Dim pTemplateDataset As IRasterDataset2
  Dim pTemplateProps As IRasterProps
  Dim pTemplateSpRef As ISpatialReference
  Dim pTemplatePnt As IPnt
  
37:   Set pTemplateBand = pParams.Element(0)
38:   strTemplateName = pParams.Element(1)
39:   Set pTemplateDataset = pTemplateBand.RasterDataset
40:   Set pTemplateRaster = pTemplateDataset.CreateFullRaster
41:   Set pTemplateProps = pTemplateBand
42:   Set pTemplateSpRef = pTemplateProps.SpatialReference
43:   Set pTemplatePnt = pTemplateProps.MeanCellSize
  
  Dim strInputName As String
  Dim strInputBand As IRasterBand
  
  Dim pAdditionalRasters As esriSystem.IVariantArray
  Dim pAdditionalFeatLayers As esriSystem.IVariantArray
  Dim booShouldClip As Boolean
  Dim pClipLayer As IFeatureLayer
  Dim strOutputFormat As String
  Dim strFullFilename As String
  Dim pWS As IWorkspace
  Dim booAllHaveNoSpRef As Boolean
  Dim strFullFilePath As String
  Dim booAddCombinedToMap As Boolean
  
59:   Set pAdditionalRasters = pParams.Element(2)
60:   Set pAdditionalFeatLayers = pParams.Element(3)
61:   booShouldClip = pParams.Element(4)
62:   Set pClipLayer = pParams.Element(5)
63:   strOutputFormat = pParams.Element(6)
'  strFullFilename = pParams.Element(7)
65:   Set pWS = pParams.Element(7)
66:   booAllHaveNoSpRef = pParams.Element(8)
'  strFullFilePath = pParams.Element(10)
68:   booAddCombinedToMap = pParams.Element(9)
  
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  Dim lngIndex3 As Long
  
  ' MAKE ARRAY OF UNIQUE SLOPE CLASS VALUES
  ' AFTER THIS FIRST RUN, THE pSlopeValArray WILL CONTAIN A SEPARATE pSlopeSubArray FOR EACH UNIQUE RASTER VALUE
  ' EACH pSlopeSubArray WILL CONTAIN 6 ITEMS:
  ' 0) ATTRIBUTE VALUE CORRESPONDING WITH [VALUE] FIELD, OR [OID] FIELD IF [VALUE] IS MISSING FOR SOME REASON
  ' 1) pSlopeValAttVals IVariantArray CONTAINING ALL ATTRIBUTE VALUES FOR THIS ROW
  ' 2) FOLDER NAME FOR TABLE
  ' 3) FILE NAME FOR TABLE
  ' 4) FULL FILE NAME FOR TABLE
  ' 5) EMPTY pAddFields FOR TABLE ATTRIBUTE FIELDS
  ' 6) BOOLEAN STATING WHETHER TABLE HAS BEEN CREATED YET OR NOT
  '
  ' WILL EVENTUALLY ADD:
  ' 7) POTENTIALLY pTable OR LONG (for CSV File Number) FOR ACTUAL TABLE
  '    TABLE WILL ONLY BE CREATED IF WE ENCOUNTER A SLOPE CLASS CELL WITH THIS VALUE IN THE SPECIFIED AREA.
  ' 8) pFieldIndexes:  FIELD INDICES CORRESPONDING WITH EACH DATASET ATTRIBUTE VALUE IN TABLE  (NOTHING IF CSV FILE)
  ' 9) pCursor: CURSOR FOR NEW TABLE (NOTHING IF CSV FILE)
  ' 10) pBuffer: INSERT BUFFER FOR NEW TABLE (NOTHING IF CSV FILE)
  '
  ' WILL ALSO HAVE AN IStringArray NAMED pSlopeValAttNames CONTAINING ALL FIELD NAMES, IN THE SAME ORDER
  ' AS pSlopeValAttVals ABOVE
  '
  ' WILL ALSO HAVE A COLLECTION collSlopeTables WITH:
  '   KEY = Slope Class Value
  '   ITEM = INDEX VALUE FOR ASSOCIATED pSlopeSubArray INSIDE pSlopeValArray
  
  Dim strTempName As String
  
  Dim strWsName As String
102:   strWsName = pWS.PathName
103:   If Right(strWsName, 1) = " \" Then
104:     strWsName = Left(strWsName, Len(strWsName) - 1)
105:   End If
  
  Dim pSlopeValArray As esriSystem.IVariantArray
  Dim pSlopeValAttVals As esriSystem.IVariantArray
  Dim pSlopeValAttNames As esriSystem.IStringArray
  Dim pSlopeAttTable As ITable
  Dim pSlopeSubArray As esriSystem.IVariantArray
  Dim pSlopeCursor As ICursor
  Dim pSlopeRow As IRow
  Dim pSlopeField As IField
  Dim pSlopeFields As IFields
  Dim varSlopeVal As Variant
  Dim lngValueIndex As Long
  Dim varIDValue As Variant
  Dim pAddFields As esriSystem.IVariantArray
  Dim booUseDBase As Boolean
  Dim booTableCreated As Boolean
  Dim collSlopeTables As Collection
  Dim lngSlopeDataIndex As Long
  
'  MsgBox strTemplateName & vbCrLf & _
      "Band is Nothing = " & CStr(pTemplateBand Is Nothing) & vbCrLf & _
      "Table is nothing = " & CStr(pTemplateBand.AttributeTable Is Nothing)
  
129:   booUseDBase = strOutputFormat = "dBASE (*.dbf)"
  
131:   Set pSlopeAttTable = TopoCorridor.TopoCor_Functions.ReturnRasterTable(pTemplateBand)
'  Set pSlopeAttTable = pTemplateBand.AttributeTable
133:   Set pSlopeFields = pSlopeAttTable.Fields
134:   Set pSlopeValAttNames = New esriSystem.strArray
135:   Set pSlopeValArray = New esriSystem.VarArray
136:   Set collSlopeTables = New Collection
  
138:   For lngIndex = 0 To pSlopeFields.FieldCount - 1
139:     Set pSlopeField = pSlopeFields.Field(lngIndex)
140:     pSlopeValAttNames.Add pSlopeField.Name
141:     If StrComp(pSlopeField.Name, "Value", vbTextCompare) = 0 Then
142:       lngValueIndex = lngIndex
143:     End If
144:   Next lngIndex
  
146:   Set pSlopeCursor = pSlopeAttTable.Search(Nothing, True)
147:   Set pSlopeRow = pSlopeCursor.NextRow
148:   Do Until pSlopeRow Is Nothing
149:     Set pSlopeSubArray = New esriSystem.VarArray
150:     Set pSlopeValAttVals = New esriSystem.VarArray
    
152:     For lngIndex = 0 To pSlopeFields.FieldCount - 1
153:       varSlopeVal = pSlopeRow.Value(lngIndex)
154:       pSlopeValAttVals.Add varSlopeVal
      
156:       Set pSlopeField = pSlopeFields.Field(lngIndex)
157:       If lngIndex = lngValueIndex Then
158:         varIDValue = varSlopeVal
159:       End If
      
161:     Next lngIndex
    
163:     If booUseDBase Then
164:       strTempName = TopoCorridor.MyGeneralOperations.MakeUniquedBASEName(strWsName & _
          "\FuzzyData_Class" & CStr(varIDValue) & ".dbf")
166:     Else
167:       strTempName = TopoCorridor.MyGeneralOperations.MakeUniqueShapeFilename(strWsName & _
          "\FuzzyData_Class" & CStr(varIDValue) & ".csv")
169:     End If
    
171:     Set pAddFields = New esriSystem.VarArray
    
173:     pSlopeSubArray.Add varIDValue
174:     pSlopeSubArray.Add pSlopeValAttVals
175:     pSlopeSubArray.Add strWsName           ' DOES NOT END IN "\"
176:     pSlopeSubArray.Add TopoCorridor.aml_func_mod.ReturnFilename(strTempName)
177:     pSlopeSubArray.Add strTempName
178:     pSlopeSubArray.Add pAddFields
179:     pSlopeSubArray.Add False               ' WHETHER TABLE HAS BEEN CREATED YET
180:     pSlopeValArray.Add pSlopeSubArray
    
182:     collSlopeTables.Add pSlopeValArray.Count - 1, CStr(varIDValue)
 '   MsgBox "Value Field = " & pSlopeFields.Field(lngValueIndex).Name & vbCrLf & _
           "Raster Value = " & CStr(varIDValue) & vbCrLf & _
           "pSlopeValArray Index Value = " & CStr(collSlopeTables.Item(CStr(varIDValue)))
186:     Set pSlopeRow = pSlopeCursor.NextRow
187:   Loop
  
'  MsgBox "pSlopeValArray.Count = " & CStr(pSlopeValArray.Count)
  
  Dim lngTransformCount As Long
192:   lngTransformCount = 1 + (pAdditionalRasters.Count) + (pAdditionalFeatLayers.Count * 3)
  Dim lngTransformCounter As Long
194:   lngTransformCounter = 1
  
  ' FOLLOWING CODE SHOULD BE INSERTED INTO A FUNCTION.  NEEDS MINOR EDITS IF GOING INTO A SUBROUTINE
  ' SPECIFICALLY, CHANGE "Exit Function" to "Exit Sub" IN BAILOUT SECTION
  ' PROGRESS BAR STUFF
    ' PROGRESS BAR STUFF
  Dim psbar As IStatusBar
201:   Set psbar = pApp.StatusBar
  Dim pPro As IStepProgressor
203:   Set pPro = psbar.ProgressBar
  Dim theTimeBegan As Date
  Dim dateRunningTime As Date
206:   theTimeBegan = Now
207:   dateRunningTime = Now
  Dim strHeader As String
209:   strHeader = "Pass 1 of 2: Aligning Datasets"
  Dim strMessage As String
211:   strMessage = strHeader
  
  Dim frmProgress As New TopoCorridor.frmJenProgressMeterBasic
  Dim theDetailedDescription As String
  Dim theProgressSpecific As String

217:   frmProgress.SetExpanded = pThisExt.ProgressDialogSetExpanded
218:   frmProgress.SetExpanded = True
219:   frmProgress.SetAutoClose = pThisExt.ProgressDialogAutoClose

221:   theTimeBegan = Now
222:   frmProgress.ShouldContinue = True
223:   frmProgress.ProgBeginTime = Now
224:   frmProgress.ProgRecCount = lngTransformCount
225:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
226:   frmProgress.lblBeginTime.Caption = "Began Job: " & Format(theTimeBegan, "ttttt,  ddddd")

228:   theDetailedDescription = "Calculating..." & vbCrLf & _
      "Began Job: " & Format(theTimeBegan, "ttttt,  ddddd") & vbCrLf & _
      "-------------------------------------------" & vbCrLf
231:   frmProgress.txtDetails.Text = theDetailedDescription

233:   frmProgress.Frame.Caption = " Current Status:"
234:   frmProgress.Frame.Visible = True
235:   frmProgress.cmdDetails.Visible = True
236:   frmProgress.cmdStop.Visible = True
  
  ' ---- UPDATE PROGRESS METER
239:   DoEvents
240:   theProgressSpecific = "Working on Step " & CStr(lngTransformCounter) & " of " & CStr(lngTransformCount) & "..."
241:   theDetailedDescription = theDetailedDescription & _
    " --> " & theProgressSpecific & vbCrLf & _
    "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
244:   frmProgress.txtDetails.Text = theDetailedDescription
245:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)

247:   frmProgress.Est_Time_Left lngTransformCounter, "Pass 1 of 2:  Aligning '" & strTemplateName & "'...", "Calculating Data..."
248:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
249:   frmProgress.Refresh
250:   frmProgress.icnProgressLine.Refresh
251:   DoEvents
252:   If CS_BailOutOfProgress(frmProgress, pApp) Then

   ' ********************************************************************************************
    ' SIMPLE CODE
256:     ExportRastersToTable = "Exited Function Prematurely..." & vbCrLf

   ' ********************************************************************************************

260:     frmProgress.cmdStop.Caption = "Close"
261:     pPro.position = 0
262:     psbar.HideProgressBar
    Exit Function
264:   End If
  ' ---- END PROGRESS METER STUFF --------------------------------
  
  
268:   psbar.ShowProgressBar "Pass 1 of 2:  Aligning projection, extent and cell size for '" & strTemplateName & "'...", 0, lngTransformCount, 1, True
269:   pPro.MaxRange = lngTransformCount
270:   pPro.position = lngTransformCounter
271:   pPro.Show
  
  
  Dim pAddLayer As IRasterLayer
  
  Dim strReport As String
277:   strReport = "Export Data to R Function failed..."
  
279:   ExportRastersToTable = strReport
    
  Dim pAddFeatLayer As IFeatureLayer
  Dim pFeatClassDesc As IFeatureClassDescriptor
  Dim pFeatClass As IFeatureClass
  Dim pField As IField
  Dim pSubArray As esriSystem.IVariantArray
  
  Dim strAddName As String
  
''  ' DO ANALYSIS
''  lngWorkOrderCounter = lngWorkOrderCounter + 1
''  psbar.ShowProgressBar "Exporting " & pTemplate.Name & " to " & strTemplateName & "...", 1, lngWorkOrderCount, lngWorkOrderCounter, True
''  DoEvents
'
'  MsgBox "lngTransformCount = " & CStr(lngTransformCount) & vbCrLf & _
         "pAdditionalRasters.Count = " & CStr(pAdditionalRasters.Count) & vbCrLf & _
         "pAdditionalFeatLayers.Count = " & CStr(pAdditionalFeatLayers.Count)
  
  Dim pRastMakerOp As IRasterMakerOp
299:   Set pRastMakerOp = New RasterMakerOp
  Dim pRastAnEnv As IRasterAnalysisEnvironment
301:   Set pRastAnEnv = pRastMakerOp

  Dim dblCellSize As Double
304:   dblCellSize = (pTemplatePnt.X + pTemplatePnt.Y) / 2
  Dim pEnv As IEnvelope
306:   Set pEnv = pTemplateProps.Extent
  Dim pClone As IClone
308:   Set pClone = pEnv
309:   Set pEnv = pClone.Clone
310:   Set pClone = Nothing
311:   pEnv.Expand -(pTemplatePnt.X) / 10, -(pTemplatePnt.Y) / 10, False
312:   Set pRastAnEnv.OutSpatialReference = pTemplateSpRef
'   Set pRastAnEnv.OutWorkspace = pWS
314:   pRastAnEnv.SetCellSize esriRasterEnvValue, dblCellSize
315:   pRastAnEnv.SetExtent esriRasterEnvValue, pEnv, pTemplateRaster
316:   If booShouldClip Then
317:     Set pRastAnEnv.Mask = pClipLayer.FeatureClass
318:   End If
  
  Dim pOneDataset As IRasterDataset
  Dim pTempRaster As IRaster2
322:   Set pTempRaster = pRastMakerOp.MakeConstant(1, False)
323:   Set pOneDataset = pTempRaster.RasterDataset
  
  Dim pMathOp As IMathOp
326:   Set pMathOp = New RasterMathOps
  Dim pMathEnv As IRasterAnalysisEnvironment
328:   Set pMathEnv = pMathOp
329:   Set pMathEnv.OutSpatialReference = pTemplateSpRef
'   Set pMathEnv.OutWorkspace = pWS
331:   pMathEnv.SetCellSize esriRasterEnvValue, dblCellSize
332:   pMathEnv.SetExtent esriRasterEnvValue, pEnv, pTemplateRaster
333:   If booShouldClip Then
334:     Set pMathEnv.Mask = pClipLayer.FeatureClass
335:   End If
  
  Dim pInBandColl As IRasterBandCollection
338:   Set pInBandColl = New Raster
  
  Dim pAddBand As IRasterBand
  Dim pNewBand As IRasterBand
  Dim pNewDataset As IRasterDataset
  Dim pNewBandColl As IRasterBandCollection
    
345:   Set pTempRaster = pMathOp.Times(pTemplateBand, pOneDataset)
346:   Set pNewDataset = pTempRaster.RasterDataset
347:   Set pNewBandColl = pNewDataset
348:   Set pNewBand = pNewBandColl.Item(0)
349:   pInBandColl.AppendBand pNewBand
  ' FOR DEBUGGING
 ' MsgBox "Added Template:" & vbCrLf & "pInBandColl.Count = " & CStr(pInBandColl.Count)
    
  Dim pInRaster As IRaster
354:   Set pInRaster = pInBandColl
355:   pInRaster.ResampleMethod = RSP_NearestNeighbor
  Dim pInProps As IRasterProps
357:   Set pInProps = pInBandColl
358:   pInProps.SpatialReference = pTemplateSpRef
359:   pInProps.Extent = pTemplateProps.Extent
360:   pInProps.Height = pTemplateProps.Height
361:   pInProps.Width = pTemplateProps.Width
  
  ' ALIGN ADDITIONAL RASTERS
364:   For lngIndex = 0 To pAdditionalRasters.Count - 1
        
366:     Set pSubArray = pAdditionalRasters.Element(lngIndex)
367:     Set pAddBand = pSubArray.Element(0)
368:     strAddName = pSubArray.Element(1)
    
370:     lngTransformCounter = lngTransformCounter + 1
371:     pPro.MaxRange = lngTransformCount
372:     pPro.position = lngTransformCounter
373:     pPro.Message = "Pass 1 of 2:  Aligning projection, extent and cell size for '" & strAddName & "'..."
374:     pPro.Show
        
    ' ---- UPDATE PROGRESS METER
377:     DoEvents
378:     theProgressSpecific = "Working on Step " & CStr(lngTransformCounter) & " of " & CStr(lngTransformCount) & "..."
379:     theDetailedDescription = theDetailedDescription & _
      " --> " & theProgressSpecific & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
382:     frmProgress.txtDetails.Text = theDetailedDescription
383:     frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
  
385:     frmProgress.Est_Time_Left lngTransformCounter, "Pass 1 of 2:  Aligning '" & strAddName & "'...", "Calculating Data..."
386:     frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
387:     frmProgress.Refresh
388:     frmProgress.icnProgressLine.Refresh
389:     DoEvents
390:     If CS_BailOutOfProgress(frmProgress, pApp) Then
  
     ' ********************************************************************************************
      ' SIMPLE CODE
394:       ExportRastersToTable = "Exited Function Prematurely..." & vbCrLf
  
     ' ********************************************************************************************
  
398:       frmProgress.cmdStop.Caption = "Close"
399:       pPro.position = 0
400:       psbar.HideProgressBar
      Exit Function
402:     End If
    ' ---- END PROGRESS METER STUFF --------------------------------
    
405:     DoEvents
    
407:     Set pTempRaster = pMathOp.Times(pAddBand, pOneDataset)
408:     Set pNewDataset = pTempRaster.RasterDataset
409:     Set pNewBandColl = pNewDataset
410:     Set pNewBand = pNewBandColl.Item(0)
411:     pInBandColl.AppendBand pNewBand
  ' FOR DEBUGGING
'    MsgBox "Added Additional Raster #" & CStr(lngIndex) & ":" & vbCrLf & "pInBandColl.Count = " & CStr(pInBandColl.Count)
'    MsgBox "Position " & CStr(pPro.position) & " of " & CStr(pPro.MaxRange)

416:   Next lngIndex

  ' EXPORT ADDITIONAL FEATURE CLASSES
  Dim pOutputFNames As esriSystem.IStringArray
420:   Set pOutputFNames = New esriSystem.strArray
  Dim pTempConversionRasterDataset As IRasterDataset
  Dim pConOp As IConversionOp
423:   Set pConOp = New RasterConversionOp
  Dim pTempAnEnv As IRasterAnalysisEnvironment
425:   Set pTempAnEnv = pConOp
426:   Set pTempAnEnv.OutSpatialReference = pTemplateSpRef
  
'   Set pTempAnEnv.OutWorkspace = pWS
429:   pTempAnEnv.SetCellSize esriRasterEnvValue, dblCellSize
430:   pTempAnEnv.SetExtent esriRasterEnvValue, pEnv, pTemplateRaster
431:   If booShouldClip Then
432:     Set pTempAnEnv.Mask = pClipLayer.FeatureClass
433:   End If

  Dim pFeatGeoDataset As IGeoDataset
  Dim pTempRastDataset As IDataset
  
  Dim pRastWS As IRasterWorkspace
  Dim pWsFact As IWorkspaceFactory
440:   Set pWsFact = New RasterWorkspaceFactory
441:   Set pRastWS = pWsFact.OpenFromFile(pWS.PathName, 0)
  
  Dim booAlreadyHasInfoFolder As Boolean
444:   booAlreadyHasInfoFolder = TopoCorridor.aml_func_mod.ExistFileDir(strWsName & "\info")
  
446:   For lngIndex = 0 To pAdditionalFeatLayers.Count - 1
447:     Set pSubArray = pAdditionalFeatLayers.Element(lngIndex)
448:     Set pAddFeatLayer = pSubArray.Element(0)
449:     Set pField = pSubArray.Element(1)
450:     Set pFeatClass = pAddFeatLayer.FeatureClass
451:     Set pFeatGeoDataset = pFeatClass
    
453:     lngTransformCounter = lngTransformCounter + 1
454:     pPro.MaxRange = lngTransformCount
455:     pPro.position = lngTransformCounter
456:     pPro.Message = "Pass 1 of 2:  Aligning projection, extent and cell size for '" & pAddFeatLayer.Name & "'..."
457:     pPro.Show
        
    ' ---- UPDATE PROGRESS METER
460:     DoEvents
461:     theProgressSpecific = "Working on Step " & CStr(lngTransformCounter) & " of " & CStr(lngTransformCount) & "..."
462:     theDetailedDescription = theDetailedDescription & _
      " --> " & theProgressSpecific & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
465:     frmProgress.txtDetails.Text = theDetailedDescription
466:     frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
  
468:     frmProgress.Est_Time_Left lngTransformCounter, "Pass 1 of 2:  Aligning '" & pAddFeatLayer.Name & "'...", "Calculating Data..."
469:     frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
470:     frmProgress.Refresh
471:     frmProgress.icnProgressLine.Refresh
472:     DoEvents
473:     If CS_BailOutOfProgress(frmProgress, pApp) Then
  
     ' ********************************************************************************************
      ' SIMPLE CODE
477:       ExportRastersToTable = "Exited Function Prematurely..." & vbCrLf
  
     ' ********************************************************************************************
  
481:       frmProgress.cmdStop.Caption = "Close"
482:       pPro.position = 0
483:       psbar.HideProgressBar
      Exit Function
485:     End If
    ' ---- END PROGRESS METER STUFF --------------------------------
487:     DoEvents

489:     Set pFeatClassDesc = New FeatureClassDescriptor
490:     pFeatClassDesc.Create pFeatClass, Nothing, pField.Name
    
492:     strTempName = TopoCorridor.MyGeneralOperations.MakeUniqueShapeFilename(strWsName & "\" & "CS_Temp")
493:     strTempName = TopoCorridor.aml_func_mod.ReturnFilename(strTempName)
  '  MsgBox "strTempName = " & strTempName & vbCrLf & _
      "strWsName = " & strWsName
496:     Set pTempConversionRasterDataset = pConOp.ToRasterDataset(pFeatClassDesc, "GRID", pRastWS, strTempName)
        
498:     lngTransformCounter = lngTransformCounter + 1
499:     pPro.MaxRange = lngTransformCount
500:     pPro.position = lngTransformCounter
501:     pPro.Message = "Pass 1 of 2:  Aligning projection, extent and cell size for '" & pAddFeatLayer.Name & "'..."
502:     pPro.Show
    
    
    
    ' ---- UPDATE PROGRESS METER
507:     DoEvents
508:     theProgressSpecific = "Working on Step " & CStr(lngTransformCounter) & " of " & CStr(lngTransformCount) & "..."
509:     theDetailedDescription = theDetailedDescription & _
      " --> " & theProgressSpecific & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
512:     frmProgress.txtDetails.Text = theDetailedDescription
513:     frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
  
515:     frmProgress.Est_Time_Left lngTransformCounter, "Pass 1 of 2:  Aligning '" & pAddFeatLayer.Name & "'...", "Calculating Data..."
516:     frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
517:     frmProgress.Refresh
518:     frmProgress.icnProgressLine.Refresh
519:     DoEvents
520:     If CS_BailOutOfProgress(frmProgress, pApp) Then
  
     ' ********************************************************************************************
      ' SIMPLE CODE
524:       ExportRastersToTable = "Exited Function Prematurely..." & vbCrLf
  
     ' ********************************************************************************************
  
528:       frmProgress.cmdStop.Caption = "Close"
529:       pPro.position = 0
530:       psbar.HideProgressBar
      Exit Function
532:     End If
    ' ---- END PROGRESS METER STUFF --------------------------------
534:     DoEvents
    
536:     Set pTempRastDataset = pTempConversionRasterDataset
537:     Set pTempRaster = pMathOp.Float(pTempConversionRasterDataset)
538:     Set pNewDataset = pTempRaster.RasterDataset
    
    
        
542:     lngTransformCounter = lngTransformCounter + 1
543:     pPro.MaxRange = lngTransformCount
544:     pPro.position = lngTransformCounter
545:     pPro.Message = "Pass 1 of 2:  Aligning projection, extent and cell size for '" & pAddFeatLayer.Name & "'..."
546:     pPro.Show
    
    
    
    ' ---- UPDATE PROGRESS METER
551:     DoEvents
552:     theProgressSpecific = "Working on Step " & CStr(lngTransformCounter) & " of " & CStr(lngTransformCount) & "..."
553:     theDetailedDescription = theDetailedDescription & _
      " --> " & theProgressSpecific & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
556:     frmProgress.txtDetails.Text = theDetailedDescription
557:     frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
  
559:     frmProgress.Est_Time_Left lngTransformCounter, "Pass 1 of 2:  Aligning '" & pAddFeatLayer.Name & "'...", "Calculating Data..."
560:     frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
561:     frmProgress.Refresh
562:     frmProgress.icnProgressLine.Refresh
563:     DoEvents
564:     If CS_BailOutOfProgress(frmProgress, pApp) Then
  
     ' ********************************************************************************************
      ' SIMPLE CODE
568:       ExportRastersToTable = "Exited Function Prematurely..." & vbCrLf
  
     ' ********************************************************************************************
  
572:       frmProgress.cmdStop.Caption = "Close"
573:       pPro.position = 0
574:       psbar.HideProgressBar
      Exit Function
576:     End If
    ' ---- END PROGRESS METER STUFF --------------------------------
578:     DoEvents
    
    
581:     Set pTempRaster = pMathOp.Times(pNewDataset, pOneDataset)
582:     Set pNewDataset = pTempRaster.RasterDataset
    
    ' for debugging
'    Dim pTempLayer As IRasterLayer
'    Set pTempLayer = New RasterLayer
'    pTempLayer.CreateFromDataset pNewDataset
'    pMxDoc.FocusMap.AddLayer pTempLayer
    
590:     Set pNewBandColl = pNewDataset
591:     Set pNewBand = pNewBandColl.Item(0)
    
593:     pNewBand.ComputeStatsAndHist
594:     pInBandColl.AppendBand pNewBand
  ' FOR DEBUGGING
'    MsgBox "Added Additional Feature #" & CStr(lngIndex) & ":" & vbCrLf & "pInBandColl.Count = " & CStr(pInBandColl.Count)

598:     Set pTempConversionRasterDataset = Nothing
599:     If pTempRastDataset.CanDelete Then
600:       pTempRastDataset.DELETE
601:     End If
602:     Set pTempRastDataset = Nothing

604:   Next lngIndex

606:   pRastAnEnv.RestoreToPreviousDefaultEnvironment
  
  ' PREPARE OUTPUT DATA
  Dim strFieldOutputReport As String
610:   strFieldOutputReport = "\par \b All tables contain the following attribute fields:\b0 " & vbNewLine
 
  Dim pAddFieldNames As esriSystem.IVariantArray
613:   Set pAddFieldNames = New esriSystem.VarArray
  Dim strFieldName As String
  
  ' DON'T ADD TEMPLATE TO OUTPUT DATA!
  ' MODIFIED APRIL 1 TO SPLIT OUTPUT INTO SEPARATE TABLES FOR EACH UNIQUE SLOPE CLASS VALUE
  ' SLOPE CLASSES TAKEN FROM TEMPLATE RASTER
'  strTempName = strTemplateName
'  strFieldName = TopoCorridor.MyGeneralOperations.ReturnDBASEFieldName(strTempName, pAddFieldNames)
'  strFieldOutputReport = strFieldOutputReport & _
'      "\par \pard   --> [" & strFieldName & "], From Data Source '" & strTempName & "'" & vbNewLine
'  pAddFieldNames.Add strFieldName
  
625:   For lngIndex = 0 To pAdditionalRasters.Count - 1
626:     Set pSubArray = pAdditionalRasters.Element(lngIndex)
627:     strTempName = pSubArray.Element(1)
628:     strFieldName = TopoCorridor.MyGeneralOperations.ReturnDBASEFieldName(strTempName, pAddFieldNames)
629:     strFieldOutputReport = strFieldOutputReport & _
        "\par \pard   --> [" & strFieldName & "], From Data Source '" & strTempName & "'" & vbNewLine
631:     pAddFieldNames.Add strFieldName
632:   Next lngIndex
  
634:   For lngIndex = 0 To pAdditionalFeatLayers.Count - 1
635:     Set pSubArray = pAdditionalFeatLayers.Element(lngIndex)
636:     Set pAddFeatLayer = pSubArray.Element(0)
637:     Set pField = pSubArray.Element(1)
'    strTempName = pField.Name
639:     strTempName = pAddFeatLayer.Name
640:     strFieldName = TopoCorridor.MyGeneralOperations.ReturnDBASEFieldName(strTempName, pAddFieldNames)
641:     strFieldOutputReport = strFieldOutputReport & _
        "\par \pard   --> [" & strFieldName & "], From Data Source '" & strTempName & _
        "',  Field = [" & pField.Name & "]" & vbNewLine
644:     pAddFieldNames.Add strFieldName
645:   Next lngIndex
  
  Dim pFieldEdit As IFieldEdit
  
  Dim pTable As ITable
  Dim pFieldIndexes As esriSystem.ILongArray
  Dim lngFieldIndex As Long
  
  Dim lngFormatStyle As Long          ' 0 = dBASE, 1 = CSV
  
  Dim lngFileNumber As Long
  Dim strRow As String
  Dim strTestOutput As String
  
  Dim pCursor As ICursor
  Dim pBuffer As IRowBuffer
  
'  MsgBox "pAddFieldNames.Count-1 = " & CStr(pAddFieldNames.Count - 1)
  
664:   For lngIndex = 0 To pSlopeValArray.Count - 1
665:     Set pSlopeSubArray = pSlopeValArray.Element(lngIndex)
666:     Set pAddFields = pSlopeSubArray.Element(5)
    
668:     If booUseDBase Then
669:       lngFormatStyle = 0
670:       Set pAddFields = New esriSystem.VarArray
671:       For lngIndex2 = 0 To pAddFieldNames.Count - 1
672:         strFieldName = pAddFieldNames.Element(lngIndex2)
673:         Set pField = New Field
674:         Set pFieldEdit = pField
675:         With pFieldEdit
676:           .Name = strFieldName
677:           .Type = esriFieldTypeDouble
678:           .Precision = 12
679:           .Scale = 8
680:         End With
681:         pAddFields.Add pField
682:       Next lngIndex2
      
'      Set pTable = TopoCorridor.MyGeneralOperations.CreatedBASETableInFolder2(pWS.PathName, strFullFilename, _
           pAddFields, False)
      
'      Set pFieldIndexes = New esriSystem.LongArray
'      For lngIndex2 = 0 To pAddFields.Count - 1
'        Set pField = pAddFields.Element(lngIndex2)
'        strFieldName = pField.Name
'        lngFieldIndex = pTable.FindField(strFieldName)
'        pFieldIndexes.Add lngFieldIndex
'      Next lngIndex2
'
'
'      Set pBuffer = pTable.CreateRowBuffer
'      Set pCursor = pTable.Insert(True)
698:       pSlopeSubArray.Remove (5)
699:       pSlopeSubArray.Insert 5, pAddFields
700:     ElseIf strOutputFormat = "Comma-Delimited (*.csv)" Then
701:       lngFormatStyle = 1
'      lngFileNumber = FreeFile(0)
'      Open strFullFilePath For Output Access Write As #lngFileNumber
'      strRow = ""
'      For lngIndex2 = 0 To pAddFieldNames.Count - 1
'        strFieldName = pAddFieldNames.Element(lngIndex2)
'        strRow = strRow & strFieldName & ","
'      Next lngIndex2
'      strRow = Left(strRow, Len(strRow) - 1)
'      Print #lngFileNumber, strRow
711:     End If
712:   Next lngIndex
  
  ' FOR DEBUGGING
'  Dim strTemp As String
'  strTemp = ""
'  For lngIndex = 0 To pAddFields.Count - 1
'    Set pField = pAddFields.Element(lngIndex)
'    strFieldName = pField.Name
'    lngFieldIndex = pFieldIndexes.Element(lngIndex)
'    strTemp = strTemp & CStr(lngIndex) & "] " & strFieldName & " at Index " & CStr(lngFieldIndex) & vbCrLf
'  Next lngIndex
'  Dim pEnum As IEnumRasterBand
'  Set pEnum = pInBandColl.Bands
  Dim pTempBand As IRasterBand
'  pEnum.Reset
'  lngIndex = 0
'  Set pTempBand = pEnum.Next
'  Do Until pTempBand Is Nothing
'    If Not pTempBand Is Nothing Then lngIndex = lngIndex + 1
'    Set pTempBand = pEnum.Next
'  Loop
'  MsgBox strTemp & vbCrLf & vbCrLf & "pInBandColl.Count = " & CStr(pInBandColl.Count) & vbCrLf & _
      "Band Count = " & CStr(lngIndex) & " using BandEnum method.", , "Field indices:"
  
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  '/////////// |                  | \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
  '----------  |   CALCULATIONS   |  ---------------------------------------------------------------------------
  '\\\\\\\\\\\ |                  | ///////////////////////////////////////////////////////////////////////////
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  Dim pRastProps As IRasterProps
743:   Set pRastProps = pInRaster
  
  Dim lngHeight As Long
  Dim lngWidth As Long
  Dim dblNoData As Double
  Dim varNoData As Variant
  
750:   lngWidth = pRastProps.Width
751:   lngHeight = pRastProps.Height
752:   varNoData = pRastProps.NoDataValue
  
754:   psbar.ShowProgressBar "Pass 2 of 2:  Extracting Cell Values...", 0, lngHeight, 1, True
755:   pPro.position = 1
756:   pPro.Show
757:   frmProgress.ProgRecCount = lngHeight
  
  Dim pPnt As IPnt
760:   Set pPnt = New Pnt
761:   pPnt.SetCoords lngWidth, 1

  Dim pPixBlock As IPixelBlock3

  Dim pTopLeft As IPnt

  Dim varNoDataArray As Variant
  Dim booHasData() As Boolean
  Dim varData As Variant
  Dim varAllData() As Variant
  Dim varEmptyData() As Variant
  ReDim varEmptyData(lngWidth - 1)
  
  ' FOR DEBUGGING
'  Dim strDebugReport As String
'  strDebugReport = ""
'  For lngIndex = 0 To (lngNbrHoodRadius * 2)
'    For lngIndex2 = 0 To (lngNbrHoodRadius * 2)
'      strDebugReport = strDebugReport & Left(UCase(CStr(booMaskArray(lngIndex, lngIndex2))), 1) & "  "
'    Next lngIndex2
'    strDebugReport = strDebugReport & vbCrLf
'  Next lngIndex
 
  ' ADD FIRST LINE OF DATA
  Dim pRastCursor As IRasterCursor
  Dim pInputRaster As IRaster2
787:   Set pInputRaster = pInRaster
788:   Set pRastCursor = pInputRaster.CreateCursorEx(pPnt)

790:   Set pPixBlock = pRastCursor.PixelBlock
791:   pPixBlock.PixelType(0) = PT_FLOAT
792:   pRastCursor.Reset
793:   Set pPixBlock = pRastCursor.PixelBlock
794:   varData = pPixBlock.PixelData(0)
  
  Dim lngNumBands As Long
797:   lngNumBands = pInBandColl.Count - 1
  Dim booAllHasData() As Boolean
  ReDim booAllHasData(lngWidth - 1)
  ReDim varAllData(lngWidth - 1, lngNumBands)   ' BAND NUMBER IS FIRST INDEX, VALUE IS SECOND INDEX
  
  Dim varNoDataVals() As Variant
  ReDim varNoDataVals(lngNumBands)
  Dim pTempProps As IRasterProps
805:   For lngIndex = 0 To lngNumBands
806:     Set pTempBand = pInBandColl.Item(lngIndex)
807:     Set pTempProps = pTempBand
808:     varNoDataVals(lngIndex) = CDbl(pTempProps.NoDataValue)
'    MsgBox "Band #" & CStr(lngIndex) & ":  NoData = " & CStr(pTempProps.NoDataValue), , "Checking NoData Values:"
810:   Next lngIndex
  
  ' RESET HAS-DATA ARRAY TO ALL TRUE
813:   For lngIndex = 0 To lngWidth - 1
814:     booAllHasData(lngIndex) = True
815:   Next lngIndex
  ' CHECK HAS DATA ARRAYS FOR ALL LEVELS; RESET booAllHasData TO FALSE IF ANY VALUES ARE FALSE
817:   For lngIndex = 0 To lngNumBands
818:     booHasData = ConvertNoDataMaskToSomethingUseful(pPixBlock.NoDataMask(0))
819:     For lngIndex2 = 0 To lngWidth - 1
820:       If booHasData(lngIndex2) = False Then booAllHasData(lngIndex2) = False
821:     Next lngIndex2
822:   Next lngIndex
'
'  ' BELOW IS WORKAROUND FOR PROBLEM IN WHICH PIXEL BLOCK APPEARS TO NOT BE ABLE TO CORRECTLY READ FIRST ROW.
'  ' I THINK THIS HAS SOMETHING TO DO WITH SETTING THE PIXEL TYPE TO PT_FLOAT.  IT READS IT CORRECTLY AFTER
'  ' THAT POINT, BUT I CANNOT SET IT BEFORE READING THE FIRST ROW FROM RASTER CURSOR, EVEN AFTER USING 'RESET'.
'  ' CODE BELOW SIMPLY EXTRACTS varData1 and booHasData1 FROM A DIFFERENT PIXEL BLOCK THAT IS READ CORRECTLY.
  Dim pTempPixBlock As IPixelBlock3
  Dim pTemporaryRaster As IRaster
830:   Set pTemporaryRaster = pInputRaster
  Dim pTempPoint As IPnt
832:   Set pTempPoint = New Pnt
833:   pTempPoint.SetCoords CInt(lngWidth), 1
834:   Set pTempPixBlock = pTemporaryRaster.CreatePixelBlock(pTempPoint)
835:   pTempPixBlock.PixelType(0) = PT_FLOAT
  Dim pTempOrigin As IPnt
837:   Set pTempOrigin = New Pnt

  Dim lngRowCounter As Long
840:   lngRowCounter = -1
  Dim varVal As Variant
  Dim dblVal As Double

844:   pTempOrigin.SetCoords 0, 0
845:   pTemporaryRaster.Read pTempOrigin, pTempPixBlock
  
'  ' FOR DEBUGGING
'  strTestOutput = TopoCorridor.MyGeneralOperations.MakeUniqueShapeFilename(strWsName & "\test.txt")
'  lngFileNumber = FreeFile(0)
'  Open strTestOutput For Output Access Write As #lngFileNumber
'  varData = pTempPixBlock.PixelDataByRef(0)
'  For lngIndex = 0 To UBound(varData, 2)
'    strRow = ""
'    For lngIndex2 = 0 To UBound(varData, 1)
'      strRow = strRow & CStr(varData(lngIndex2, lngIndex)) & ","
'    Next lngIndex2
'    strRow = Left(strRow, Len(strRow) - 1) & "   ...   " & vbCrLf
'    Print #lngFileNumber, strRow
'  Next lngIndex
'  Close #lngFileNumber
'  booHasData = ConvertNoDataMaskToSomethingUseful(pTempPixBlock.NoDataMaskByRef(0))

  ' RESET HAS-DATA ARRAY TO ALL TRUE
864:   For lngIndex = 0 To lngWidth - 1
865:     pTempPixBlock.PixelType(lngIndex) = PT_FLOAT
866:     booAllHasData(lngIndex) = True
867:   Next lngIndex
  ' CHECK HAS DATA ARRAYS FOR ALL LEVELS; RESET booAllHasData TO FALSE IF ANY VALUES ARE FALSE
869:   For lngIndex = 0 To lngNumBands
870:     varData = pTempPixBlock.PixelDataByRef(lngIndex)
    
'    ' FOR DEBUGGING
'    strTestOutput = TopoCorridor.MyGeneralOperations.MakeUniqueShapeFilename(strWsName & _
'        "\varData_" & CStr(lngIndex) & ".txt")
'    lngFileNumber = FreeFile(0)
'    Open strTestOutput For Output Access Write As #lngFileNumber
'    For lngIndex2 = 0 To UBound(varData, 1)
'      strRow = ""
'      For lngIndex3 = 0 To UBound(varData, 2)
'        strRow = strRow & CStr(varData(lngIndex2, lngIndex3)) & ","
'      Next lngIndex3
'      strRow = Left(strRow, Len(strRow) - 1)
'      Print #lngFileNumber, strRow
'    Next lngIndex2
'    Close #lngFileNumber
    
887:     booHasData = ConvertNoDataMaskToSomethingUseful(pTempPixBlock.NoDataMask(lngIndex))
888:     For lngIndex2 = 0 To lngWidth - 1
889:       varAllData(lngIndex2, lngIndex) = varData(lngIndex2, 0)
890:       If booHasData(lngIndex2) = False Then booAllHasData(lngIndex2) = False
891:     Next lngIndex2
    
893:   Next lngIndex

  ' FOR DEBUGGING
'  strTestOutput = TopoCorridor.MyGeneralOperations.MakeUniqueShapeFilename(strWsName & _
'      "\varAllData.txt")
'  lngFileNumber = FreeFile(0)
'  Open strTestOutput For Output Access Write As #lngFileNumber
'  For lngIndex2 = 0 To UBound(varAllData, 1)
'    strRow = ""
'    For lngIndex3 = 0 To UBound(varAllData, 2)
'      strRow = strRow & CStr(varAllData(lngIndex2, lngIndex3)) & ","
'    Next lngIndex3
'    strRow = Left(strRow, Len(strRow) - 1)
'    Print #lngFileNumber, strRow
'  Next lngIndex2
'  Close #lngFileNumber
  
  ' FROM HERE ON OUT, RASTER CURSOR SEEMS TO WORK

  ' IF lngFocalRow ACTUALLY IS lngHeight, THEN IT IS ONE ROW TOO FAR.
  Dim lngFocalRow As Long
914:   lngFocalRow = 0
  Dim lngLastColumn As Long
916:   lngLastColumn = lngWidth - 1
  Dim dblOut As Double
  Dim booAddThisRow As Boolean
  
  Dim datRunningTime As Date
921:   datRunningTime = Now
'    ' UPDATE PROGRESS BAR MESSAGE ONCE PER 2 SECONDS
'    If (lngTimer = lngValCount) Or DateDiff("s", dateRunningTime, Now) >= 1 Then
  
  
926:   Do Until lngFocalRow = lngHeight
927:     pPro.Step
928:     DoEvents

930:     For lngIndex = 0 To lngLastColumn

  '    If MsgBox("lngFocalRow = " & CStr(lngFocalRow) & vbCrLf & _
        "booAllHasData(lngIndex = " & CStr(lngIndex) & ") = " & CStr(booAllHasData(lngIndex)), vbYesNo) = vbYes Then Exit Function

'      dblOut = -9999
936:       If booAllHasData(lngIndex) Then

        ' GET DATA FROM ALL BANDS; IF ANY ARE NODATA, THEN DO NOT ADD ROW TO OUTPUT
939:         booAddThisRow = True
940:         strRow = ""
        
        ' GET SLOPE CLASS VALUE, AND ASSOCIATED TABLES
943:         varIDValue = varAllData(lngIndex, 0)
944:         If IsNull(varIDValue) Then
945:           booAddThisRow = False
946:           Exit For
947:         Else
          
          ' GET ASSOCIATED TABLES
950:           lngSlopeDataIndex = collSlopeTables.Item(CStr(varIDValue))
951:           Set pSlopeSubArray = pSlopeValArray.Element(lngSlopeDataIndex)
952:           booTableCreated = pSlopeSubArray.Element(6)
953:           If booTableCreated Then
954:             If booUseDBase Then
955:               Set pTable = pSlopeSubArray.Element(7)
956:               Set pFieldIndexes = pSlopeSubArray.Element(8)
957:               Set pCursor = pSlopeSubArray.Element(9)
958:               Set pBuffer = pSlopeSubArray.Element(10)
959:             Else
960:               lngFileNumber = pSlopeSubArray.Element(7)
961:             End If
962:           Else
963:             If booUseDBase Then
964:               pSlopeSubArray.Remove (6)
965:               pSlopeSubArray.Add True
              
              ' MAKE NEW TABLE
968:               strFullFilename = pSlopeSubArray.Element(3)
969:               Set pAddFields = pSlopeSubArray.Element(5)
970:               Set pTable = TopoCorridor.MyGeneralOperations.CreatedBASETableInFolder2(pWS.PathName, strFullFilename, _
                   pAddFields, False)
              
973:               Set pFieldIndexes = New esriSystem.LongArray
974:               For lngIndex2 = 0 To pAddFields.Count - 1
975:                 Set pField = pAddFields.Element(lngIndex2)
976:                 strFieldName = pField.Name
977:                 lngFieldIndex = pTable.FindField(strFieldName)
978:                 pFieldIndexes.Add lngFieldIndex
979:               Next lngIndex2
        
981:               Set pBuffer = pTable.CreateRowBuffer           ' <---------------
982:               Set pCursor = pTable.Insert(True)
              
984:               pSlopeSubArray.Add pTable
985:               pSlopeSubArray.Add pFieldIndexes
986:               pSlopeSubArray.Add pCursor
987:               pSlopeSubArray.Add pBuffer
            
989:             Else
990:               pSlopeSubArray.Remove (6)
991:               pSlopeSubArray.Add True
              
993:               strFullFilePath = pSlopeSubArray.Element(4)
994:               lngFileNumber = FreeFile(0)
995:               Open strFullFilePath For Output Access Write As #lngFileNumber
996:               strRow = ""
997:               For lngIndex2 = 0 To pAddFieldNames.Count - 1
998:                 strFieldName = pAddFieldNames.Element(lngIndex2)
999:                 strRow = strRow & strFieldName & ","
1000:               Next lngIndex2
1001:               strRow = Left(strRow, Len(strRow) - 1)
1002:               Print #lngFileNumber, strRow
1003:               pSlopeSubArray.Add lngFileNumber
1004:               strRow = ""
1005:             End If
1006:           End If
          
          ' EXTRACT VALUES FOR ALL OTHER NON-SLOPE-CLASS RASTER BANDS
1009:           For lngIndex2 = 1 To lngNumBands
          
1011:             varVal = varAllData(lngIndex, lngIndex2)
              
  '          If MsgBox("lngFocalRow = " & CStr(lngFocalRow) & vbCrLf & _
              "booAllHasData(lngIndex = " & CStr(lngIndex) & ") = " & CStr(booAllHasData(lngIndex)) & vbCrLf & _
              "Not IsNull(varVal) = " & CStr(Not IsNull(varVal)), vbYesNo) = vbYes Then Exit Function
1016:             If IsNull(varVal) Then
1017:               booAddThisRow = False
1018:               Exit For
1019:             Else
1020:               varNoData = varNoDataVals(lngIndex2)
1021:               If varNoData = varVal Then
1022:                 booAddThisRow = False
1023:                 Exit For
1024:               Else
    
  '            If MsgBox("lngFocalRow = " & CStr(lngFocalRow) & ", Band " & CStr(lngIndex2) & vbCrLf & _
                "booAllHasData(lngIndex = " & CStr(lngIndex) & ") = " & CStr(booAllHasData(lngIndex)) & vbCrLf & _
                "Not IsNull(varVal) = " & CStr(Not IsNull(varVal)) & vbCrLf & _
                "dblVal = " & CStr(dblVal), vbYesNo) = vbYes Then Exit Function
      
1031:                 dblVal = CDbl(varVal)
                       
1033:                 If lngFormatStyle = 0 Then              ' dBASE
1034:                   lngFieldIndex = pFieldIndexes.Element(lngIndex2 - 1)     ' <---------------
1035:                   pBuffer.Value(lngFieldIndex) = dblVal
1036:                 ElseIf lngFormatStyle = 1 Then          ' CSV
1037:                   strRow = strRow & CStr(Format(dblVal, "0.00000000")) & ","
1038:                 End If
1039:               End If
1040:             End If
1041:           Next lngIndex2
1042:         End If
        
1044:         If booAddThisRow Then
1045:           If lngFormatStyle = 0 Then             ' dBASE
1046:             pCursor.InsertRow pBuffer
1047:           ElseIf lngFormatStyle = 1 Then          ' CSV
1048:             strRow = Left(strRow, Len(strRow) - 1)
1049:             Print #lngFileNumber, strRow
1050:           End If
1051:         End If
1052:       End If
      
      ' ---- UPDATE PROGRESS METER
1055:       If (lngFocalRow = lngHeight) Or DateDiff("s", dateRunningTime, Now) >= 2 Then
1056:         dateRunningTime = Now
1057:         DoEvents
1058:         theProgressSpecific = "Working on Raster Row " & CStr(lngFocalRow) & " of " & CStr(lngHeight) & "..."
1059:         theDetailedDescription = theDetailedDescription & _
          " --> " & theProgressSpecific & vbCrLf & _
          "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
1062:         frmProgress.txtDetails.Text = theDetailedDescription
1063:         frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
      
1065:         frmProgress.Est_Time_Left lngFocalRow, "Pass 2 of 2:  Extracting Cell Values...", "Extracting Data: Row "
1066:         frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
1067:         frmProgress.Refresh
1068:         frmProgress.icnProgressLine.Refresh
1069:         DoEvents
1070:         If CS_BailOutOfProgress(frmProgress, pApp) Then
      
         ' ********************************************************************************************
          ' SIMPLE CODE
1074:           ExportRastersToTable = "Exited Function Prematurely..." & vbCrLf
      
         ' ********************************************************************************************
      
1078:           frmProgress.cmdStop.Caption = "Close"
1079:           pPro.position = 0
1080:           psbar.HideProgressBar
          Exit Function
1082:         End If
        ' ---- END PROGRESS METER STUFF --------------------------------
1084:         DoEvents
1085:       End If
1086:     Next lngIndex
    
    ' FLUSH CURRENT SET OF DATA TO TABLES
1089:     If lngFormatStyle = 0 Then
1090:       For lngIndex = 0 To pSlopeValArray.Count - 1
1091:         Set pSlopeSubArray = pSlopeValArray.Element(lngIndex)
1092:         booTableCreated = pSlopeSubArray.Element(6)
1093:         If booTableCreated Then
1094:           Set pCursor = pSlopeSubArray.Element(9)
1095:           pCursor.Flush
1096:         End If
1097:       Next lngIndex
1098:     End If
    
    ' MOVE TO NEXT RASTER ROW
1101:     lngFocalRow = lngFocalRow + 1

1103:     pRastCursor.Next
1104:     Set pPixBlock = pRastCursor.PixelBlock
        
'    pPixBlock.PixelType(0) = PT_FLOAT
'    varData = pPixBlock.PixelDataByRef(0)
'    booHasData = ConvertNoDataMaskToSomethingUseful(pPixBlock.NoDataMaskByRef(0))

    ' RESET HAS-DATA ARRAY TO ALL TRUE
1111:     For lngIndex = 0 To lngWidth - 1
1112:       pPixBlock.PixelType(lngIndex) = PT_FLOAT
1113:       booAllHasData(lngIndex) = True
1114:     Next lngIndex
    ' CHECK HAS DATA ARRAYS FOR ALL LEVELS; RESET booAllHasData TO FALSE IF ANY VALUES ARE FALSE
1116:     For lngIndex = 0 To lngNumBands
1117:       varData = pPixBlock.PixelDataByRef(lngIndex)
1118:       booHasData = ConvertNoDataMaskToSomethingUseful(pPixBlock.NoDataMaskByRef(lngIndex))
1119:       For lngIndex2 = 0 To lngWidth - 1
1120:         varAllData(lngIndex2, lngIndex) = varData(lngIndex2, 0)
1121:         If booHasData(lngIndex2) = False Then booAllHasData(lngIndex2) = False
1122:       Next lngIndex2
1123:     Next lngIndex
    
1125:   Loop
  
  Dim pStandaloneTable As IStandaloneTable
  Dim pStTabColl As IStandaloneTableCollection
  Dim pTableWindow2 As ITableWindow2
  
  Dim strOutputString As String
1132:   If lngFormatStyle = 0 Then              ' dBASE
1133:     strOutputString = "\par \b Output Tables:\b0    [dBASE format, *.dbf]" & vbNewLine
1134:   Else
1135:     strOutputString = "\par \b Output Tables:\b0    [Comma-delimited text format, *.csv]" & vbNewLine
1136:   End If
  
1138:   For lngIndex = 0 To pSlopeValArray.Count - 1
1139:     Set pSlopeSubArray = pSlopeValArray.Element(lngIndex)
    
1141:     booTableCreated = pSlopeSubArray.Element(6)
1142:     If booTableCreated Then
1143:       varIDValue = pSlopeSubArray.Element(0)
1144:       Set pSlopeValAttVals = pSlopeSubArray.Element(1)
1145:       strTempName = pSlopeSubArray.Element(4)
      
1147:       strOutputString = strOutputString & _
        "\par   Slope Class Value = \b " & CStr(varIDValue) & "\b0" & vbNewLine
1149:       For lngIndex2 = 0 To pSlopeValAttVals.Count - 1
1150:         strOutputString = strOutputString & _
          "\par        aka [" & pSlopeValAttNames.Element(lngIndex2) & "] = " & CStr(pSlopeValAttVals.Element(lngIndex2)) & vbNewLine
1152:       Next lngIndex2
1153:       strOutputString = strOutputString & "\par       Saved To " & Replace(strTempName, "\", "\\") & vbNewLine
1154:       strOutputString = strOutputString & "\par " & vbNewLine
      
1156:       If lngFormatStyle = 0 Then              ' dBASE
        
1158:         Set pTable = pSlopeSubArray.Element(7)
1159:         Set pCursor = pSlopeSubArray.Element(9)
1160:         pCursor.Flush
1161:         Set pStTabColl = pMxDoc.FocusMap
1162:         Set pStandaloneTable = New StandaloneTable
1163:         Set pStandaloneTable.Table = pTable
1164:         pStTabColl.AddStandaloneTable pStandaloneTable
1165:         pMxDoc.UpdateContents
        
1167:         Set pTableWindow2 = New TableWindow
1168:         With pTableWindow2
1169:           Set .StandaloneTable = pStandaloneTable
1170:           Set .Application = pApp
1171:           .TableSelectionAction = esriSelectFeatures
1172:           .ShowAliasNamesInColumnHeadings = True
1173:           .ShowSelected = False
1174:           .Show True
1175:         End With
        
1177:       ElseIf lngFormatStyle = 1 Then          ' CSV
1178:         lngFileNumber = pSlopeSubArray.Element(7)
1179:         Close #lngFileNumber
1180:       End If
      
1182:     End If
1183:   Next lngIndex
  
  ' FOR DEBUGGING
  Dim pCombineLayer As IRasterLayer
1187:   If booAddCombinedToMap Then
1188:     Set pCombineLayer = New RasterLayer
1189:     pCombineLayer.CreateFromRaster pInRaster
1190:     pCombineLayer.Name = "Combined"
1191:     pMxDoc.AddLayer pCombineLayer
1192:   End If
  
1194:   psbar.HideProgressBar
1195:   pPro.position = 0
1196:   If frmProgress.chkClose.Value = 1 Then
1197:     frmProgress.Frame.Visible = False
1198:     Set frmProgress = Nothing
1199:   Else
1200:     theProgressSpecific = "Calculations Complete..."
1201:     theDetailedDescription = theDetailedDescription & _
      " --> " & theProgressSpecific & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf & _
      TopoCorridor.MyGeneralOperations.ReturnTimeElapsed(theTimeBegan, Now)
1205:       frmProgress.txtDetails.Text = theDetailedDescription
1206:       frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
1207:       frmProgress.cmdStop.Caption = "Close"
1208:   End If
  
  Dim strBaseString As String
1211:   strBaseString = strBaseString & "{\rtf1\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f0\fswiss\fcharset0 Arial;}{\f1\fnil\fcharset0 MS Sans Serif;}}" & vbNewLine
1212:   strBaseString = strBaseString & "\viewkind4\uc1\pard\qc\b\f0\fs16 -- Function Complete --\b0 " & vbNewLine
1213:   strBaseString = strBaseString & "\par \pard " & vbNewLine
1214:   strBaseString = strBaseString & "\par \b Initial Categorical Raster Layer:\b0 " & vbNewLine
1215:   strBaseString = strBaseString & "\par   --> Layer = " & strTemplateName & vbNewLine
1216:   strBaseString = strBaseString & "\par   --> Spatial Reference = " & pTemplateSpRef.Name & vbNewLine
1217:   strBaseString = strBaseString & "\par   --> Cell Size:  X = " & CStr(pTemplatePnt.X) & ", Y = " & CStr(pTemplatePnt.Y) & vbNewLine
1218:   strBaseString = strBaseString & "\par   --> Raster Size:  " & CStr(pTemplateProps.Width) & " Columns, " & _
        CStr(pTemplateProps.Height) & " Rows" & vbNewLine
1220:   strBaseString = strBaseString & "\par " & vbNewLine
1221:   strBaseString = strBaseString & "\par \b Rasters Layers Included:\b0 " & vbNewLine

1223:   If pAdditionalRasters.Count > 0 Then
1224:     For lngIndex = 0 To pAdditionalRasters.Count - 1
1225:       Set pSubArray = pAdditionalRasters.Element(lngIndex)
1226:       strInputName = pSubArray.Element(1)
1227:       strBaseString = strBaseString & "\par   " & CStr(lngIndex + 1) & "] " & strInputName & vbNewLine
1228:     Next lngIndex
1229:   Else
1230:     strBaseString = strBaseString & "\par   --> None..." & vbNewLine
1231:   End If

1233:   strBaseString = strBaseString & "\par " & vbNewLine
1234:   strBaseString = strBaseString & "\par \b Feature Layers Included:\b0 " & vbNewLine

1236:   If pAdditionalFeatLayers.Count > 0 Then
1237:     For lngIndex = 0 To pAdditionalFeatLayers.Count - 1
1238:       Set pSubArray = pAdditionalFeatLayers.Element(lngIndex)
1239:       Set pAddFeatLayer = pSubArray.Element(0)
1240:       Set pField = pSubArray.Element(1)
 '     Set pAddFeatLayer = pAdditionalFeatLayers.Element(lngIndex)
1242:       strBaseString = strBaseString & "\par   " & CStr(lngIndex + 1) & "] " & pAddFeatLayer.Name & _
          ",  Attribute Field = [" & pField.Name & "]" & vbNewLine
1244:     Next lngIndex
1245:   Else
1246:     strBaseString = strBaseString & "\par   --> None..." & vbNewLine
1247:   End If
  
1249:   strBaseString = strBaseString & "\par " & vbNewLine
1250:   strBaseString = strBaseString & "\par \b Option to Clip Data to Polygon Layer: " & UCase(CStr(booShouldClip)) & "\b0 " & vbNewLine
1251:   If booShouldClip Then
1252:     strBaseString = strBaseString & "\par   --> Clip Layer = '" & pClipLayer.Name & "'" & vbNewLine
1253:   End If
  
1255:   strBaseString = strBaseString & "\par " & vbNewLine
  
1257:   strBaseString = strBaseString & strOutputString
  
'  strBaseString = strBaseString & "\par \b Output Data Table:\b0 " & vbNewLine
'  strBaseString = strBaseString & "\par \pard   --> " & Replace(strFullFilePath, "\", "\\") & vbNewLine
'  strBaseString = strBaseString & "\par \pard   --> Format = " & strOutputFormat & vbNewLine
1262:   strBaseString = strBaseString & strFieldOutputReport
  
1264:   strBaseString = strBaseString & "\par " & vbNewLine
1265:   strBaseString = strBaseString & "\par --------------------------- \par " & vbNewLine
1266:   strBaseString = strBaseString & TopoCorridor.MyGeneralOperations.ReturnTimeElapsedRTF(theTimeBegan, Now, 7)
1267:   strBaseString = strBaseString & "\par }" & vbNewLine
1268:   strBaseString = strBaseString & " " & vbNewLine

1270:   strReport = strBaseString
  
1272:   ExportRastersToTable = strReport
1273:   If Not booAlreadyHasInfoFolder Then
1274:     If TopoCorridor.aml_func_mod.ExistFileDir(strWsName & "\info") Then
1275:       Kill strWsName & "\info\*.*"
1276:       RmDir strWsName & "\info"
1277:     End If
1278:   End If
  
1280:   GoTo CleanUp
  
ErrorHandler:
  HandleError True, "ExportRastersToTable " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

CleanUp:

1287:   Set pApp = Nothing
1288:   Set pThisExt = Nothing
1289:   Set pMxDoc = Nothing
1290:   Set pTemplateBand = Nothing
1291:   Set pTemplateRaster = Nothing
1292:   Set pTemplateDataset = Nothing
1293:   Set pTemplateProps = Nothing
1294:   Set pTemplateSpRef = Nothing
1295:   Set pTemplatePnt = Nothing
1296:   Set strInputBand = Nothing
1297:   Set pAdditionalRasters = Nothing
1298:   Set pAdditionalFeatLayers = Nothing
1299:   Set pClipLayer = Nothing
1300:   Set pWS = Nothing
1301:   Set pSlopeValArray = Nothing
1302:   Set pSlopeValAttVals = Nothing
1303:   Set pSlopeValAttNames = Nothing
1304:   Set pSlopeAttTable = Nothing
1305:   Set pSlopeSubArray = Nothing
1306:   Set pSlopeCursor = Nothing
1307:   Set pSlopeRow = Nothing
1308:   Set pSlopeField = Nothing
1309:   Set pSlopeFields = Nothing
1310:   Set pAddFields = Nothing
1311:   Set collSlopeTables = Nothing
1312:   Set psbar = Nothing
1313:   Set pPro = Nothing
1314:   Set frmProgress = Nothing
1315:   Set pAddLayer = Nothing
1316:   Set pAddFeatLayer = Nothing
1317:   Set pFeatClassDesc = Nothing
1318:   Set pFeatClass = Nothing
1319:   Set pField = Nothing
1320:   Set pSubArray = Nothing
1321:   Set pRastMakerOp = Nothing
1322:   Set pRastAnEnv = Nothing
1323:   Set pEnv = Nothing
1324:   Set pOneDataset = Nothing
1325:   Set pTempRaster = Nothing
1326:   Set pMathOp = Nothing
1327:   Set pMathEnv = Nothing
1328:   Set pInBandColl = Nothing
1329:   Set pAddBand = Nothing
1330:   Set pNewBand = Nothing
1331:   Set pNewDataset = Nothing
1332:   Set pNewBandColl = Nothing
1333:   Set pInRaster = Nothing
1334:   Set pInProps = Nothing
1335:   Set pOutputFNames = Nothing
1336:   Set pTempConversionRasterDataset = Nothing
1337:   Set pConOp = Nothing
1338:   Set pTempAnEnv = Nothing
1339:   Set pFeatGeoDataset = Nothing
1340:   Set pTempRastDataset = Nothing
1341:   Set pRastWS = Nothing
1342:   Set pWsFact = Nothing
1343:   Set pAddFieldNames = Nothing
1344:   Set pFieldEdit = Nothing
1345:   Set pTable = Nothing
1346:   Set pFieldIndexes = Nothing
1347:   Set pCursor = Nothing
1348:   Set pBuffer = Nothing
1349:   Set pTempBand = Nothing
1350:   Set pRastProps = Nothing
1351:   Set pPnt = Nothing
1352:   Set pPixBlock = Nothing
1353:   Set pTopLeft = Nothing
1354:   Set pRastCursor = Nothing
1355:   Set pInputRaster = Nothing
1356:   Set pTempProps = Nothing
1357:   Set pTempPixBlock = Nothing
1358:   Set pTemporaryRaster = Nothing
1359:   Set pTempPoint = Nothing
1360:   Set pTempOrigin = Nothing
1361:   Set pStandaloneTable = Nothing
1362:   Set pStTabColl = Nothing
1363:   Set pTableWindow2 = Nothing
1364:   Set pCombineLayer = Nothing

End Function

Public Function ExportRastersToAscii(pDocParams As esriSystem.IVariantArray, _
    pParams As esriSystem.IVariantArray) As String
  On Error GoTo ErrorHandler
  
  Dim datTimeBegan As Date
1373:   datTimeBegan = Now
  
  Dim pApp As IApplication
  Dim pThisExt As TopoCorridor.clsExtension
  Dim pMxDoc As IMxDocument
  
1379:   Set pApp = pDocParams.Element(0)
1380:   Set pThisExt = pDocParams.Element(1)
1381:   Set pMxDoc = pDocParams.Element(2)
  
  ' RETURN ARRAY CONTAINS THE FOLLOWING:
  ' 0) IRasterLayer:  TEMPLATE RASTER LAYER
  ' 1) IVariantArray:  ADDITIONAL RASTER LAYERS; COULD BE EMPTY
  ' 2) String:  OUTPUT FOLDER TO SAVE ASCII RASTERS
  Dim pTemplate As IRasterLayer
  Dim pTemplateRaster As IRaster
  Dim pTemplateProps As IRasterProps
  Dim pTemplateSpRef As ISpatialReference
  Dim pTemplatePnt As IPnt
  
1393:   Set pTemplate = pParams.Element(0)
1394:   Set pTemplateRaster = pTemplate.Raster
1395:   Set pTemplateProps = pTemplateRaster
1396:   Set pTemplateSpRef = pTemplateProps.SpatialReference
1397:   Set pTemplatePnt = pTemplateProps.MeanCellSize
  
  Dim pAdditionalRasters As esriSystem.IVariantArray
1400:   Set pAdditionalRasters = pParams.Element(1)
  Dim pAdditionalFeatLayers As esriSystem.IVariantArray
1402:   Set pAdditionalFeatLayers = pParams.Element(2)
  Dim strOutputFolder As String
1404:   strOutputFolder = pParams.Element(3)
  Dim booAllHaveNoSpRef As Boolean
1406:   booAllHaveNoSpRef = pParams.Element(4)
  
  Dim booAlreadyHasInfoFolder As Boolean
1409:   booAlreadyHasInfoFolder = TopoCorridor.aml_func_mod.ExistFileDir(strOutputFolder & "info")
'  MsgBox "booAlreadyHasInfoFolder = " & CStr(booAlreadyHasInfoFolder)
  
  ' FOR PROGRESS
  Dim lngWorkOrderCount As Long
  Dim lngWorkOrderCounter As Long
1415:   lngWorkOrderCount = pAdditionalRasters.Count + 1 + (2 * pAdditionalFeatLayers.Count)
1416:   lngWorkOrderCounter = 0
  
    ' PROGRESS BAR STUFF
  Dim psbar As IStatusBar
1420:   Set psbar = pApp.StatusBar
  Dim pPro As IStepProgressor
1422:   Set pPro = psbar.ProgressBar
  
  Dim pAddLayer As IRasterLayer
  Dim lngIndex As Long
  
  Dim strReport As String
1428:   strReport = "Function failed..."
  
  ' WORKSPACE
  Dim pWsFact As IWorkspaceFactory
1432:   Set pWsFact = New RasterWorkspaceFactory
  Dim pWS As IRasterWorkspace
  Dim strWsName As String
1435:   strWsName = strOutputFolder
1436:   If Right(strWsName, 1) = "\" Then
'    strWsName = strWsName & "\"
1438:     strWsName = Left(strWsName, Len(strWsName) - 1)
1439:   End If
1440:   Set pWS = pWsFact.OpenFromFile(strWsName, 0)
  Dim pOutputNames As esriSystem.IStringArray
1442:   Set pOutputNames = New esriSystem.strArray
  Dim strNewName As String
  
  Dim pWorkspace As IWorkspace
1446:   Set pWorkspace = pWS
'  MsgBox "Workspace Path = " & pWorkspace.PathName & vbCrLf & "Is Directory = " & CStr(pWorkspace.IsDirectory)
  
  Dim strTemplateName As String
1450:   strTemplateName = TopoCorridor.MyGeneralOperations.ReplaceBadChars(pTemplate.Name, True, True, True, False) & ".asc"
1451:   strTemplateName = MyGeneralOperations.MakeUniqueShapeFilename(strWsName & "\" & strTemplateName)
  
1453:   ExportRastersToAscii = strReport
  
  ' DO ANALYSIS
1456:   lngWorkOrderCounter = lngWorkOrderCounter + 1
1457:   psbar.ShowProgressBar "Exporting " & pTemplate.Name & " to " & strTemplateName & "...", 1, lngWorkOrderCount, lngWorkOrderCounter, True
1458:   DoEvents
  
  Dim pRastExportOp As IRasterExportOp
1461:   Set pRastExportOp = New RasterConversionOp
  Dim pRastAnEnv As IRasterAnalysisEnvironment
1463:   Set pRastAnEnv = pRastExportOp
  
  Dim dblCellSize As Double
1466:   dblCellSize = (pTemplatePnt.X + pTemplatePnt.Y) / 2
  Dim pEnv As IEnvelope
1468:   Set pEnv = pTemplateProps.Extent
  Dim pClone As IClone
1470:   Set pClone = pEnv
1471:   Set pEnv = pClone.Clone
1472:   Set pClone = Nothing
1473:   pEnv.Expand -(pTemplatePnt.X) / 10, -(pTemplatePnt.Y) / 10, False
1474:   Set pRastAnEnv.OutSpatialReference = pTemplateSpRef
1475:   Set pRastAnEnv.OutWorkspace = pWS
1476:   pRastAnEnv.SetCellSize esriRasterEnvValue, dblCellSize
1477:   pRastAnEnv.SetExtent esriRasterEnvValue, pEnv, pTemplateRaster
  
  ' EXPORT TEMPLATE RASTER
1480:   pRastExportOp.ExportToASCII pTemplateRaster, strTemplateName
  
  ' EXPORT ADDITIONAL RASTERS
  Dim pAddRaster As IRaster
1484:   For lngIndex = 0 To pAdditionalRasters.Count - 1
1485:     Set pAddLayer = pAdditionalRasters.Element(lngIndex)
1486:     Set pAddRaster = pAddLayer.Raster
1487:     strNewName = TopoCorridor.MyGeneralOperations.ReplaceBadChars(pAddLayer.Name, True, True, True, False) & ".asc"
1488:     strNewName = TopoCorridor.MyGeneralOperations.MakeUniqueShapeFilename(strWsName & "\" & strNewName)
1489:     pOutputNames.Add strNewName
    
1491:     lngWorkOrderCounter = lngWorkOrderCounter + 1
1492:     psbar.Message(1) = "Exporting " & pAddLayer.Name & " to " & strNewName & "..."
1493:     pPro.Step
1494:     DoEvents
    
1496:     pRastExportOp.ExportToASCII pAddRaster, strNewName
1497:   Next lngIndex
  
  ' EXPORT ADDITIONAL FEATURE CLASSES
  Dim pAddFeatLayer As IFeatureLayer
  Dim pOutputFNames As esriSystem.IStringArray
1502:   Set pOutputFNames = New esriSystem.strArray
  Dim pTempConversionRasterDataset As IRasterDataset
  Dim pConOp As IConversionOp
1505:   Set pConOp = New RasterConversionOp
  Dim pTempAnEnv As IRasterAnalysisEnvironment
1507:   Set pTempAnEnv = pConOp
1508:   Set pTempAnEnv.OutSpatialReference = pTemplateSpRef
1509:   Set pTempAnEnv.OutWorkspace = pWS
1510:   pTempAnEnv.SetCellSize esriRasterEnvValue, dblCellSize
1511:   pTempAnEnv.SetExtent esriRasterEnvValue, pEnv, pTemplateRaster
  
  ' ADDED OCT 31 2009
  Dim pFeatClassDesc As IFeatureClassDescriptor
  Dim pFeatClass As IFeatureClass
  Dim pField As IField
  Dim pSubArray As esriSystem.IVariantArray
  
  
  Dim pFeatGeoDataset As IGeoDataset
  Dim pTempRastDataset As IDataset
  Dim strTempName As String
  
1524:   For lngIndex = 0 To pAdditionalFeatLayers.Count - 1
1525:     Set pSubArray = pAdditionalFeatLayers.Element(lngIndex)
1526:     Set pAddFeatLayer = pSubArray.Element(0)
1527:     Set pField = pSubArray.Element(1)
1528:     Set pFeatClass = pAddFeatLayer.FeatureClass
1529:     Set pFeatGeoDataset = pFeatClass
1530:     strNewName = TopoCorridor.MyGeneralOperations.ReplaceBadChars(pAddFeatLayer.Name, True, True, True, False) & ".asc"
1531:     strNewName = TopoCorridor.MyGeneralOperations.MakeUniqueShapeFilename(strWsName & "\" & strNewName)
1532:     strTempName = TopoCorridor.MyGeneralOperations.MakeUniqueShapeFilename(strWsName & "\" & "CS_Temp")
1533:     strTempName = TopoCorridor.aml_func_mod.ReturnFilename(strTempName)
1534:     pOutputFNames.Add strNewName
    
1536:     lngWorkOrderCounter = lngWorkOrderCounter + 1
1537:     psbar.ShowProgressBar "Exporting " & pAddFeatLayer.Name & " to " & strTempName & "...", 1, lngWorkOrderCount, lngWorkOrderCounter, True
'    pPro.Step
1539:     DoEvents
    
1541:     Set pFeatClassDesc = New FeatureClassDescriptor
1542:     pFeatClassDesc.Create pFeatClass, Nothing, pField.Name
    
1544:     Set pTempConversionRasterDataset = pConOp.ToRasterDataset(pFeatClassDesc, "GRID", pWS, strTempName)
1545:     Set pTempRastDataset = pTempConversionRasterDataset
    
1547:     lngWorkOrderCounter = lngWorkOrderCounter + 1
1548:     psbar.ShowProgressBar "Exporting " & strTempName & " to " & strNewName & "...", 1, lngWorkOrderCount, lngWorkOrderCounter, True
'    pPro.Step
1550:     DoEvents
    
1552:     pRastExportOp.ExportToASCII pTempConversionRasterDataset, strNewName
    
1554:     Set pTempConversionRasterDataset = Nothing
1555:     If pTempRastDataset.CanDelete Then
1556:       pTempRastDataset.DELETE
1557:     End If
1558:     Set pTempRastDataset = Nothing
    
1560:   Next lngIndex
  
1562:   pRastAnEnv.RestoreToPreviousDefaultEnvironment
  
1564:   psbar.HideProgressBar
1565:   pPro.position = 0
  
  Dim strBaseString As String
1568:   strBaseString = strBaseString & "{\rtf1\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f0\fswiss\fcharset0 Arial;}{\f1\fnil\fcharset0 MS Sans Serif;}}" & vbNewLine
1569:   strBaseString = strBaseString & "\viewkind4\uc1\pard\qc\b\f0\fs16 -- Analysis Complete --\b0 " & vbNewLine
1570:   strBaseString = strBaseString & "\par \pard " & vbNewLine
1571:   strBaseString = strBaseString & "\par \b Template Raster Layer:\b0 " & vbNewLine
1572:   strBaseString = strBaseString & "\par   --> Template Layer = " & pTemplate.Name & vbNewLine
1573:   strBaseString = strBaseString & "\par   --> Spatial Reference = " & pTemplateSpRef.Name & vbNewLine
1574:   strBaseString = strBaseString & "\par   --> Cell Size:  X = " & CStr(pTemplatePnt.X) & ", Y = " & CStr(pTemplatePnt.Y) & vbNewLine
1575:   strBaseString = strBaseString & "\par   --> Raster Size:  " & CStr(pTemplateProps.Width) & " Columns, " & _
        CStr(pTemplateProps.Height) & " Rows" & vbNewLine
1577:   strBaseString = strBaseString & "\par   --> Saved as '" & Replace(strTemplateName, "\", "\\") & "'" & vbNewLine
1578:   strBaseString = strBaseString & "\par " & vbNewLine
1579:   strBaseString = strBaseString & "\par \b Additional Rasters Exported:\b0 " & vbNewLine
  
1581:   If pAdditionalRasters.Count > 0 Then
1582:     For lngIndex = 0 To pAdditionalRasters.Count - 1
1583:       Set pAddLayer = pAdditionalRasters.Element(lngIndex)
1584:       strNewName = pOutputNames.Element(lngIndex)
1585:       strBaseString = strBaseString & "\par   " & CStr(lngIndex + 1) & "] " & pAddLayer.Name & vbNewLine
1586:       strBaseString = strBaseString & "\par       --> Saved as '" & Replace(strNewName, "\", "\\") & "'" & vbNewLine
1587:     Next lngIndex
1588:   Else
1589:     strBaseString = strBaseString & "\par   --> None..." & vbNewLine
1590:   End If
  
1592:   strBaseString = strBaseString & "\par " & vbNewLine
1593:   strBaseString = strBaseString & "\par \b Additional Feature Layers Exported:\b0 " & vbNewLine
  
1595:   If pAdditionalFeatLayers.Count > 0 Then
1596:     For lngIndex = 0 To pAdditionalFeatLayers.Count - 1
1597:       Set pSubArray = pAdditionalFeatLayers.Element(lngIndex)
1598:       Set pAddFeatLayer = pSubArray.Element(0)
1599:       Set pField = pSubArray.Element(1)
 '     Set pAddFeatLayer = pAdditionalFeatLayers.Element(lngIndex)
1601:       strNewName = pOutputFNames.Element(lngIndex)
1602:       strBaseString = strBaseString & "\par   " & CStr(lngIndex + 1) & "] " & pAddFeatLayer.Name & _
          ",  Attribute Field = [" & pField.Name & "]" & vbNewLine
1604:       strBaseString = strBaseString & "\par       --> Saved as '" & Replace(strNewName, "\", "\\") & "'" & vbNewLine
1605:     Next lngIndex
1606:   Else
1607:     strBaseString = strBaseString & "\par   --> None..." & vbNewLine
1608:   End If
  
1610:   strBaseString = strBaseString & "\par " & vbNewLine
1611:   strBaseString = strBaseString & "\par \b All ASCII Rasters Exported To:\b0 " & vbNewLine
1612:   strBaseString = strBaseString & "\par \pard   --> " & Replace(strOutputFolder, "\", "\\") & vbNewLine
1613:   strBaseString = strBaseString & "\par " & vbNewLine
1614:   strBaseString = strBaseString & "\par --------------------------- \par " & vbNewLine
1615:   strBaseString = strBaseString & TopoCorridor.MyGeneralOperations.ReturnTimeElapsedRTF(datTimeBegan, Now, 7)
1616:   strBaseString = strBaseString & "\par }" & vbNewLine
1617:   strBaseString = strBaseString & " " & vbNewLine

1619:   strReport = strBaseString
  
1621:   ExportRastersToAscii = strReport
  

1624:   Set pFeatClassDesc = Nothing
1625:   Set pFeatClass = Nothing
1626:   Set pSubArray = Nothing
1627:   Set pField = Nothing
1628:   Set pApp = Nothing
1629:   Set pThisExt = Nothing
1630:   Set pMxDoc = Nothing
1631:   Set pTemplate = Nothing
1632:   Set pTemplateRaster = Nothing
1633:   Set pTemplateProps = Nothing
1634:   Set pTemplateSpRef = Nothing
1635:   Set pTemplatePnt = Nothing
1636:   Set pAdditionalRasters = Nothing
1637:   Set pAdditionalFeatLayers = Nothing
1638:   Set psbar = Nothing
1639:   Set pPro = Nothing
1640:   Set pAddLayer = Nothing
1641:   Set pWsFact = Nothing
1642:   Set pWS = Nothing
1643:   Set pOutputNames = Nothing
1644:   Set pWorkspace = Nothing
1645:   Set pRastExportOp = Nothing
1646:   Set pRastAnEnv = Nothing
1647:   Set pEnv = Nothing
1648:   Set pAddRaster = Nothing
1649:   Set pAddFeatLayer = Nothing
1650:   Set pOutputFNames = Nothing
1651:   Set pTempConversionRasterDataset = Nothing
1652:   Set pConOp = Nothing
1653:   Set pTempAnEnv = Nothing
1654:   Set pFeatGeoDataset = Nothing
1655:   Set pTempRastDataset = Nothing


1658:   If Not booAlreadyHasInfoFolder Then
1659:     If TopoCorridor.aml_func_mod.ExistFileDir(strWsName & "\info") Then
1660:       Kill strWsName & "\info\*.*"
1661:       RmDir strWsName & "\info"
1662:     End If
1663:   End If
  
  Exit Function
ErrorHandler:
  HandleError True, "ExportRastersToAscii " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

' ------------- IN THIS MODULE, NEEDS CS_GetExtension FUNCTION
Private Function CS_GetExtension(pApp As IApplication) As TopoCorridor.clsExtension
  On Error GoTo ErrorHandler

  Dim newUid As New UID
  Dim pExt As IExtension
  Dim pThisExt As TopoCorridor.clsExtension
1677:   newUid.Value = "TopoCorridor.clsExtension"
1678:   Set pExt = pApp.FindExtensionByCLSID(newUid)
1679:   Set CS_GetExtension = pExt
  
1681:   Set newUid = Nothing
1682:   Set pExt = Nothing
1683:   Set pThisExt = Nothing

  Exit Function
ErrorHandler:
  HandleError False, "CS_GetExtension " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

' ------------- IN THIS MODULE, NEEDS CS_CalcTimeLeft FUNCTION
Private Function CS_CalcTimeLeft(strHeader As String, dateStart As Date, lngCurrent As Long, lngTotal As Long) As String
  On Error GoTo ErrorHandler

' ESTIMATED TIME LEFT CODE

  Dim thePercentDone As Double
1697:   If lngTotal <= 0 Then
1698:     thePercentDone = 0
1699:   Else
1700:     thePercentDone = (lngCurrent / lngTotal)
1701:   End If

  Dim thePercentForCalcs As Double
1704:   thePercentForCalcs = Round(thePercentDone * 100, 2)

'    lblPercentDone.Caption = "(" & thePercentForCalcs & "%)"
'    lblPercentDone.Refresh

  Dim theDuration As Double
1710:   theDuration = DateDiff("s", dateStart, Now)

  Dim PredictedDuration As Double
1713:   PredictedDuration = (theDuration * lngTotal) / (lngCurrent)

  Dim EstTimeLeft As Double
1716:   EstTimeLeft = PredictedDuration - theDuration

  Dim EstHoursLeft As Long
  Dim EstMinutesLeft As Long
  Dim EstSecondsLeft As Long

1722:   EstHoursLeft = Fix(EstTimeLeft / 3600)
1723:   EstMinutesLeft = Fix((EstTimeLeft - (EstHoursLeft * 3600)) / 60)
1724:   EstSecondsLeft = Fix(EstTimeLeft - (EstHoursLeft * 3600) - (EstMinutesLeft * 60))

  Dim EstTimeLeftStr As String
1727:   EstTimeLeftStr = EstHoursLeft & ":" & Format(EstMinutesLeft, "00") & ":" & Format(EstSecondsLeft, "00")

1729:   CS_CalcTimeLeft = strHeader & ".....[record #" & CStr(lngCurrent) & " of " & CStr(lngTotal) & "  ---  " & thePercentForCalcs & "%]....." & _
      "Est. Time Left = " & EstTimeLeftStr & "..."

  Exit Function
ErrorHandler:
  HandleError False, "CS_CalcTimeLeft " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
' ------------- IN THIS MODULE, NEEDS CS_BailOutOfProgress FUNCTION
Private Function CS_BailOutOfProgress(frmProgressDialog As Object, pApp As IApplication) As Boolean
  On Error GoTo ErrorHandler

  Dim pThisExt As TopoCorridor.clsExtension
1741:   Set pThisExt = CS_GetExtension(pApp)

  Dim frmProgress As TopoCorridor.frmJenProgressMeterBasic
1744:   Set frmProgress = frmProgressDialog

1746:   pThisExt.ProgressDialogAutoClose = (frmProgress.chkClose.Value = 1)
1747:   pThisExt.ProgressDialogSetExpanded = (frmProgress.SetExpanded)

'  MsgBox "Checking Autoclose Stuff: " & vbCrLf & _
      "Check-Box checked? " & CStr(frmProgress.chkClose.Value = 1) & vbCrLf & _
      "Extension Property Checked? " & CStr(pThisExt.ProgressDialogAutoClose)

  ' PROGRESS METER STUFF --------------------------------------------------------
1754:   CS_BailOutOfProgress = Not frmProgress.ShouldContinue
1755:   If CS_BailOutOfProgress Then
    Dim theDateString As String
    Dim theElapsedTimeString As String

1759:     theDateString = Format(Now, "long date") & ";  " & Format(Now, "long time")
1760:     Screen.MousePointer = vbDefault
    Dim theTimeBegan As Date
    Dim theTimeEnd As Date
1763:     theTimeBegan = frmProgress.ProgBeginTime
1764:     theTimeEnd = Now

1766:     theElapsedTimeString = MyGeneralOperations.ReturnTimeElapsed(theTimeBegan, theTimeEnd)

    Dim theReport As String
1769:     theReport = "Operation cancelled prematurely at " & theDateString & vbCrLf & _
        "---------------------------------------------------------" & vbCrLf & theElapsedTimeString

    Dim strCurrentReport As String
1773:     strCurrentReport = frmProgress.txtDetails.Text
1774:     frmProgress.txtDetails.Text = strCurrentReport & vbCrLf & theReport
1775:     frmProgress.txtDetails.SelStart = Len(frmProgress.txtDetails.Text)

1777:     If frmProgress.chkClose.Value = 1 Then
    '  Unload frmProgress
    '  Set frmProgress = Nothing
1780:       frmProgress.Frame.Visible = False
1781:     End If

    Exit Function
1784:   End If
  
1786:   Set pThisExt = Nothing
1787:   Set frmProgress = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "CS_BailOutOfProgress " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


