VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Matrix_jen"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Description = "Matrix class, based on 2-dimensional arrays.  The class includes several common matrix functions, including inverses, transposes, multiplications, etc."
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
' MATRIX CLASS
'------------------------------------------------------------------------------------------------------------
'   Matrix class is based on 2-dimensional array
'   Matrix object is called "Matrix_jen" to avoid any potential conflicts with other custom "matrix" objects
'
'   Author:  Jeff Jenness
'            Jenness Enterprises
'            http://www.jennessent.com
'            jeffj@jennessent.com
'            3020 N. Schevene Blvd.
'            Flagstaff, Arizona, USA            86004
'            Last Modified August 2008
'
' PROPERTIES ---------------------
'
'          Dimension:  String value indicating number of rows and columns, as in "5 x 6"
'                      Set automatically when matrix is created and array is added
'                      Public "Get":  Use syntax yourMatrix.Dimension
'               Rows:  Long value indicating number of rows
'                      Set automatically when matrix is created and array is added
'                      Public "Get":  Use syntax yourMatrix.Rows
'            Columns:  Long value indicating number of rows.
'                      Set automatically when matrix is created and array is added
'                      Public "Get":  Use syntax yourMatrix.Columns
'        MatrixArray:  2-dimensional Array value (array components are Doubles), reflecting the array
'                          associated with this matrix
'                      Public "Let": Set using syntax "yourMatrix.MatrixArray = anArray"
'                      Public "Get": Returned using syntax "yourArray = yourMatrix.MatrixArray"
'             IsNull:  Boolean value stating whether the matrix has any data.  IsNull = True upon instantiation, False if
'                      data gets added to it
'         MatrixType:  String value:  Matrix, Row Vector, Column Vector, Number, Null
'                      Set automatically when matrix is created and array is added
'                      Public "Get": Returned using syntax "yourArray = yourMatrix.MatrixType"
'
' METHODS -------------------------
'
'                Add:  Adds current matrix to another matrix and returns the new matrix.
'                      Should be called the following way:
'                        Dim theAddSucceeded As Boolean
'                        Dim theAddMatrix As Matrix_jen
'                        theAddSucceeded = False
'                        Set theAddMatrix = OrigAsMatrix.Add(OrigAsMatrix, theAddSucceeded)
'                        If (Not theAddSucceeded) Then
'                           ' OPERATION FAILED FOR SOME REASON, PROBABLY BECAUSE OF INCOMPATIBLE DIMENSIONS
'                        End If
'  CheckMatrixCollectionForKey:  Just returns a boolean stating whether collection has a particular key
'              Clone:  Clones current matrix, producing an exact duplicate.
'                      Should be called the following way:
'                        Dim theCloneMatrix As Matrix_jen
'                        Set theCloneMatrix = theMultMatrix.Clone
'             Invert:  Returns an inverted matrix and an error number, given an input matrix
'                      Should be called the following way:
'                        Dim theInverseSucceeded As Byte
'                        Dim theInverse As Matrix_jen
'                        theInverseSucceeded = 0
'                        Set theInverse = OrigAsMatrix.Invert(theInverseSucceeded)
'                        If (theInverseSucceeded <> 0) Then
'                         ' OPERATION FAILED FOR SOME REASON, INDICATED BY THE BYTE VALUE OF theInverseSucceeded
'                         1) FAILED AT MatrixLUDecomp_jen BECAUSE OF NON-SQUARE MATRIX
'                         2) FAILED AT MatrixLUDecomp_jen BECAUSE OF SINGULAR MATRIX
'                         3) FAILED AT MatrixFBSub_jen BECAUSE OF UNKNOWN REASONS
'                        End If
'     MakeAvenueCode:  Returns a String representing the Avenue code necessary to create the matrix.  Optionally
'                      you can specify the precision of the numbers.
'       MakeConstant:  Returns an M x N matrix with a constant value X in all cells
'                      Requires 3 numeric values corresponding with X, M and N
'       MakeIdentity:  Returns an N x N identity matrix (all 1's down the diagonal, 0's everywhere else)
'                      Requires a single numeric value indicating the size of the matrix
'           MakeText:  Returns a String representing the matrix.  Optionally you can specify the precision of the numbers
'                      and/or a title (i.e. "The Inverse Matrix = ...").
'    MatrixFBSub_jen:  Produce a vector to substitute into return vector
'                      Will return 10 if successful
'                      Should be called in the following way:
'                        Using theDecomp() from MatrixLUDecomp_jen
'                        Using theIndexVector() as from MatrixLUDecomp_jen
'                        Using a new theBVec() as created in Invert
'                        dim FBSubSuccessful as byte
'                        FBSubSuccessful = MatrixFBSub_jen(theDecomp() As Double, _
'                                 theIndexVec() As Double, theBVec() As Double) As Byte
' MatrixLUDecomp_jen:  Forces Lower/Upper Decomposition of matrix, modifying 'theDecomp' and 'theIndexVector' in the process
'                      WILL RETURN 0 IF SUCCESSFUL
'                                  1 IF FAILED DUE TO NON-SQUARE MATRIX ENTERED
'                                  2 IF FAILED DUE TO SINGULAR MATRIX ENTERED
'                      Should be called in the following way:
'                        dim theDecomp() as double
'                        dim theIndexVector() as double
'                        dim DecompSuccessful as byte
'                        DecompSuccessful = yourMatrix.MatrixLUDecomp (theDecomp, theIndexVector)
'        MultByConst:  Multiplies current matrix with a constant number and returns the new matrix.
'                      Should be called the following way:
'                        Dim theMultSucceeded As Boolean
'                        Dim theMultCMatrix As Matrix_jen
'                        theMultSucceeded = False
'                        Set theMultCMatrix = OrigAsMatrix.MultByConst(YourNumber, theMultSucceeded)
'                        If (Not theMultSucceeded) Then
'                           ' OPERATION FAILED FOR SOME REASON; PROBABLY BECAUSE YourNumber WAS NON-NUMERIC
'                        End If
'           Multiply:  Multiplies current matrix with matrix in argument and returns the new multiple.
'                      Should be called the following way:
'                        Dim theMultiplySucceeded As Boolean
'                        Dim theMultMatrix As Matrix_jen
'                        theMultiplySucceeded = False
'                        Set theMultMatrix = OrigAsMatrix.Multiply(theInverse, theMultiplySucceeded)
'                        If (Not theMultiplySucceeded) Then
'                          ' OPERATION FAILED FOR SOME REASON, PROBABLY BECAUSE OF INCOMPATIBLE DIMENSIONS
'                        End If
' ReturnStatMatrices:  Considers current matrix as a multivariate sample with variables in columns and samples in rows.
'                      Optionally Returns vector of mean values, variance/covariance matrix and/or correlation matrix.
'                      Should be called the following way:
'                        Dim booStatsSucceeded As Boolean
'                        Dim pSumVector As new Matrix_jen
'                        Dim pMeanVector As new Matrix_jen
'                        Dim pVarCovarMatrix new Matrix_jen
'                        Dim pCorrMatrix As new Matrix_jen
'                        Dim strFailureReason as string
'                        theMultiplySucceeded = False
'                        booStatsSucceeded = OrigAsMatrix.ReturnStatMatrices(strFailureReason, pSumVector, pMeanVector, pVarCovarMatrix, _
'                           pCorrMatrix)
'                        If (Not theMultiplySucceeded) Then
'                          ' OPERATION FAILED FOR SOME REASON, SPECIFIED BY strFailureReason
'                        End If
'    ReturnCellValue:  RETURNS A DOUBLE VALUE FOR THE ROW AND COLUMN SPECIFIED.  ASSUMES ROW AND COLUMN INDICES STARTING AT 1
'                      Should be called the following way:
'                        Dim dblValue as double
'                        dim lngFailureReason as long
'                        dblValue = OrigAsMatrix.ReturnCellValue(3,3,lngFailureReason)
'                        If (Not lngFailureReason = 0) Then
'                           ' OPERATION FAILED FOR SOME REASON
'                           ' lngFailureReason = 1 means row subscript out of range
'                           ' lngFailureReason = 2 means column subscript out of range
'                        End If
' ReturnSumOfColumns:  Considers current matrix as a multivariate sample with variables in columns and samples in rows.
'                      Returns a matrix with a single row containing column sums
'                      Should be called the following way:
'                        Dim booSucceeded As Boolean
'                        Dim pSums As Matrix_jen
'                        booSucceeded = False
'                        Set pSums = OrigAsMatrix.ReturnSumOfColumns(booSucceeded)
'                        If (Not booSucceeded) Then
'                           ' OPERATION FAILED FOR SOME REASON
'                        End If
'       SetcellValue:  SETS A DOUBLE VALUE FOR THE ROW AND COLUMN SPECIFIED.  ASSUMES ROW AND COLUMN INDICES STARTING AT 1
'                      Should be called the following way:
'                        Dim dblValue as double
'                        dim lngFailureReason as long
'                        OrigAsMatrix.ReturnCellValue(3, 3, dblValue, lngFailureReason)
'                        If (Not lngFailureReason = 0) Then
'                           ' OPERATION FAILED FOR SOME REASON
'                           ' lngFailureReason = 1 means row subscript out of range
'                           ' lngFailureReason = 2 means column subscript out of range
'                        End If
' SortDoubleAscendingWithObjects:  GIVEN A DOUBLE ARRAY AND OBJECT ARRAY, SORTS BOTH ARRAYS ACCORDING TO DOUBLE VALUES
'           Subtract:  Subtracts matrix from current matrix and returns the new matrix.
'                      Should be called the following way:
'                        Dim theSubtractSucceeded As Boolean
'                        Dim theSubtractMatrix As Matrix_jen
'                        theSubtractSucceeded = False
'                        Set theSubtractMatrix = OrigAsMatrix.Add(OrigAsMatrix, theSubtractSucceeded)
'                        If (Not theSubtractSucceeded) Then
'                           ' OPERATION FAILED FOR SOME REASON, PROBABLY BECAUSE OF INCOMPATIBLE DIMENSIONS
'                        End If
'          Transpose:  Transposes current matrix and returns it.
'                      Should be called the following way:
'                        Dim theTransposeSucceeded As Byte
'                        Dim theTranspose As Matrix_jen
'                        theTransposeSucceeded = False
'                        Set theTranspose = OrigAsMatrix.Transpose(theTransposeSucceeded)
'                        If (Not theTransposeSucceeded) Then
'                         ' OPERATION FAILED FOR SOME REASON
'                        End If

Option Explicit

'local variable(s) to hold property value(s)
Private mvarRows As Long 'local copy
Private mvarColumns As Long 'local copy
Private mvarMatrixArray As Variant 'local copy
Private mvarDimension As String 'local copy
Private mvarIsNull As Boolean ' local copy
Private mvarMatrixType As String ' local copy
Const c_sModuleFileName As String = "D:\arcGIS_stuff\consultation\brost_climate_change\code\Matrix_jen.cls"


Public Sub ExtractMatrixFromTable(pTable As ITable, booUseSelected As Boolean, _
  pFieldIndices As esriSystem.ILongArray, booSucceed As Boolean, _
  Optional strFailure As String = "Operation Successful")
  On Error GoTo ErrorHandler

  
190:   booSucceed = True
  
  ' FIRST CHECK TO MAKE SURE SELECTED FIELD INDICES ARE AVAILABLE AND NUMERIC
  Dim lngIndex As Long
  Dim lngFieldIndex As Long
  Dim pField As IField
  Dim pFieldType As esriFieldType
  
  ' IF pFieldIndices IS NOTHING, THEN USE ALL FIELD INDICES
  Dim lngFieldCount As Long
200:   If pFieldIndices Is Nothing Then
201:     Set pFieldIndices = New esriSystem.LongArray
202:     For lngIndex = 0 To pTable.Fields.FieldCount - 1
203:       Set pField = pTable.Fields.Field(lngIndex)
204:       If pField.Type <> esriFieldTypeOID Then
205:         pFieldIndices.Add (lngIndex)
206:       End If
207:     Next lngIndex
208:   End If
  
210:   lngFieldCount = pFieldIndices.Count - 1
  
212:   For lngIndex = 0 To lngFieldCount
213:     lngFieldIndex = pFieldIndices.Element(lngIndex)
214:     If pTable.Fields.FieldCount <= lngFieldIndex Then    ' IF FIELD INDEX HIGHER THAN ANY EXISTING FIELD
215:       strFailure = "Field Index (" & CStr(lngFieldIndex) & " higher than existing field count..."
216:       booSucceed = False
      Exit Sub
218:     ElseIf lngFieldIndex < 0 Then                        ' IF FIELD INDEX BELOW 0
219:       strFailure = "Field Index (" & CStr(lngFieldIndex) & " less than 0..."
220:       booSucceed = False
      Exit Sub
222:     Else                                                 ' CHECK IF FIELD IS NUMERIC
223:       Set pField = pTable.Fields.Field(lngFieldIndex)
224:       pFieldType = pField.Type
225:       If Not (pFieldType = esriFieldTypeDouble Or pFieldType = esriFieldTypeInteger Or _
        pFieldType = esriFieldTypeOID Or pFieldType = esriFieldTypeSingle Or _
        pFieldType = esriFieldTypeSmallInteger) Then                       ' IF FIELD INDEX BELOW 0
228:         strFailure = "Field at Index (" & CStr(lngFieldIndex) & " [" & pField.Name & _
          " is not a numeric field..."
230:         booSucceed = False
        Exit Sub
232:       End If
233:     End If
234:   Next lngIndex
  
  ' DETERMINE WHICH RECORDS TO EXTRACT DATA FROM
  Dim pCursor As ICursor
  Dim lngCount As Long
239:   If booUseSelected Then
    Dim pTableSelection As ITableSelection
241:     Set pTableSelection = pTable
    Dim pSelSet As ISelectionSet
243:     Set pSelSet = pTableSelection.SelectionSet
244:     pSelSet.Search Nothing, True, pCursor
245:     lngCount = pSelSet.Count
246:   Else
247:     Set pCursor = pTable.Search(Nothing, True)
248:     lngCount = pTable.RowCount(Nothing)
249:   End If
  
  ' MAKE NEW ARRAY
  Dim dblArray() As Double
  ReDim dblArray(lngCount - 1, lngFieldCount)
  Dim lngRowIndex As Long
  
  Dim pRow As IRow
257:   Set pRow = pCursor.NextRow
258:   lngRowIndex = 0
259:   Do Until pRow Is Nothing
260:     For lngIndex = 0 To lngFieldCount
261:       lngFieldIndex = pFieldIndices.Element(lngIndex)
262:       dblArray(lngRowIndex, lngIndex) = pRow.Value(lngFieldIndex)
263:     Next lngIndex
264:     Set pRow = pCursor.NextRow
265:     lngRowIndex = lngRowIndex + 1
266:   Loop
  
  ' CREATE MATRIX OBJECT
269:   Me.MatrixArray = dblArray
  

  Exit Sub
ErrorHandler:
  HandleError True, "ExtractMatrixFromTable " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub SetCellValue(lngRow As Long, lngColumn As Long, dblVal As Double, lngFailureReason As Long)
  On Error GoTo ErrorHandler


  Dim anArray() As Double
282:   anArray = Me.MatrixArray
283:   If lngRow < 1 Or (lngRow - 1) > UBound(anArray, 1) Then
284:     lngFailureReason = 1
285:   ElseIf lngColumn < 1 Or (lngColumn - 1) > UBound(anArray, 2) Then
286:     lngFailureReason = 2
287:   Else
288:     lngFailureReason = 0
289:     anArray(lngRow - 1, lngColumn - 1) = dblVal
290:   End If
291:   Me.MatrixArray = anArray


  Exit Sub
ErrorHandler:
  HandleError True, "SetCellValue " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function ReturnCellValue(lngRow As Long, lngColumn As Long, lngFailureReason As Long) As Double
  On Error GoTo ErrorHandler


  Dim anArray() As Double
304:   anArray = Me.MatrixArray
305:   If lngRow < 1 Or (lngRow - 1) > UBound(anArray, 1) Then
306:     lngFailureReason = 1
307:   ElseIf lngColumn < 1 Or (lngColumn - 1) > UBound(anArray, 2) Then
308:     lngFailureReason = 2
309:   Else
310:     lngFailureReason = 0
311:     ReturnCellValue = anArray(lngRow - 1, lngColumn - 1)
312:   End If


  Exit Function
ErrorHandler:
  HandleError True, "ReturnCellValue " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnSumOfColumns(booSucceeded As Boolean) As Matrix_jen
  On Error GoTo ErrorHandler
  
  Dim anArray() As Double
324:   anArray = Me.MatrixArray
  
  Dim lngCols As Long
  Dim lngRows As Long
328:   lngRows = Me.Rows
329:   lngCols = Me.Columns
  
  Dim dblSums() As Double
  ReDim dblSums(0, lngCols - 1)
  
  Dim lngIndexRow As Long
  Dim lngIndexCol As Long
  Dim dblSum As Double
  
338:   For lngIndexCol = 0 To lngCols - 1
339:     dblSum = 0
340:     For lngIndexRow = 0 To lngRows - 1
341:       dblSum = dblSum + anArray(lngIndexRow, lngIndexCol)
342:     Next lngIndexRow
343:     dblSums(0, lngIndexCol) = dblSum
344:   Next lngIndexCol
  
  Dim SumOfColumns As Matrix_jen
347:   Set SumOfColumns = New Matrix_jen
348:   SumOfColumns.MatrixArray = dblSums
349:   Set ReturnSumOfColumns = SumOfColumns
  
351:   booSucceeded = True
  
  Exit Function
ErrorHandler:
355:   booSucceeded = False
  
End Function

Public Function CheckMatrixCollectionForKey(colCollection As Collection, strKey As String) As Boolean
  On Error GoTo errHandler
  
362:   CheckMatrixCollectionForKey = True
  Dim lngVarType As Long
364:   lngVarType = VarType(colCollection.Item(strKey))
  
  Exit Function
errHandler:
368:   CheckMatrixCollectionForKey = False

End Function
Public Function ReturnSpearmansRho( _
    pSpearmansRho As Matrix_jen, _
    Optional pRankCovMatrix As Matrix_jen, _
    Optional pRankValues As Matrix_jen, _
    Optional strFailureReason As String = "Operation Successful") As Boolean
  On Error GoTo ErrorHandler


  Dim lngColumns As Long
  Dim lngRows As Long
  Dim dblArray() As Double
382:   dblArray = Me.MatrixArray
383:   lngColumns = UBound(dblArray, 2)
384:   lngRows = UBound(dblArray, 1)
  
  Dim pCollection As Collection
  Dim pRowIndices As esriSystem.ILongArray
  
  Dim dblRankArray() As Double
  ReDim dblRankArray(UBound(dblArray, 1), UBound(dblArray, 2))
  Dim pCollItem As esriSystem.IVariantArray
  Dim varIndices() As Variant
  Dim dblUniqueValues() As Double
  
  Dim pAllVariablesUniqueValues As esriSystem.IArray
396:   Set pAllVariablesUniqueValues = New esriSystem.Array
  
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  Dim lngRowIndex As Long
  Dim lngColIndex As Long
  Dim dblVal As Double
  Dim strVal As String
  
405:   For lngColIndex = 0 To lngColumns
406:     Set pCollection = New Collection
407:     For lngRowIndex = 0 To lngRows
408:       dblVal = dblArray(lngRowIndex, lngColIndex)
409:       strVal = CStr(dblVal)
410:       If CheckMatrixCollectionForKey(pCollection, strVal) Then
411:         Set pCollItem = pCollection.Item(strVal)
412:         Set pRowIndices = pCollItem.Element(1)
413:         pRowIndices.Add lngRowIndex
414:       Else
415:         Set pCollItem = New esriSystem.VarArray
416:         Set pRowIndices = New esriSystem.LongArray
417:         pRowIndices.Add lngRowIndex
418:         pCollItem.Add dblVal
419:         pCollItem.Add pRowIndices
420:         pCollection.Add pCollItem, strVal
421:       End If
422:     Next lngRowIndex
    
    ' NOW HAVE A COLLECTION CONTAINING ARRAYS
    ' EACH ARRAY HAS 2 OBJECTS:
    ' 1) A VALUE FROM THE ORIGINAL COLUMN
    ' 2) A LONG ARRAY CONTAINING ALL THE INDEX NUMBERS OF ROWS THAT CONTAIN THIS VALUE
    ' NEED TO SORT THEM AND POPULATE NEW ARRAY
    
    ReDim dblUniqueValues(pCollection.Count - 1)
    ReDim varIndices(pCollection.Count - 1)
432:     For lngRowIndex = 1 To pCollection.Count
433:       Set pCollItem = pCollection.Item(lngRowIndex)
434:       dblUniqueValues(lngRowIndex - 1) = pCollItem.Element(0)
435:       Set varIndices(lngRowIndex - 1) = pCollItem.Element(1)
436:     Next lngRowIndex
    
    ' SORT VALUES
439:     SortDoubleAscendingWithObjects dblUniqueValues, varIndices, 0, pCollection.Count - 1
    
    ' POPULATE THIS COLUMN IN NEW RANK ARRAY
    Dim dblRankValue As Double
    Dim dblMidRowCount As Double
    Dim lngRowCount As Long
445:     dblRankValue = 0
446:     For lngIndex = 0 To UBound(dblUniqueValues)
447:       Set pRowIndices = varIndices(lngIndex)
448:       lngRowCount = pRowIndices.Count
449:       dblMidRowCount = dblRankValue + ((lngRowCount + 1) / 2)
450:       dblRankValue = dblRankValue + lngRowCount
451:       For lngIndex2 = 0 To pRowIndices.Count - 1
452:         dblRankArray(pRowIndices.Element(lngIndex2), lngColIndex) = dblMidRowCount
453:       Next lngIndex2
454:     Next lngIndex
455:   Next lngColIndex
  
  Dim pSpearmansRhoTemp As Matrix_jen
458:   Set pSpearmansRhoTemp = New Matrix_jen
459:   pSpearmansRhoTemp.MatrixArray = dblRankArray
  
  Dim pSums As Matrix_jen
462:   Set pSums = New Matrix_jen
  Dim pMeanVector As Matrix_jen
464:   Set pMeanVector = New Matrix_jen
  Dim pCovMatrix As Matrix_jen
466:   Set pCovMatrix = New Matrix_jen
  Dim pCorrMatrix As Matrix_jen
468:   Set pCorrMatrix = New Matrix_jen
  
  Dim booSucceed As Boolean
  Dim strNewMessage As String
472:   booSucceed = pSpearmansRhoTemp.ReturnStatMatrices(strNewMessage, pSums, pMeanVector, pCovMatrix, pCorrMatrix)
473:   If Not booSucceed Then
474:     strFailureReason = "Failed in General Statistical Function:  " & strNewMessage
475:     ReturnSpearmansRho = False
    Exit Function
477:   End If
478:   ReturnSpearmansRho = True
479:   Set pSpearmansRho = pCorrMatrix
  
481:   If Not pRankValues Is Nothing Then
482:     Set pRankValues = pSpearmansRhoTemp
483:   End If
484:   If Not pRankCovMatrix Is Nothing Then
485:     Set pRankCovMatrix = pCovMatrix
486:   End If
'
'' CONVERT TO MATRIX OF RANKS
'theVariableCount = theMatrix.Get(0).Count
'theRowCount = theMatrix.Count
'
'theDictionary = Dictionary.Make(theVariableCount)
'for each aNum in 0..(theVariableCount-1)
'  theDictionary.Set(aNum, Dictionary.Make(theRowCount))
'end
'
'theCounter = 0
'av.ShowMsg("Generating Matrix of Ranks...")
'for each aRowIndex in 0..(theRowCount-1)
'  theCounter = theCounter+1
'  av.SetStatus((theCounter/theRowCount)*100)
'  theRow = theMatrix.Get(aRowIndex)
'  for each aValIndex in 0..(theVariableCount-1)
'    theVal = theRow.Get(aValIndex)
'    theSubDictionary1 = theDictionary.Get(aValIndex)
'    theData = theSubDictionary1.Get(theVal)
'    if (theData = nil) then
'      theSubDictionary1.Set(theVal, {1, {aRowIndex}})
'    else
'      theData.Set(0, theData.Get(0)+1)
'      theData.Get(1).Add(aRowIndex)
'      theSubDictionary1.Set(theVal, theData)
'    end
'  end
'end



  Exit Function
ErrorHandler:
  HandleError True, "ReturnSpearmansRho " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function ReturnStatMatrices( _
    Optional strFailureReason As String = "Operation Successful", _
    Optional pSums As Matrix_jen, _
    Optional pMeanVector As Matrix_jen, _
    Optional pCovMatrix As Matrix_jen, _
    Optional pCorrMatrix As Matrix_jen) As Boolean
  On Error GoTo ErrorHandler

  
534:   If pSums Is Nothing And pMeanVector Is Nothing And pCovMatrix Is Nothing And pCorrMatrix Is Nothing Then
535:     strFailureReason = "No Statistical Matrices Specified!"
536:     ReturnStatMatrices = False
    Exit Function
538:   Else
539:     ReturnStatMatrices = True
540:     strFailureReason = "Operation Successful"
541:   End If

  Dim lngIndex1 As Long
  Dim lngIndex2 As Long
  Dim booSucceeded As Boolean
  Dim pTransposeMatrix As Matrix_jen
547:   Set pTransposeMatrix = Me.Transpose(booSucceeded)
548:   If Not booSucceeded Then
549:     ReturnStatMatrices = False
550:     strFailureReason = "Failed to transpose original matrix..."
    Exit Function
552:   End If
  
  Dim lngMatrixCount As Long
555:   lngMatrixCount = Me.Rows
  Dim lngNMinus1 As Long
557:   lngNMinus1 = lngMatrixCount - 1

  ' TO REDUCE ROUNDING ERROR, FIRST MAKE VECTOR OF SUMS
560:   Set pSums = ReturnSumOfColumns(booSucceeded)
561:   If Not booSucceeded Then
562:     ReturnStatMatrices = False
563:     strFailureReason = "Failed to create sum of columns..."
    Exit Function
565:   End If
  
'  Debug.Print "Got Sums..."
  
  ' BAIL OUT IF NO OTHER STATS SPECIFIED
570:   If pMeanVector Is Nothing And pCovMatrix Is Nothing And pCorrMatrix Is Nothing Then
    Exit Function
572:   End If
  
  ' CALCULATE MEAN VECTOR
575:   Set pMeanVector = pSums.MultByConst(1 / lngMatrixCount, booSucceeded)
576:   If Not booSucceeded Then
577:     ReturnStatMatrices = False
578:     strFailureReason = "Failed to create mean vector..."
    Exit Function
580:   End If
  
'  Debug.Print "Got Means..."

  ' BAIL OUT IF NO OTHER STATS SPECIFIED
585:   If pCovMatrix Is Nothing And pCorrMatrix Is Nothing Then
    Exit Function
587:   End If
  
' MAKE BLANK CORRELATION MATRIX
  Dim dblDataArray() As Double
591:   dblDataArray = Me.MatrixArray
  Dim dblMeanArray() As Double
593:   dblMeanArray = pMeanVector.MatrixArray
  
  Dim lngMaxStatArrayIndex As Long
596:   lngMaxStatArrayIndex = pMeanVector.Columns - 1
  
  Dim dblPrelimCorrelationArray() As Double
  Dim dblNewCorMatrix() As Double
  ReDim dblPrelimCorrelationArray(lngMaxStatArrayIndex, lngMaxStatArrayIndex)
  
602:   For lngIndex1 = 0 To lngMaxStatArrayIndex
603:     For lngIndex2 = 0 To lngMaxStatArrayIndex
604:       dblPrelimCorrelationArray(lngIndex1, lngIndex2) = 0

606:     Next lngIndex2
607:   Next lngIndex1
  
  Dim dblRowMean As Double
  Dim dblRowVal As Double
  Dim dblRowValMinusMean As Double
  Dim dblColMean As Double
  Dim dblColVal As Double
  Dim dblColValMinusMean
  Dim dblCurrentVal As Double
  Dim dblNewVal As Double
  Dim lngIndex3 As Double
  
619:   For lngIndex1 = 0 To UBound(dblDataArray, 1)
620:     For lngIndex2 = 0 To lngMaxStatArrayIndex
621:       dblRowMean = dblMeanArray(0, lngIndex2)
622:       dblRowVal = dblDataArray(lngIndex1, lngIndex2)
623:       dblRowValMinusMean = dblRowVal - dblRowMean
624:       For lngIndex3 = 0 To lngMaxStatArrayIndex
625:         dblColMean = dblMeanArray(0, lngIndex3)
626:         dblColVal = dblDataArray(lngIndex1, lngIndex3)
627:         dblColValMinusMean = dblColVal - dblColMean
628:         dblCurrentVal = dblPrelimCorrelationArray(lngIndex2, lngIndex3)
629:         dblNewVal = dblCurrentVal + (dblRowValMinusMean * dblColValMinusMean)
630:         dblPrelimCorrelationArray(lngIndex2, lngIndex3) = dblNewVal
631:       Next lngIndex3
632:     Next lngIndex2
633:   Next lngIndex1
  
635:   Debug.Print "Got Correlation..."
  
  ' MAKE COVARIANCE MATRIX
  Dim pCorrelationMatrix As New Matrix_jen
639:   pCorrelationMatrix.MatrixArray = dblPrelimCorrelationArray
640:   Set pCovMatrix = pCorrelationMatrix.MultByConst(1 / lngNMinus1, booSucceeded)
641:   If Not booSucceeded Then
642:     ReturnStatMatrices = False
643:     strFailureReason = "Failed to create Covariance Matrix..."
    Exit Function
645:   End If

  ' BAIL OUT IF NO OTHER STATS SPECIFIED
648:   If pCorrMatrix Is Nothing Then
    Exit Function
650:   End If
  
  Dim pNewCorMatrix As New Matrix_jen
  Dim pCorFromCov As New Matrix_jen
  

  Dim dblFirstDiagonal As Double
  Dim dblSecondDiagonal As Double
  Dim dblCovMatrix() As Double
659:   dblCovMatrix = pCovMatrix.MatrixArray
  Dim dblCorFromCov() As Double
  ReDim dblCorFromCov(lngMaxStatArrayIndex, lngMaxStatArrayIndex)
  
663:   For lngIndex1 = 0 To lngMaxStatArrayIndex
664:     dblFirstDiagonal = Sqr(dblCovMatrix(lngIndex1, lngIndex1))
665:     For lngIndex2 = 0 To lngMaxStatArrayIndex
666:       dblSecondDiagonal = Sqr(dblCovMatrix(lngIndex2, lngIndex2))
667:       dblCurrentVal = dblCovMatrix(lngIndex1, lngIndex2)
668:       dblNewVal = dblCurrentVal / (dblFirstDiagonal * dblSecondDiagonal)
669:       dblCorFromCov(lngIndex1, lngIndex2) = dblNewVal
670:     Next lngIndex2
671:   Next lngIndex1
  
673:   Set pCorrMatrix = New Matrix_jen
674:   pCorrMatrix.MatrixArray = dblCorFromCov
  
'
'return ({theMeanMatrix, theCovMatrix, theCorFromCovMatrix})
  


  Exit Function
ErrorHandler:
  HandleError True, "ReturnStatMatrices " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function MakeAvenueCode(Optional nPrecision As Variant = -9) As String
  On Error GoTo ErrorHandler


Dim anArray() As Double
Dim theRow As Long
Dim theCol As Long
Dim ArrayString As String
Dim IsFirstRow As Boolean

696: If ((Not IsNumeric(nPrecision)) Or (IsEmpty(nPrecision))) Then
697:   ArrayString = "Error: Invalid precision value for 'MakeAvenueCode' method." & Chr(13) & _
                "Please use a numeric value for precision, or skip optional value entirely..."
699:   MakeAvenueCode = ArrayString
  Exit Function
701: End If
  
703: anArray = Me.MatrixArray
704: IsFirstRow = True

706: ArrayString = "theMatrix = " & Chr(13)
707: For theRow = LBound(anArray, 1) To UBound(anArray, 1)
708:     If (IsFirstRow) Then
709:       ArrayString = ArrayString & "  {{"
710:       IsFirstRow = False
711:     Else
712:       ArrayString = ArrayString & "   {"
713:     End If
714:     For theCol = LBound(anArray, 2) To UBound(anArray, 2)
715:         ArrayString = ArrayString & anArray(theRow, theCol) & ", "
716:     Next theCol
717:     ArrayString = Left(ArrayString, Len(ArrayString) - 2) & "}," & Chr(13)
718: Next theRow

720: MakeAvenueCode = Left(ArrayString, Len(ArrayString) - 2) & "}" & Chr(13)


  Exit Function
ErrorHandler:
  HandleError True, "MakeAvenueCode " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function MakeText(Optional nPrecision As Variant = -9, _
                         Optional strTitle As String = "defaultTitlezzz") As String
  On Error GoTo ErrorHandler

Dim anArray() As Double
Dim theRow As Long
Dim theCol As Long
Dim ArrayString As String

737: If ((Not IsNumeric(nPrecision)) Or (IsEmpty(nPrecision))) Then
738:   ArrayString = "Error: Invalid precision value for 'MakeText' method." & Chr(13) & _
                "Please use a numeric value for precision, or skip optional value entirely..."
740:   MakeText = ArrayString
  Exit Function
742: End If

Dim strFormatString As String
745: If nPrecision = 0 Then
746:   strFormatString = "0"
747: ElseIf nPrecision > 0 Then
748:   strFormatString = "0." & String(nPrecision, "0")
749: End If

751: anArray = Me.MatrixArray

' GET MAXIMUM LENGTH
Dim dblVal As Double
Dim strVal As String
Dim lngMaxLength As Long
757: lngMaxLength = 0
758: For theRow = LBound(anArray, 1) To UBound(anArray, 1)
759:     For theCol = LBound(anArray, 2) To UBound(anArray, 2)
760:         dblVal = anArray(theRow, theCol)
761:         If nPrecision < 0 Then
762:           strVal = CStr(dblVal)
763:         Else
764:           strVal = CStr(Format(dblVal, strFormatString))
765:         End If
766:         If Len(strVal) > lngMaxLength Then lngMaxLength = Len(strVal)
767:     Next theCol
'    ArrayString = RTrim(ArrayString) & Chr(13)
769: Next theRow

'Debug.Print lngMaxLength

773: If (strTitle <> "defaultTitlezzz") Then
774:   ArrayString = strTitle & " = " & Chr(13)
775: Else
776:   ArrayString = ""
777: End If
778: For theRow = LBound(anArray, 1) To UBound(anArray, 1)
779:     For theCol = LBound(anArray, 2) To UBound(anArray, 2)
780:         dblVal = anArray(theRow, theCol)
781:         If nPrecision < 0 Then
782:           strVal = CStr(dblVal)
783:         Else
784:           strVal = CStr(Format(dblVal, strFormatString))
785:         End If
786:         ArrayString = ArrayString & Space(lngMaxLength - Len(strVal)) & strVal & "   "
787:     Next theCol
788:     ArrayString = RTrim(ArrayString) & Chr(13)
789: Next theRow

791: MakeText = ArrayString & vbCrLf


  Exit Function
ErrorHandler:
  HandleError True, "MakeText " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function MakeTextWithHeaders(pHeaders As esriSystem.IStringArray, booColumnHeaders As Boolean, _
                         booRowHeaders As Boolean, booHasError As Boolean, Optional nPrecision As Variant = -9, _
                         Optional strTitle As String = "defaultTitlezzz") As String
  On Error GoTo ErrorHandler

                         
'  Test This Function
  
  
  Dim anArray() As Double
  Dim theRow As Long
  Dim theCol As Long
  Dim ArrayString As String
  
813:   booHasError = False
  
815:   If Not booColumnHeaders And Not booRowHeaders Then
816:     MakeTextWithHeaders = MakeText(nPrecision, strTitle)
    Exit Function
818:   End If
  
820:   anArray = Me.MatrixArray
  
  ' CHECK TO MAKE SURE THE NUMBER OF HEADERS MATCHES THE NUMBER OF COLUMNS
823:   If booColumnHeaders Then
824:     If pHeaders.Count <> (UBound(anArray, 2) + 1) Then
825:       booHasError = True
826:       MakeTextWithHeaders = "Number of Headers [" & CStr(pHeaders.Count) & "] <> Number of Columns [" & _
            CStr(UBound(anArray, 2) + 1) & "]"
      Exit Function
829:     End If
830:   End If
  
  ' CHECK TO MAKE SURE THE NUMBER OF HEADERS MATCHES THE NUMBER OF ROWS
833:   If booRowHeaders Then
834:     If pHeaders.Count <> (UBound(anArray, 1) + 1) Then
835:       booHasError = True
836:       MakeTextWithHeaders = "Number of Headers [" & CStr(pHeaders.Count) & "] <> Number of Rows [" & _
            CStr(UBound(anArray, 1) + 1) & "]"
      Exit Function
839:     End If
840:   End If
  
842:   If ((Not IsNumeric(nPrecision)) Or (IsEmpty(nPrecision))) Then
843:     MakeTextWithHeaders = "Error: Invalid precision value for 'MakeText' method." & Chr(13) & _
                  "Please use a numeric value for precision, or skip optional value entirely..."
845:     booHasError = True
    Exit Function
847:   End If
  
  Dim strFormatString As String
850:   If nPrecision = 0 Then
851:     strFormatString = "0"
852:   ElseIf nPrecision > 0 Then
853:     strFormatString = "0." & String(nPrecision, "0")
854:   End If
    
  ' GET MAXIMUM LENGTH
  Dim dblVal As Double
  Dim strVal As String
  Dim lngMaxLength As Long
860:   lngMaxLength = 0
  
  Dim lngIndex As Long
863:   If booColumnHeaders Or booRowHeaders Then
864:     For lngIndex = 0 To pHeaders.Count - 1
865:       strVal = pHeaders.Element(lngIndex)
866:       If Len(strVal) > lngMaxLength Then lngMaxLength = Len(strVal)
867:     Next lngIndex
868:   End If
  
870:   For theRow = LBound(anArray, 1) To UBound(anArray, 1)
871:       For theCol = LBound(anArray, 2) To UBound(anArray, 2)
872:           dblVal = anArray(theRow, theCol)
873:           If nPrecision < 0 Then
874:             strVal = CStr(dblVal)
875:           Else
876:             strVal = CStr(Format(dblVal, strFormatString))
877:           End If
878:           If Len(strVal) > lngMaxLength Then lngMaxLength = Len(strVal)
879:       Next theCol
'      ArrayString = RTrim(ArrayString) & Chr(13)
881:   Next theRow
  
  'Debug.Print lngMaxLength
  
885:   If (strTitle <> "defaultTitlezzz") Then
886:     ArrayString = strTitle & " = " & Chr(13)
887:   Else
888:     ArrayString = ""
889:   End If
  
  ' ADD ROW OF HEADERS IF REQUESTED
892:   If booColumnHeaders Then
893:     If booRowHeaders Then     ' ADD EMPTY SPACE IN UPPER LEFT CORNER OF MATRIX IF INCLUDING ROW HEADERS
894:       strVal = " "
895:       ArrayString = ArrayString & Space(lngMaxLength - Len(strVal)) & strVal & "   "
896:     End If
897:     For theCol = LBound(anArray, 2) To UBound(anArray, 2)
898:       strVal = pHeaders.Element(theCol)
899:       ArrayString = ArrayString & Space(lngMaxLength - Len(strVal)) & strVal & "   "
900:     Next theCol
901:     ArrayString = RTrim(ArrayString) & Chr(13)
902:   End If
  
904:   For theRow = LBound(anArray, 1) To UBound(anArray, 1)
905:       If booRowHeaders Then
906:         strVal = pHeaders.Element(theRow)
907:         ArrayString = ArrayString & Space(lngMaxLength - Len(strVal)) & strVal & "   "
908:       End If
909:       For theCol = LBound(anArray, 2) To UBound(anArray, 2)
910:           dblVal = anArray(theRow, theCol)
911:           If nPrecision < 0 Then
912:             strVal = CStr(dblVal)
913:           Else
914:             strVal = CStr(Format(dblVal, strFormatString))
915:           End If
916:           ArrayString = ArrayString & Space(lngMaxLength - Len(strVal)) & strVal & "   "
917:       Next theCol
918:       ArrayString = RTrim(ArrayString) & Chr(13)
919:   Next theRow
  
921:   MakeTextWithHeaders = ArrayString & vbCrLf


  Exit Function
ErrorHandler:
  HandleError True, "MakeTextWithHeaders " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Property Get MatrixType() As String
  On Error GoTo ErrorHandler

'used when retrieving value of a property, on the right side of an assignment.
'Syntax: Debug.Print X.Dimension
934:     MatrixType = mvarMatrixType

  Exit Property
ErrorHandler:
  HandleError True, "MatrixType " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Property

Public Property Get IsNull() As Boolean
  On Error GoTo ErrorHandler

'used when retrieving value of a property, on the right side of an assignment.
'Syntax: Debug.Print X.Dimension
946:     IsNull = mvarIsNull

  Exit Property
ErrorHandler:
  HandleError True, "IsNull " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Property

Public Property Get Dimension() As String
  On Error GoTo ErrorHandler

'used when retrieving value of a property, on the right side of an assignment.
'Syntax: Debug.Print X.Dimension
958:     Dimension = mvarDimension

  Exit Property
ErrorHandler:
  HandleError True, "Dimension " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Property

Public Property Let MatrixArray(ByVal vData As Variant)
  On Error GoTo ErrorHandler

'used when assigning a value to the property, on the left side of an assignment.
'Syntax: X.MatrixArray = 5
  
971:   If (IsArray(vData)) Then
972:     mvarRows = UBound(vData, 1) + 1
973:     mvarColumns = UBound(vData, 2) + 1
974:     mvarDimension = mvarRows & " x " & mvarColumns
975:     mvarIsNull = False
976:     If (VarType(vData(0, 0)) = vbDouble) Then
977:       mvarMatrixArray = vData
978:     ElseIf (IsNumeric(VarType(vData(0, 0)))) Then
      Dim aRow, aCol As Long
      Dim aNewArray() As Double
      ReDim aNewArray(UBound(vData, 1), UBound(vData, 2))
982:       For aRow = 0 To UBound(vData, 1)
983:         For aCol = 0 To UBound(vData, 2)
984:           aNewArray(aRow, aCol) = CDbl(vData(aRow, aCol))
985:         Next aCol
986:       Next aRow
987:       mvarMatrixArray = aNewArray
      'mvarMatrixArray = CDbl(vData)
989:     Else             ' NON-NUMERIC ARRAY; INVALID
990:       mvarRows = 0
991:       mvarColumns = 0
992:       mvarDimension = "0 x 0"
993:       mvarIsNull = True
994:       mvarMatrixType = "Null"
      Exit Property
996:     End If
    
998:     SetMatrixType
    
1000:   Else
    ' IF SOME OTHER TYPE OF INPUT DATA
1002:   End If
  

  Exit Property
ErrorHandler:
  HandleError True, "MatrixArray " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Property

Private Sub SetMatrixType()
  On Error GoTo ErrorHandler


1014:     If (UBound(mvarMatrixArray, 1) = 0 And (UBound(mvarMatrixArray, 2) = 0)) Then
1015:       mvarMatrixType = "Number"
1016:     ElseIf (UBound(mvarMatrixArray, 1) > 0 And (UBound(mvarMatrixArray, 2) = 0)) Then
1017:       mvarMatrixType = "Column Vector"
1018:     ElseIf (UBound(mvarMatrixArray, 1) = 0 And (UBound(mvarMatrixArray, 2) > 0)) Then
1019:       mvarMatrixType = "Row Vector"
1020:     ElseIf (UBound(mvarMatrixArray, 1) > 0 And (UBound(mvarMatrixArray, 2) > 0)) Then
1021:       mvarMatrixType = "Matrix"
1022:     Else
1023:       mvarMatrixType = "Null"
1024:     End If
    

  Exit Sub
ErrorHandler:
  HandleError False, "SetMatrixType " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Property Get MatrixArray() As Variant
  On Error GoTo ErrorHandler

'used when retrieving value of a property, on the right side of an assignment.
'Syntax: Debug.Print X.MatrixArray
1037:     MatrixArray = mvarMatrixArray

  Exit Property
ErrorHandler:
  HandleError True, "MatrixArray " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Property

Public Property Get Columns() As Long
  On Error GoTo ErrorHandler

'used when retrieving value of a property, on the right side of an assignment.
'Syntax: Debug.Print X.Columns
1049:     Columns = mvarColumns

  Exit Property
ErrorHandler:
  HandleError True, "Columns " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Property

Public Function MakeConstant(aValue As Variant, NumRows As Variant, NumCols As Variant) As Boolean
  On Error GoTo ErrorHandler

  
1060:   If (Not (IsNumeric(aValue) And IsNumeric(NumRows) And IsNumeric(NumCols))) Then
1061:     MakeConstant = False
    Exit Function
1063:   End If
  
  Dim theReturn() As Double
  ReDim theReturn(NumRows - 1, NumCols - 1)
  Dim theCounter As Long
  Dim theCounter2 As Long
  
1070:   For theCounter = 0 To (NumRows - 1)
1071:     For theCounter2 = 0 To (NumCols - 1)
1072:       theReturn(theCounter, theCounter2) = aValue
1073:     Next theCounter2
1074:   Next theCounter
  
1076:   mvarRows = NumRows
1077:   mvarColumns = NumCols
1078:   mvarMatrixArray = theReturn
1079:   mvarDimension = NumRows & " x " & NumCols
1080:   mvarIsNull = False
  
1082:   SetMatrixType
  
1084:   MakeConstant = True
  

  Exit Function
ErrorHandler:
  HandleError True, "MakeConstant " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function MakeIdentity(Size As Long) As Boolean
  On Error GoTo ErrorHandler

  Dim theReturn() As Double
  ReDim theReturn(Size - 1, Size - 1)
  Dim theCounter As Byte
  Dim theCounter2 As Byte
  
1100:   For theCounter = 0 To (Size - 1)
1101:     For theCounter2 = 0 To (Size - 1)
1102:       If (theCounter = theCounter2) Then
1103:         theReturn(theCounter, theCounter2) = 1
1104:       Else
1105:         theReturn(theCounter, theCounter2) = 0
1106:       End If
1107:     Next theCounter2
1108:   Next theCounter
  
1110:   mvarRows = Size
1111:   mvarColumns = Size
1112:   mvarMatrixArray = theReturn
1113:   mvarDimension = Size & " x " & Size
1114:   mvarIsNull = False
  
1116:   SetMatrixType
  
1118:   MakeIdentity = True
  

  Exit Function
ErrorHandler:
  HandleError True, "MakeIdentity " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Subtract(aSubtractMatrix As Matrix_jen, FunctionSucceeded As Boolean) As Matrix_jen
  On Error GoTo ErrorHandler


Dim OrigArray() As Double
Dim SecondArray() As Double
Dim SubtractArray() As Double

1134: OrigArray = Me.MatrixArray
1135: SecondArray = aSubtractMatrix.MatrixArray

ReDim SubtractArray(UBound(OrigArray, 1), UBound(OrigArray, 2)) As Double

Dim aRow, aCol As Long

Dim theSubtractMatrix As Matrix_jen
1142: Set theSubtractMatrix = New Matrix_jen

1144: If ((UBound(OrigArray, 1) <> UBound(SecondArray, 1)) Or (UBound(OrigArray, 2) <> UBound(SecondArray, 2))) Then
1145:   FunctionSucceeded = False        ' BECAUSE OF INCOMPATIBLE DIMENSIONS
1146:   Set Subtract = theSubtractMatrix
  Exit Function
1148: End If

1150: For aRow = 0 To UBound(OrigArray, 1)
1151:   For aCol = 0 To UBound(OrigArray, 2)
1152:     SubtractArray(aRow, aCol) = OrigArray(aRow, aCol) - SecondArray(aRow, aCol)
1153:   Next aCol
1154: Next aRow

FunctionSucceeded = True
1157: theSubtractMatrix.MatrixArray = SubtractArray

1159: Set Subtract = theSubtractMatrix

  Exit Function
ErrorHandler:
  HandleError True, "Subtract " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Add(anAddMatrix As Matrix_jen, FunctionSucceeded As Boolean) As Matrix_jen
  On Error GoTo ErrorHandler


Dim OrigArray() As Double
Dim SecondArray() As Double
Dim AddArray() As Double

1174: OrigArray = Me.MatrixArray
1175: SecondArray = anAddMatrix.MatrixArray

ReDim AddArray(UBound(OrigArray, 1), UBound(OrigArray, 2)) As Double

Dim aRow, aCol As Long

Dim theAddMatrix As Matrix_jen
1182: Set theAddMatrix = New Matrix_jen

1184: If ((UBound(OrigArray, 1) <> UBound(SecondArray, 1)) Or (UBound(OrigArray, 2) <> UBound(SecondArray, 2))) Then
1185:   FunctionSucceeded = False        ' BECAUSE OF INCOMPATIBLE DIMENSIONS
1186:   Set Add = theAddMatrix
  Exit Function
1188: End If

1190: For aRow = 0 To UBound(OrigArray, 1)
1191:   For aCol = 0 To UBound(OrigArray, 2)
1192:     AddArray(aRow, aCol) = OrigArray(aRow, aCol) + SecondArray(aRow, aCol)
1193:   Next aCol
1194: Next aRow

FunctionSucceeded = True
1197: theAddMatrix.MatrixArray = AddArray

1199: Set Add = theAddMatrix


  Exit Function
ErrorHandler:
  HandleError True, "Add " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Multiply(aMultiplyMatrix As Matrix_jen, MultiplySucceeded As Boolean) As Matrix_jen
  On Error GoTo ErrorHandler


Dim OrigArray() As Double
Dim SecondArray() As Double
Dim MultArray() As Double

1215: OrigArray = Me.MatrixArray
1216: SecondArray = aMultiplyMatrix.MatrixArray
ReDim MultArray(UBound(OrigArray, 1), UBound(SecondArray, 2)) As Double

Dim theRows1, theCols1, theRows2, theCols2, aRow, aColumn, aValue As Long
Dim theTempSum As Double

Dim theMultiplyMatrix As Matrix_jen
1223: Set theMultiplyMatrix = New Matrix_jen

1225: theRows1 = UBound(OrigArray, 1)
1226: theCols1 = UBound(OrigArray, 2)
1227: theRows2 = UBound(SecondArray, 1)
1228: theCols2 = UBound(SecondArray, 2)

1230: If (theCols1 <> theRows2) Then
1231:   MultiplySucceeded = False
1232:   Set Multiply = theMultiplyMatrix
  Exit Function
1234: Else
1235:   For aRow = 0 To theRows1
1236:     For aColumn = 0 To theCols2
1237:       theTempSum = 0
1238:       For aValue = 0 To theCols1
1239:         theTempSum = theTempSum + (OrigArray(aRow, aValue) * SecondArray(aValue, aColumn))
1240:       Next aValue
1241:       MultArray(aRow, aColumn) = theTempSum
1242:     Next aColumn
1243:   Next aRow
1244: End If

1246: MultiplySucceeded = True
1247: theMultiplyMatrix.MatrixArray = MultArray
1248: Set Multiply = theMultiplyMatrix


  Exit Function
ErrorHandler:
  HandleError True, "Multiply " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function MultByConst(aConstant As Variant, OperationSucceeded As Boolean) As Matrix_jen
  On Error GoTo ErrorHandler


Dim theMByConstMatrix As Matrix_jen
1261: Set theMByConstMatrix = New Matrix_jen

Dim anArray() As Double
Dim theMByConstArray() As Double

1266: If (Not IsNumeric(aConstant)) Then     ' BAIL OUT IF USER DOESN'T SEND A NUMBER TO THIS FUNCTION
1267:   OperationSucceeded = False
1268:   Set MultByConst = theMByConstMatrix
  Exit Function
1270: End If

1272: anArray = Me.MatrixArray
ReDim theMByConstArray(UBound(anArray, 1), UBound(anArray, 2)) As Double

Dim theRow As Long
Dim theCol As Long

1278: For theRow = LBound(anArray, 1) To UBound(anArray, 1)
1279:     For theCol = LBound(anArray, 2) To UBound(anArray, 2)
1280:         theMByConstArray(theRow, theCol) = anArray(theRow, theCol) * aConstant
1281:     Next theCol
1282: Next theRow

1284: OperationSucceeded = True
1285: theMByConstMatrix.MatrixArray = theMByConstArray
1286: Set MultByConst = theMByConstMatrix


  Exit Function
ErrorHandler:
  HandleError True, "MultByConst " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function Clone() As Matrix_jen
  On Error GoTo ErrorHandler


Dim theCloneMatrix As Matrix_jen
1298: Set theCloneMatrix = New Matrix_jen

Dim anArray() As Double
1301: anArray = Me.MatrixArray

Dim theCloneArray() As Double
1304: theCloneArray = anArray

1306: theCloneMatrix.MatrixArray = theCloneArray
1307: Set Clone = theCloneMatrix


  Exit Function
ErrorHandler:
  HandleError True, "Clone " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Transpose(TransposeSucceeded As Boolean) As Matrix_jen
  On Error GoTo ErrorHandler


1319: TransposeSucceeded = False

Dim theTransposeMatrix As Matrix_jen
1322: Set theTransposeMatrix = New Matrix_jen

Dim anArray() As Double
Dim theTransposeArray() As Double

1327: anArray = Me.MatrixArray

ReDim theTransposeArray(UBound(anArray, 2), UBound(anArray, 1)) As Double
Dim theRow As Long
Dim theCol As Long

1333: For theRow = LBound(anArray, 1) To UBound(anArray, 1)
1334:     For theCol = LBound(anArray, 2) To UBound(anArray, 2)
1335:         theTransposeArray(theCol, theRow) = anArray(theRow, theCol)
1336:     Next theCol
1337: Next theRow

1339: theTransposeMatrix.MatrixArray = theTransposeArray

1341: Set Transpose = theTransposeMatrix
1342: TransposeSucceeded = True


  Exit Function
ErrorHandler:
  HandleError True, "Transpose " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Invert(InvertSucceeded As Byte) As Matrix_jen
  On Error GoTo ErrorHandler

' MODIFIES BYTE VALUE
'   0) IF OPERATION SUCCESSFUL
'   1) FAILED AT MatrixLUDecomp_jen BECAUSE OF NON-SQUARE MATRIX
'   2) FAILED AT MatrixLUDecomp_jen BECAUSE OF SINGULAR MATRIX
'   3) FAILED AT MatrixFBSub_jen BECAUSE OF UNKNOWN REASONS

Dim theInvertMatrix As Matrix_jen
1360: Set theInvertMatrix = New Matrix_jen

Dim anArray() As Double
Dim N As Double

1365: anArray = Me.MatrixArray
1366: N = UBound(anArray, 1)

Dim theDecomp() As Double
Dim theIndexVector() As Double
Dim DecompSuccessful As Byte

' LOWER-UPPER DECOMPOSITION
1373: DecompSuccessful = Me.MatrixLUDecomp_jen(theDecomp, theIndexVector)

1375: If (DecompSuccessful <> 0) Then
' SOME ERROR FOUND
1377:   Set Invert = theInvertMatrix
1378:   InvertSucceeded = DecompSuccessful
  Exit Function
1380: End If

' WORK ON COPY OF ORIGINAL MATRIX
Dim theMatrixClone() As Double
ReDim theMatrixClone(UBound(anArray, 1), UBound(anArray, 2))
1385: theMatrixClone = anArray

Dim J As Long
Dim i As Long
Dim theInputVector() As Double
Dim BacksubSuccessful As Byte

' BACK SUBSTITUTION
1393: For J = 0 To N
  ReDim theInputVector(N) As Double
1395:   For i = 0 To N
1396:     theInputVector(i) = 0
1397:   Next i
1398:   theInputVector(J) = 1
  
1400:   BacksubSuccessful = MatrixFBSub_jen(theDecomp, theIndexVector, theInputVector)
1401:   If (BacksubSuccessful <> 10) Then
  ' SOME ERROR FOUND
1403:     Set Invert = theInvertMatrix
1404:     InvertSucceeded = 3
    Exit Function
1406:   End If
  
1408:   For i = 0 To N
1409:     theMatrixClone(i, J) = theInputVector(i)
1410:   Next i
1411: Next J

1413: theInvertMatrix.MatrixArray = theMatrixClone

1415: InvertSucceeded = 0
1416: Set Invert = theInvertMatrix


  Exit Function
ErrorHandler:
  HandleError True, "Invert " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function MatrixFBSub_jen(theDecomp() As Double, theIndexVec() As Double, theBVec() As Double) As Byte
  On Error GoTo ErrorHandler


' DERIVED FROM Jennessent.MatrixFBSub
1428: MatrixFBSub_jen = 3   ' DEFAULT VALUE SAYING OPERATION FAILED

Dim N, IP, II, i, J As Long
Dim theSum As Double

1433: N = UBound(theDecomp, 1)

1435: IP = 0
1436: theSum = 0
1437: II = 0

1439: For i = 0 To N
  
1441:   IP = theIndexVec(i)
1442:   theSum = theBVec(IP)
1443:   theBVec(IP) = theBVec(i)
  
1445:   If (II <> 0) Then
1446:     For J = (II - 1) To (i - 1) Step 1
1447:       theSum = theSum - (theDecomp(i, J) * theBVec(J))
1448:     Next J
1449:   Else
1450:     If (theSum <> 0) Then
1451:       II = i + 1
1452:     End If
1453:   End If
1454:   theBVec(i) = theSum

1456: Next i
        
1458: For i = N To 0 Step -1
1459:   theSum = theBVec(i)
1460:   For J = (i + 1) To N Step 1
1461:     theSum = theSum - (theDecomp(i, J) * theBVec(J))
1462:   Next J
1463:   theBVec(i) = theSum / theDecomp(i, i)
1464: Next i
  
1466: MatrixFBSub_jen = 10   ' RETURN VALUE SAYING IT WAS SUCCESSFUL


  Exit Function
ErrorHandler:
  HandleError True, "MatrixFBSub_jen " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Property Get Rows() As Long
  On Error GoTo ErrorHandler

'used when retrieving value of a property, on the right side of an assignment.
'Syntax: Debug.Print X.Count
1480:     Rows = mvarRows

  Exit Property
ErrorHandler:
  HandleError True, "Rows " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Property

Public Function MatrixLUDecomp_jen(theSaveMatrix() As Double, theIndexVector() As Double) As Byte
  On Error GoTo ErrorHandler


'' Jennessent.MatrixLUDecomp
'
'' ADAPTED FROM NUMERICAL RECIPIES IN C
'
' Should be called by the following:
' dim theDecomp() as double
' dim theIndexVector() as double
' dim DecompSuccessful as byte
' DecompSuccessful = yourMatrix.MatrixLUDecomp (theDecomp, theIndexVector)
' WILL RETURN 0 IF SUCCESSFUL
'             1 IF NON-SQUARE MATRIX ENTERED
'             2 IF SINGULAR MATRIX ENTERED

Dim theDim1, theDim2 As Long
Dim i, J, K, D As Long

ReDim theSaveMatrix(theDim1, theDim2)
Dim thePivotVector() As Double
ReDim thePivotVector(theDim1)
Dim lCount As Long
Dim dTiny As Double
Dim dBig As Double
Dim lRow As Long
Dim lCol As Long
Dim dTempValue As Double
Dim dMax As Double
Dim dTheSum As Double
Dim dDummyVal As Double

1520: MatrixLUDecomp_jen = 0
Dim anArray() As Double
1522: anArray = Me.MatrixArray

1524: theDim1 = UBound(anArray, 1)
1525: theDim2 = UBound(anArray, 2)

ReDim theIndexVector(theDim1)
ReDim thePivotVector(theDim1)

1530: If (UBound(anArray, 1) <> UBound(anArray, 2)) Then
1531:   MatrixLUDecomp_jen = 1
  Exit Function
1533: End If

1535: theSaveMatrix = anArray
1536: lCount = UBound(anArray)
1537: dTiny = 1E-21
1538: D = 1

' CREATE A VECTOR OF 1/EXTREME ROW VALUES; FOR PIVOTING

1542: For lRow = 0 To lCount
1543:   dBig = 0
1544:   For lCol = 0 To lCount
1545:     dTempValue = Abs(anArray(lRow, lCol))
1546:     If (dTempValue > dBig) Then dBig = dTempValue
1547:   Next lCol
1548:   If (dBig = 0) Then
    ' code singular matrix error
    ' stop function
1551:     MatrixLUDecomp_jen = 2
    Exit Function
1553:   End If
1554:   thePivotVector(lRow) = (1 / dBig)
1555: Next lRow

1557: dMax = 0

1559: For J = 0 To lCount
1560:   For i = 0 To (J - 1) Step 1
1561:     dTheSum = theSaveMatrix(i, J)
1562:     For K = 0 To (i - 1) Step 1
1563:       dTheSum = dTheSum - (theSaveMatrix(i, K) * theSaveMatrix(K, J))
1564:     Next K
1565:     theSaveMatrix(i, J) = dTheSum
1566:   Next i
  
1568:   dBig = 0
  
1570:   For i = J To lCount Step 1
1571:     dTheSum = theSaveMatrix(i, J)
1572:     For K = 0 To (J - 1) Step 1
1573:       dTheSum = dTheSum - (theSaveMatrix(i, K) * theSaveMatrix(K, J))
1574:     Next K
    
1576:     theSaveMatrix(i, J) = dTheSum
1577:     dTempValue = thePivotVector(i) * Abs(dTheSum)
    
1579:     If (dTempValue >= dBig) Then
1580:       dBig = dTempValue
1581:       dMax = i
1582:     End If
1583:   Next i

1585:   If (J <> dMax) Then
1586:     For K = 0 To lCount Step 1
1587:       dDummyVal = theSaveMatrix(dMax, K)
1588:       theSaveMatrix(dMax, K) = theSaveMatrix(J, K)
1589:       theSaveMatrix(J, K) = dDummyVal
1590:     Next K
1591:     D = -D
1592:     thePivotVector(dMax) = thePivotVector(J)
1593:   End If
 
1595:   theIndexVector(J) = dMax

1597:   If (theSaveMatrix(J, J) = 0) Then
1598:     theSaveMatrix(J, J) = dTiny
1599:   End If

1601:   If (J <> lCount) Then
1602:     dDummyVal = 1 / (theSaveMatrix(J, J))
1603:     For i = (J + 1) To lCount Step 1
1604:       dTempValue = theSaveMatrix(i, J)
1605:       theSaveMatrix(i, J) = dTempValue * dDummyVal
1606:     Next i
1607:   End If
  
1609: Next J
    

  Exit Function
ErrorHandler:
  HandleError True, "MatrixLUDecomp_jen " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Sub Class_Initialize()
  On Error GoTo ErrorHandler

1620:   mvarRows = 0
1621:   mvarColumns = 0
1622:   mvarMatrixArray = 0
1623:   mvarDimension = "0 x 0"
1624:   mvarIsNull = True
1625:   mvarMatrixType = "Null"

  Exit Sub
ErrorHandler:
  HandleError True, "Class_Initialize " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub SortDoubleAscendingWithObjects(narray() As Double, varObjArray() As Variant, inLow As Long, inHi As Long)
  On Error GoTo ErrorHandler


  Dim pivot As Double
  Dim tmpSwap As Double
  Dim tmpSizeSwap As Variant
  Dim tmpLow As Long
  Dim tmpHi  As Long
   
1642:   tmpLow = inLow
1643:   tmpHi = inHi
   
1645:   pivot = narray((inLow + inHi) / 2)
1646:   While (tmpLow <= tmpHi)
       
1648:     While (narray(tmpLow) < pivot And tmpLow < inHi)
1649:        tmpLow = tmpLow + 1
1650:     Wend
  
1652:     While (pivot < narray(tmpHi) And tmpHi > inLow)
1653:        tmpHi = tmpHi - 1
1654:     Wend
1655:     If (tmpLow <= tmpHi) Then
1656:        tmpSwap = narray(tmpLow)
1657:        narray(tmpLow) = narray(tmpHi)
1658:        narray(tmpHi) = tmpSwap
       
1660:        tmpSizeSwap = varObjArray(tmpLow)
1661:        varObjArray(tmpLow) = varObjArray(tmpHi)
1662:        varObjArray(tmpHi) = tmpSizeSwap
       
1664:        tmpLow = tmpLow + 1
1665:        tmpHi = tmpHi - 1
1666:     End If
     
1668:   Wend
    
1670:   If (inLow < tmpHi) Then SortDoubleAscendingWithObjects narray(), varObjArray(), inLow, tmpHi
1671:   If (tmpLow < inHi) Then SortDoubleAscendingWithObjects narray(), varObjArray(), tmpLow, inHi


  Exit Sub
ErrorHandler:
  HandleError True, "SortDoubleAscendingWithObjects " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub






