Attribute VB_Name = "TPI_Code"
Option Explicit
Const c_sModuleFileName As String = "D:\arcGIS_stuff\consultation\brost_climate_change\code\TPI_Code.bas"


Public Function Create_TPI(pDocParams As esriSystem.IVariantArray, pParams As esriSystem.IVariantArray) As String
  On Error GoTo ErrorHandler


  ' This subroutine calculates 3 variations on TPI:
  ' 1) TPI: [Cell Elevation Value - Neighborhood Mean Elevation]
  ' 2) Standardized Elevation:  [TPI / Neighborhood Standard Deviation]
  ' 3) Standardized TPI:  [TPI - TPI Mean from Entire Raster] / [TPI Standard Deviation from Entire Raster]

  Dim pApp As IApplication
  Dim pMxDoc As IMxDocument
  Dim pThisExt As TopoCorridor.clsExtension
  
18:   Set pApp = pDocParams.Element(0)
19:   Set pMxDoc = pDocParams.Element(1)
20:   Set pThisExt = pDocParams.Element(2)
  
  Dim psbar As IStatusBar
23:   Set psbar = pApp.StatusBar
  Dim pProgAnim As IAnimationProgressor
25:   Set pProgAnim = psbar.ProgressAnimation
26:   pProgAnim.Show
27:   pProgAnim.Play
  
  ' RETURN ARRAY IS IVariantArray, CONTAINING:
  ' 0) IRasterBand: SELECTED RASTER BAND
  ' 1) STRING: Save Format
  ' 2) STRING: TEXT VERSION OF SAVE FORMAT
  ' 3) STRING: New Raster Name
  ' 4) BOOLEAN: Whether new workspace is File Geodatabase
  ' 5) IWORKSPACE: Either an IRasterWorkspace2 or an IRasterWorkspaceEx
  ' 6) STRING: Extension for output
  ' 7) STRING: Name of Selected Raster Band
  ' 8) STRING: TPI Option; "TPI", "Standardized Elevation", "Standardized TPI"
  ' 9) INbrHood:  DEFINED NEIGHBORHOOD
  '10) STRING: Description of neighborhood
  '11) BOOLEAN: SET NODATA IF ANY NULL

  Dim datBegan As Date
44:   datBegan = Now
  
  Dim strReport As String
47:   strReport = "Function failed..."
  
49:   Create_TPI = strReport
  
  ' DEFINE ADDITIONAL VARIABLES
  Dim pMakerOp As IRasterMakerOp
53:   Set pMakerOp = New RasterMakerOp
  Dim pRastAnEnv As IRasterAnalysisEnvironment
  Dim pRastProps As IRasterProps
  Dim pSpRef As ISpatialReference
  Dim pGeoDataset As IGeoDataset
  Dim pMathOp As IMathOp
59:   Set pMathOp = New RasterMathOps
  Dim pNbrHoodOp As INeighborhoodOp
61:   Set pNbrHoodOp = New RasterNeighborhoodOp
  Dim pNbrHood As IRasterNeighborhood
  Dim strTPIOption As String
  Dim strNbrHoodDesc As String
  
  Dim pInputBand As IRasterBand
  Dim pRastLayer As IRasterLayer
  Dim pInputDataset As IRasterDataset
  Dim pInputRaster As IRaster2
  Dim pEnv As IEnvelope
  Dim strSaveFormat As String
  Dim strNewRasterName As String
  Dim booIsFGB As Boolean
  Dim pRastWS As IRasterWorkspace2
  Dim pRastWsEx As IRasterWorkspaceEx
  Dim strSaveFormatString As String
  Dim pWS As IWorkspace
  Dim strExtension As String
  Dim strInputName As String
  Dim booExcludeIfAnyNull As Boolean
  Dim pSaveAs As ISaveAs2
  
  ' GET PARAMETERS FROM DIALOG
84:   Set pInputBand = pParams.Element(0)
85:   Set pGeoDataset = pInputBand
86:   Set pEnv = pGeoDataset.Extent
  
88:   strSaveFormat = pParams.Element(1)
89:   strSaveFormatString = pParams.Element(2)
90:   strNewRasterName = pParams.Element(3)
91:   booIsFGB = pParams.Element(4)
92:   Set pWS = pParams.Element(5)
93:   If booIsFGB Then
94:     Set pRastWsEx = pWS
95:   Else
96:     Set pRastWS = pWS
97:   End If
98:   strExtension = pParams.Element(6)
99:   strInputName = pParams.Element(7)
100:   strTPIOption = pParams.Element(8)
101:   Set pNbrHood = pParams.Element(9)
102:   strNbrHoodDesc = pParams.Element(10)
103:   booExcludeIfAnyNull = pParams.Element(11)
  
  ' ANALYSIS ==========================================================================

  ' FIND DEM LAYER IN MAP AND GET RASTER DATASET
108:   Set pInputDataset = pInputBand
109:   Set pSpRef = pGeoDataset.SpatialReference
110:   Set pRastProps = pInputBand
111:   Set pRastAnEnv = pMakerOp
112:   pRastAnEnv.SetCellSize esriRasterEnvValue, pInputDataset
113:   pRastAnEnv.SetExtent esriRasterEnvValue, pInputDataset, pInputDataset
114:   Set pRastAnEnv.OutSpatialReference = pSpRef

  ' CALCULATE EITHER STANDARDIZED DEM, SIMPLE TPI, OR STANDARDIZED TPI
  
  Dim pTempDataset As IRasterDataset2
  Dim pTPIDataset As IRasterDataset2

121:   If strTPIOption = "TPI" Then
122:     Set pTempDataset = ReturnTPIDataset(pInputDataset, psbar, strInputName, pNbrHoodOp, pNbrHood, pMathOp, _
          pMxDoc, False, booExcludeIfAnyNull)
124:   ElseIf strTPIOption = "Standardized Elevation" Then
125:     Set pTempDataset = ReturnStandardizedElevationDataset(pInputDataset, pNbrHoodOp, psbar, _
          pMakerOp, pNbrHood, strInputName, pMxDoc, booExcludeIfAnyNull, False)
127:   ElseIf strTPIOption = "Standardized TPI" Then
128:     Set pTempDataset = ReturnStandardizedTPIDataset(pInputDataset, pNbrHood, "TPI", strInputName, _
          psbar, pNbrHoodOp, pMxDoc, pMakerOp, booExcludeIfAnyNull, False)
130:   End If

  
'  MsgBox "strNewRasterName = '" & strNewRasterName & "'" & vbCrLf & _
         "strExtension = '" & strExtension & "'" & vbCrLf & _
         "strSaveFormat = '" & strSaveFormat & "'"
         
'  Dim pTemp As IRasterLayer
'  Set pTemp = New RasterLayer
'  pTemp.CreateFromDataset pTempDataset
'  pMxDoc.FocusMap.AddLayer pTemp
  
  Dim pTempRaster As IRaster
  Dim pTempProps As IRasterProps
144:   Set pTempRaster = pTempDataset.CreateFullRaster
145:   Set pTempProps = pTempRaster
146:   pTempProps.PixelType = PT_FLOAT
  
  Dim pRastStorDef As IRasterStorageDef
149:   Set pRastStorDef = New RasterStorageDef
150:   Set pRastStorDef.CellSize = pRastProps.MeanCellSize
  
152:   If booIsFGB Then
153:     Set pTPIDataset = pRastWsEx.SaveAsRasterDataset(strNewRasterName, pTempDataset.CreateFullRaster, Nothing, "", Nothing, Nothing)
154:   Else
155:     Set pSaveAs = pTempRaster
'    MsgBox "pTempDataset is nothing = " & CStr(pTempDataset Is Nothing)
'    MsgBox "TypeOf pTempDataset is IRasterDataset = " & CStr(TypeOf pTempDataset Is IRasterDataset)
'    MsgBox "TypeOf pRastWS is IRasterWorkspace = " & CStr(TypeOf pRastWS Is IRasterWorkspace)
159:     Set pTPIDataset = pSaveAs.SaveAsRasterDataset(strNewRasterName & strExtension, pRastWS, strSaveFormat, pRastStorDef)
160:     Set pSaveAs = Nothing
161:     Set pTempRaster = Nothing
162:     Set pTPIDataset = Nothing
163:     Set pTempProps = Nothing
164:     Set pTempDataset = Nothing
165:     Set pTPIDataset = pRastWS.OpenRasterDataset(strNewRasterName & strExtension)
166:   End If

168:   Set pTempRaster = Nothing
169:   Set pTempProps = Nothing
'  ----------------------------
  Dim pNewLayer As IRasterLayer
  
  Dim pOutputRaster As IRaster
  
  Dim pStatsBandColl As IRasterBandCollection
  Dim pStatsBand As IRasterBand
177:   Set pStatsBandColl = pTPIDataset
178:   Set pStatsBand = pStatsBandColl.Item(0)
179:   pStatsBand.ComputeStatsAndHist
  
  ' ADD RASTER TO MAP DOCUMENT
182:   Set pNewLayer = New RasterLayer
183:   pNewLayer.CreateFromDataset pTPIDataset
184:   pNewLayer.Name = strNewRasterName

  ' SYMBOLIZE RASTER LAYER

  ' Create renderer and QI RasterRenderer
  Dim pStretchRen As IRasterStretchColorRampRenderer
190:   Set pStretchRen = New RasterStretchColorRampRenderer
  Dim pRasRen As IRasterRenderer
192:   Set pRasRen = pStretchRen
  
  Dim pRasterStretch As IRasterStretch
195:   Set pRasterStretch = pStretchRen
196:   pRasterStretch.StretchType = esriRasterStretch_StandardDeviations

  ' Set raster for the renderer and update
199:   Set pOutputRaster = pTPIDataset.CreateFullRaster
200:   Set pRasRen.Raster = pOutputRaster
201:   pRasRen.Update

  ' COLORS
  Dim pRedColor As IColor
  Dim pYellowColor As IColor
  Dim pGreenColor As IColor

208:   Set pRedColor = New RgbColor
209:   Set pYellowColor = New RgbColor
210:   Set pGreenColor = New RgbColor

212:   pRedColor.RGB = RGB(190, 0, 0)
213:   pYellowColor.RGB = RGB(255, 255, 200)
214:   pGreenColor.RGB = RGB(0, 140, 0)

  ' CREATE COLOR RAMPS
  Dim pRamp1 As IAlgorithmicColorRamp
218:   Set pRamp1 = New AlgorithmicColorRamp
219:   pRamp1.Algorithm = esriLabLChAlgorithm
220:   pRamp1.Size = 130
221:   pRamp1.FromColor = pGreenColor
222:   pRamp1.ToColor = pYellowColor
223:   pRamp1.CreateRamp True

  Dim pRamp2 As IAlgorithmicColorRamp
226:   Set pRamp2 = New AlgorithmicColorRamp
227:   pRamp2.Algorithm = esriLabLChAlgorithm
228:   pRamp2.Size = 130
229:   pRamp2.FromColor = pYellowColor
230:   pRamp2.ToColor = pRedColor
231:   pRamp2.CreateRamp True

  Dim pMultiRamp As IMultiPartColorRamp
234:   Set pMultiRamp = New MultiPartColorRamp
235:   pMultiRamp.Ramp(0) = pRamp1
236:   pMultiRamp.Ramp(1) = pRamp2
237:   pMultiRamp.Size = 260
238:   pMultiRamp.CreateRamp True

  ' Plug this colorramp into renderer and select a band
241:   pStretchRen.BandIndex = 0
242:   pStretchRen.ColorRamp = pMultiRamp

  ' Update the renderer with new settings and plug into layer
245:   pRasRen.Update
246:   Set pNewLayer.Renderer = pStretchRen
247:   pMxDoc.AddLayer pNewLayer
'-----------------------------------
  
250:   pMxDoc.UpdateContents

252:   strReport = "Calculate TPI Function Complete:" & vbCrLf & _
              "----------------------------------------------------------" & vbCrLf & _
              "Analysis Parameters:" & vbCrLf & _
              " --> TPI Type = " & strTPIOption & vbCrLf & _
              " --> Input Raster Layer: " & strInputName & vbCrLf & _
              " --> " & strNbrHoodDesc & vbCrLf & _
              " --> Option to Mark as NoData if any NoData Cells in Neighborhood: " & _
                      UCase(CStr(booExcludeIfAnyNull)) & vbCrLf & _
              " --> Output Format: " & strSaveFormatString & vbCrLf & _
              " --> Filename: " & strNewRasterName & strExtension & vbCrLf & _
              " --> Workspace = " & pWS.PathName & vbCrLf & _
              "----------------------------------------------------------" & vbCrLf & _
              TopoCorridor.MyGeneralOperations.ReturnTimeElapsed(datBegan, Now)
  
266:   pMxDoc.UpdateContents
267:   pMxDoc.ActiveView.PartialRefresh esriViewGeography, Nothing, pEnv
  
269:   pProgAnim.Stop
270:   pProgAnim.Hide
271:   Set pProgAnim = Nothing
  
273:   Create_TPI = strReport
  
275:   GoTo CleanUp
ErrorHandler:

278:   pProgAnim.Stop
279:   pProgAnim.Hide
280:   Set pProgAnim = Nothing
  HandleError True, "Create_TPI " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
  

CleanUp:
285:   Set pApp = Nothing
286:   Set pMxDoc = Nothing
287:   Set pThisExt = Nothing
288:   Set psbar = Nothing
289:   Set pProgAnim = Nothing
290:   Set pMakerOp = Nothing
291:   Set pRastAnEnv = Nothing
292:   Set pRastProps = Nothing
293:   Set pSpRef = Nothing
294:   Set pGeoDataset = Nothing
295:   Set pMathOp = Nothing
296:   Set pNbrHoodOp = Nothing
297:   Set pNbrHood = Nothing
298:   Set pInputBand = Nothing
299:   Set pRastLayer = Nothing
300:   Set pInputDataset = Nothing
301:   Set pInputRaster = Nothing
302:   Set pEnv = Nothing
303:   Set pRastWS = Nothing
304:   Set pRastWsEx = Nothing
305:   Set pWS = Nothing
306:   Set pSaveAs = Nothing
307:   Set pTempDataset = Nothing
308:   Set pTPIDataset = Nothing
309:   Set pTempRaster = Nothing
310:   Set pTempProps = Nothing
311:   Set pRastStorDef = Nothing
312:   Set pNewLayer = Nothing
313:   Set pOutputRaster = Nothing
314:   Set pStatsBandColl = Nothing
315:   Set pStatsBand = Nothing
316:   Set pStretchRen = Nothing
317:   Set pRasRen = Nothing
318:   Set pRasterStretch = Nothing
319:   Set pRedColor = Nothing
320:   Set pYellowColor = Nothing
321:   Set pGreenColor = Nothing
322:   Set pRamp1 = Nothing
323:   Set pRamp2 = Nothing
324:   Set pMultiRamp = Nothing

End Function

Public Function Create_Sample_3_Class_SlopeClass(pDocParams As esriSystem.IVariantArray, _
    pParams As esriSystem.IVariantArray) As String
  On Error GoTo ErrorHandler

    
  Dim pApp As IApplication
  Dim pMxDoc As IMxDocument
  Dim pThisExt As TopoCorridor.clsExtension
  
337:   Set pApp = pDocParams.Element(0)
338:   Set pMxDoc = pDocParams.Element(1)
339:   Set pThisExt = pDocParams.Element(2)

  ' RETURN ARRAY IS IVariantArray, CONTAINING:
  ' 0) IRasterBand: SELECTED RASTER BAND
  ' 1) STRING: Save Format
  ' 2) STRING: TEXT VERSION OF SAVE FORMAT
  ' 3) STRING: New Raster Name
  ' 4) BOOLEAN: Whether new workspace is File Geodatabase
  ' 5) IWORKSPACE: Either an IRasterWorkspace2 or an IRasterWorkspaceEx
  ' 6) STRING: Extension for output
  ' 7) STRING: Name of Selected Raster Band
  ' 8) STRING: TPI Option; "TPI", "Standardized Elevation", "Standardized TPI"
  ' 9) INbrHood:  DEFINED NEIGHBORHOOD
  '10) STRING: Description of neighborhood
  '11) BOOLEAN: SET NODATA IF ANY NULL
  '12) STRING: Class 1 Name
  '13) STRING: Class 2 Name
  '14) STRING: Class 3 Name
  '15) DOUBLE: Threshold [A]
  '16) DOUBLE: Threshold [B]
  '17) STRING: CLASS DEFINITIONS FROM RICH TEXT BOX

  Dim datBegan As Date
362:   datBegan = Now
  
  Dim strReport As String
365:   strReport = "Function failed..."
  
367:   Create_Sample_3_Class_SlopeClass = strReport
  
  ' DEFINE ADDITIONAL VARIABLES
  Dim psbar As IStatusBar
371:   Set psbar = pApp.StatusBar
  Dim pProgAnim As IAnimationProgressor
373:   Set pProgAnim = psbar.ProgressAnimation
374:   pProgAnim.Show
375:   pProgAnim.Play
  
  Dim pMakerOp As IRasterMakerOp
378:   Set pMakerOp = New RasterMakerOp
  Dim pRastAnEnv As IRasterAnalysisEnvironment
  Dim pRastProps As IRasterProps
  Dim pSpRef As ISpatialReference
  Dim pGeoDataset As IGeoDataset
  Dim pMathOp As IMathOp
384:   Set pMathOp = New RasterMathOps
  Dim pNbrHoodOp As INeighborhoodOp
386:   Set pNbrHoodOp = New RasterNeighborhoodOp
  Dim pNbrHood As IRasterNeighborhood
  Dim pLogOp As ILogicalOp
389:   Set pLogOp = New RasterMathOps
  Dim pConOp As IConditionalOp
391:   Set pConOp = New RasterConditionalOp
  Dim strTPIOption As String
  Dim strNbrHoodDesc As String
  
  Dim strClass1Name As String
  Dim strClass2Name As String
  Dim strClass3Name As String
  Dim dblThresholdA As Double
  Dim dblThresholdB As Double
  Dim strClassDescription As String
  
  Dim pInputBand As IRasterBand
  Dim pRastLayer As IRasterLayer
  Dim pInputDataset As IRasterDataset
  Dim pInputRaster As IRaster2
  Dim pEnv As IEnvelope
  Dim strSaveFormat As String
  Dim strNewRasterName As String
  Dim booIsFGB As Boolean
  Dim pRastWS As IRasterWorkspace2
  Dim pRastWsEx As IRasterWorkspaceEx
  Dim strSaveFormatString As String
  Dim pWS As IWorkspace
  Dim strExtension As String
  Dim strInputName As String
  Dim booExcludeIfAnyNull As Boolean
  Dim pSaveAs As ISaveAs2
  
  ' GET PARAMETERS FROM DIALOG
420:   Set pInputBand = pParams.Element(0)
421:   Set pGeoDataset = pInputBand
422:   Set pEnv = pGeoDataset.Extent
  
424:   strSaveFormat = pParams.Element(1)
425:   strSaveFormatString = pParams.Element(2)
426:   strNewRasterName = pParams.Element(3)
427:   booIsFGB = pParams.Element(4)
428:   Set pWS = pParams.Element(5)
429:   If booIsFGB Then
430:     Set pRastWsEx = pWS
431:   Else
432:     Set pRastWS = pWS
433:   End If
434:   strExtension = pParams.Element(6)
435:   strInputName = pParams.Element(7)
436:   strTPIOption = pParams.Element(8)
437:   Set pNbrHood = pParams.Element(9)
438:   strNbrHoodDesc = pParams.Element(10)
439:   booExcludeIfAnyNull = pParams.Element(11)
440:   strClass1Name = pParams.Element(12)
441:   strClass2Name = pParams.Element(13)
442:   strClass3Name = pParams.Element(14)
443:   dblThresholdA = pParams.Element(15)
444:   dblThresholdB = pParams.Element(16)
445:   strClassDescription = pParams.Element(17)
  
  ' ANALYSIS ==========================================================================

  ' FIND DEM LAYER IN MAP AND GET RASTER DATASET
450:   Set pInputDataset = pInputBand.RasterDataset
451:   Set pSpRef = pGeoDataset.SpatialReference
452:   Set pRastProps = pInputBand
453:   Set pRastAnEnv = pMakerOp
454:   pRastAnEnv.SetCellSize esriRasterEnvValue, pInputDataset
455:   pRastAnEnv.SetExtent esriRasterEnvValue, pInputDataset, pInputDataset
456:   Set pRastAnEnv.OutSpatialReference = pSpRef

  ' CALCULATE EITHER STANDARDIZED DEM, SIMPLE TPI, OR STANDARDIZED TPI
  
  Dim pTempDataset As IRasterDataset2
  Dim pTPIDataset As IRasterDataset2

463:   If strTPIOption = "TPI" Then
464:     Set pTPIDataset = ReturnTPIDataset(pInputDataset, psbar, strInputName, pNbrHoodOp, pNbrHood, pMathOp, _
          pMxDoc, False, booExcludeIfAnyNull)
466:   ElseIf strTPIOption = "Standardized Elevation" Then
467:     Set pTPIDataset = ReturnStandardizedElevationDataset(pInputDataset, pNbrHoodOp, psbar, _
          pMakerOp, pNbrHood, strInputName, pMxDoc, booExcludeIfAnyNull, False)
469:   ElseIf strTPIOption = "Standardized TPI" Then
470:     Set pTPIDataset = ReturnStandardizedTPIDataset(pInputDataset, pNbrHood, "TPI", strInputName, _
          psbar, pNbrHoodOp, pMxDoc, pMakerOp, booExcludeIfAnyNull, False)
472:   End If
  
  Dim pTempRasterA As IRaster
  Dim pTempProps As IRasterProps
476:   Set pTempRasterA = pTPIDataset.CreateFullRaster
477:   Set pTempProps = pTempRasterA
478:   pTempProps.PixelType = PT_FLOAT
  
  ' CONVERT TPI INTO 3-CATEGORY CLASSIFICATION RASTER
  ' FIRST CREATE CONSTANT-VALUE RASTERS FOR THE 4 CATEGORIES
482:   psbar.Message(0) = "Calculating 3-Category Topographic Position Classification Raster..."
  Dim pCanyonBottomDataset As IRasterDataset
  Dim pSlopeDataset As IRasterDataset
  Dim pRidgelineDataset As IRasterDataset
  Dim pTempRaster As IRaster2
  
488:   Set pTempRaster = pMakerOp.MakeConstant(1, True)
489:   Set pCanyonBottomDataset = pTempRaster.RasterDataset

491:   Set pTempRaster = pMakerOp.MakeConstant(2, True)
492:   Set pSlopeDataset = pTempRaster.RasterDataset

494:   Set pTempRaster = pMakerOp.MakeConstant(3, True)
495:   Set pRidgelineDataset = pTempRaster.RasterDataset

  Dim pTPI_ThresholdA_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = -8
  Dim pTPI_ThresholdB_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = 8

500:   Set pTempRaster = pMakerOp.MakeConstant(dblThresholdA, False)
501:   Set pTPI_ThresholdA_Dataset = pTempRaster.RasterDataset

503:   Set pTempRaster = pMakerOp.MakeConstant(dblThresholdB, False)
504:   Set pTPI_ThresholdB_Dataset = pTempRaster.RasterDataset

  Dim pSlopePositionDataset As IRasterDataset2
  
'  MsgBox "pTPIDataset is Nothing = " & CStr(pTPIDataset Is Nothing)
'  MsgBox "pTPI_ThresholdA_Dataset is Nothing = " & CStr(pTPI_ThresholdA_Dataset Is Nothing)
'  MsgBox "pTPI_ThresholdB_Dataset is Nothing = " & CStr(pTPI_ThresholdB_Dataset Is Nothing)
'  MsgBox "pCanyonBottomDataset is Nothing = " & CStr(pCanyonBottomDataset Is Nothing)
'  MsgBox "pSlopeDataset is Nothing = " & CStr(pSlopeDataset Is Nothing)
'  MsgBox "pRidgelineDataset is Nothing = " & CStr(pRidgelineDataset Is Nothing)
  
  Dim pTempTPIRaster As IRaster2
516:   Set pTempTPIRaster = _
      pConOp.Con(pLogOp.LessThanEqual(pTPIDataset, pTPI_ThresholdA_Dataset), pCanyonBottomDataset, _
      pConOp.Con(pLogOp.LessThanEqual(pTPIDataset, pTPI_ThresholdB_Dataset), pSlopeDataset, pRidgelineDataset))

520:   Set pSlopePositionDataset = pTempTPIRaster.RasterDataset
  
  Dim pRastBandColl As IRasterBandCollection
  Dim pRastBand As IRasterBand
  
525:   Set pRastBandColl = pSlopePositionDataset
526:   Set pRastBand = pRastBandColl.Item(0)
527:   pRastBand.ComputeStatsAndHist
  
  Dim pSupportOp As IMathSupportOp
530:   Set pSupportOp = New RasterMathSupportOp
  
  ' CONVERT TO INTEGER RASTER AND SAVE
  Dim pSlopePositionIntegerDataset As IRasterDataset2
  Dim pTempRaster3 As IRaster2
535:   Set pTempRaster3 = pSupportOp.Int(pSlopePositionDataset)
  
  ' FOR DEBUGGING
'  Dim pTempLayer As IRasterLayer
'  Set pTempLayer = New RasterLayer
'  pTempLayer.CreateFromRaster pTempRaster3
'  pMxDoc.FocusMap.AddLayer pTempLayer
  
'  MsgBox "strNewRasterName = " & strNewRasterName & vbCrLf & _
         "strExtension = " & strExtension & vbCrLf & _
         "strSaveFormat = " & strSaveFormat & vbCrLf & _
         "pWS is Nothing = " & CStr(pWS Is Nothing) & vbCrLf & _
         "Path = " & pWS.PathName
  
  Dim pRastStorDef As IRasterStorageDef
550:   Set pRastStorDef = New RasterStorageDef
         
552:   If booIsFGB Then
553:     Set pSlopePositionIntegerDataset = pRastWsEx.SaveAsRasterDataset(strNewRasterName, pTempRaster3, Nothing, "", Nothing, Nothing)
554:     Set pSlopePositionIntegerDataset = Nothing
555:     Set pTempRaster = Nothing
556:     Set pTempRaster3 = Nothing
557:     Set pSlopePositionIntegerDataset = pRastWsEx.OpenRasterDataset(strNewRasterName)
558:   Else
559:     Set pSaveAs = pTempRaster3
560:     Set pSlopePositionIntegerDataset = pSaveAs.SaveAsRasterDataset(strNewRasterName & strExtension, _
          pRastWS, strSaveFormat, pRastStorDef)  ' <----------------
562:     Set pSaveAs = Nothing
563:     Set pTempRaster3 = Nothing
564:     Set pSupportOp = Nothing
565:     Set pRastBand = Nothing
566:     Set pRastBandColl = Nothing
567:     Set pSlopePositionDataset = Nothing
568:     Set pTempTPIRaster = Nothing
569:     Set pSlopePositionIntegerDataset = pRastWS.OpenRasterDataset(strNewRasterName & strExtension)
570:   End If
  
572:   Set pRastBandColl = pSlopePositionIntegerDataset
573:   Set pRastBand = pRastBandColl.Item(0)
574:   pRastBand.ComputeStatsAndHist

  ' ADD CORRECT CLASS NAMES TO RASTER ATTRIBUTE TABLE
  ' FIRST ADD NAME FIELD
  
579:   psbar.Message(0) = "Updating 3-Category Topographic Position Table..."
  Dim pTable As ITable
581:   Set pTable = TopoCorridor.TopoCor_Functions.ReturnRasterTable(pRastBand)
'  Set pTable = pRastBand.AttributeTable
  Dim pNameField As IField
584:   Set pNameField = New Field
  Dim pNameFieldEdit As IFieldEdit
586:   Set pNameFieldEdit = pNameField
587:   With pNameFieldEdit
588:     .Name = "Class_Name"
589:     .Type = esriFieldTypeString
590:     .length = 25
591:   End With
592:   pTable.AddField pNameField
  ' NEXT ADD CLASS VALUES TO NEW FIELD
  Dim lngValue As Long
  Dim lngValueIndex As Long
  Dim lngNameIndex As Long

598:   lngValueIndex = pTable.FindField("VALUE")
599:   lngNameIndex = pTable.FindField("Class_Name")

  Dim pCursor As ICursor
  Dim pRow As IRow
603:   Set pCursor = pTable.Update(Nothing, False)
604:   Set pRow = pCursor.NextRow
605:   Do Until pRow Is Nothing
606:     lngValue = pRow.Value(lngValueIndex)
    Select Case lngValue
      Case 1
609:         pRow.Value(lngNameIndex) = strClass1Name
      Case 2
611:         pRow.Value(lngNameIndex) = strClass2Name
      Case 3
613:         pRow.Value(lngNameIndex) = strClass3Name
      Case Else
615:         pRow.Value(lngNameIndex) = "Unknown"
616:     End Select
617:     pCursor.UpdateRow pRow

619:     Set pRow = pCursor.NextRow
620:   Loop






  ' SYMBOLIZE ADD 3-CATEGORY SLOPE POSITION CLASSIFICATION RASTER TO MAP
628:   psbar.Message(0) = "  --> Adding 3-Category Topographic Position to map..."
  Dim p3_CategoryLayer As IRasterLayer
630:   Set p3_CategoryLayer = New RasterLayer
631:   p3_CategoryLayer.CreateFromDataset pSlopePositionIntegerDataset
632:   p3_CategoryLayer.Name = strNewRasterName
  
'  Dim strTemplateFile As String
'  If Right(App.Path, 1) = "\" Then
'    strTemplateFile = App.Path & "slope_3_class_template.lyr"
'  Else
'    strTemplateFile = App.Path & "\slope_3_class_template.lyr"
'  End If
'
'  If TopoCorridor.aml_func_mod.FileExists(strTemplateFile) Then
'    TopoCorridor.MyGeneralOperations.SetRasterSymbols p3_CategoryLayer, strTemplateFile
'  End If
     
  Dim NumOfValues As Integer
646:   NumOfValues = pTable.RowCount(Nothing)
  
  ' CREATE CLASS COLORS
  
  ' CANYON = MEDIUM BLUE
  Dim pCanyonColor As IRgbColor
652:   Set pCanyonColor = New RgbColor
653:   pCanyonColor.RGB = RGB(102, 204, 255)
  
  ' MIDDLE SLOPE = LIGHT GREEN
  Dim pMiddleColor As IRgbColor
657:   Set pMiddleColor = New RgbColor
658:   pMiddleColor.RGB = RGB(204, 255, 204)
  
  ' RIDGES = MEDIUM BRICK
  Dim pRidgesColor As IRgbColor
662:   Set pRidgesColor = New RgbColor
663:   pRidgesColor.RGB = RGB(255, 153, 102)
  
  ' OTHER = BLACK
  Dim pOtherColor As IRgbColor
667:   Set pOtherColor = New RgbColor
668:   pOtherColor.RGB = RGB(0, 0, 0)
  
  Dim pCatRaster As IRaster
671:   Set pCatRaster = p3_CategoryLayer.Raster
  
  Dim pFSymbol As ISimpleFillSymbol
  
  ' Create UniqueValue renderer and QI RasterRenderer
  Dim pUVRen As IRasterUniqueValueRenderer
677:   Set pUVRen = New RasterUniqueValueRenderer
  Dim pRasRen As IRasterRenderer
679:   Set pRasRen = pUVRen

  ' Connect the renderer and the raster
682:   Set pRasRen.Raster = pCatRaster
683:   pRasRen.Update
  
  ' Set UniqueValue renerer
686:   pUVRen.HeadingCount = 1   ' Use one heading
687:   pUVRen.Heading(0) = "Slope Classes"
688:   pUVRen.ClassCount(0) = NumOfValues
689:   pUVRen.Field = "Class_Name"
  
  Dim varLabelValue As Variant
692:   Set pCursor = pTable.Update(Nothing, False)
693:   Set pRow = pCursor.NextRow
  Dim lngOID As Long
695:   Do Until pRow Is Nothing
696:     lngValue = pRow.Value(lngValueIndex)
697:     lngOID = pRow.OID
    Select Case lngValue
      Case 1
700:         varLabelValue = strClass1Name
701:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
702:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
703:         Set pFSymbol = New SimpleFillSymbol
704:         pFSymbol.Color = pCanyonColor
705:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case 2
707:         varLabelValue = strClass2Name
708:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
709:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
710:         Set pFSymbol = New SimpleFillSymbol
711:         pFSymbol.Color = pMiddleColor
712:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case 3
714:         varLabelValue = strClass3Name
715:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
716:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
717:         Set pFSymbol = New SimpleFillSymbol
718:         pFSymbol.Color = pRidgesColor
719:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case Else
721:         varLabelValue = "Unknown"
722:         pUVRen.AddValue 0, lngOID, varLabelValue 'Get a row from the table
723:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
724:         Set pFSymbol = New SimpleFillSymbol
725:         pFSymbol.Color = pOtherColor
726:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
727:     End Select
728:     pCursor.UpdateRow pRow

730:     Set pRow = pCursor.NextRow
731:   Loop
 

  'Update render and refresh layer
735:   pRasRen.Update
736:   Set p3_CategoryLayer.Renderer = pUVRen
  
  
  
740:   pMxDoc.FocusMap.AddLayer p3_CategoryLayer

742:   strReport = "Calculate 3-Category Topographic Position Function Complete:" & vbCrLf & _
              "----------------------------------------------------------" & vbCrLf & _
              "Analysis Parameters:" & vbCrLf & _
              " --> Classification Scheme:" & vbCrLf & vbCrLf & strClassDescription & vbCrLf & _
              " --> TPI Type = " & strTPIOption & vbCrLf & _
              " --> Input Raster Layer: " & strInputName & vbCrLf & _
              " --> " & strNbrHoodDesc & vbCrLf & _
              " --> Option to Mark as NoData if any NoData Cells in Neighborhood: " & _
                      UCase(CStr(booExcludeIfAnyNull)) & vbCrLf & _
              " --> Output Format: " & strSaveFormatString & vbCrLf & _
              " --> Filename: " & strNewRasterName & strExtension & vbCrLf & _
              " --> Workspace = " & pWS.PathName & vbCrLf & _
              "----------------------------------------------------------" & vbCrLf & _
              TopoCorridor.MyGeneralOperations.ReturnTimeElapsed(datBegan, Now)
  
757:   pMxDoc.UpdateContents
758:   pMxDoc.ActiveView.PartialRefresh esriViewGeography, Nothing, pEnv


761:   Create_Sample_3_Class_SlopeClass = strReport


764:   pProgAnim.Stop
765:   pProgAnim.Hide
766:   Set pProgAnim = Nothing

768:   GoTo CleanUp
ErrorHandler:

771:   pProgAnim.Stop
772:   pProgAnim.Hide
773:   Set pProgAnim = Nothing
  HandleError True, "Create_Sample_3_Class_SlopeClass " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

CleanUp:
777:   Set pApp = Nothing
778:   Set pMxDoc = Nothing
779:   Set pThisExt = Nothing
780:   Set psbar = Nothing
781:   Set pProgAnim = Nothing
782:   Set pMakerOp = Nothing
783:   Set pRastAnEnv = Nothing
784:   Set pRastProps = Nothing
785:   Set pSpRef = Nothing
786:   Set pGeoDataset = Nothing
787:   Set pMathOp = Nothing
788:   Set pNbrHoodOp = Nothing
789:   Set pNbrHood = Nothing
790:   Set pLogOp = Nothing
791:   Set pConOp = Nothing
792:   Set pInputBand = Nothing
793:   Set pRastLayer = Nothing
794:   Set pInputDataset = Nothing
795:   Set pInputRaster = Nothing
796:   Set pEnv = Nothing
797:   Set pRastWS = Nothing
798:   Set pRastWsEx = Nothing
799:   Set pWS = Nothing
800:   Set pSaveAs = Nothing
801:   Set pTempDataset = Nothing
802:   Set pTPIDataset = Nothing
803:   Set pTempRasterA = Nothing
804:   Set pTempProps = Nothing
805:   Set pCanyonBottomDataset = Nothing
806:   Set pSlopeDataset = Nothing
807:   Set pRidgelineDataset = Nothing
808:   Set pTempRaster = Nothing
809:   Set pTPI_ThresholdA_Dataset = Nothing
810:   Set pTPI_ThresholdB_Dataset = Nothing
811:   Set pSlopePositionDataset = Nothing
812:   Set pTempTPIRaster = Nothing
813:   Set pRastBandColl = Nothing
814:   Set pRastBand = Nothing
815:   Set pSupportOp = Nothing
816:   Set pSlopePositionIntegerDataset = Nothing
817:   Set pTempRaster3 = Nothing
818:   Set pRastStorDef = Nothing
819:   Set pTable = Nothing
820:   Set pNameField = Nothing
821:   Set pNameFieldEdit = Nothing
822:   Set pCursor = Nothing
823:   Set pRow = Nothing
824:   Set p3_CategoryLayer = Nothing
825:   Set pCanyonColor = Nothing
826:   Set pMiddleColor = Nothing
827:   Set pRidgesColor = Nothing
828:   Set pOtherColor = Nothing
829:   Set pCatRaster = Nothing
830:   Set pFSymbol = Nothing
831:   Set pUVRen = Nothing
832:   Set pRasRen = Nothing

End Function


Public Function Create_Sample_6_Class_SlopeClass(pDocParams As esriSystem.IVariantArray, _
    pParams As esriSystem.IVariantArray) As String
  On Error GoTo ErrorHandler


  ' THIS SAMPLE FROM THE MANUAL CREATES A 6-CATEGORY SLOPE CLASSIFICATION ACCORDING TO THE
  ' FOLLOWING CRITERIA

  '       Valley [Class Value = 6]:  TPI <= -1 SD
  '  Lower Slope [Class Value = 5]:  -1 SD < TPI >= -0.5 SD
  '   Flat Slope [Class Value = 4]:  -0.5 SD < TPI < 0.5 SD,   Slope <= 5
  ' Middle Slope [Class Value = 3]:  -0.5 SD < TPI < 0.5 SD,   Slope > 5
  '  Upper Slope [Class Value = 2]:  0.5 SD < TPI >= 1 SD
  '        Ridge [Class Value = 1]:  TPI > 1 SD

 
  Dim pApp As IApplication
  Dim pMxDoc As IMxDocument
  Dim pThisExt As TopoCorridor.clsExtension
  
857:   Set pApp = pDocParams.Element(0)
858:   Set pMxDoc = pDocParams.Element(1)
859:   Set pThisExt = pDocParams.Element(2)

  ' RETURN ARRAY IS IVariantArray, CONTAINING:
  ' 0) IRasterBand: SELECTED RASTER BAND
  ' 1) STRING: Save Format
  ' 2) STRING: TEXT VERSION OF SAVE FORMAT
  ' 3) STRING: New Raster Name
  ' 4) BOOLEAN: Whether new workspace is File Geodatabase
  ' 5) IWORKSPACE: Either an IRasterWorkspace2 or an IRasterWorkspaceEx
  ' 6) STRING: Extension for output
  ' 7) STRING: Name of Selected Raster Band
  ' 8) STRING: TPI Option; "TPI", "Standardized Elevation", "Standardized TPI"
  ' 9) INbrHood:  DEFINED NEIGHBORHOOD
  '10) STRING: Description of neighborhood
  '11) BOOLEAN: SET NODATA IF ANY NULL
  '12) STRING: Class 1 Name
  '13) STRING: Class 2 Name
  '14) STRING: Class 3 Name
  '15) STRING: Class 4 Name
  '16) STRING: Class 5 Name
  '17) STRING: Class 6 Name
  '18) DOUBLE: Threshold [A]
  '19) DOUBLE: Threshold [B]
  '20) DOUBLE: Threshold [C]
  '21) DOUBLE: Threshold [D]
  '22) DOUBLE: Threshold [S]
  '23) STRING: CLASS DEFINITIONS FROM RICH TEXT BOX

  Dim datBegan As Date
888:   datBegan = Now
  
  Dim strReport As String
891:   strReport = "Function failed..."
  
893:   Create_Sample_6_Class_SlopeClass = strReport
  
  ' DEFINE ADDITIONAL VARIABLES
  Dim psbar As IStatusBar
897:   Set psbar = pApp.StatusBar
  Dim pProgAnim As IAnimationProgressor
899:   Set pProgAnim = psbar.ProgressAnimation
900:   pProgAnim.Show
901:   pProgAnim.Play
  
  Dim pMakerOp As IRasterMakerOp
904:   Set pMakerOp = New RasterMakerOp
  Dim pRastAnEnv As IRasterAnalysisEnvironment
  Dim pRastProps As IRasterProps
  Dim pSpRef As ISpatialReference
  Dim pGeoDataset As IGeoDataset
  Dim pMathOp As IMathOp
910:   Set pMathOp = New RasterMathOps
  Dim pNbrHoodOp As INeighborhoodOp
912:   Set pNbrHoodOp = New RasterNeighborhoodOp
  Dim pNbrHood As IRasterNeighborhood
  Dim pLogOp As ILogicalOp
915:   Set pLogOp = New RasterMathOps
  Dim pConOp As IConditionalOp
917:   Set pConOp = New RasterConditionalOp
  Dim strTPIOption As String
  Dim strNbrHoodDesc As String
  
  Dim strClass1Name As String
  Dim strClass2Name As String
  Dim strClass3Name As String
  Dim strClass4Name As String
  Dim strClass5Name As String
  Dim strClass6Name As String
  Dim dblThresholdA As Double
  Dim dblThresholdB As Double
  Dim dblThresholdC As Double
  Dim dblThresholdD As Double
  Dim dblThresholdS As Double
  Dim strClassDescription As String
  
  Dim pInputBand As IRasterBand
  Dim pRastLayer As IRasterLayer
  Dim pInputDataset As IRasterDataset
  Dim pInputRaster As IRaster2
  Dim pEnv As IEnvelope
  Dim strSaveFormat As String
  Dim strNewRasterName As String
  Dim booIsFGB As Boolean
  Dim pRastWS As IRasterWorkspace2
  Dim pRastWsEx As IRasterWorkspaceEx
  Dim strSaveFormatString As String
  Dim pWS As IWorkspace
  Dim strExtension As String
  Dim strInputName As String
  Dim booExcludeIfAnyNull As Boolean
  Dim pSaveAs As ISaveAs2
  
  ' GET PARAMETERS FROM DIALOG
952:   Set pInputBand = pParams.Element(0)
953:   Set pGeoDataset = pInputBand
954:   Set pEnv = pGeoDataset.Extent
  
956:   strSaveFormat = pParams.Element(1)
957:   strSaveFormatString = pParams.Element(2)
958:   strNewRasterName = pParams.Element(3)
959:   booIsFGB = pParams.Element(4)
960:   Set pWS = pParams.Element(5)
961:   If booIsFGB Then
962:     Set pRastWsEx = pWS
963:   Else
964:     Set pRastWS = pWS
965:   End If
966:   strExtension = pParams.Element(6)
967:   strInputName = pParams.Element(7)
968:   strTPIOption = pParams.Element(8)
969:   Set pNbrHood = pParams.Element(9)
970:   strNbrHoodDesc = pParams.Element(10)
971:   booExcludeIfAnyNull = pParams.Element(11)
972:   strClass1Name = pParams.Element(12)
973:   strClass2Name = pParams.Element(13)
974:   strClass3Name = pParams.Element(14)
975:   strClass4Name = pParams.Element(15)
976:   strClass5Name = pParams.Element(16)
977:   strClass6Name = pParams.Element(17)
978:   dblThresholdA = pParams.Element(18)
979:   dblThresholdB = pParams.Element(19)
980:   dblThresholdC = pParams.Element(20)
981:   dblThresholdD = pParams.Element(21)
982:   dblThresholdS = pParams.Element(22)
983:   strClassDescription = pParams.Element(23)
  
  ' ANALYSIS ==========================================================================

  ' FIND DEM LAYER IN MAP AND GET RASTER DATASET
988:   Set pInputDataset = pInputBand.RasterDataset
989:   Set pSpRef = pGeoDataset.SpatialReference
990:   Set pRastProps = pInputBand
991:   Set pRastAnEnv = pMakerOp
992:   pRastAnEnv.SetCellSize esriRasterEnvValue, pInputDataset
993:   pRastAnEnv.SetExtent esriRasterEnvValue, pInputDataset, pInputDataset
994:   Set pRastAnEnv.OutSpatialReference = pSpRef

  ' CALCULATE EITHER STANDARDIZED DEM, SIMPLE TPI, OR STANDARDIZED TPI
  
  Dim pTempDataset As IRasterDataset2
  Dim pTPIDataset As IRasterDataset2

1001:   If strTPIOption = "TPI" Then
1002:     Set pTPIDataset = ReturnTPIDataset(pInputDataset, psbar, strInputName, pNbrHoodOp, pNbrHood, pMathOp, _
          pMxDoc, False, booExcludeIfAnyNull)
1004:   ElseIf strTPIOption = "Standardized Elevation" Then
1005:     Set pTPIDataset = ReturnStandardizedElevationDataset(pInputDataset, pNbrHoodOp, psbar, _
          pMakerOp, pNbrHood, strInputName, pMxDoc, booExcludeIfAnyNull, False)
1007:   ElseIf strTPIOption = "Standardized TPI" Then
1008:     Set pTPIDataset = ReturnStandardizedTPIDataset(pInputDataset, pNbrHood, "TPI", strInputName, _
          psbar, pNbrHoodOp, pMxDoc, pMakerOp, booExcludeIfAnyNull, False)
1010:   End If

  ' GET SLOPE DATASET
  Dim pSlopeDataset As IRasterDataset
1014:   Set pSlopeDataset = ReturnSlopeDataset(pInputDataset, psbar, strInputName, pMxDoc, False)

  ' CONVERT TPI AND SLOPE INTO 6-CATEGORY CLASSIFICATION RASTER
  ' FIRST CREATE CONSTANT-VALUE RASTERS FOR THE 4 CATEGORIES
1018:   psbar.Message(0) = "Calculating 6-Category Topographic Position Classification Raster..."

  ' FIRST CREATE CONSTANT-VALUE RASTERS FOR THE 6 CATEGORIES
  Dim pValleyDataset As IRasterDataset
  Dim pLowerSlopeDataset As IRasterDataset
  Dim pFlatSlopeDataset As IRasterDataset
  Dim pMiddleSlopeDataset As IRasterDataset
  Dim pUpperSlopeDataset As IRasterDataset
  Dim pRidgeDataset As IRasterDataset
  Dim pTempRaster As IRaster2

1029:   Set pTempRaster = pMakerOp.MakeConstant(1, True)
1030:   Set pValleyDataset = pTempRaster.RasterDataset

1032:   Set pTempRaster = pMakerOp.MakeConstant(2, True)
1033:   Set pLowerSlopeDataset = pTempRaster.RasterDataset

1035:   Set pTempRaster = pMakerOp.MakeConstant(3, True)
1036:   Set pFlatSlopeDataset = pTempRaster.RasterDataset

1038:   Set pTempRaster = pMakerOp.MakeConstant(4, True)
1039:   Set pMiddleSlopeDataset = pTempRaster.RasterDataset

1041:   Set pTempRaster = pMakerOp.MakeConstant(5, True)
1042:   Set pUpperSlopeDataset = pTempRaster.RasterDataset

1044:   Set pTempRaster = pMakerOp.MakeConstant(6, True)
1045:   Set pRidgeDataset = pTempRaster.RasterDataset

  ' NEXT MAKE CONSTANT-VALUE RASTERS FOR THE VARIOUS TPI AND SLOPE THRESHOLDS

  Dim pTPI_Negative_1_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = -1
  Dim pTPI_Negative_0_5_Dataset As IRasterDataset   ' FOR TPI THRESHOLD = -0.5
  Dim pTPI_Positive_0_5_Dataset As IRasterDataset   ' FOR TPI THRESHOLD = 0.5
  Dim pTPI_Positive_1_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = 1
  Dim pSlope_5_Dataset As IRasterDataset            ' FOR SLOPE THRESHOLD = 5

1055:   Set pTempRaster = pMakerOp.MakeConstant(dblThresholdA, False)
1056:   Set pTPI_Negative_1_Dataset = pTempRaster.RasterDataset

1058:   Set pTempRaster = pMakerOp.MakeConstant(dblThresholdB, False)
1059:   Set pTPI_Negative_0_5_Dataset = pTempRaster.RasterDataset

1061:   Set pTempRaster = pMakerOp.MakeConstant(dblThresholdC, False)
1062:   Set pTPI_Positive_0_5_Dataset = pTempRaster.RasterDataset

1064:   Set pTempRaster = pMakerOp.MakeConstant(dblThresholdD, False)
1065:   Set pTPI_Positive_1_Dataset = pTempRaster.RasterDataset

1067:   Set pTempRaster = pMakerOp.MakeConstant(dblThresholdS, False)
1068:   Set pSlope_5_Dataset = pTempRaster.RasterDataset

  Dim pSlopePositionDataset As IRasterDataset
  Dim pRastBandColl As IRasterBandCollection
  Dim pRastBand As IRasterBand
  Dim pSupportOp As IMathSupportOp
1074:   Set pSupportOp = New RasterMathSupportOp
  
'  MsgBox "pTPIDataset is IRasterDataset = " & CStr(TypeOf pTPIDataset Is IRasterDataset) & vbCrLf & _
'         "pTPI_Negative_1_Dataset is IRasterDataset = " & CStr(TypeOf pTPI_Negative_1_Dataset Is IRasterDataset) & vbCrLf & _
'         "pTPI_Negative_0_5_Dataset is IRasterDataset = " & CStr(TypeOf pTPI_Negative_0_5_Dataset Is IRasterDataset) & vbCrLf & _
'         "pTPI_Positive_0_5_Dataset is IRasterDataset = " & CStr(TypeOf pTPI_Positive_0_5_Dataset Is IRasterDataset) & vbCrLf & _
'         "pTPI_Positive_1_Dataset is IRasterDataset = " & CStr(TypeOf pTPI_Positive_1_Dataset Is IRasterDataset) & vbCrLf & _
'         "pSlopeDataset is IRasterDataset = " & CStr(TypeOf pSlopeDataset Is IRasterDataset) & vbCrLf & _
'         "pValleyDataset is IRasterDataset = " & CStr(TypeOf pValleyDataset Is IRasterDataset) & vbCrLf & _
'         "pLowerSlopeDataset is IRasterDataset = " & CStr(TypeOf pLowerSlopeDataset Is IRasterDataset) & vbCrLf & _
'         "pFlatSlopeDataset is IRasterDataset = " & CStr(TypeOf pFlatSlopeDataset Is IRasterDataset) & vbCrLf & _
'         "pMiddleSlopeDataset is IRasterDataset = " & CStr(TypeOf pMiddleSlopeDataset Is IRasterDataset) & vbCrLf & _
'         "pUpperSlopeDataset is IRasterDataset = " & CStr(TypeOf pUpperSlopeDataset Is IRasterDataset) & vbCrLf & _
'         "pRidgeDataset is IRasterDataset = " & CStr(TypeOf pRidgeDataset Is IRasterDataset)
'
'  MsgBox "pTPIDataset is nothing = " & CStr(pTPIDataset Is Nothing) & vbCrLf & _
'         "pTPI_Negative_1_Dataset is nothing = " & CStr(pTPI_Negative_1_Dataset Is Nothing) & vbCrLf & _
'         "pTPI_Negative_0_5_Dataset is nothing = " & CStr(pTPI_Negative_0_5_Dataset Is Nothing) & vbCrLf & _
'         "pTPI_Positive_0_5_Dataset is nothing = " & CStr(pTPI_Positive_0_5_Dataset Is Nothing) & vbCrLf & _
'         "pTPI_Positive_1_Dataset is nothing = " & CStr(pTPI_Positive_1_Dataset Is Nothing) & vbCrLf & _
'         "pSlopeDataset is nothing = " & CStr(pSlopeDataset Is Nothing) & vbCrLf & _
'         "pValleyDataset is nothing = " & CStr(pValleyDataset Is Nothing) & vbCrLf & _
'         "pLowerSlopeDataset is nothing = " & CStr(pLowerSlopeDataset Is Nothing) & vbCrLf & _
'         "pFlatSlopeDataset is nothing = " & CStr(pFlatSlopeDataset Is Nothing) & vbCrLf & _
'         "pMiddleSlopeDataset is nothing = " & CStr(pMiddleSlopeDataset Is Nothing) & vbCrLf & _
'         "pUpperSlopeDataset is nothing = " & CStr(pUpperSlopeDataset Is Nothing) & vbCrLf & _
'         "pRidgeDataset is nothing = " & CStr(pRidgeDataset Is Nothing)
  
  ' KEEPS CRASHING; TRY CONVERTING THEM TO RASTERS
  
  
  
  
1107:   Set pTempRaster = _
      pConOp.Con(pLogOp.LessThanEqual(pTPIDataset, pTPI_Negative_1_Dataset), pValleyDataset, _
        pConOp.Con(pLogOp.LessThanEqual(pTPIDataset, pTPI_Negative_0_5_Dataset), pLowerSlopeDataset, _
          pConOp.Con(pLogOp.LessThanEqual(pTPIDataset, pTPI_Positive_0_5_Dataset), _
                     pConOp.Con(pLogOp.LessThanEqual(pSlopeDataset, pSlope_5_Dataset), pFlatSlopeDataset, pMiddleSlopeDataset), _
          pConOp.Con(pLogOp.LessThanEqual(pTPIDataset, pTPI_Positive_1_Dataset), pUpperSlopeDataset, pRidgeDataset))))
  
1114:   Set pSlopePositionDataset = pTempRaster.RasterDataset
1115:   Set pRastBandColl = pSlopePositionDataset
1116:   Set pRastBand = pRastBandColl.Item(0)
1117:   pRastBand.ComputeStatsAndHist

  ' CONVERT TO INTEGER RASTER
  Dim pSlopePositionIntegerDataset As IRasterDataset
1121:   Set pTempRaster = pSupportOp.Int(pSlopePositionDataset)
  
  Dim pRastStorDef As IRasterStorageDef
1124:   Set pRastStorDef = New RasterStorageDef
         
1126:   If booIsFGB Then
1127:     Set pSlopePositionIntegerDataset = pRastWsEx.SaveAsRasterDataset(strNewRasterName, pTempRaster, Nothing, "", Nothing, Nothing)
1128:   Else
1129:     Set pSaveAs = pTempRaster
1130:     Set pSlopePositionIntegerDataset = pSaveAs.SaveAsRasterDataset(strNewRasterName & strExtension, _
          pRastWS, strSaveFormat, pRastStorDef)  ' <----------------
1132:     Set pSaveAs = Nothing
1133:     Set pTempRaster = Nothing
1134:     Set pSupportOp = Nothing
1135:     Set pRastBand = Nothing
1136:     Set pRastBandColl = Nothing
1137:     Set pSlopePositionDataset = Nothing
1138:     Set pConOp = Nothing
1139:     Set pSlopePositionIntegerDataset = pRastWS.OpenRasterDataset(strNewRasterName & strExtension)
1140:   End If
    
1142:   Set pRastBandColl = pSlopePositionIntegerDataset
1143:   Set pRastBand = pRastBandColl.Item(0)
1144:   pRastBand.ComputeStatsAndHist

  ' ADD CORRECT CLASS NAMES TO RASTER ATTRIBUTE TABLE
  ' FIRST ADD NAME FIELD
1148:   Debug.Print "  --> Updating 6-Category Topographic Position Table..."
  Dim pTable As ITable
'  Set pTable = pRastBand.AttributeTable
1151:   Set pTable = TopoCorridor.TopoCor_Functions.ReturnRasterTable(pRastBand)
  Dim pNameField As IField
1153:   Set pNameField = New Field
  Dim pNameFieldEdit As IFieldEdit
1155:   Set pNameFieldEdit = pNameField
1156:   With pNameFieldEdit
1157:     .Name = "Class_Name"
1158:     .Type = esriFieldTypeString
1159:     .length = 25
1160:   End With
1161:   pTable.AddField pNameField
  ' NEXT ADD CLASS VALUES TO NEW FIELD
  Dim lngValue As Long
  Dim lngValueIndex As Long
  Dim lngNameIndex As Long

1167:   lngValueIndex = pTable.FindField("VALUE")
1168:   lngNameIndex = pTable.FindField("Class_Name")

  Dim pCursor As ICursor
  Dim pRow As IRow
1172:   Set pCursor = pTable.Update(Nothing, False)
1173:   Set pRow = pCursor.NextRow
1174:   Do Until pRow Is Nothing
1175:     lngValue = pRow.Value(lngValueIndex)
    Select Case lngValue
      Case 1
1178:         pRow.Value(lngNameIndex) = strClass1Name
      Case 2
1180:         pRow.Value(lngNameIndex) = strClass2Name
      Case 3
1182:         pRow.Value(lngNameIndex) = strClass3Name
      Case 4
1184:         pRow.Value(lngNameIndex) = strClass4Name
      Case 5
1186:         pRow.Value(lngNameIndex) = strClass5Name
      Case 6
1188:         pRow.Value(lngNameIndex) = strClass6Name
      Case Else
1190:         pRow.Value(lngNameIndex) = "Unknown"
1191:     End Select
1192:     pCursor.UpdateRow pRow

1194:     Set pRow = pCursor.NextRow
1195:   Loop
  




  ' SYMBOLIZE ADD 3-CATEGORY SLOPE POSITION CLASSIFICATION RASTER TO MAP
1202:   psbar.Message(0) = "  --> Adding 6-Category Topographic Position to map..."
  Dim p6_CategoryLayer As IRasterLayer
1204:   Set p6_CategoryLayer = New RasterLayer
1205:   p6_CategoryLayer.CreateFromDataset pSlopePositionIntegerDataset
1206:   p6_CategoryLayer.Name = strNewRasterName
  
'  Dim strTemplateFile As String
'  If Right(App.Path, 1) = "\" Then
'    strTemplateFile = App.Path & "slope_6_class_template.lyr"
'  Else
'    strTemplateFile = App.Path & "\slope_6_class_template.lyr"
'  End If
'
'  If TopoCorridor.aml_func_mod.FileExists(strTemplateFile) Then
'    TopoCorridor.MyGeneralOperations.SetRasterSymbols p6_CategoryLayer, strTemplateFile
'  End If
  
  Dim NumOfValues As Integer
1220:   NumOfValues = pTable.RowCount(Nothing)
  
  ' CREATE CLASS COLORS
  
  ' CANYON = MEDIUM BLUE
  Dim pCanyonColor As IRgbColor
1226:   Set pCanyonColor = New RgbColor
1227:   pCanyonColor.RGB = RGB(102, 204, 255)
  
  ' FLAT AREAS = LIGHT YELLOW GREEN
  Dim pFlatColor As IRgbColor
1231:   Set pFlatColor = New RgbColor
1232:   pFlatColor.RGB = RGB(255, 255, 204)
  
  ' LOWER SLOPE = LIGHT GREEN
  Dim pLowerColor As IRgbColor
1236:   Set pLowerColor = New RgbColor
1237:   pLowerColor.RGB = RGB(204, 255, 204)
  
  ' MIDDLE SLOPE = MEDIUM GREEN
  Dim pMiddleColor As IRgbColor
1241:   Set pMiddleColor = New RgbColor
1242:   pMiddleColor.RGB = RGB(0, 255, 0)
  
  ' UPPER SLOPE = DARK GREEN
  Dim pUpperColor As IRgbColor
1246:   Set pUpperColor = New RgbColor
1247:   pUpperColor.RGB = RGB(0, 102, 0)
  
  ' RIDGES = MEDIUM BRICK
  Dim pRidgesColor As IRgbColor
1251:   Set pRidgesColor = New RgbColor
1252:   pRidgesColor.RGB = RGB(255, 153, 102)
  
  ' OTHER = BLACK
  Dim pOtherColor As IRgbColor
1256:   Set pOtherColor = New RgbColor
1257:   pOtherColor.RGB = RGB(0, 0, 0)
  
  Dim pCatRaster As IRaster
1260:   Set pCatRaster = p6_CategoryLayer.Raster
  
  Dim pFSymbol As ISimpleFillSymbol
  
  ' Create UniqueValue renderer and QI RasterRenderer
  Dim pUVRen As IRasterUniqueValueRenderer
1266:   Set pUVRen = New RasterUniqueValueRenderer
  Dim pRasRen As IRasterRenderer
1268:   Set pRasRen = pUVRen

  ' Connect the renderer and the raster
1271:   Set pRasRen.Raster = pCatRaster
1272:   pRasRen.Update
  
  ' Set UniqueValue renerer
1275:   pUVRen.HeadingCount = 1   ' Use one heading
1276:   pUVRen.Heading(0) = "Slope Classes"
1277:   pUVRen.ClassCount(0) = NumOfValues
1278:   pUVRen.Field = "Class_Name"
  
  Dim varLabelValue As Variant
1281:   Set pCursor = pTable.Update(Nothing, False)
1282:   Set pRow = pCursor.NextRow
  Dim lngOID As Long
1284:   Do Until pRow Is Nothing
1285:     lngValue = pRow.Value(lngValueIndex)
1286:     lngOID = pRow.OID
    Select Case lngValue
      Case 1
1289:         varLabelValue = strClass1Name
1290:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
1291:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1292:         Set pFSymbol = New SimpleFillSymbol
1293:         pFSymbol.Color = pCanyonColor
1294:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case 2
1296:         varLabelValue = strClass2Name
1297:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
1298:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1299:         Set pFSymbol = New SimpleFillSymbol
1300:         pFSymbol.Color = pLowerColor
1301:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case 3
1303:         varLabelValue = strClass3Name
1304:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
1305:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1306:         Set pFSymbol = New SimpleFillSymbol
1307:         pFSymbol.Color = pFlatColor
1308:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case 4
1310:         varLabelValue = strClass4Name
1311:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
1312:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1313:         Set pFSymbol = New SimpleFillSymbol
1314:         pFSymbol.Color = pMiddleColor
1315:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case 5
1317:         varLabelValue = strClass5Name
1318:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
1319:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1320:         Set pFSymbol = New SimpleFillSymbol
1321:         pFSymbol.Color = pUpperColor
1322:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case 6
1324:         varLabelValue = strClass6Name
1325:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
1326:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1327:         Set pFSymbol = New SimpleFillSymbol
1328:         pFSymbol.Color = pRidgesColor
1329:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case Else
1331:         varLabelValue = "Unknown"
1332:         pUVRen.AddValue 0, lngOID, varLabelValue 'Get a row from the table
1333:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1334:         Set pFSymbol = New SimpleFillSymbol
1335:         pFSymbol.Color = pOtherColor
1336:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
1337:     End Select
1338:     pCursor.UpdateRow pRow

1340:     Set pRow = pCursor.NextRow
1341:   Loop
 

  'Update render and refresh layer
1345:   pRasRen.Update
1346:   Set p6_CategoryLayer.Renderer = pUVRen
  
  
1349:   pMxDoc.FocusMap.AddLayer p6_CategoryLayer

1351:   strReport = "Calculate 6-Category Topographic Position Function Complete:" & vbCrLf & _
              "----------------------------------------------------------" & vbCrLf & _
              "Analysis Parameters:" & vbCrLf & _
              " --> Classification Scheme:" & vbCrLf & vbCrLf & strClassDescription & vbCrLf & _
              " --> TPI Type = " & strTPIOption & vbCrLf & _
              " --> Input Raster Layer: " & strInputName & vbCrLf & _
              " --> " & strNbrHoodDesc & vbCrLf & _
              " --> Option to Mark as NoData if any NoData Cells in Neighborhood: " & _
                      UCase(CStr(booExcludeIfAnyNull)) & vbCrLf & _
              " --> Output Format: " & strSaveFormatString & vbCrLf & _
              " --> Filename: " & strNewRasterName & strExtension & vbCrLf & _
              " --> Workspace = " & pWS.PathName & vbCrLf & _
              "----------------------------------------------------------" & vbCrLf & _
              TopoCorridor.MyGeneralOperations.ReturnTimeElapsed(datBegan, Now)
  
1366:   pMxDoc.UpdateContents
1367:   pMxDoc.ActiveView.PartialRefresh esriViewGeography, Nothing, pEnv

1369:   Create_Sample_6_Class_SlopeClass = strReport

1371:   pProgAnim.Stop
1372:   pProgAnim.Hide
1373:   Set pProgAnim = Nothing
1374:   GoTo CleanUp
ErrorHandler:

1377:   pProgAnim.Stop
1378:   pProgAnim.Hide
1379:   Set pProgAnim = Nothing
  HandleError True, "Create_Sample_6_Class_SlopeClass " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

CleanUp:

1384:   Set pApp = Nothing
1385:   Set pMxDoc = Nothing
1386:   Set pThisExt = Nothing
1387:   Set psbar = Nothing
1388:   Set pProgAnim = Nothing
1389:   Set pMakerOp = Nothing
1390:   Set pRastAnEnv = Nothing
1391:   Set pRastProps = Nothing
1392:   Set pSpRef = Nothing
1393:   Set pGeoDataset = Nothing
1394:   Set pMathOp = Nothing
1395:   Set pNbrHoodOp = Nothing
1396:   Set pNbrHood = Nothing
1397:   Set pLogOp = Nothing
1398:   Set pConOp = Nothing
1399:   Set pInputBand = Nothing
1400:   Set pRastLayer = Nothing
1401:   Set pInputDataset = Nothing
1402:   Set pInputRaster = Nothing
1403:   Set pEnv = Nothing
1404:   Set pRastWS = Nothing
1405:   Set pRastWsEx = Nothing
1406:   Set pWS = Nothing
1407:   Set pSaveAs = Nothing
1408:   Set pTempDataset = Nothing
1409:   Set pTPIDataset = Nothing
1410:   Set pSlopeDataset = Nothing
1411:   Set pValleyDataset = Nothing
1412:   Set pLowerSlopeDataset = Nothing
1413:   Set pFlatSlopeDataset = Nothing
1414:   Set pMiddleSlopeDataset = Nothing
1415:   Set pUpperSlopeDataset = Nothing
1416:   Set pRidgeDataset = Nothing
1417:   Set pTempRaster = Nothing
1418:   Set pTPI_Negative_1_Dataset = Nothing
1419:   Set pTPI_Negative_0_5_Dataset = Nothing
1420:   Set pTPI_Positive_0_5_Dataset = Nothing
1421:   Set pTPI_Positive_1_Dataset = Nothing
1422:   Set pSlope_5_Dataset = Nothing
1423:   Set pSlopePositionDataset = Nothing
1424:   Set pRastBandColl = Nothing
1425:   Set pRastBand = Nothing
1426:   Set pSupportOp = Nothing
1427:   Set pSlopePositionIntegerDataset = Nothing
1428:   Set pRastStorDef = Nothing
1429:   Set pTable = Nothing
1430:   Set pNameField = Nothing
1431:   Set pNameFieldEdit = Nothing
1432:   Set pCursor = Nothing
1433:   Set pRow = Nothing
1434:   Set p6_CategoryLayer = Nothing
1435:   Set pCanyonColor = Nothing
1436:   Set pFlatColor = Nothing
1437:   Set pLowerColor = Nothing
1438:   Set pMiddleColor = Nothing
1439:   Set pUpperColor = Nothing
1440:   Set pRidgesColor = Nothing
1441:   Set pOtherColor = Nothing
1442:   Set pCatRaster = Nothing
1443:   Set pFSymbol = Nothing
1444:   Set pUVRen = Nothing
1445:   Set pRasRen = Nothing

End Function

Public Function Create_Sample_4_Class_SlopeClass(pDocParams As esriSystem.IVariantArray, _
    pParams As esriSystem.IVariantArray) As String
  On Error GoTo ErrorHandler


  ' THIS SAMPLE FROM THE MANUAL CREATES A 6-CATEGORY SLOPE CLASSIFICATION ACCORDING TO THE
  ' FOLLOWING CRITERIA

'  '  Canyon Bottom [Class Value = 4]: TPI <= -8
'  '   Gentle Slope [Class Value = 3]: -8 < TPI <= 8, Slope < 6
'  '    Steep Slope [Class Value = 2]: -8 < TPI <= 8, Slope >= 6
'  '      Ridgeline [Class Value = 1]: TPI >= 8

 
  Dim pApp As IApplication
  Dim pMxDoc As IMxDocument
  Dim pThisExt As TopoCorridor.clsExtension
  
1467:   Set pApp = pDocParams.Element(0)
1468:   Set pMxDoc = pDocParams.Element(1)
1469:   Set pThisExt = pDocParams.Element(2)


  ' RETURN ARRAY IS IVariantArray, CONTAINING:
  ' 0) IRasterBand: SELECTED RASTER BAND
  ' 1) STRING: Save Format
  ' 2) STRING: TEXT VERSION OF SAVE FORMAT
  ' 3) STRING: New Raster Name
  ' 4) BOOLEAN: Whether new workspace is File Geodatabase
  ' 5) IWORKSPACE: Either an IRasterWorkspace2 or an IRasterWorkspaceEx
  ' 6) STRING: Extension for output
  ' 7) STRING: Name of Selected Raster Band
  ' 8) STRING: TPI Option; "TPI", "Standardized Elevation", "Standardized TPI"
  ' 9) INbrHood:  DEFINED NEIGHBORHOOD
  '10) STRING: Description of neighborhood
  '11) BOOLEAN: SET NODATA IF ANY NULL
  '12) STRING: Class 1 Name
  '13) STRING: Class 2 Name
  '14) STRING: Class 3 Name
  '15) STRING: Class 4 Name
  '16) DOUBLE: Threshold [A]
  '17) DOUBLE: Threshold [B]
  '18) DOUBLE: Threshold [S]
  '19) STRING: CLASS DEFINITIONS FROM RICH TEXT BOX

  Dim datBegan As Date
1495:   datBegan = Now
  
  Dim strReport As String
1498:   strReport = "Function failed..."
  
1500:   Create_Sample_4_Class_SlopeClass = strReport
  
  ' DEFINE ADDITIONAL VARIABLES
  Dim psbar As IStatusBar
1504:   Set psbar = pApp.StatusBar
  Dim pProgAnim As IAnimationProgressor
1506:   Set pProgAnim = psbar.ProgressAnimation
1507:   pProgAnim.Show
1508:   pProgAnim.Play
  
  Dim pMakerOp As IRasterMakerOp
1511:   Set pMakerOp = New RasterMakerOp
  Dim pRastAnEnv As IRasterAnalysisEnvironment
  Dim pRastProps As IRasterProps
  Dim pSpRef As ISpatialReference
  Dim pGeoDataset As IGeoDataset
  Dim pMathOp As IMathOp
1517:   Set pMathOp = New RasterMathOps
  Dim pNbrHoodOp As INeighborhoodOp
1519:   Set pNbrHoodOp = New RasterNeighborhoodOp
  Dim pNbrHood As IRasterNeighborhood
  Dim pLogOp As ILogicalOp
1522:   Set pLogOp = New RasterMathOps
  Dim pConOp As IConditionalOp
1524:   Set pConOp = New RasterConditionalOp
  Dim strTPIOption As String
  Dim strNbrHoodDesc As String
  
  Dim strClass1Name As String
  Dim strClass2Name As String
  Dim strClass3Name As String
  Dim strClass4Name As String
  Dim dblThresholdA As Double
  Dim dblThresholdB As Double
  Dim dblThresholdS As Double
  Dim strClassDescription As String
  
  Dim pInputBand As IRasterBand
  Dim pRastLayer As IRasterLayer
  Dim pInputDataset As IRasterDataset
  Dim pInputRaster As IRaster2
  Dim pEnv As IEnvelope
  Dim strSaveFormat As String
  Dim strNewRasterName As String
  Dim booIsFGB As Boolean
  Dim pRastWS As IRasterWorkspace2
  Dim pRastWsEx As IRasterWorkspaceEx
  Dim strSaveFormatString As String
  Dim pWS As IWorkspace
  Dim strExtension As String
  Dim strInputName As String
  Dim booExcludeIfAnyNull As Boolean
  Dim pSaveAs As ISaveAs2
  
  ' GET PARAMETERS FROM DIALOG
1555:   Set pInputBand = pParams.Element(0)
1556:   Set pGeoDataset = pInputBand
1557:   Set pEnv = pGeoDataset.Extent
  
1559:   strSaveFormat = pParams.Element(1)
1560:   strSaveFormatString = pParams.Element(2)
1561:   strNewRasterName = pParams.Element(3)
1562:   booIsFGB = pParams.Element(4)
1563:   Set pWS = pParams.Element(5)
1564:   If booIsFGB Then
1565:     Set pRastWsEx = pWS
1566:   Else
1567:     Set pRastWS = pWS
1568:   End If
1569:   strExtension = pParams.Element(6)
1570:   strInputName = pParams.Element(7)
1571:   strTPIOption = pParams.Element(8)
1572:   Set pNbrHood = pParams.Element(9)
1573:   strNbrHoodDesc = pParams.Element(10)
1574:   booExcludeIfAnyNull = pParams.Element(11)
1575:   strClass1Name = pParams.Element(12)
1576:   strClass2Name = pParams.Element(13)
1577:   strClass3Name = pParams.Element(14)
1578:   strClass4Name = pParams.Element(15)
1579:   dblThresholdA = pParams.Element(16)
1580:   dblThresholdB = pParams.Element(17)
1581:   dblThresholdS = pParams.Element(18)
1582:   strClassDescription = pParams.Element(19)
  
  ' ANALYSIS ==========================================================================

  ' FIND DEM LAYER IN MAP AND GET RASTER DATASET
1587:   Set pInputDataset = pInputBand.RasterDataset
1588:   Set pSpRef = pGeoDataset.SpatialReference
1589:   Set pRastProps = pInputBand
1590:   Set pRastAnEnv = pMakerOp
1591:   pRastAnEnv.SetCellSize esriRasterEnvValue, pInputDataset
1592:   pRastAnEnv.SetExtent esriRasterEnvValue, pInputDataset, pInputDataset
1593:   Set pRastAnEnv.OutSpatialReference = pSpRef

  ' CALCULATE EITHER STANDARDIZED DEM, SIMPLE TPI, OR STANDARDIZED TPI
  
  Dim pTempDataset As IRasterDataset2
  Dim pTPIDataset As IRasterDataset2

1600:   If strTPIOption = "TPI" Then
1601:     Set pTPIDataset = ReturnTPIDataset(pInputDataset, psbar, strInputName, pNbrHoodOp, pNbrHood, pMathOp, _
          pMxDoc, False, booExcludeIfAnyNull)
1603:   ElseIf strTPIOption = "Standardized Elevation" Then
1604:     Set pTPIDataset = ReturnStandardizedElevationDataset(pInputDataset, pNbrHoodOp, psbar, _
          pMakerOp, pNbrHood, strInputName, pMxDoc, booExcludeIfAnyNull, False)
1606:   ElseIf strTPIOption = "Standardized TPI" Then
1607:     Set pTPIDataset = ReturnStandardizedTPIDataset(pInputDataset, pNbrHood, "TPI", strInputName, _
          psbar, pNbrHoodOp, pMxDoc, pMakerOp, booExcludeIfAnyNull, False)
1609:   End If
  
'  Dim pTempRasterA As IRaster
'  Dim pTempProps As IRasterProps
'  Set pTempRasterA = pTPIDataset.CreateFullRaster
'  Set pTempProps = pTempRasterA
'  pTempProps.PixelType = PT_FLOAT

  ' GET SLOPE DATASET
  Dim pSlopeDataset As IRasterDataset
1619:   Set pSlopeDataset = ReturnSlopeDataset(pInputDataset, psbar, strInputName, pMxDoc, False)

  ' CONVERT TPI AND SLOPE INTO 4-CATEGORY CLASSIFICATION RASTER
  ' FIRST CREATE CONSTANT-VALUE RASTERS FOR THE 4 CATEGORIES
'  psbar.Message(0) = "Calculating 4-Category Topographic Position Classification Raster..."

'  ' FIRST CREATE CONSTANT-VALUE RASTERS FOR THE 4 CATEGORIES
'  Dim pValleyDataset As IRasterDataset
'  Dim pLowerSlopeDataset As IRasterDataset
'  Dim pFlatSlopeDataset As IRasterDataset
'  Dim pMiddleSlopeDataset As IRasterDataset
'  Dim pUpperSlopeDataset As IRasterDataset
'  Dim pRidgeDataset As IRasterDataset
  Dim pTempRaster As IRaster2
'
'  Set pTempRaster = pMakerOp.MakeConstant(1, True)
'  Set pValleyDataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(2, True)
'  Set pLowerSlopeDataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(3, True)
'  Set pFlatSlopeDataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(4, True)
'  Set pMiddleSlopeDataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(5, True)
'  Set pUpperSlopeDataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(6, True)
'  Set pRidgeDataset = pTempRaster.RasterDataset

  ' FIRST CREATE CONSTANT-VALUE RASTERS FOR THE 4 CATEGORIES
1653:   psbar.Message(0) = "Calculating 4-Category Topographic Position Classification Raster..."
  Dim pCanyonBottomDataset As IRasterDataset
  Dim pGentleSlopeDataset As IRasterDataset
  Dim pSteepSlopeDataset As IRasterDataset
  Dim pRidgelineDataset As IRasterDataset

1659:   Set pTempRaster = pMakerOp.MakeConstant(1, True)
1660:   Set pCanyonBottomDataset = pTempRaster.RasterDataset

1662:   Set pTempRaster = pMakerOp.MakeConstant(2, True)
1663:   Set pGentleSlopeDataset = pTempRaster.RasterDataset

1665:   Set pTempRaster = pMakerOp.MakeConstant(3, True)
1666:   Set pSteepSlopeDataset = pTempRaster.RasterDataset

1668:   Set pTempRaster = pMakerOp.MakeConstant(4, True)
1669:   Set pRidgelineDataset = pTempRaster.RasterDataset

  ' NEXT MAKE CONSTANT-VALUE RASTERS FOR THE VARIOUS TPI AND SLOPE THRESHOLDS
  ' --> TPI THRESHOLDS AT -8, 8
  ' --> SLOPE THRESHOLD AT 6

  Dim pTPI_Negative_8_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = -8
  Dim pTPI_Positive_8_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = 8
  Dim pSlope_6_Dataset As IRasterDataset            ' FOR SLOPE THRESHOLD = 6

1679:   Set pTempRaster = pMakerOp.MakeConstant(dblThresholdA, False)
1680:   Set pTPI_Negative_8_Dataset = pTempRaster.RasterDataset

1682:   Set pTempRaster = pMakerOp.MakeConstant(dblThresholdB, False)
1683:   Set pTPI_Positive_8_Dataset = pTempRaster.RasterDataset

1685:   Set pTempRaster = pMakerOp.MakeConstant(dblThresholdS, False)
1686:   Set pSlope_6_Dataset = pTempRaster.RasterDataset

  Dim pSlopePositionDataset As IRasterDataset
  
'  MsgBox "pTPIDataset is Nothing = " & CStr(pTPIDataset Is Nothing)
'  MsgBox "pTPI_Negative_8_Dataset is Nothing = " & CStr(pTPI_Negative_8_Dataset Is Nothing)
'  MsgBox "pCanyonBottomDataset is Nothing = " & CStr(pCanyonBottomDataset Is Nothing)
'  MsgBox "pTPI_Positive_8_Dataset is Nothing = " & CStr(pTPI_Positive_8_Dataset Is Nothing)
'  MsgBox "pSlopeDataset is Nothing = " & CStr(pSlopeDataset Is Nothing)
'  MsgBox "pSlope_6_Dataset is Nothing = " & CStr(pSlope_6_Dataset Is Nothing)
'  MsgBox "pGentleSlopeDataset is Nothing = " & CStr(pGentleSlopeDataset Is Nothing)
'  MsgBox "pSteepSlopeDataset is Nothing = " & CStr(pSteepSlopeDataset Is Nothing)
'  MsgBox "pRidgelineDataset is Nothing = " & CStr(pRidgelineDataset Is Nothing)
  
1700:   Set pTempRaster = _
      pConOp.Con(pLogOp.LessThanEqual(pTPIDataset, pTPI_Negative_8_Dataset), pCanyonBottomDataset, _
      pConOp.Con(pLogOp.LessThanEqual(pTPIDataset, pTPI_Positive_8_Dataset), _
                 pConOp.Con(pLogOp.LessThanEqual(pSlopeDataset, pSlope_6_Dataset), pGentleSlopeDataset, pSteepSlopeDataset), _
                      pRidgelineDataset))
  
  Dim pRastBandColl As IRasterBandCollection
  Dim pRastBand As IRasterBand
  Dim pSupportOp As IMathSupportOp
1709:   Set pSupportOp = New RasterMathSupportOp
  
1711:   Set pSlopePositionDataset = pTempRaster.RasterDataset
1712:   Set pRastBandColl = pSlopePositionDataset
1713:   Set pRastBand = pRastBandColl.Item(0)
1714:   pRastBand.ComputeStatsAndHist

  ' CONVERT TO INTEGER RASTER
  Dim pSlopePositionIntegerDataset As IRasterDataset
1718:   Set pTempRaster = pSupportOp.Int(pSlopePositionDataset)
  
  Dim pRastStorDef As IRasterStorageDef
1721:   Set pRastStorDef = New RasterStorageDef
         
1723:   If booIsFGB Then
1724:     Set pSlopePositionIntegerDataset = pRastWsEx.SaveAsRasterDataset(strNewRasterName, pTempRaster, Nothing, "", Nothing, Nothing)
1725:   Else
1726:     Set pSaveAs = pTempRaster
1727:     Set pSlopePositionIntegerDataset = pSaveAs.SaveAsRasterDataset(strNewRasterName & strExtension, _
          pRastWS, strSaveFormat, pRastStorDef)  ' <----------------
1729:     Set pSaveAs = Nothing
1730:     Set pTempRaster = Nothing
1731:     Set pSupportOp = Nothing
1732:     Set pRastBand = Nothing
1733:     Set pRastBandColl = Nothing
1734:     Set pSlopePositionDataset = Nothing
1735:     Set pConOp = Nothing
1736:     Set pSlopePositionIntegerDataset = pRastWS.OpenRasterDataset(strNewRasterName & strExtension)
1737:   End If
  
1739:   Set pRastBandColl = pSlopePositionIntegerDataset
1740:   Set pRastBand = pRastBandColl.Item(0)
1741:   pRastBand.ComputeStatsAndHist

  ' ADD CORRECT CLASS NAMES TO RASTER ATTRIBUTE TABLE
  ' FIRST ADD NAME FIELD
1745:   Debug.Print "  --> Updating 4-Category Topographic Position Table..."
  Dim pTable As ITable
'  Set pTable = pRastBand.AttributeTable
1748:   Set pTable = TopoCorridor.TopoCor_Functions.ReturnRasterTable(pRastBand)
  Dim pNameField As IField
1750:   Set pNameField = New Field
  Dim pNameFieldEdit As IFieldEdit
1752:   Set pNameFieldEdit = pNameField
1753:   With pNameFieldEdit
1754:     .Name = "Class_Name"
1755:     .Type = esriFieldTypeString
1756:     .length = 25
1757:   End With
1758:   pTable.AddField pNameField
  ' NEXT ADD CLASS VALUES TO NEW FIELD
  Dim lngValue As Long
  Dim lngValueIndex As Long
  Dim lngNameIndex As Long

1764:   lngValueIndex = pTable.FindField("VALUE")
1765:   lngNameIndex = pTable.FindField("Class_Name")

  Dim pCursor As ICursor
  Dim pRow As IRow
1769:   Set pCursor = pTable.Update(Nothing, False)
1770:   Set pRow = pCursor.NextRow
1771:   Do Until pRow Is Nothing
1772:     lngValue = pRow.Value(lngValueIndex)
    Select Case lngValue
      Case 1
1775:         pRow.Value(lngNameIndex) = strClass1Name
      Case 2
1777:         pRow.Value(lngNameIndex) = strClass2Name
      Case 3
1779:         pRow.Value(lngNameIndex) = strClass3Name
      Case 4
1781:         pRow.Value(lngNameIndex) = strClass4Name
      Case Else
1783:         pRow.Value(lngNameIndex) = "Unknown"
1784:     End Select
1785:     pCursor.UpdateRow pRow

1787:     Set pRow = pCursor.NextRow
1788:   Loop


  ' SYMBOLIZE ADD 4-CATEGORY SLOPE POSITION CLASSIFICATION RASTER TO MAP
1792:   psbar.Message(0) = "  --> Adding 4-Category Topographic Position to map..."
  Dim p4_CategoryLayer As IRasterLayer
1794:   Set p4_CategoryLayer = New RasterLayer
1795:   p4_CategoryLayer.CreateFromDataset pSlopePositionIntegerDataset
1796:   p4_CategoryLayer.Name = strNewRasterName
  
'  Dim strTemplateFile As String
'  If Right(App.Path, 1) = "\" Then
'    strTemplateFile = App.Path & "slope_4_class_template.lyr"
'  Else
'    strTemplateFile = App.Path & "\slope_4_class_template.lyr"
'  End If
'
'  If TopoCorridor.aml_func_mod.FileExists(strTemplateFile) Then
'    TopoCorridor.MyGeneralOperations.SetRasterSymbols p4_CategoryLayer, strTemplateFile
'  End If
     
  Dim NumOfValues As Integer
1810:   NumOfValues = pTable.RowCount(Nothing)
  
  ' CREATE CLASS COLORS
  
  ' CANYON = MEDIUM BLUE
  Dim pCanyonColor As IRgbColor
1816:   Set pCanyonColor = New RgbColor
1817:   pCanyonColor.RGB = RGB(102, 204, 255)
  
  ' FLAT SLOPE = LIGHT YELLOW GREEN
  Dim pFlatColor As IRgbColor
1821:   Set pFlatColor = New RgbColor
1822:   pFlatColor.RGB = RGB(255, 255, 204)
  
  ' STEEP SLOPE = DARK GREEN
  Dim pSteepColor As IRgbColor
1826:   Set pSteepColor = New RgbColor
1827:   pSteepColor.RGB = RGB(0, 102, 0)
  
  ' RIDGES = MEDIUM BRICK
  Dim pRidgesColor As IRgbColor
1831:   Set pRidgesColor = New RgbColor
1832:   pRidgesColor.RGB = RGB(255, 153, 102)
  
  ' OTHER = BLACK
  Dim pOtherColor As IRgbColor
1836:   Set pOtherColor = New RgbColor
1837:   pOtherColor.RGB = RGB(0, 0, 0)
  
  Dim pCatRaster As IRaster
1840:   Set pCatRaster = p4_CategoryLayer.Raster
  
  Dim pFSymbol As ISimpleFillSymbol
  
  ' Create UniqueValue renderer and QI RasterRenderer
  Dim pUVRen As IRasterUniqueValueRenderer
1846:   Set pUVRen = New RasterUniqueValueRenderer
  Dim pRasRen As IRasterRenderer
1848:   Set pRasRen = pUVRen

  ' Connect the renderer and the raster
1851:   Set pRasRen.Raster = pCatRaster
1852:   pRasRen.Update
  
  ' Set UniqueValue renerer
1855:   pUVRen.HeadingCount = 1   ' Use one heading
1856:   pUVRen.Heading(0) = "Slope Classes"
1857:   pUVRen.ClassCount(0) = NumOfValues
1858:   pUVRen.Field = "Class_Name"
  
  Dim varLabelValue As Variant
1861:   Set pCursor = pTable.Update(Nothing, False)
1862:   Set pRow = pCursor.NextRow
  Dim lngOID As Long
1864:   Do Until pRow Is Nothing
1865:     lngValue = pRow.Value(lngValueIndex)
1866:     lngOID = pRow.OID
    Select Case lngValue
      Case 1
1869:         varLabelValue = strClass1Name
1870:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
1871:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1872:         Set pFSymbol = New SimpleFillSymbol
1873:         pFSymbol.Color = pCanyonColor
1874:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case 2
1876:         varLabelValue = strClass2Name
1877:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
1878:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1879:         Set pFSymbol = New SimpleFillSymbol
1880:         pFSymbol.Color = pFlatColor
1881:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case 3
1883:         varLabelValue = strClass3Name
1884:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
1885:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1886:         Set pFSymbol = New SimpleFillSymbol
1887:         pFSymbol.Color = pSteepColor
1888:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case 4
1890:         varLabelValue = strClass4Name
1891:         pUVRen.AddValue 0, lngOID, varLabelValue  'Get a row from the table
1892:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1893:         Set pFSymbol = New SimpleFillSymbol
1894:         pFSymbol.Color = pRidgesColor
1895:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
      Case Else
1897:         varLabelValue = "Unknown"
1898:         pUVRen.AddValue 0, lngOID, varLabelValue 'Get a row from the table
1899:         pUVRen.Label(0, lngOID) = CStr(varLabelValue) ' Set label
1900:         Set pFSymbol = New SimpleFillSymbol
1901:         pFSymbol.Color = pOtherColor
1902:         pUVRen.Symbol(0, lngOID) = pFSymbol  'Set symbol
1903:     End Select
1904:     pCursor.UpdateRow pRow

1906:     Set pRow = pCursor.NextRow
1907:   Loop
 

  'Update render and refresh layer
1911:   pRasRen.Update
1912:   Set p4_CategoryLayer.Renderer = pUVRen
  
1914:   pMxDoc.FocusMap.AddLayer p4_CategoryLayer

1916:   strReport = "Calculate 4-Category Topographic Position Function Complete:" & vbCrLf & _
              "----------------------------------------------------------" & vbCrLf & _
              "Analysis Parameters:" & vbCrLf & _
              " --> Classification Scheme:" & vbCrLf & vbCrLf & strClassDescription & vbCrLf & _
              " --> TPI Type = " & strTPIOption & vbCrLf & _
              " --> Input Raster Layer: " & strInputName & vbCrLf & _
              " --> " & strNbrHoodDesc & vbCrLf & _
              " --> Option to Mark as NoData if any NoData Cells in Neighborhood: " & _
                      UCase(CStr(booExcludeIfAnyNull)) & vbCrLf & _
              " --> Output Format: " & strSaveFormatString & vbCrLf & _
              " --> Filename: " & strNewRasterName & strExtension & vbCrLf & _
              " --> Workspace = " & pWS.PathName & vbCrLf & _
              "----------------------------------------------------------" & vbCrLf & _
              TopoCorridor.MyGeneralOperations.ReturnTimeElapsed(datBegan, Now)
  
1931:   pMxDoc.UpdateContents
1932:   pMxDoc.ActiveView.PartialRefresh esriViewGeography, Nothing, pEnv

1934:   Create_Sample_4_Class_SlopeClass = strReport

1936:   pProgAnim.Stop
1937:   pProgAnim.Hide
1938:   Set pProgAnim = Nothing

1940:   GoTo CleanUp
ErrorHandler:

1943:   pProgAnim.Stop
1944:   pProgAnim.Hide
1945:   Set pProgAnim = Nothing
  HandleError True, "Create_Sample_4_Class_SlopeClass " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

CleanUp:

1950:   Set pApp = Nothing
1951:   Set pMxDoc = Nothing
1952:   Set pThisExt = Nothing
1953:   Set psbar = Nothing
1954:   Set pProgAnim = Nothing
1955:   Set pMakerOp = Nothing
1956:   Set pRastAnEnv = Nothing
1957:   Set pRastProps = Nothing
1958:   Set pSpRef = Nothing
1959:   Set pGeoDataset = Nothing
1960:   Set pMathOp = Nothing
1961:   Set pNbrHoodOp = Nothing
1962:   Set pNbrHood = Nothing
1963:   Set pLogOp = Nothing
1964:   Set pConOp = Nothing
1965:   Set pInputBand = Nothing
1966:   Set pRastLayer = Nothing
1967:   Set pInputDataset = Nothing
1968:   Set pInputRaster = Nothing
1969:   Set pEnv = Nothing
1970:   Set pRastWS = Nothing
1971:   Set pRastWsEx = Nothing
1972:   Set pWS = Nothing
1973:   Set pSaveAs = Nothing
1974:   Set pTempDataset = Nothing
1975:   Set pTPIDataset = Nothing
1976:   Set pSlopeDataset = Nothing
1977:   Set pTempRaster = Nothing
1978:   Set pCanyonBottomDataset = Nothing
1979:   Set pGentleSlopeDataset = Nothing
1980:   Set pSteepSlopeDataset = Nothing
1981:   Set pRidgelineDataset = Nothing
1982:   Set pTPI_Negative_8_Dataset = Nothing
1983:   Set pTPI_Positive_8_Dataset = Nothing
1984:   Set pSlope_6_Dataset = Nothing
1985:   Set pSlopePositionDataset = Nothing
1986:   Set pRastBandColl = Nothing
1987:   Set pRastBand = Nothing
1988:   Set pSupportOp = Nothing
1989:   Set pSlopePositionIntegerDataset = Nothing
1990:   Set pRastStorDef = Nothing
1991:   Set pTable = Nothing
1992:   Set pNameField = Nothing
1993:   Set pNameFieldEdit = Nothing
1994:   Set pCursor = Nothing
1995:   Set pRow = Nothing
1996:   Set p4_CategoryLayer = Nothing
1997:   Set pCanyonColor = Nothing
1998:   Set pFlatColor = Nothing
1999:   Set pSteepColor = Nothing
2000:   Set pRidgesColor = Nothing
2001:   Set pOtherColor = Nothing
2002:   Set pCatRaster = Nothing
2003:   Set pFSymbol = Nothing
2004:   Set pUVRen = Nothing
2005:   Set pRasRen = Nothing

End Function


'
'Public Sub Create_Sample_10_Class_Landform()
'
'  ' THIS SAMPLE FROM THE MANUAL CREATES A 10-CATEGORY LANDFORM CLASSIFICATION ACCORDING TO THE
'  ' FOLLOWING CRITERIA
'
'  '         Canyons, Deeply Incised Streams [1]:  Small Neighborhood TPI:  TPI <= -1 Standardized Units
'  '                                               Large Neighborhood TPI:  TPI <= -1 Standardized Units
'  '     Midslope Drainages, Shallow Valleys [2]:  Small Neighborhood TPI:  TPI <= -1 Standardized Units
'  '                                               Large Neighborhood TPI:  -1 < TPI < 1 Standardized Units
'  '            Upland Drainages, Headwaters [3]:  Small Neighborhood TPI:  TPI <= -1 Standardized Units
'  '                                               Large Neighborhood TPI:  TPI >= 1 Standardized Units
'  '                        U-shaped Valleys [4]:  Small Neighborhood TPI:  -1 < TPI < 1 Standardized Units
'  '                                               Large Neighborhood TPI:  TPI <= -1 Standardized Units
'  '                                  Plains [5]:  Small  Neighborhood TPI:  -1 < TPI < 1 Standardized Units
'  '                                               Large Neighborhood TPI:  -1 < TPI < 1 Standardized Units
'  '                                               Slope <= 5
'  '                             Open Slopes [6]:  Small Neighborhood TPI:  -1 < TPI < 1 Standardized Units
'  '                                               Large Neighborhood TPI:  -1 < TPI < 1 Standardized Units
'  '                                               Slope > 5
'  '                     Upper Slopes, Mesas [7]:  Small Neighborhood TPI:  -1 < TPI < 1 Standardized Units
'  '                                               Large Neighborhood TPI:  TPI >= 1 Standardized Units
'  '           Local Ridges/Hills in Valleys [8]:  Small Neighborhood TPI:  TPI >= 1 Standardized Units
'  '                                               Large Neighborhood TPI:  TPI <= -1 Standardized Units
'  '  Midslope Ridges, Small Hills in Plains [9]:  Small Neighborhood TPI:  TPI >= 1 Standardized Units
'  '                                               Large Neighborhood TPI:  -1 < TPI < 1 Standardized Units
'  '             Mountain Tops, High Ridges [10]:  Small Neighborhood TPI:  TPI >= 1 Standardized Units
'  '                                               Large Neighborhood TPI:  TPI >= 1 Standardized Units
'  '
'
'  ' TPI thresholds are in standardized TPI units.  Therefore we first calculate a TPI raster (equal to
'  ' [Elevation Value] - [Neighborhood Mean Elevation Value] for both the small and large
'  ' neighborhood sizes.  We then standardize that TPI raster according to the mean and standard deviation
'  ' of the entire TPI raster.
'
'  ' DEFINING VARIABLES:  DO NOT WORRY ABOUT THIS PART =================================
'  Dim strDEMLayerName As String
'  Set pFrm10CategoryParams = Nothing
''  Dim dblSmallNeighborhoodRadius As Double
''  Dim lngSmallNeighborhoodUnits As esriGeoAnalysisUnitsEnum
''  Dim dblLargeNeighborhoodRadius As Double
''  Dim lngLargeNeighborhoodUnits As esriGeoAnalysisUnitsEnum
''
''  ' VARIABLES FOR YOU TO SET ==========================================================
''  ' NAME OF ORIGINAL DEM LAYER IN ARCMAP
''  strDEMLayerName = "dem"
''
''  ' SMALL NEIGHBORHOOD PARAMETERS; USED WHEN CALCULATING TPI AND STANDARDIZED TPI
''  ' SETTINGS BELOW SPECIFY A CIRCULAR NEIGHBORHOOD WTIH A RADIUS OF 5 CELLS
''  dblSmallNeighborhoodRadius = 5  '..................RADIUS OF CIRCULAR NEIGHBORHOOD
''  lngSmallNeighborhoodUnits = esriUnitsCells   '.....MEANS RADIUS IS DEFINED AS CELLS
''                                               '     CHANGE THIS TO esriUnitsMap IF YOU WANT TO DEFINE RADIUS
''                                               '        AS METERS OR FEET
''
''  ' LARGE NEIGHBORHOOD PARAMETERS; USED WHEN CALCULATING TPI AND STANDARDIZED TPI
''  ' SETTINGS BELOW SPECIFY A CIRCULAR NEIGHBORHOOD WTIH A RADIUS OF 25 CELLS
''  dblLargeNeighborhoodRadius = 25  '..................RADIUS OF CIRCULAR NEIGHBORHOOD
''  lngLargeNeighborhoodUnits = esriUnitsCells   '.....MEANS RADIUS IS DEFINED AS CELLS
'                                               '     CHANGE THIS TO esriUnitsMap IF YOU WANT TO DEFINE RADIUS
'                                               '        AS METERS OR FEET
'
'  ' DEFINE ADDITIONAL VARIABLES
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
'  Dim pRastBandColl As IRasterBandCollection
'  Dim pRastBand As IRasterBand
'  Dim pRastStats As IRasterStatistics
'  Dim pApp As IApplication
'  Set pApp = Application
'  Dim pSBar As IStatusBar
'  Set pSBar = pApp.StatusBar
'  Dim pMakerOp As IRasterMakerOp
'  Set pMakerOp = New RasterMakerOp
'  Dim pRastAnEnv As IRasterAnalysisEnvironment
'  Dim pRastProps As IRasterProps
'  Dim pSpRef As ISpatialReference
'  Dim pGeoDataset As IGeoDataset
'  Dim pMathOp As IMathOp
'  Set pMathOp = New RasterMathOps
'  Dim pNbrHoodOp As INeighborhoodOp
'  Set pNbrHoodOp = New RasterNeighborhoodOp
'  Dim pConOp As IConditionalOp
'  Set pConOp = New RasterConditionalOp
'  Dim pLogOp As ILogicalOp
'  Set pLogOp = New RasterMathOps
'  Dim pSupportOp As IMathSupportOp
'  Set pSupportOp = New RasterMathSupportOp
'  Dim pTempRaster As IRaster2
'  Dim pSmallNbrHood As IRasterNeighborhood
''  Set pSmallNbrHood = New RasterNeighborhood
''  pSmallNbrHood.SetCircle dblSmallNeighborhoodRadius, lngSmallNeighborhoodUnits
'  Dim pLargeNbrHood As IRasterNeighborhood
''  Set pLargeNbrHood = New RasterNeighborhood
''  pLargeNbrHood.SetCircle dblLargeNeighborhoodRadius, lngLargeNeighborhoodUnits
'  Dim dblMean As Double
'  Dim dblSD As Double
'  Dim pTempDataset As IRasterDataset
'  Dim pMeanDataset As IRasterDataset
'  Dim pSDDataset As IRasterDataset
'
'  ' GET PARAMETERS FROM DIALOG
'
'  Dim frmParams As frm10CategoryLandform
'  Set frmParams = New frm10CategoryLandform
'  Dim pDocArray As esriSystem.IVariantArray
'  Set pDocArray = New esriSystem.VarArray
'  pDocArray.Add pMxDoc
'  pDocArray.Add pApp
'  Set frmParams.ArcStuff = pDocArray
'  frmParams.Show vbModal
'
'  Set frmParams = Nothing
'  Set pDocArray = Nothing
'
'  If pFrm10CategoryParams Is Nothing Then
'    Set pMxDoc = Nothing
'    Set pApp = Nothing
'    Set pRastBandColl = Nothing
'    Set pRastBand = Nothing
'    Set pRastStats = Nothing
'    Set pSBar = Nothing
'    Set pMakerOp = Nothing
'    Set pRastAnEnv = Nothing
'    Set pRastProps = Nothing
'    Set pSpRef = Nothing
'    Set pGeoDataset = Nothing
'    Set pMathOp = Nothing
'    Set pNbrHoodOp = Nothing
'    Set pConOp = Nothing
'    Set pLogOp = Nothing
'    Set pSupportOp = Nothing
'    Set pTempRaster = Nothing
'    Set pSmallNbrHood = Nothing
'    Set pLargeNbrHood = Nothing
'
'    Exit Sub
'
'  End If
'  ' ANALYSIS ==========================================================================
'
'  ' FIND DEM LAYER IN MAP AND GET RASTER DATASET
'  Dim pDEMLayer As IRasterLayer
'  Set pDEMLayer = pFrm10CategoryParams.Element(0)
'  strDEMLayerName = pDEMLayer.Name
'
'  Dim strTPIVariation As String
'  strTPIVariation = pFrm10CategoryParams.Element(1)
'
'  Dim booAddIntermediate As Boolean
'  booAddIntermediate = pFrm10CategoryParams.Element(2)
'
'  Set pSmallNbrHood = pFrm10CategoryParams.Element(3)
'  Set pLargeNbrHood = pFrm10CategoryParams.Element(4)
'
'  Dim dblThresholdA As Double
'  Dim dblThresholdB As Double
'  Dim dblThresholdX As Double
'  Dim dblThresholdY As Double
'  Dim dblThresholdSlope As Double
'
'  dblThresholdA = pFrm10CategoryParams.Element(5)
'  dblThresholdB = pFrm10CategoryParams.Element(6)
'  dblThresholdX = pFrm10CategoryParams.Element(7)
'  dblThresholdY = pFrm10CategoryParams.Element(8)
'  dblThresholdSlope = pFrm10CategoryParams.Element(9)
'
'  Set pFrm10CategoryParams = Nothing
'
'  ' ANALYSIS ==========================================================================
'
'  ' FIND DEM LAYER IN MAP AND GET RASTER DATASET
'  Dim pDEMRaster As IRaster2
'  Set pDEMRaster = pDEMLayer.Raster
'  Dim pDEMDataset As IRasterDataset
'  Set pDEMDataset = pDEMRaster.RasterDataset
'  Set pGeoDataset = pDEMDataset
'  Set pSpRef = pGeoDataset.SpatialReference
'  Set pRastProps = pDEMRaster
'  Set pRastAnEnv = pMakerOp
'  pRastAnEnv.SetCellSize esriRasterEnvValue, pDEMRaster
'  pRastAnEnv.SetExtent esriRasterEnvValue, pDEMRaster, pDEMRaster
'  Set pRastAnEnv.OutSpatialReference = pSpRef
'
'
'  Dim pSmallStandTPIDataset As IRasterDataset
'  Dim pLargeStandTPIDataset As IRasterDataset
'
'  ' CALCULATE TPI VARIATION:  EITHER STANDARDIZED DEM, SIMPLE TPI, OR STANDARDIZED TPI
'
'  Dim ptpidataset As IRasterDataset
'  If strTPIVariation = "Standardized Elevation" Then
'    Set pSmallStandTPIDataset = ReturnStandardizedElevationDataset(pDEMDataset, pNbrHoodOp, pSBar, _
'          pMakerOp, pSmallNbrHood, strDEMLayerName, pMxDoc, booAddIntermediate, " Small-Neighborhood ")
'    Set pLargeStandTPIDataset = ReturnStandardizedElevationDataset(pDEMDataset, pNbrHoodOp, pSBar, _
'          pMakerOp, pLargeNbrHood, strDEMLayerName, pMxDoc, booAddIntermediate, " Large-Neighborhood ")
'
'  ElseIf strTPIVariation = "Standardized TPI" Then
'    Set pSmallStandTPIDataset = ReturnStandardizedTPIDataset(pDEMDataset, pSmallNbrHood, "Small-Neighborhood TPI", _
'          strDEMLayerName, pSBar, pNbrHoodOp, pMxDoc, pMakerOp, booAddIntermediate)
'    Set pLargeStandTPIDataset = ReturnStandardizedTPIDataset(pDEMDataset, pLargeNbrHood, "Large-Neighborhood TPI", _
'          strDEMLayerName, pSBar, pNbrHoodOp, pMxDoc, pMakerOp, booAddIntermediate)
'
'  ElseIf strTPIVariation = "TPI" Then
'    Set pSmallStandTPIDataset = ReturnTPIDataset(pDEMDataset, pSBar, strDEMLayerName, pNbrHoodOp, pSmallNbrHood, pMathOp, _
'          pMxDoc, booAddIntermediate, " Small-Neighborhood ")
'    Set pLargeStandTPIDataset = ReturnTPIDataset(pDEMDataset, pSBar, strDEMLayerName, pNbrHoodOp, pLargeNbrHood, pMathOp, _
'          pMxDoc, booAddIntermediate, " Large-Neighborhood ")
'
'  End If
'
'  Dim pSlopeDataset As IRasterDataset
'  Set pSlopeDataset = ReturnSlopeDataset(pDEMDataset, pSBar, strDEMLayerName, pMxDoc, booAddIntermediate)
'
'
'  ' NEXT MAKE CONSTANT-VALUE RASTERS FOR THE VARIOUS TPI AND SLOPE THRESHOLDS
'  ' --> TPI THRESHOLDS AT -1, 1
'  ' --> SLOPE THRESHOLD AT 5
'
'  Dim pTPI_Negative_1_Small_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = -1
'  Dim pTPI_Positive_1_Small_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = 1
'  Dim pTPI_Negative_1_Large_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = -1
'  Dim pTPI_Positive_1_Large_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = 1
'  Dim pSlope_5_Dataset As IRasterDataset            ' FOR SLOPE THRESHOLD = 5
'  Dim pZeroDataset As IRasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(dblThresholdA, False)
'  Set pTPI_Negative_1_Small_Dataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(dblThresholdB, False)
'  Set pTPI_Positive_1_Small_Dataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(dblThresholdX, False)
'  Set pTPI_Negative_1_Large_Dataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(dblThresholdY, False)
'  Set pTPI_Positive_1_Large_Dataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(5, False)
'  Set pSlope_5_Dataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(0, False)
'  Set pZeroDataset = pTempRaster.RasterDataset
'
'  ' COMBINE SLOPE AND TPI INTO 10-CATEGORY LANDFORM CLASSIFICATION RASTER
'  ' FIRST CREATE CONSTANT-VALUE RASTERS FOR THE 10 CATEGORIES
'  pSBar.Message(0) = "Calculating 10-Category Landform Classification Raster..."
'  Debug.Print vbCrLf & "Calculating 10-Category Landform Classification Raster..."
'  Dim p1_CanyonsDataset As IRasterDataset
'  Dim p2_MidslopeDrainagesDataset As IRasterDataset
'  Dim p3_UplandDataset As IRasterDataset
'  Dim p4_UShapedDataset As IRasterDataset
'  Dim p5_PlainsDataset As IRasterDataset
'  Dim p6_OpenDataset As IRasterDataset
'  Dim p7_UpperDataset As IRasterDataset
'  Dim p8_LocalRidgesDataset As IRasterDataset
'  Dim p9_MidslopeRidgesDataset As IRasterDataset
'  Dim p10_MountainTopsDataset As IRasterDataset
'
'  Debug.Print "  --> Calculating Class 1:  Canyons, Deeply Incised Streams..."
'  Set pTempRaster = pMakerOp.MakeConstant(1, True)
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempRaster = pConOp.Con(pLogOp.BooleanAnd( _
'      pLogOp.LessThanEqual(pSmallStandTPIDataset, pTPI_Negative_1_Small_Dataset), _
'      pLogOp.LessThanEqual(pLargeStandTPIDataset, pTPI_Negative_1_Large_Dataset)), pTempDataset, pZeroDataset)
'  Set p1_CanyonsDataset = pTempRaster.RasterDataset
'
'  Debug.Print "  --> Calculating Class 2:  Midslope Drainages, Shallow Valleys..."
'  Set pTempRaster = pMakerOp.MakeConstant(2, True)
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempRaster = pMathOp.Plus(p1_CanyonsDataset, _
'      pConOp.Con(pLogOp.BooleanAnd( _
'                  pLogOp.LessThanEqual(pSmallStandTPIDataset, pTPI_Negative_1_Small_Dataset), _
'                  pLogOp.BooleanAnd(pLogOp.LessThan(pLargeStandTPIDataset, pTPI_Positive_1_Large_Dataset), _
'                                    pLogOp.GreaterThan(pLargeStandTPIDataset, pTPI_Negative_1_Large_Dataset))), _
'                  pTempDataset, pZeroDataset))
'  Set p2_MidslopeDrainagesDataset = pTempRaster.RasterDataset
'
'  Debug.Print "  --> Calculating Class 3:  Upland Drainages, Headwaters..."
'  Set pTempRaster = pMakerOp.MakeConstant(3, True)
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempRaster = pMathOp.Plus(p2_MidslopeDrainagesDataset, _
'                        pConOp.Con(pLogOp.BooleanAnd( _
'                            pLogOp.LessThanEqual(pSmallStandTPIDataset, pTPI_Negative_1_Small_Dataset), _
'                            pLogOp.GreaterThanEqual(pLargeStandTPIDataset, pTPI_Positive_1_Large_Dataset)), _
'                            pTempDataset, pZeroDataset))
'  Set p3_UplandDataset = pTempRaster.RasterDataset
'
'  Debug.Print "  --> Calculating Class 4:  U-shaped Valleys..."
'  Set pTempRaster = pMakerOp.MakeConstant(4, True)
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempRaster = pMathOp.Plus(p3_UplandDataset, _
'      pConOp.Con(pLogOp.BooleanAnd( _
'                  pLogOp.BooleanAnd(pLogOp.LessThan(pSmallStandTPIDataset, pTPI_Positive_1_Small_Dataset), _
'                                    pLogOp.GreaterThan(pSmallStandTPIDataset, pTPI_Negative_1_Small_Dataset)), _
'                  pLogOp.LessThanEqual(pLargeStandTPIDataset, pTPI_Negative_1_Large_Dataset)), _
'                  pTempDataset, pZeroDataset))
'  Set p4_UShapedDataset = pTempRaster.RasterDataset
'
'  Debug.Print "  --> Calculating Class 5:  Plains..."
'  Set pTempRaster = pMakerOp.MakeConstant(5, True)
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempRaster = pMathOp.Plus(p4_UShapedDataset, _
'      pConOp.Con(pLogOp.BooleanAnd( _
'          pLogOp.BooleanAnd( _
'                  pLogOp.BooleanAnd(pLogOp.LessThan(pSmallStandTPIDataset, pTPI_Positive_1_Small_Dataset), _
'                                    pLogOp.GreaterThan(pSmallStandTPIDataset, pTPI_Negative_1_Small_Dataset)), _
'                  pLogOp.BooleanAnd(pLogOp.LessThan(pLargeStandTPIDataset, pTPI_Positive_1_Large_Dataset), _
'                                    pLogOp.GreaterThan(pLargeStandTPIDataset, pTPI_Negative_1_Large_Dataset))), _
'          pLogOp.LessThanEqual(pSlopeDataset, pSlope_5_Dataset)), _
'          pTempDataset, pZeroDataset))
'  Set p5_PlainsDataset = pTempRaster.RasterDataset
'
'  Debug.Print "  --> Calculating Class 6:  Open Slopes..."
'  Set pTempRaster = pMakerOp.MakeConstant(6, True)
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempRaster = pMathOp.Plus(p5_PlainsDataset, _
'      pConOp.Con(pLogOp.BooleanAnd( _
'          pLogOp.BooleanAnd( _
'                  pLogOp.BooleanAnd(pLogOp.LessThan(pSmallStandTPIDataset, pTPI_Positive_1_Small_Dataset), _
'                                    pLogOp.GreaterThan(pSmallStandTPIDataset, pTPI_Negative_1_Small_Dataset)), _
'                  pLogOp.BooleanAnd(pLogOp.LessThan(pLargeStandTPIDataset, pTPI_Positive_1_Large_Dataset), _
'                                    pLogOp.GreaterThan(pLargeStandTPIDataset, pTPI_Negative_1_Large_Dataset))), _
'          pLogOp.GreaterThan(pSlopeDataset, pSlope_5_Dataset)), _
'          pTempDataset, pZeroDataset))
'  Set p6_OpenDataset = pTempRaster.RasterDataset
'
'  Debug.Print "  --> Calculating Class 7:  Upper Slopes, Mesas..."
'  Set pTempRaster = pMakerOp.MakeConstant(7, True)
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempRaster = pMathOp.Plus(p6_OpenDataset, _
'      pConOp.Con(pLogOp.BooleanAnd( _
'                  pLogOp.BooleanAnd(pLogOp.LessThan(pSmallStandTPIDataset, pTPI_Positive_1_Small_Dataset), _
'                                    pLogOp.GreaterThan(pSmallStandTPIDataset, pTPI_Negative_1_Small_Dataset)), _
'                  pLogOp.GreaterThanEqual(pLargeStandTPIDataset, pTPI_Positive_1_Large_Dataset)), _
'                  pTempDataset, pZeroDataset))
'  Set p7_UpperDataset = pTempRaster.RasterDataset
'
'  Debug.Print "  --> Calculating Class 8:  Local Ridges/Hills in Valleys..."
'  Set pTempRaster = pMakerOp.MakeConstant(8, True)
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempRaster = pMathOp.Plus(p7_UpperDataset, _
'                        pConOp.Con(pLogOp.BooleanAnd( _
'                            pLogOp.GreaterThanEqual(pSmallStandTPIDataset, pTPI_Positive_1_Small_Dataset), _
'                            pLogOp.LessThanEqual(pLargeStandTPIDataset, pTPI_Negative_1_Small_Dataset)), _
'                            pTempDataset, pZeroDataset))
'  Set p8_LocalRidgesDataset = pTempRaster.RasterDataset
'
'  Debug.Print "  --> Calculating Class 9:  Midslope Ridges, Small Hills in Plains..."
'  Set pTempRaster = pMakerOp.MakeConstant(9, True)
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempRaster = pMathOp.Plus(p8_LocalRidgesDataset, _
'      pConOp.Con(pLogOp.BooleanAnd( _
'                  pLogOp.GreaterThanEqual(pSmallStandTPIDataset, pTPI_Positive_1_Small_Dataset), _
'                  pLogOp.BooleanAnd(pLogOp.LessThan(pLargeStandTPIDataset, pTPI_Positive_1_Large_Dataset), _
'                                    pLogOp.GreaterThan(pLargeStandTPIDataset, pTPI_Negative_1_Large_Dataset))), _
'                  pTempDataset, pZeroDataset))
'  Set p9_MidslopeRidgesDataset = pTempRaster.RasterDataset
'
'  Debug.Print "  --> Calculating Class 10:  Mountain Tops, High Ridges..."
'  Set pTempRaster = pMakerOp.MakeConstant(10, True)
'  Set pTempDataset = pTempRaster.RasterDataset
'  Set pTempRaster = pMathOp.Plus(p9_MidslopeRidgesDataset, _
'                        pConOp.Con(pLogOp.BooleanAnd( _
'                            pLogOp.GreaterThanEqual(pSmallStandTPIDataset, pTPI_Positive_1_Small_Dataset), _
'                            pLogOp.GreaterThanEqual(pLargeStandTPIDataset, pTPI_Positive_1_Large_Dataset)), _
'                            pTempDataset, pZeroDataset))
'  Set p10_MountainTopsDataset = pTempRaster.RasterDataset
'
'  Dim pLandformDataset As IRasterDataset
'
'  Set pLandformDataset = p10_MountainTopsDataset
'  Set pRastBandColl = p10_MountainTopsDataset
'  Set pRastBand = pRastBandColl.Item(0)
'  pRastBand.ComputeStatsAndHist
'
'  ' CONVERT TO INTEGER RASTER
'  Dim pLandformIntegerDataset As IRasterDataset
'  Set pTempRaster = pSupportOp.Int(pLandformDataset)
'  Set pLandformIntegerDataset = pTempRaster.RasterDataset
'  Set pRastBandColl = pLandformIntegerDataset
'  Set pRastBand = pRastBandColl.Item(0)
'  pRastBand.ComputeStatsAndHist
'
'  ' ADD CORRECT CLASS NAMES TO RASTER ATTRIBUTE TABLE
'  ' FIRST ADD NAME FIELD
'  Debug.Print "  --> Updating 10-Category Landform Table..."
'  Dim pTable As ITable
'  Set pTable = pTempRaster.AttributeTable
'  Dim pNameField As IField
'  Set pNameField = New Field
'  Dim pNameFieldEdit As IFieldEdit
'  Set pNameFieldEdit = pNameField
'  With pNameFieldEdit
'    .Name = "Class_Name"
'    .Type = esriFieldTypeString
'    .length = 125
'  End With
'  pTable.AddField pNameField
'  ' NEXT ADD CLASS VALUES TO NEW FIELD
'  Dim lngValue As Long
'  Dim lngValueIndex As Long
'  Dim lngNameIndex As Long
'
'  lngValueIndex = pTable.FindField("VALUE")
'  lngNameIndex = pTable.FindField("Class_Name")
'
'  Dim pCursor As ICursor
'  Dim pRow As IRow
'  Set pCursor = pTable.Update(Nothing, False)
'  Set pRow = pCursor.NextRow
'  Do Until pRow Is Nothing
'    lngValue = pRow.Value(lngValueIndex)
'    Select Case lngValue
'      Case 1
'        pRow.Value(lngNameIndex) = "Canyons, Deeply Incised Streams"
'      Case 2
'        pRow.Value(lngNameIndex) = "Midslope Drainages, Shallow Valleys"
'      Case 3
'        pRow.Value(lngNameIndex) = "Upland Drainages, Headwaters"
'      Case 4
'        pRow.Value(lngNameIndex) = "U-shaped Valleys"
'      Case 5
'        pRow.Value(lngNameIndex) = "Plains"
'      Case 6
'        pRow.Value(lngNameIndex) = "Open Slopes"
'      Case 7
'        pRow.Value(lngNameIndex) = "Upper Slopes, Mesas"
'      Case 8
'        pRow.Value(lngNameIndex) = "Local Ridges/Hills in Valleys"
'      Case 9
'        pRow.Value(lngNameIndex) = "Midslope Ridges, Small Hills in Plains"
'      Case 10
'        pRow.Value(lngNameIndex) = "Mountain Tops, High Ridges"
'      Case Else
'        pRow.Value(lngNameIndex) = "Unknown"
'    End Select
'    pCursor.UpdateRow pRow
'
'    Set pRow = pCursor.NextRow
'  Loop
'
'  ' ADD 6-CATEGORY SLOPE POSITION CLASSIFICATION RASTER TO MAP
'  Debug.Print "  --> Adding 10-Category Landform to map..."
'  Dim p10_CategoryLayer As IRasterLayer
'  Set p10_CategoryLayer = New RasterLayer
'  p10_CategoryLayer.CreateFromDataset pLandformIntegerDataset
'  p10_CategoryLayer.Name = "10-Category Landform from " & pDEMLayer.Name
'  pMxDoc.FocusMap.AddLayer p10_CategoryLayer
'
'  pMxDoc.UpdateContents
'  Dim pDocDirty As IDocumentDirty
'  Set pDocDirty = pMxDoc
'  pDocDirty.SetDirty
'
'  Debug.Print vbCrLf & "Done..."
'  Debug.Print
'
'  ' CLEAN UP CODE
'  Set pMxDoc = Nothing
'  Set pRastBandColl = Nothing
'  Set pRastBand = Nothing
'  Set pRastStats = Nothing
'  Set pApp = Nothing
'  Set pSBar = Nothing
'  Set pMakerOp = Nothing
'  Set pRastAnEnv = Nothing
'  Set pRastProps = Nothing
'  Set pSpRef = Nothing
'  Set pGeoDataset = Nothing
'  Set pMathOp = Nothing
'  Set pNbrHoodOp = Nothing
'  Set pConOp = Nothing
'  Set pLogOp = Nothing
'  Set pSupportOp = Nothing
'  Set pTempRaster = Nothing
'  Set pSmallNbrHood = Nothing
'  Set pLargeNbrHood = Nothing
'  Set pMeanDataset = Nothing
'  Set pSDDataset = Nothing
'  Set pDEMLayer = Nothing
'  Set pDEMRaster = Nothing
'  Set pDEMDataset = Nothing
''  Set pSmallTPIDataset = Nothing
'  Set pTempDataset = Nothing
'  Set pSmallStandTPIDataset = Nothing
''  Set pSmallTPILayer = Nothing
''  Set pLargeTPIDataset = Nothing
'  Set pLargeStandTPIDataset = Nothing
''  Set pLargeTPILayer = Nothing
'  Set pSlopeDataset = Nothing
''  Set pSurfaceOp = Nothing
''  Set pSlopeLayer = Nothing
'  Set pTPI_Negative_1_Small_Dataset = Nothing
'  Set pTPI_Positive_1_Small_Dataset = Nothing
'  Set pTPI_Negative_1_Large_Dataset = Nothing
'  Set pTPI_Positive_1_Large_Dataset = Nothing
'  Set pSlope_5_Dataset = Nothing
'  Set pZeroDataset = Nothing
'  Set p1_CanyonsDataset = Nothing
'  Set p2_MidslopeDrainagesDataset = Nothing
'  Set p3_UplandDataset = Nothing
'  Set p4_UShapedDataset = Nothing
'  Set p5_PlainsDataset = Nothing
'  Set p6_OpenDataset = Nothing
'  Set p7_UpperDataset = Nothing
'  Set p8_LocalRidgesDataset = Nothing
'  Set p9_MidslopeRidgesDataset = Nothing
'  Set p10_MountainTopsDataset = Nothing
'  Set pLandformDataset = Nothing
'  Set pLandformIntegerDataset = Nothing
'  Set pTable = Nothing
'  Set pNameField = Nothing
'  Set pNameFieldEdit = Nothing
'  Set pCursor = Nothing
'  Set pRow = Nothing
'  Set p10_CategoryLayer = Nothing
'  Set pDocDirty = Nothing
'
'End Sub
'
'
'Public Sub Create_Sample_4_Class_SlopeClass()
'
'  ' THIS SAMPLE FROM THE MANUAL CREATES A 4-CATEGORY SLOPE CLASSIFICATION ACCORDING TO THE
'  ' FOLLOWING CRITERIA
'
'  '  Canyon Bottom [Class Value = 4]: TPI <= -8
'  '   Gentle Slope [Class Value = 3]: -8 < TPI <= 8, Slope < 6
'  '    Steep Slope [Class Value = 2]: -8 < TPI <= 8, Slope >= 6
'  '      Ridgeline [Class Value = 1]: TPI >= 8
'
'  ' TPI thresholds are in elevation units.  Therefore we first calculate a TPI raster (equal to
'  ' [Elevation Value] - [Neighborhood Mean Elevation Value].  We use the TPI values to identify
'  ' which regions of the raster lie below the threshold.
'
'
'  ' DEFINING VARIABLES:  DO NOT WORRY ABOUT THIS PART =================================
'  Dim strDEMLayerName As String
'  Set pFrm4CategoryParams = Nothing
'
'  ' VARIABLES FOR YOU TO SET ==========================================================
'  ' NAME OF ORIGINAL DEM LAYER IN ARCMAP
''  strDEMLayerName = "dem"
'
''  ' NEIGHBORHOOD PARAMETERS; USED WHEN CALCULATING TPI AND STANDARDIZED TPI
''  ' SETTINGS BELOW SPECIFY A CIRCULAR NEIGHBORHOOD WTIH A RADIUS OF 5 CELLS
''  dblNeighborhoodRadius = 5  '..................RADIUS OF CIRCULAR NEIGHBORHOOD
''  lngNeighborhoodUnits = esriUnitsCells   '.....MEANS RADIUS IS DEFINED AS CELLS
'                                          '     CHANGE THIS TO esriUnitsMap IF YOU WANT TO DEFINE RADIUS
'                                          '        AS METERS OR FEET
'
'
'  ' DEFINE ADDITIONAL VARIABLES
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
'  Dim pRastBandColl As IRasterBandCollection
'  Dim pRastBand As IRasterBand
'  Dim pRastStats As IRasterStatistics
'  Dim pApp As IApplication
'  Set pApp = Application
'  Dim pSBar As IStatusBar
'  Set pSBar = pApp.StatusBar
'  Dim pMakerOp As IRasterMakerOp
'  Set pMakerOp = New RasterMakerOp
'  Dim pRastAnEnv As IRasterAnalysisEnvironment
'  Dim pRastProps As IRasterProps
'  Dim pSpRef As ISpatialReference
'  Dim pGeoDataset As IGeoDataset
'  Dim pMathOp As IMathOp
'  Set pMathOp = New RasterMathOps
'  Dim pNbrHoodOp As INeighborhoodOp
'  Set pNbrHoodOp = New RasterNeighborhoodOp
'  Dim pConOp As IConditionalOp
'  Set pConOp = New RasterConditionalOp
'  Dim pLogOp As ILogicalOp
'  Set pLogOp = New RasterMathOps
'  Dim pSupportOp As IMathSupportOp
'  Set pSupportOp = New RasterMathSupportOp
'  Dim pTempRaster As IRaster2
'  Dim pNbrHood As IRasterNeighborhood
'  Set pNbrHood = New RasterNeighborhood
'
'  ' GET PARAMETERS FROM DIALOG
'
'  Dim frmParams As frm4CategorySlope
'  Set frmParams = New frm4CategorySlope
'  Dim pDocArray As esriSystem.IVariantArray
'  Set pDocArray = New esriSystem.VarArray
'  pDocArray.Add pMxDoc
'  pDocArray.Add pApp
'  Set frmParams.ArcStuff = pDocArray
'  frmParams.Show vbModal
'
'  Set frmParams = Nothing
'  Set pDocArray = Nothing
'
'  If pFrm4CategoryParams Is Nothing Then
'    Set pMxDoc = Nothing
'    Set pApp = Nothing
'    Set pRastBandColl = Nothing
'    Set pRastBand = Nothing
'    Set pRastStats = Nothing
'    Set pSBar = Nothing
'    Set pMakerOp = Nothing
'    Set pRastAnEnv = Nothing
'    Set pRastProps = Nothing
'    Set pSpRef = Nothing
'    Set pGeoDataset = Nothing
'    Set pMathOp = Nothing
'    Set pNbrHoodOp = Nothing
'    Set pConOp = Nothing
'    Set pLogOp = Nothing
'    Set pSupportOp = Nothing
'    Set pTempRaster = Nothing
'    Set pNbrHood = Nothing
'
'    Exit Sub
'
'  End If
'  ' ANALYSIS ==========================================================================
'
'  ' FIND DEM LAYER IN MAP AND GET RASTER DATASET
'  Dim pDEMLayer As IRasterLayer
'  Set pDEMLayer = pFrm4CategoryParams.Element(0)
'  strDEMLayerName = pDEMLayer.Name
'
'  Dim strTPIVariation As String
'  strTPIVariation = pFrm4CategoryParams.Element(1)
'
'  Dim booAddIntermediate As Boolean
'  booAddIntermediate = pFrm4CategoryParams.Element(2)
'
'  Set pNbrHood = pFrm4CategoryParams.Element(3)
'
'  Dim dblThresholdA As Double
'  Dim dblThresholdB As Double
'  Dim dblThresholdSlope As Double
'
'  dblThresholdA = pFrm4CategoryParams.Element(4)
'  dblThresholdB = pFrm4CategoryParams.Element(5)
'  dblThresholdSlope = pFrm4CategoryParams.Element(6)
'  Set pFrm4CategoryParams = Nothing
'
'  ' ANALYSIS ==========================================================================
'
'  ' FIND DEM LAYER IN MAP AND GET RASTER DATASET
'  Dim pDEMRaster As IRaster2
'  Set pDEMRaster = pDEMLayer.Raster
'  Dim pDEMDataset As IRasterDataset
'  Set pDEMDataset = pDEMRaster.RasterDataset
'  Set pGeoDataset = pDEMDataset
'  Set pSpRef = pGeoDataset.SpatialReference
'  Set pRastProps = pDEMRaster
'  Set pRastAnEnv = pMakerOp
'  pRastAnEnv.SetCellSize esriRasterEnvValue, pDEMRaster
'  pRastAnEnv.SetExtent esriRasterEnvValue, pDEMRaster, pDEMRaster
'  Set pRastAnEnv.OutSpatialReference = pSpRef
'
'  ' CALCULATE TPI VARIATION:  EITHER STANDARDIZED DEM, SIMPLE TPI, OR STANDARDIZED TPI
'
'  Dim pTPIDataset As IRasterDataset
'  If strTPIVariation = "Standardized Elevation" Then
'    Set pTPIDataset = ReturnStandardizedElevationDataset(pDEMDataset, pNbrHoodOp, pSBar, _
'          pMakerOp, pNbrHood, strDEMLayerName, pMxDoc, booAddIntermediate)
'
'  ElseIf strTPIVariation = "Standardized TPI" Then
'    Set pTPIDataset = ReturnStandardizedTPIDataset(pDEMDataset, pNbrHood, "TPI", strDEMLayerName, _
'          pSBar, pNbrHoodOp, pMxDoc, pMakerOp, booAddIntermediate)
'
'  ElseIf strTPIVariation = "TPI" Then
'    Set pTPIDataset = ReturnTPIDataset(pDEMDataset, pSBar, strDEMLayerName, pNbrHoodOp, pNbrHood, pMathOp, _
'          pMxDoc, booAddIntermediate)
'
'  End If
'
'  Dim pSlopeDataset As IRasterDataset
'  Set pSlopeDataset = ReturnSlopeDataset(pDEMDataset, pSBar, strDEMLayerName, pMxDoc, booAddIntermediate)
'
'  ' COMBINE SLOPE AND TPI INTO 4-CATEGORY CLASSIFICATION RASTER
'  ' FIRST CREATE CONSTANT-VALUE RASTERS FOR THE 4 CATEGORIES
'  pSBar.Message(0) = "Calculating 4-Category Topographic Position Classification Raster..."
'  Debug.Print vbCrLf & "Calculating 4-Category Topographic Position Classification Raster..."
'  Dim pCanyonBottomDataset As IRasterDataset
'  Dim pGentleSlopeDataset As IRasterDataset
'  Dim pSteepSlopeDataset As IRasterDataset
'  Dim pRidgelineDataset As IRasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(4, True)
'  Set pCanyonBottomDataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(3, True)
'  Set pGentleSlopeDataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(2, True)
'  Set pSteepSlopeDataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(1, True)
'  Set pRidgelineDataset = pTempRaster.RasterDataset
'
'  ' NEXT MAKE CONSTANT-VALUE RASTERS FOR THE VARIOUS TPI AND SLOPE THRESHOLDS
'  ' --> TPI THRESHOLDS AT -8, 8
'  ' --> SLOPE THRESHOLD AT 6
'
'
'  Dim pTPI_Negative_8_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = -8
'  Dim pTPI_Positive_8_Dataset As IRasterDataset     ' FOR TPI THRESHOLD = 8
'  Dim pSlope_6_Dataset As IRasterDataset            ' FOR SLOPE THRESHOLD = 6
'
'  Set pTempRaster = pMakerOp.MakeConstant(dblThresholdA, False)
'  Set pTPI_Negative_8_Dataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(dblThresholdB, False)
'  Set pTPI_Positive_8_Dataset = pTempRaster.RasterDataset
'
'  Set pTempRaster = pMakerOp.MakeConstant(dblThresholdSlope, False)
'  Set pSlope_6_Dataset = pTempRaster.RasterDataset
'
'  Dim pSlopePositionDataset As IRasterDataset
'
'  Set pTempRaster = _
'      pConOp.Con(pLogOp.LessThanEqual(pTPIDataset, pTPI_Negative_8_Dataset), pCanyonBottomDataset, _
'      pConOp.Con(pLogOp.LessThanEqual(pTPIDataset, pTPI_Positive_8_Dataset), _
'                 pConOp.Con(pLogOp.LessThanEqual(pSlopeDataset, pSlope_6_Dataset), pGentleSlopeDataset, pSteepSlopeDataset), _
'                      pRidgelineDataset))
'
'  Set pSlopePositionDataset = pTempRaster.RasterDataset
'  Set pRastBandColl = pSlopePositionDataset
'  Set pRastBand = pRastBandColl.Item(0)
'  pRastBand.ComputeStatsAndHist
'
'  ' CONVERT TO INTEGER RASTER
'  Dim pSlopePositionIntegerDataset As IRasterDataset
'  Set pTempRaster = pSupportOp.Int(pSlopePositionDataset)
'  Set pSlopePositionIntegerDataset = pTempRaster.RasterDataset
'  Set pRastBandColl = pSlopePositionIntegerDataset
'  Set pRastBand = pRastBandColl.Item(0)
'  pRastBand.ComputeStatsAndHist
'
'  ' ADD CORRECT CLASS NAMES TO RASTER ATTRIBUTE TABLE
'  ' FIRST ADD NAME FIELD
'  Debug.Print "  --> Updating 4-Category Topographic Position Table..."
'  Dim pTable As ITable
'  Set pTable = pTempRaster.AttributeTable
'  Dim pNameField As IField
'  Set pNameField = New Field
'  Dim pNameFieldEdit As IFieldEdit
'  Set pNameFieldEdit = pNameField
'  With pNameFieldEdit
'    .Name = "Class_Name"
'    .Type = esriFieldTypeString
'    .length = 25
'  End With
'  pTable.AddField pNameField
'  ' NEXT ADD CLASS VALUES TO NEW FIELD
'  Dim lngValue As Long
'  Dim lngValueIndex As Long
'  Dim lngNameIndex As Long
'
'  lngValueIndex = pTable.FindField("VALUE")
'  lngNameIndex = pTable.FindField("Class_Name")
'
'  Dim pCursor As ICursor
'  Dim pRow As IRow
'  Set pCursor = pTable.Update(Nothing, False)
'  Set pRow = pCursor.NextRow
'  Do Until pRow Is Nothing
'    lngValue = pRow.Value(lngValueIndex)
'    Select Case lngValue
'      Case 1
'        pRow.Value(lngNameIndex) = "Ridgeline"
'      Case 2
'        pRow.Value(lngNameIndex) = "Steep Slope"
'      Case 3
'        pRow.Value(lngNameIndex) = "Gentle Slope"
'      Case 4
'        pRow.Value(lngNameIndex) = "Canyon Bottom"
'      Case Else
'        pRow.Value(lngNameIndex) = "Unknown"
'    End Select
'    pCursor.UpdateRow pRow
'
'    Set pRow = pCursor.NextRow
'  Loop
'
'  ' ADD 6-CATEGORY SLOPE POSITION CLASSIFICATION RASTER TO MAP
'  Debug.Print "  --> Adding 4-Category Topographic Position to map..."
'  Dim p4_CategoryLayer As IRasterLayer
'  Set p4_CategoryLayer = New RasterLayer
'  p4_CategoryLayer.CreateFromDataset pSlopePositionIntegerDataset
'  p4_CategoryLayer.Name = "4-Category Topographic Position from " & pDEMLayer.Name
'  pMxDoc.FocusMap.AddLayer p4_CategoryLayer
'
'  pMxDoc.UpdateContents
'  Dim pDocDirty As IDocumentDirty
'  Set pDocDirty = pMxDoc
'  pDocDirty.SetDirty
'
'  Debug.Print vbCrLf & "Done..."
'  Debug.Print
'
'  ' CLEAN UP CODE
'  Set pMxDoc = Nothing
'  Set pRastBandColl = Nothing
'  Set pRastBand = Nothing
'  Set pRastStats = Nothing
'  Set pApp = Nothing
'  Set pSBar = Nothing
'  Set pMakerOp = Nothing
'  Set pRastAnEnv = Nothing
'  Set pRastProps = Nothing
'  Set pSpRef = Nothing
'  Set pGeoDataset = Nothing
'  Set pMathOp = Nothing
'  Set pNbrHoodOp = Nothing
'  Set pConOp = Nothing
'  Set pLogOp = Nothing
'  Set pSupportOp = Nothing
'  Set pTempRaster = Nothing
'  Set pNbrHood = Nothing
'  Set pDEMLayer = Nothing
'  Set pDEMRaster = Nothing
'  Set pDEMDataset = Nothing
'  Set pTPIDataset = Nothing
''  Set pMeanDataset = Nothing
''  Set pTempDataset = Nothing
''  Set pTPILayer = Nothing
'  Set pSlopeDataset = Nothing
''  Set pSurfaceOp = Nothing
''  Set pSlopeLayer = Nothing
'  Set pCanyonBottomDataset = Nothing
'  Set pGentleSlopeDataset = Nothing
'  Set pSteepSlopeDataset = Nothing
'  Set pRidgelineDataset = Nothing
'  Set pTPI_Negative_8_Dataset = Nothing
'  Set pTPI_Positive_8_Dataset = Nothing
'  Set pSlope_6_Dataset = Nothing
'  Set pSlopePositionDataset = Nothing
'  Set pSlopePositionIntegerDataset = Nothing
'  Set pTable = Nothing
'  Set pNameField = Nothing
'  Set pNameFieldEdit = Nothing
'  Set pCursor = Nothing
'  Set pRow = Nothing
'  Set p4_CategoryLayer = Nothing
'  Set pDocDirty = Nothing
'
'End Sub
'
'
'
Public Function ReturnSlopeDataset(pDEMDataset As IRasterDataset, psbar As IStatusBar, _
  strDEMLayerName As String, pMxDoc As IMxDocument, _
  Optional booAddIntermediate As Boolean = True) As IRasterDataset
  On Error GoTo ErrorHandler



  ' CALCULATE SLOPE FROM DEM
  ' THIS ASSUMES THAT DEM IS PROJECTED, AND THAT ELEVATION UNITS ARE THE SAME AS X/Y UNITS OF SPATIAL REFERENCE
2870:   psbar.Message(0) = "Calculating Slope from " & strDEMLayerName & "..."
2871:   Debug.Print vbCrLf & "Calculating Slope from " & strDEMLayerName & "..."
  Dim pSlopeDataset As IRasterDataset
  Dim pSurfaceOp As ISurfaceOp
  Dim pTempRaster As IRaster2
  Dim pRastBandColl As IRasterBandCollection
  Dim pRastBand As IRasterBand

2878:   Set pSurfaceOp = New RasterSurfaceOp
2879:   Set pTempRaster = pSurfaceOp.Slope(pDEMDataset, esriGeoAnalysisSlopeDegrees)
2880:   Set pSlopeDataset = pTempRaster.RasterDataset
2881:   Set pRastBandColl = pSlopeDataset
2882:   Set pRastBand = pRastBandColl.Item(0)
2883:   pRastBand.ComputeStatsAndHist

  ' ADD SLOPE RASTER TO MAP; COMMENT THIS OUT IF YOU DO NOT WANT TO ADD NEW LAYER
  Dim pSlopeLayer As IRasterLayer
2887:   If booAddIntermediate Then
2888:     Set pSlopeLayer = New RasterLayer
2889:     pSlopeLayer.CreateFromDataset pSlopeDataset
2890:     Debug.Print "  --> Adding Slope Layer to map..."
2891:     pSlopeLayer.Name = "Slope from " & strDEMLayerName
2892:     pMxDoc.FocusMap.AddLayer pSlopeLayer
2893:   End If

2895:   Set ReturnSlopeDataset = pSlopeDataset

2897:   GoTo CleanUp
ErrorHandler:
  HandleError True, "ReturnSlopeDataset " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

CleanUp:

2903:   Set pSlopeDataset = Nothing
2904:   Set pSurfaceOp = Nothing
2905:   Set pTempRaster = Nothing
2906:   Set pRastBandColl = Nothing
2907:   Set pRastBand = Nothing
2908:   Set pSlopeLayer = Nothing


End Function

Public Function ReturnStandardizedElevationDataset(pDEMDataset As IRasterDataset, _
    pNbrHoodOp As INeighborhoodOp, psbar As IStatusBar, pMakerOp As IRasterMakerOp, _
    pNbrHood As IRasterNeighborhood, strDEMLayerName As String, pMxDoc As IMxDocument, _
    booExcludeIfNull As Boolean, booAddIntermediate As Boolean, _
    Optional strInsertInName As String = " ") As IRasterDataset
  On Error GoTo ErrorHandler


  ' CONVERT ELEVATION RASTER TO STANDARDIZED [BY NEIGHBORHOOD] ELEVATION RASTER --------------------------------------
  ' NOTE THAT THIS IS STANDARDIZED ACCORDING TO THE MEAN AND STANDARD DEVIATION OF THE NEIGHBORHOOD,
  ' RATHER THAN THE MEAN AND STANDARD DEVIATION OF THE ENTIRE DEM.
2924:   psbar.Message(0) = "Calculating" & strInsertInName & "Standardized [by neighborhood] Elevation from " & strDEMLayerName & "..."
2925:   Debug.Print vbCrLf & "Calculating" & strInsertInName & "Standardized [by neighborhood] Elevation from " & strDEMLayerName & "..."
  Dim pStandardizedElevDataset As IRasterDataset
  Dim pSDDataset As IRasterDataset
  Dim pMeanDataset As IRasterDataset
  Dim pTempRaster As IRaster2
  Dim pRastBandColl As IRasterBandCollection
  Dim pRastBand As IRasterBand
  Dim pLogOp As ILogicalOp
2933:   Set pLogOp = New RasterMathOps
  Dim pMathOp As IMathOp
2935:   Set pMathOp = New RasterMathOps
  Dim pConOp As IConditionalOp
2937:   Set pConOp = New RasterConditionalOp

2939:   Debug.Print "  --> Calculating" & strInsertInName & "Focal Standard Deviation from " & strDEMLayerName & "..."
2940:   Set pTempRaster = pNbrHoodOp.FocalStatistics(pDEMDataset, esriGeoAnalysisStatsStd, pNbrHood, True)
2941:   Set pSDDataset = pTempRaster.RasterDataset
2942:   Debug.Print "  --> Calculating" & strInsertInName & "Focal Mean from " & strDEMLayerName & "..."
2943:   Set pTempRaster = pNbrHoodOp.FocalStatistics(pDEMDataset, esriGeoAnalysisStatsMean, pNbrHood, True)
2944:   Set pMeanDataset = pTempRaster.RasterDataset
2945:   Debug.Print "  --> Calculating" & strInsertInName & "Standardized Elevation from " & strDEMLayerName & "..."

  ' IF STANDARD DEVIATION = 0, THEN SET STANDARDIZED ELEVATION EQUAL TO ZERO, TOO.  OTHERWISE IT GETS SET AS NODATA.
  Dim pForceZeroDataset As IRasterDataset
2949:   Set pTempRaster = pMakerOp.MakeConstant(0, True)
2950:   Set pForceZeroDataset = pTempRaster.RasterDataset

2952:   Set pTempRaster = pConOp.Con(pLogOp.EqualTo(pSDDataset, pForceZeroDataset), pForceZeroDataset, _
        pMathOp.Divide(pMathOp.Minus(pDEMDataset, pMeanDataset), pSDDataset))
2954:   Set pStandardizedElevDataset = pTempRaster.RasterDataset
2955:   Set pRastBandColl = pStandardizedElevDataset
2956:   Set pRastBand = pRastBandColl.Item(0)
2957:   pRastBand.ComputeStatsAndHist

  ' ADD NEIGHBORHOOD-STANDARDIZED ELEVATION RASTER TO MAP; COMMENT THIS OUT IF YOU DO NOT WANT TO ADD NEW LAYER
  Dim pStandElevLayer As IRasterLayer
2961:   If booAddIntermediate Then
2962:     Set pStandElevLayer = New RasterLayer
2963:     pStandElevLayer.CreateFromDataset pStandardizedElevDataset
2964:     Debug.Print "  --> Adding" & strInsertInName & "Standardized [by Neighborhood] Elevation Layer to map..."
2965:     pStandElevLayer.Name = Trim(strInsertInName & "Standardized DEM from " & strDEMLayerName)
2966:     pMxDoc.FocusMap.AddLayer pStandElevLayer
2967:   End If


2970:   Set ReturnStandardizedElevationDataset = pStandardizedElevDataset


2973:   GoTo CleanUp
ErrorHandler:
  HandleError True, "ReturnStandardizedElevationDataset " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

CleanUp:

2979:   Set pStandardizedElevDataset = Nothing
2980:   Set pSDDataset = Nothing
2981:   Set pMeanDataset = Nothing
2982:   Set pTempRaster = Nothing
2983:   Set pRastBandColl = Nothing
2984:   Set pRastBand = Nothing
2985:   Set pLogOp = Nothing
2986:   Set pMathOp = Nothing
2987:   Set pConOp = Nothing
2988:   Set pForceZeroDataset = Nothing
2989:   Set pStandElevLayer = Nothing

End Function

Public Function ReturnStandardizedTPIDataset(pDEMDataset As IRasterDataset, pNbrHood As IRasterNeighborhood, _
    strName As String, strDEMLayerName As String, psbar As IStatusBar, pNbrHoodOp As INeighborhoodOp, _
    pMxDoc As IMxDocument, pMakerOp As IRasterMakerOp, booExcludeIfNull As Boolean, _
    booAddIntermediate As Boolean) As IRasterDataset
  On Error GoTo ErrorHandler


  Dim pRastBandColl As IRasterBandCollection
  Dim pRastBand As IRasterBand
  Dim pRastStats As IRasterStatistics
  Dim pTempRaster As IRaster2
  Dim pMathOp As IMathOp
3005:   Set pMathOp = New RasterMathOps
  Dim dblMean As Double
  Dim dblSD As Double
  Dim pMeanDataset As IRasterDataset
  Dim pSDDataset As IRasterDataset

  ' CONVERT ELEVATION RASTER TO SMALL-NEIGHBORHOOD TPI RASTER --------------------------------------
3012:   psbar.Message(0) = "Calculating " & strName & " from " & strDEMLayerName & "..."
3013:   Debug.Print vbCrLf & "Calculating " & strName & " from " & strDEMLayerName & "..."
  Dim pTPIDataset As IRasterDataset

  ' START BY CALCULATING FOCAL MEAN USING NEIGHBORHOOD DEFINED ABOVE
3017:   Debug.Print "  --> Calculating " & strName & " Focal Mean from " & strDEMLayerName & "..."
  Dim pTempDataset As IRasterDataset
3019:   Set pTempRaster = pNbrHoodOp.FocalStatistics(pDEMDataset, esriGeoAnalysisStatsMean, pNbrHood, True)
3020:   Set pTempDataset = pTempRaster.RasterDataset

  ' TPI RASTER EQUALS ORGINAL RASTER MINUS FOCAL MEAN
3023:   Debug.Print "  --> Subracting " & strName & " Focal Mean from " & strDEMLayerName & "..."
3024:   Set pTempRaster = pMathOp.Minus(pDEMDataset, pTempDataset)
3025:   Set pTPIDataset = pTempRaster.RasterDataset
3026:   Set pRastBandColl = pTPIDataset
3027:   Set pRastBand = pRastBandColl.Item(0)
3028:   pRastBand.ComputeStatsAndHist

  ' STANDARDIZE SMALL-NEIGHBORHOOD TPI RASTER
3031:   Debug.Print "  --> Standardizing " & strName & "..."
3032:   Set pRastStats = pRastBand.Statistics
3033:   dblMean = pRastStats.Mean
3034:   dblSD = pRastStats.StandardDeviation
3035:   Set pTempRaster = pMakerOp.MakeConstant(dblMean, False)
3036:   Set pMeanDataset = pTempRaster.RasterDataset
3037:   Set pTempRaster = pMakerOp.MakeConstant(dblSD, False)
3038:   Set pSDDataset = pTempRaster.RasterDataset
  Dim pStandTPIDataset As IRasterDataset
3040:   Set pTempRaster = pMathOp.Divide(pMathOp.Minus(pTPIDataset, pMeanDataset), pSDDataset)
3041:   Set pStandTPIDataset = pTempRaster.RasterDataset
3042:   Set pRastBandColl = pStandTPIDataset
3043:   Set pRastBand = pRastBandColl.Item(0)
3044:   pRastBand.ComputeStatsAndHist

  ' ADD STANDARDIZED SMALL-NEIGHBORHOOD TPI RASTER TO MAP; COMMENT THIS OUT IF YOU DO NOT WANT TO ADD NEW LAYER
  Dim pTPILayer As IRasterLayer
3048:   If booAddIntermediate Then
3049:     Set pTPILayer = New RasterLayer
3050:     pTPILayer.CreateFromDataset pStandTPIDataset
3051:     Debug.Print "  --> Adding Standardized " & strName & " Layer to map..."
3052:     pTPILayer.Name = "Standardized " & strName & " from " & strDEMLayerName
3053:     pMxDoc.FocusMap.AddLayer pTPILayer
3054:   End If

3056:   Set ReturnStandardizedTPIDataset = pStandTPIDataset

3058:   GoTo CleanUp
ErrorHandler:
  HandleError True, "ReturnStandardizedTPIDataset " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

CleanUp:

3064:   Set pRastBandColl = Nothing
3065:   Set pRastBand = Nothing
3066:   Set pRastStats = Nothing
3067:   Set pTempRaster = Nothing
3068:   Set pMathOp = Nothing
3069:   Set pMeanDataset = Nothing
3070:   Set pSDDataset = Nothing
3071:   Set pTPIDataset = Nothing
3072:   Set pTempDataset = Nothing
3073:   Set pStandTPIDataset = Nothing
3074:   Set pTPILayer = Nothing

End Function

Public Function ReturnTPIDataset(pDEMDataset As IRasterDataset, psbar As IStatusBar, _
    strDEMLayerName As String, pNbrHoodOp As INeighborhoodOp, pNbrHood As IRasterNeighborhood, _
    pMathOp As IMathOp, pMxDoc As IMxDocument, booAddIntermediate As Boolean, booExcludeIfNull As Boolean, _
    Optional strInsertInName As String = " ") As IRasterDataset
  On Error GoTo ErrorHandler


  Dim pTempRaster As IRaster2
  Dim pRastBandColl As IRasterBandCollection
  Dim pRastBand As IRasterBand

  ' CONVERT ELEVATION RASTER TO TPI RASTER --------------------------------------
3090:   psbar.Message(0) = "Calculating" & strInsertInName & "TPI from " & strDEMLayerName & "..."
3091:   Debug.Print vbCrLf & "Calculating" & strInsertInName & "TPI from " & strDEMLayerName & "..."
  Dim pTPIDataset As IRasterDataset
  Dim pMeanDataset As IRasterDataset

  ' START BY CALCULATING FOCAL MEAN USING NEIGHBORHOOD DEFINED ABOVE
3096:   Debug.Print "  --> Calculating" & strInsertInName & "Focal Mean from " & strDEMLayerName & "..."
  Dim pTempDataset As IRasterDataset
3098:   Set pTempRaster = pNbrHoodOp.FocalStatistics(pDEMDataset, esriGeoAnalysisStatsMean, pNbrHood, Not booExcludeIfNull)
3099:   Set pTempDataset = pTempRaster.RasterDataset
  
  ' TPI RASTER EQUALS ORGINAL RASTER MINUS FOCAL MEAN
3102:   Debug.Print "  --> Subracting" & strInsertInName & "Focal Mean from " & strDEMLayerName & "..."
3103:   Set pTempRaster = pMathOp.Minus(pDEMDataset, pTempDataset)
3104:   Set pTPIDataset = pTempRaster.RasterDataset
3105:   Set pRastBandColl = pTPIDataset
3106:   Set pRastBand = pRastBandColl.Item(0)
3107:   pRastBand.ComputeStatsAndHist

  ' ADD TPI RASTER TO MAP; COMMENT THIS OUT IF YOU DO NOT WANT TO ADD NEW LAYER
  Dim pTPILayer As IRasterLayer
3111:   If booAddIntermediate Then
3112:     Set pTPILayer = New RasterLayer
3113:     Debug.Print "  --> Adding" & strInsertInName & "TPI Layer to map..."
3114:     pTPILayer.CreateFromDataset pTPIDataset
3115:     pTPILayer.Name = Trim(strInsertInName & "TPI from " & strDEMLayerName)
3116:     pMxDoc.FocusMap.AddLayer pTPILayer
3117:   End If

3119:   Set ReturnTPIDataset = pTPIDataset

3121:   GoTo CleanUp
ErrorHandler:
  HandleError True, "ReturnTPIDataset " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

CleanUp:

3127:   Set pTempRaster = Nothing
3128:   Set pRastBandColl = Nothing
3129:   Set pRastBand = Nothing
3130:   Set pTPIDataset = Nothing
3131:   Set pMeanDataset = Nothing
3132:   Set pTempDataset = Nothing
3133:   Set pTPILayer = Nothing


End Function



