Attribute VB_Name = "aml_func_mod"
'             Environmental Systems Research Institute, Inc.
'Module Name: aml_func.bas
'Description: Used to perform AML-like functions for string manipipulation.
'             Also parses out tokens in a string to elements in an array.
'
'   Requires: ParseString and ParseStringR require that you Dim the named array arg in
'             the calling propgram: Must be a zero dimensioned string array:
'             Dim yourarray() as string

'    Methods: After   - Returns the substring to the right of the leftmost occurrence of
'                       searchStr
'             Before  - Returns the substring to the left of the leftmost occurrence of
'                       searchStr.
'        ExistFileDir - Returns True/False if file or Directory exists.
'             Extract - Returns an element from a list of elements
'             Index   - Returns the position of the leftmost occurrence of a specified
'                       string in a target string.
'             Keyword - Returns the position of a keyword within a list of keywords.
'             Search  - Returns the position of the first character of a search string
'                       in a target string.
'             Sort    - Returns a sorted a list of elements.
'             Subst   - Returns a string that has had one string subsituted for another
'             Substr  - Returns a substring that starts at a specified character position.
'             Token   - Allows tokens in a list to be manipulated.
'
'                       Count - the number of tokens in a list
'                       Find  - the position of a token in a list
'                       Move  - tokens in a list
'                       Insert- a new token into a list
'                       Delete- a token in a list
'                       Replace-one token in a list for another
'                       Switch -one token in a list for another
'
'        ParseString  - Populates string array with tokens in a string.
'        ParseStringR - Same as ParseString except blanks and commas are
'                       treated as delimiters.
'
'    History: DMA     - 03/04/97 - Original coding
'       Glenn Meister - 12/19/97 - Added ExistFileDir function
'
'   MODIFICATIONS:  JEFF JENNESS
'   GetFullFileString - ' CONVERTS 8.3 FILESTRING TO FULL TEXT
'           ReturnDir - GIVEN A STRING, RETURNS TEXT PRECEDING LAST "\" CHARACTER
'      ReturnFilename - GIVEN A STRING, RETURNS TEXT FOLLOWING LAST "\" CHARACTER
'         ReturnFiles - GIVEN A STRING PATH, RETURNS COLLECTION CONTAINING THE NUMBER OF FILES THAT MET CRITERIA
'                       AND ARRAY OF STRING FILENAMES.  OPTIONS FOR INCLUDE READ-ONLY, SYSTEM, HIDDEN FILES.
'       ReturnFolders - GIVENA STRING PATH, RETURNS COLLECTION CONTAINING THE NUMBER OF FOLDERS THAT MET CRITERIA
'                       AND ARRAY OF FOLDER FILENAMES.
'       ClipExtension - GIVEN A STRING, RETURNS TEXT PRECEDING LAST "." CHARACTER
'        SetExtension - GIVEN A STRING AND EXTENSION, RETURNS TEXT PRECEDING LAST "." CHARACTER, PLUS "." AND NEW EXTENSION
'    GetExtensionText - GIVEN A STRING, RETURNS TEXT FOLLOWING LAST "." CHARACTER
'      FieldIsNumeric - GIVEN A FIELD, RETURNS BOOLEAN
'       FieldIsString - GIVEN A FIELD, RETURNS BOOLEAN
'         FieldIsDate - GIVEN A FIELD, RETURNS BOOLEAN
'        FieldIsShape - GIVEN A FIELD, RETURNS BOOLEAN
'        InsertCommas - GIVEN A NUMBER OR STRING, INSERTS COMMAS AND RETURNS STRING
'     BasicTrimAvenue - GIVEN A STRING, RETURNS A STRING WITH VALUES TRIMMED OFF EACH SIDE
'  MakeUniqueFilename - GIVEN A STRING REPRESENTING A FILEPATH, RETURNS A UNIQUE FILENAME WHICH MAY HAVE NUMBERS APPENDED TO IT
'      GetTheUserName - JUST RETURNS THE USER NAME; CAN BE USED TO FIND THE "MY DOCUMENTS" FOLDER
'    TempPathLocation - RETURNS PATHNAME TO TEMP DIRECTORY
'     CreateShapefile - GIVEN PATHNAME AND SHAPE TYPE, RETURNS A pFeatureClass.  ESRI CODE, MODIFIED BY JENNESS
'    CreateShapefile2 - GIVEN PATHNAME, SHAPE TYPE AND ARRAY OF FIELDS, RETURNS A pFeatureClass.
'    CreatedBASETable - GIVEN PATHNAME AND OPTIONAL FIELDS, RETURNS ITABLE
'        GetMxDocPath - GIVEN APPLICATION OBJECT, RETURNS PATHNAME OF MAP DOCUMENT
'         QuoteString - GIVEN A STRING, RETURNS A QUOTED VERSION OF THAT STRING
'    SubstituteString - GIVEN A STRING, A SEARCH STRING AND A SUBSTITUTE STRING, REPLACES ALL INSTANCES OF SEARCH TEXT WITH
'                       SUBSTITUTE TEXT IN THE ORIGINAL STRING
'ReturnArcGISInstallDir - RETURNS A STRING CONTAINING THE ARCGIS INSTALL LOCATION
'ReturnArcGISGeneralDir - RETURNS DIRECTORY STRING FOR INSTALL, LAST LOCATION, LAST BROWSED, LAST EXPORT AND LAST SAVE TO
' ReturnArcGISVersion - RETURNS ARCGIS VERSION NUMBER
'ReturnArcGISVersionAlt - MORE DIRECT METHOD OF GETTING ARCGIS VERSION; RETURNS A LONG WHERE 0 = 8.3, 1 = 9.0, 2 = 9.2, 3 = 9.3
'          FileExists - GIVEN A STRING PATH, RETURNS TRUE OR FALSE
' ReturnShapeTypeName - GIVEN A SHAPE TYPE ENUMERATION, RETURNS SHAPE NAME
' QueryForNewFilename - GIVEN A SAMPLE FILENAME, OPENS DIALOG AND QUERIES USER TO SPECIFY NEW FILENAME

Option Explicit

' FILENAME 8.3 FORMAT CONVERSION BELOW COPIED FROM http://forums.esri.com/Thread.asp?c=93&f=993&t=123512&mc=1#msgid400360
'   POSTED BY  Brett N.Meroney
'              GIS Programmer / Analyst
'              Integrated Laboratory Systems, Inc.
'              contractor to US-EPA Region VIII
'              Golden , CO
' NECESSARY TO CONVERT 8.3 FILENAMES TO USEABLE FILENAMES

Private Const strMAXPATH = 260
Private Const m_Quotation = """"

'
' Win32 Registry functions
'
Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" _
    (ByVal hKey As Long, _
     ByVal lpSubKey As String, _
     ByVal ulOptions As Long, _
     ByVal samDesired As Long, _
     phkResult As Long) _
     As Long
Private Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" _
    (ByVal hKey As Long, _
     ByVal lpSubKey As String, _
     ByVal Reserved As Long, _
     ByVal lpClass As String, _
     ByVal dwOptions As Long, _
     ByVal samDesired As Long, _
     lpSecurityAttributes As Any, _
     phkResult As Long, _
     lpdwDisposition As Long) _
     As Long
Private Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" _
     (ByVal hKey As Long, _
     ByVal lpValueName As String, _
     ByVal lpReserved As Long, _
     lpType As Long, _
     lpData As Any, _
     lpcbData As Long) _
     As Long         ' Note that if you declare the lpData parameter as String, you must pass it By Value.
Private Declare Function RegQueryInfoKey Lib "advapi32.dll" Alias "RegQueryInfoKeyA" _
     (ByVal hKey As Long, _
     ByVal lpClass As String, _
     lpcbClass As Long, _
     lpReserved As Long, _
     lpcSubKeys As Long, _
     lpcbMaxSubKeyLen As Long, _
     lpcbMaxClassLen As Long, _
     lpcValues As Long, _
     lpcbMaxValueNameLen As Long, _
     lpcbMaxValueLen As Long, _
     lpcbSecurityDescriptor As Long, _
     lpftLastWriteTime As Any) _
     As Long
Private Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" _
     (ByVal hKey As Long, _
     ByVal dwIndex As Long, _
     ByVal lpName As String, _
     lpcbName As Long, _
     lpReserved As Long, _
     ByVal lpClass As String, _
     lpcbClass As Long, _
     lpftLastWriteTime As Any) _
     As Long
Private Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" _
     (ByVal hKey As Long, _
      ByVal dwIndex As Long, _
     ByVal lpValueName As String, _
     lpcbValueName As Long, _
     lpReserved As Long, _
     lpType As Long, _
     lpData As Any, _
     lpcbData As Long) _
     As Long
Private Declare Function RegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" _
     (ByVal hKey As Long, _
     ByVal lpValueName As String, _
     ByVal Reserved As Long, _
     ByVal dwType As Long, _
     lpData As Any, _
     ByVal cbData As Long) _
     As Long         ' Note that if you declare the lpData parameter as String, you must pass it By Value.
Private Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" _
     (ByVal hKey As Long, _
      ByVal lpSubKey As String) _
     As Long
Private Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" _
     (ByVal hKey As Long, _
     ByVal lpValueName As String) _
     As Long
Private Declare Function RegCloseKey Lib "advapi32.dll" _
     (ByVal hKey As Long) _
     As Long
'
' Constants for Windows 32-bit Registry API
'
Private Const HKEY_CLASSES_ROOT = &H80000000
Private Const HKEY_CURRENT_USER = &H80000001
Private Const HKEY_LOCAL_MACHINE = &H80000002
Private Const HKEY_USERS = &H80000003
Private Const HKEY_PERFORMANCE_DATA = &H80000004
Private Const HKEY_CURRENT_CONFIG = &H80000005
Private Const HKEY_DYN_DATA = &H80000006
'
' Reg result codes
'
Private Const REG_CREATED_NEW_KEY = &H1                      ' New Registry Key created
Private Const REG_OPENED_EXISTING_KEY = &H2                      ' Existing Key opened
'
' Reg Create Type Values...
'
Private Const REG_OPTION_RESERVED = 0           ' Parameter is reserved
Private Const REG_OPTION_NON_VOLATILE = 0       ' Key is preserved when system is rebooted
Private Const REG_OPTION_VOLATILE = 1           ' Key is not preserved when system is rebooted
Private Const REG_OPTION_CREATE_LINK = 2        ' Created key is a symbolic link
Private Const REG_OPTION_BACKUP_RESTORE = 4     ' open for backup or restore
'
' Reg Key Security Options
'
Private Const DELETE = &H10000
Private Const READ_CONTROL = &H20000
Private Const WRITE_DAC = &H40000
Private Const WRITE_OWNER = &H80000
Private Const SYNCHRONIZE = &H100000
Private Const STANDARD_RIGHTS_READ = (READ_CONTROL)
Private Const STANDARD_RIGHTS_WRITE = (READ_CONTROL)
Private Const STANDARD_RIGHTS_EXECUTE = (READ_CONTROL)
Private Const STANDARD_RIGHTS_REQUIRED = &HF0000
Private Const STANDARD_RIGHTS_ALL = &H1F0000
Private Const SPECIFIC_RIGHTS_ALL = &HFFFF
Private Const KEY_QUERY_VALUE = &H1
Private Const KEY_SET_VALUE = &H2
Private Const KEY_CREATE_SUB_KEY = &H4
Private Const KEY_ENUMERATE_SUB_KEYS = &H8
Private Const KEY_NOTIFY = &H10
Private Const KEY_CREATE_LINK = &H20
Private Const KEY_READ = ((STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY) And (Not SYNCHRONIZE))
Private Const KEY_WRITE = ((STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY) And (Not SYNCHRONIZE))
Private Const KEY_ALL_ACCESS = ((STANDARD_RIGHTS_ALL Or KEY_QUERY_VALUE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY Or KEY_CREATE_LINK) And (Not SYNCHRONIZE))
Private Const KEY_EXECUTE = ((KEY_READ) And (Not SYNCHRONIZE))

Private Const ERROR_SUCCESS = 0&
Private Const ERROR_MORE_DATA = 234
Private Const ERROR_NO_MORE_ITEMS = 259

Private Const REG_SZ = 1                         ' Unicode nul terminated string
'

Private Declare Function GetLongPathName Lib "Kernel32" Alias _
    "GetLongPathNameA" (ByVal lpszShortPath As String, _
    ByVal lpszLongPath As String, ByVal cchBuffer As Long) _
    As Long

Public Enum enumArcGISFolderTypes
   enumLastBrowsedLocation
   enumLastExportToLocation
   enumLastLocation
   enumLastSaveToLocation
   enumArcGISInstallLocation
End Enum

Private Declare Function GetUserName Lib "advapi32.dll" Alias "GetUserNameA" (ByVal lpBuffer As String, nSize As Long) As Long
Private Declare Function GetTempPath Lib "Kernel32" Alias "GetTempPathA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
Const Quotation = """"
Const c_sModuleFileName As String = "D:\arcGIS_stuff\consultation\brost_climate_change\code\aml_func_mod.bas"

Public Function ReturnArcGISVersionAlt(pMxDoc As IMxDocument) As Long
  On Error GoTo ErrorHandler


 '  ReturnArcGISVersionAlt = "Unable to determine ArcGIS Version!"
   
'   Dim pDocVersion As IDocumentVersion
'   Set pDocVersion = pMxDoc
'
'   ReturnArcGISVersionAlt = pDocVersion.DocumentVersion

255:   ReturnArcGISVersionAlt = 0
  Dim strVersion As String
257:   strVersion = ReturnArcGISVersion()
  Dim strStep As String
  
260:   strStep = "1"
  
262:   If InStr(1, strVersion, ".") Then
    Dim strTokens() As String
264:     strTokens = Split(strVersion, ".")
265:     strVersion = strTokens(0)
266:     If UBound(strTokens) > 0 Then
267:       strStep = strTokens(1)
268:     End If
269:   End If
  
271:   strVersion = Trim(strVersion)
272:   If strVersion = 10 Then
273:     strStep = 4
274:   End If
  
276:   ReturnArcGISVersionAlt = CLng(strStep)



  Exit Function
ErrorHandler:
  HandleError True, "ReturnArcGISVersionAlt " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function QueryForNewFilename(strSampleName As String) As String
  On Error GoTo ErrorHandler



  Exit Function
ErrorHandler:
  HandleError True, "QueryForNewFilename " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function ReturnShapeName(pEnum As esriGeometryType) As String
  On Error GoTo ErrorHandler


  Select Case pEnum
  Case 0
302:     ReturnShapeName = "Unknown Geometry"
  Case 1
304:     ReturnShapeName = "Point"
  Case 2
306:     ReturnShapeName = "Multipoint"
  Case 3
308:     ReturnShapeName = "Polyline"
  Case 4
310:     ReturnShapeName = "Polygon"
  Case 5
312:     ReturnShapeName = "Envelope"
  Case 6
314:     ReturnShapeName = "Path"
  Case 7
316:     ReturnShapeName = "Unknown Geometry"
  Case 9
318:     ReturnShapeName = "Multipatch"
  Case 11
320:     ReturnShapeName = "Ring"
  Case 13
322:     ReturnShapeName = "Line"
  Case 14
324:     ReturnShapeName = "Circular Arc"
  Case 15
326:     ReturnShapeName = "Bezier Curve"
  Case 16
328:     ReturnShapeName = "Elliptic Arc"
  Case 17
330:     ReturnShapeName = "Geometry Bag"
  Case 18
332:     ReturnShapeName = "Triangle Strip"
  Case 19
334:     ReturnShapeName = "Triangle Fan"
  Case 20
336:     ReturnShapeName = "Ray"
  Case 21
338:     ReturnShapeName = "Sphere"
  Case 22
340:     ReturnShapeName = "Triangles"
  Case Else
342:     ReturnShapeName = "Unknown Geometry"
343:   End Select


  Exit Function
ErrorHandler:
  HandleError True, "ReturnShapeName " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function FileExists(strFilename As String) As Boolean
  On Error GoTo ErrorHandler


356:   FileExists = Dir(strFilename) <> ""


  Exit Function
ErrorHandler:
  HandleError True, "FileExists " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnArcGISVersion() As String
  On Error GoTo ErrorHandler


' ORIGINAL FUNCTION:
' Public Function GetSetting(ByVal Section As String, ByVal Key As String, Optional ByVal Default As String = "") As String
   ' Section   Required. String expression containing the name of the section where the key setting is found.
   '           If omitted, key setting is assumed to be in default subkey.
   ' Key       Required. String expression containing the name of the key setting to return.
   ' Default   Optional. Expression containing the value to return if no value is set in the key setting.
   '           If omitted, default is assumed to be a zero-length string ("").
   ' ADAPTED BY JENNESS FROM Ask the VB Pro column, [Getting Started with VB", Spring 1998.]
   ' SET DEFAULT VALUE
377:    ReturnArcGISVersion = "Unable to determine ArcGIS Version!"
   
   Dim nRet As Long
   Dim hKey As Long
   Dim nType As Long
   Dim nBytes As Long
   Dim Buffer As String
   
   Dim strSection As String
386:    strSection = ""
   Dim strKey As String
388:    strKey = "RealVersion"
   Dim strDefault As String
390:    strDefault = ""
   Dim strDir As String
   
   ' Assume failure and set return to Default
'   GetSetting = Default

   ' Open key
397:    nRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, SubKey("ESRI", "ArcGIS", strSection), 0&, KEY_READ, hKey)
398:    If nRet = ERROR_SUCCESS Then
      ' Set appropriate value for default query
400:       If strKey = "*" Then strKey = vbNullString
      
      ' Determine how large the buffer needs to be
403:       nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, nBytes)
404:       If nRet = ERROR_SUCCESS Then
         ' Build buffer and get data
406:          If nBytes > 0 Then
407:             Buffer = Space(nBytes)
408:             nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, Len(Buffer))
409:             If nRet = ERROR_SUCCESS Then
               ' Trim NULL and return successful query!
411:                strDir = Left(Buffer, nBytes - 1)
412:                ReturnArcGISVersion = strDir
413:             End If
414:          End If
415:       End If
416:       Call RegCloseKey(hKey)
417:    End If


  Exit Function
ErrorHandler:
  HandleError True, "ReturnArcGISVersion " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnArcGISInstallDir() As String
  On Error GoTo ErrorHandler


' ORIGINAL FUNCTION:
' Public Function GetSetting(ByVal Section As String, ByVal Key As String, Optional ByVal Default As String = "") As String
   ' Section   Required. String expression containing the name of the section where the key setting is found.
   '           If omitted, key setting is assumed to be in default subkey.
   ' Key       Required. String expression containing the name of the key setting to return.
   ' Default   Optional. Expression containing the value to return if no value is set in the key setting.
   '           If omitted, default is assumed to be a zero-length string ("").
   ' ADAPTED BY JENNESS FROM Ask the VB Pro column, [Getting Started with VB", Spring 1998.]
   ' SET DEFAULT VALUE
438:    ReturnArcGISInstallDir = "Unable to determine ArcGIS Install location!"
   
   Dim nRet As Long
   Dim hKey As Long
   Dim nType As Long
   Dim nBytes As Long
   Dim Buffer As String
   
   Dim strSection As String
447:    strSection = ""
   Dim strKey As String
449:    strKey = "InstallDir"
   Dim strDefault As String
451:    strDefault = ""
   Dim strDir As String
   
   
   ' Assume failure and set return to Default
'   GetSetting = Default

   ' Open key
459:    nRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, SubKey("ESRI", "ArcGIS", strSection), 0&, KEY_READ, hKey)
460:    If nRet = ERROR_SUCCESS Then
      ' Set appropriate value for default query
462:       If strKey = "*" Then strKey = vbNullString
      
      ' Determine how large the buffer needs to be
465:       nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, nBytes)
466:       If nRet = ERROR_SUCCESS Then
         ' Build buffer and get data
468:          If nBytes > 0 Then
469:             Buffer = Space(nBytes)
470:             nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, Len(Buffer))
471:             If nRet = ERROR_SUCCESS Then
               ' Trim NULL and return successful query!
473:                strDir = Left(Buffer, nBytes - 1)
474:                If Right(strDir, 1) = "\" Then strDir = Left(strDir, Len(strDir) - 1)
475:                ReturnArcGISInstallDir = strDir
476:             End If
477:          End If
478:       End If
479:       Call RegCloseKey(hKey)
480:    End If


  Exit Function
ErrorHandler:
  HandleError True, "ReturnArcGISInstallDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnArcGISGeneralDir(pArcGISFolderType As enumArcGISFolderTypes) As String
  On Error GoTo ErrorHandler


' ORIGINAL FUNCTION:
' Public Function GetSetting(ByVal Section As String, ByVal Key As String, Optional ByVal Default As String = "") As String
   ' Section   Required. String expression containing the name of the section where the key setting is found.
   '           If omitted, key setting is assumed to be in default subkey.
   ' Key       Required. String expression containing the name of the key setting to return.
   ' Default   Optional. Expression containing the value to return if no value is set in the key setting.
   '           If omitted, default is assumed to be a zero-length string ("").
   ' ADAPTED BY JENNESS FROM Ask the VB Pro column, [Getting Started with VB", Spring 1998.]
   ' SET DEFAULT VALUE
   
   
  Dim nRet As Long
  Dim hKey As Long
  Dim nType As Long
  Dim nBytes As Long
  Dim Buffer As String
  Dim strKey As String
  Dim strDir As String
  
  Select Case pArcGISFolderType
    Case enumArcGISInstallLocation
513:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Install location!"
514:       nRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\ESRI\ArcGIS", 0&, KEY_READ, hKey)
515:       strKey = "InstallDir"
    Case enumLastSaveToLocation
517:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last Saved To location!"
518:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
519:       strKey = "LastSaveToLocation"
    Case enumLastBrowsedLocation
521:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last Browsed To location!"
522:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
523:       strKey = "LastBrowseLocation"
    Case enumLastExportToLocation
525:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last Exported To location!"
526:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
527:       strKey = "LastExportToLocation"
    Case enumLastLocation
529:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last location!"
530:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
531:       strKey = "LastLocation"
532:   End Select
   
  ' Assume failure and set return to Default
  '   GetSetting = Default

   ' Open key
538:    If nRet = ERROR_SUCCESS Then
      ' Set appropriate value for default query
540:       If strKey = "*" Then strKey = vbNullString
      
      ' Determine how large the buffer needs to be
543:       nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, nBytes)
544:       If nRet = ERROR_SUCCESS Then
         ' Build buffer and get data
546:          If nBytes > 0 Then
547:             Buffer = Space(nBytes)
548:             nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, Len(Buffer))
549:             If nRet = ERROR_SUCCESS Then
               ' Trim NULL and return successful query!
551:                strDir = Left(Buffer, nBytes - 1)
552:                If Right(strDir, 1) = "\" Then strDir = Left(strDir, Len(strDir) - 1)
553:                ReturnArcGISGeneralDir = strDir
554:             End If
555:          End If
556:       End If
557:       Call RegCloseKey(hKey)
558:    End If


  Exit Function
ErrorHandler:
  HandleError True, "ReturnArcGISGeneralDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Function SubKey(ByVal strCompany As String, ByVal strAppName As String, Optional ByVal Section As String = "") As String
  On Error GoTo ErrorHandler


   ' Build SubKey from known values
571:    SubKey = "Software\" & strCompany & "\" & strAppName
572:    If Len(Section) Then
573:       SubKey = SubKey & "\" & Section
574:    End If


  Exit Function
ErrorHandler:
  HandleError False, "SubKey " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function ReturnFolders(ByVal DirPath As String, Optional ByVal FolderName As String) As Collection
  On Error GoTo ErrorHandler


  ' RETURNS A COLLECTION CONTAINING:
  '    A) Count of files that met criteria
  '    B) String Array of full folder filenames
  
  Dim returnCollection As Collection
590:   Set returnCollection = New Collection
  
  Dim strFolders() As String
  
  Dim strSearchString As String
595:   If FolderName = "" Then
596:     strSearchString = "*"
597:   Else
598:     strSearchString = FolderName
599:   End If
  
  Dim strSearchPath As String
  
603:   DirPath = Trim(DirPath)
604:   If Not Right(DirPath, 1) = "\" Then DirPath = DirPath & "\"
605:   strSearchPath = DirPath & strSearchString
  
  Dim strFoundFolder As String
  Dim anIndex As Integer
609:   anIndex = -1
  
611:   strFoundFolder = Dir(strSearchPath, 16)
  
613:   Do While Not strFoundFolder = ""
614:     If (Not strFoundFolder = ".") And (Not strFoundFolder = "..") Then
615:       If GetAttr(DirPath & strFoundFolder) = vbDirectory Then
616:         anIndex = anIndex + 1
        ReDim Preserve strFolders(anIndex)
618:         strFolders(anIndex) = DirPath & strFoundFolder
619:       End If
620:     End If
621:     strFoundFolder = Dir
622:   Loop
  
624:   returnCollection.Add (UBound(strFolders) - LBound(strFolders) + 1)
625:   returnCollection.Add (strFolders)
  
627:   Set ReturnFolders = returnCollection


  Exit Function
ErrorHandler:
  HandleError True, "ReturnFolders " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnFiles(ByVal DirPath As String, Optional ByVal FileName As String, Optional ByVal IncludeReadOnlyFiles As Boolean, _
                            Optional ByVal IncludeHiddenFiles As Boolean, Optional ByVal IncludeSystemFiles As Boolean) As Collection
  On Error GoTo ErrorHandler


  ' RETURNS A COLLECTION CONTAINING:
  '    A) Count of files that met criteria
  '    B) String Array of full filenames
  
  Dim returnCollection As Collection
645:   Set returnCollection = New Collection
  
  Dim strFiles() As String
  Dim intOption As Integer
649:   intOption = 0
650:   If IncludeReadOnlyFiles Then intOption = intOption + 1
651:   If IncludeHiddenFiles Then intOption = intOption + 2
652:   If IncludeSystemFiles Then intOption = intOption + 4
  
  Dim strSearchString As String
655:   If FileName = "" Then
656:     strSearchString = "*"
657:   Else
658:     strSearchString = FileName
659:   End If
  
  Dim strSearchPath As String
  
663:   DirPath = Trim(DirPath)
664:   If Not Right(DirPath, 1) = "\" Then DirPath = DirPath & "\"
665:   strSearchPath = DirPath & strSearchString
  
  Dim strFoundFile As String
  Dim anIndex As Integer
  
670:   strFoundFile = Dir(strSearchPath, intOption)
  
672:   Do While Not strFoundFile = ""
    ReDim Preserve strFiles(anIndex)
674:     strFiles(anIndex) = DirPath & strFoundFile
675:     strFoundFile = Dir
676:     anIndex = anIndex + 1
677:   Loop
  
679:   returnCollection.Add anIndex
'  If anIndex = 0 Then
'    returnCollection.Add 0
'  Else
'    returnCollection.Add (UBound(strFiles) - LBound(strFiles) + 1)
'  End If
685:   returnCollection.Add (strFiles)
  
687:   Set ReturnFiles = returnCollection


  Exit Function
ErrorHandler:
  HandleError True, "ReturnFiles " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function QuoteString(strInput As String) As String
  On Error GoTo ErrorHandler


  Dim strQuoted As String
701:   strQuoted = m_Quotation & SubstituteString(strInput, Chr(34), m_Quotation & m_Quotation) & m_Quotation
    
703:   QuoteString = strQuoted


  Exit Function
ErrorHandler:
  HandleError True, "QuoteString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ContainsString(strInText As String, strSearchText As String) As Boolean
  On Error GoTo ErrorHandler

  
715:   If strInText = "" Or strSearchText = "" Then
716:     ContainsString = False
717:   Else
718:     ContainsString = InStr(1, strInText, strSearchText, vbTextCompare) > 0
719:   End If


  Exit Function
ErrorHandler:
  HandleError True, "ContainsString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SubstituteString(strFullText As String, strSearchText As String, strSubstituteText As String)
  On Error GoTo ErrorHandler


  Dim lngIndex As Long
  Dim lngStartPos As Long
  Dim lngSearchLength As Long
  Dim lngFullLength As Long
  
736:   lngStartPos = 1
737:   lngSearchLength = Len(strSearchText)
738:   lngFullLength = Len(strFullText)
  
740:   lngIndex = InStr(lngStartPos, strFullText, strSearchText, vbTextCompare)
  
  Dim strNewString As String
743:   If lngIndex = 0 Then
744:     strNewString = strFullText
745:   Else
746:     strNewString = Left(strFullText, lngIndex - 1) & strSubstituteText
747:     lngStartPos = lngIndex + lngSearchLength
748:   End If
  
750:   Do While lngIndex <> 0
751:     lngIndex = InStr(lngStartPos, strFullText, strSearchText, vbTextCompare)
752:     If lngIndex = 0 Then
753:       strNewString = strNewString & Right(strFullText, lngFullLength - lngStartPos + 1)
754:     Else
755:       strNewString = strNewString & Mid(strFullText, lngStartPos, lngIndex - lngStartPos) & strSubstituteText
756:       lngStartPos = lngIndex + lngSearchLength
757:     End If
  
759:   Loop
760:   SubstituteString = strNewString


  Exit Function
ErrorHandler:
  HandleError True, "SubstituteString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function GetMxDocPath(pApp As IApplication) As String
  On Error GoTo ErrorHandler


  Dim pTemplates As ITemplates
  Dim lTempCount As Long
  
775:   Set pTemplates = pApp.Templates
776:   lTempCount = pTemplates.Count
  
  ' The document is always the last item
779:   GetMxDocPath = pTemplates.Item(lTempCount - 1)


  Exit Function
ErrorHandler:
  HandleError True, "GetMxDocPath " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CreatedBASETable(strFullName As String, Optional pFields As IFields) As ITable
  On Error GoTo ErrorHandler


' createDBF: simple function to create a DBASE file.
' note: the name of the DBASE file should not contain the .dbf extension
' ESRI Sample; modified by Jenness August 20 2007
  
  Dim strName As String
  Dim strFolder As String
  
798:   strFolder = aml_func_mod.ReturnDir(strFullName)
799:   strName = aml_func_mod.ReturnFilename(strFullName)
800:   If Right(strName, 4) = ".dbf" Then strName = Left(strName, Len(strName) - 4)
  
  ' Open the Workspace
  Dim pFWS As IFeatureWorkspace
  Dim pWorkspaceFactory As IWorkspaceFactory
  Dim fs As Object
  Dim pFieldsEdit As IFieldsEdit
  Dim pFieldEdit As IFieldEdit
  Dim pField As IField

810:   Set pWorkspaceFactory = New ShapefileWorkspaceFactory
811:   If Not aml_func_mod.ExistFileDir(strFolder) Then
812:     MsgBox "Folder does not exist: " & vbCr & strFolder
    Exit Function
814:   End If

816:   Set pFWS = pWorkspaceFactory.OpenFromFile(strFolder, 0)
  
  ' if a fields collection is not passed in then create one
819:   If pFields Is Nothing Then
    ' create the fields used by our object
821:     Set pFields = New Fields
822:     Set pFieldsEdit = pFields
823:     pFieldsEdit.FieldCount = 1
    
    'Create text Field
826:     Set pField = New Field
827:     Set pFieldEdit = pField
828:     With pFieldEdit
829:         .Precision = 8
830:         .Name = "Unique_ID"
831:         .Type = esriFieldTypeInteger
832:     End With
833:     Set pFieldsEdit.Field(0) = pField
834:   End If
  
  Dim strString As String
  Dim lngIndex As Long
  Dim pFieldInfo As IFieldInfo
  
840:   Set pField = pFields.Field(0)


'  MsgBox pField.Name
'  MsgBox pField.Scale
'  MsgBox pField.Precision
'  MsgBox pField.Type

'  For lngIndex = 0 To pFields.FieldCount - 1
'    strString = strString + "-----------------" & "  Field Name = " & pFields.Field(lngIndex).Name & vbCrLf & _
'        "  Field Scale = " & pFields.Field(lngIndex).Scale & vbCrLf & "  Precision = " & pFields.Field(lngIndex).Precision & _
'        vbCrLf & "  Field Type = " & CStr(pFields.Field(lngIndex).Type) & vbCrLf
'  Next lngIndex
'  MsgBox "Problem with workspace? " & CStr(pFWS Is Nothing) & vbCrLf & "Filename = " & strFullName & vbCrLf & _
'        "Folder = " & strFolder & vbCrLf & "strName = " & strName & vbCrLf & pFields.FieldCount & " fields..." & vbCrLf & _
'        "File already exists? " & CBool(FileExists(strFullName)) & vbCrLf & strString
  
857:   Set CreatedBASETable = pFWS.CreateTable(strName, pFields, Nothing, Nothing, "")


  Exit Function
ErrorHandler:
  HandleError True, "CreatedBASETable " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function CreateShapefile(sPath As String, sName As String, pSpatialReference As ISpatialReference, strShapeType As String) As IFeatureClass   ' Don't include filename!
  On Error GoTo ErrorHandler

  
870:   If Right(sPath, 4) = ".shp" Then sPath = ReturnDir(sPath)
871:   If Right(sName, 4) = ".shp" Then sName = Left(sName, Len(sName) - 4)
  
  ' SET GEOMETRY TYPE, AND EXIT IF NOT ONE OF STANDARD OPTIONS
  Dim pGeomDef As IGeometryDef
  Dim pGeomDefEdit As IGeometryDefEdit
876:   Set pGeomDef = New GeometryDef
877:   Set pGeomDefEdit = pGeomDef
878:   With pGeomDefEdit
    Select Case strShapeType
      Case "Polygon", "polygon"
881:         .GeometryType = esriGeometryPolygon
      Case "Polyline", "polyline"
883:         .GeometryType = esriGeometryPolyline
      Case "Point", "point"
885:         .GeometryType = esriGeometryPoint
      Case "Multipoint", "multipoint", "MultiPoint"
887:         .GeometryType = esriGeometryMultipoint
      Case "Multipatch", "multipatch", "MultiPatch"
889:         .GeometryType = esriGeometryMultiPatch
      Case Else
891:         MsgBox "Invalid Shape Type [" & strShapeType & "]!  This function is only written to generate " & _
            "Point, Polyline, Polygon, Multipoint or Multipatch shapefiles...", vbCritical, "Invalid Shape Type:"
893:     End Select
'    Set .SpatialReference = New UnknownCoordinateSystem
895:     Set .SpatialReference = pSpatialReference
896:   End With
  
  ' Open the folder to contain the shapefile as a workspace
  Dim pFWS As IFeatureWorkspace
  Dim pWorkspaceFactory As IWorkspaceFactory
901:   Set pWorkspaceFactory = New ShapefileWorkspaceFactory
  
'   If Not pWorkspaceFactory.IsWorkspace(sPath) Then
904:   If Not ExistFileDir(sPath) Then
905:     MsgBox "Unable to create Feature Class:" & vbCrLf & _
           sPath & " is not a valid workspace...", , "Failed to Create Feature Class:"
907:     Set CreateShapefile = Nothing
    Exit Function
909:   End If
  
911:   Set pFWS = pWorkspaceFactory.OpenFromFile(sPath, 0)
  
  ' Set up a simple fields collection
  Dim pFields As IFields
  Dim pFieldsEdit As IFieldsEdit
916:   Set pFields = New Fields
917:   Set pFieldsEdit = pFields
  
  Dim pField As IField
  Dim pFieldEdit As IFieldEdit
  
  ' Make the shape field
  ' it will need a geometry definition, with a spatial reference
924:   Set pField = New Field
925:   Set pFieldEdit = pField
926:   pFieldEdit.Name = "Shape"
927:   pFieldEdit.Type = esriFieldTypeGeometry
  
929:   Set pFieldEdit.GeometryDef = pGeomDef
930:   pFieldsEdit.AddField pField

  ' Add an ID field
933:   Set pField = New Field
934:   Set pFieldEdit = pField
935:   With pFieldEdit
936:       .length = 8
937:       .Name = "Unique_ID"
938:       .Type = esriFieldTypeInteger
939:       .Precision = 0
940:   End With
941:   pFieldsEdit.AddField pField
  
  ' Create the shapefile
  ' (some parameters apply to geodatabase options and can be defaulted as Nothing)
  Dim booFileExists As Boolean
  Dim strCheckString As String
947:   If Right(sPath, 1) = "\" Then
948:     strCheckString = sPath & sName & ".shp"
'    MsgBox sPath & sName & ".shp" & vbCrLf & "File Exists? " & CStr(Dir(sPath & sName & ".shp") <> "")
950:   Else
951:     strCheckString = sPath & "\" & sName & ".shp"
'    MsgBox sPath & "\" & sName & ".shp" & vbCrLf & "File Exists? " & CStr(Dir(sPath & "\" & sName & ".shp") <> "")
953:   End If
  
955:   booFileExists = (Dir(strCheckString) <> "")
  
957:   If booFileExists Then
958:     MsgBox "The following file already exists:" & vbCrLf & vbCrLf & strCheckString & vbCrLf & vbCrLf & _
           "Please select a new filename...", , "Duplicate Filename:"
960:     Set CreateShapefile = Nothing
    Exit Function
962:   End If
  
  Dim pFeatClass As IFeatureClass
965:   Set pFeatClass = pFWS.CreateFeatureClass(sName, pFields, Nothing, _
                                           Nothing, esriFTSimple, "Shape", "")
                                           
968:   Set CreateShapefile = pFeatClass


  Exit Function
ErrorHandler:
  HandleError True, "CreateShapefile " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CreateShapefile2(sPath As String, sName As String, pSpatialReference As ISpatialReference, _
    strShapeType As String, pAddFields As esriSystem.IVariantArray) As IFeatureClass     ' Don't include filename!
  On Error GoTo ErrorHandler

  
981:   If Right(sPath, 4) = ".shp" Then sPath = ReturnDir(sPath)
982:   If Right(sName, 4) = ".shp" Then sName = Left(sName, Len(sName) - 4)
  
  ' SET GEOMETRY TYPE, AND EXIT IF NOT ONE OF STANDARD OPTIONS
  Dim pGeomDef As IGeometryDef
  Dim pGeomDefEdit As IGeometryDefEdit
987:   Set pGeomDef = New GeometryDef
988:   Set pGeomDefEdit = pGeomDef
989:   With pGeomDefEdit
    Select Case strShapeType
      Case "Polygon", "polygon"
992:         .GeometryType = esriGeometryPolygon
      Case "Polyline", "polyline"
994:         .GeometryType = esriGeometryPolyline
      Case "Point", "point"
996:         .GeometryType = esriGeometryPoint
      Case "Multipoint", "multipoint", "MultiPoint"
998:         .GeometryType = esriGeometryMultipoint
      Case "Multipatch", "multipatch", "MultiPatch"
1000:         .GeometryType = esriGeometryMultiPatch
      Case Else
1002:         MsgBox "Invalid Shape Type [" & strShapeType & "]!  This function is only written to generate " & _
            "Point, Polyline, Polygon, Multipoint or Multipatch shapefiles...", vbCritical, "Invalid Shape Type:"
1004:     End Select
'    Set .SpatialReference = New UnknownCoordinateSystem
1006:     Set .SpatialReference = pSpatialReference
1007:   End With
  
  ' Open the folder to contain the shapefile as a workspace
  Dim pFWS As IFeatureWorkspace
  Dim pWorkspaceFactory As IWorkspaceFactory
1012:   Set pWorkspaceFactory = New ShapefileWorkspaceFactory
  
'   If Not pWorkspaceFactory.IsWorkspace(sPath) Then
1015:   If Not ExistFileDir(sPath) Then
1016:     MsgBox "Unable to create Feature Class:" & vbCrLf & _
           sPath & " is not a valid workspace...", , "Failed to Create Feature Class:"
1018:     Set CreateShapefile2 = Nothing
    Exit Function
1020:   End If
  
1022:   Set pFWS = pWorkspaceFactory.OpenFromFile(sPath, 0)
  
'   Set up a simple fields collection
  Dim pFields As IFields
  Dim pFieldsEdit As IFieldsEdit
1027:   Set pFields = New Fields
1028:   Set pFieldsEdit = pFields

  Dim pField As IField
  Dim pFieldEdit As IFieldEdit
  
  ' Make the shape field
  ' it will need a geometry definition, with a spatial reference
1035:   Set pField = New Field
1036:   Set pFieldEdit = pField
1037:   pFieldEdit.Name = "Shape"
1038:   pFieldEdit.Type = esriFieldTypeGeometry

1040:   Set pFieldEdit.GeometryDef = pGeomDef
1041:   pFieldsEdit.AddField pField
  
  ' ADD FIELDS
  Dim lngIndex As Long
1045:   For lngIndex = 0 To pAddFields.Count - 1
1046:     pFieldsEdit.AddField pAddFields.Element(lngIndex)
1047:   Next lngIndex
    
  ' Create the shapefile
  ' (some parameters apply to geodatabase options and can be defaulted as Nothing)
  Dim booFileExists As Boolean
  Dim strCheckString As String
1053:   If Right(sPath, 1) = "\" Then
1054:     strCheckString = sPath & sName & ".shp"
'    MsgBox sPath & sName & ".shp" & vbCrLf & "File Exists? " & CStr(Dir(sPath & sName & ".shp") <> "")
1056:   Else
1057:     strCheckString = sPath & "\" & sName & ".shp"
'    MsgBox sPath & "\" & sName & ".shp" & vbCrLf & "File Exists? " & CStr(Dir(sPath & "\" & sName & ".shp") <> "")
1059:   End If
  
1061:   booFileExists = (Dir(strCheckString) <> "")
  
1063:   If booFileExists Then
1064:     MsgBox "The following file already exists:" & vbCrLf & vbCrLf & strCheckString & vbCrLf & vbCrLf & _
           "Please select a new filename...", , "Duplicate Filename:"
1066:     Set CreateShapefile2 = Nothing
    Exit Function
1068:   End If
  
  Dim pFeatClass As IFeatureClass
1071:   Set pFeatClass = pFWS.CreateFeatureClass(sName, pFields, Nothing, _
                                           Nothing, esriFTSimple, "Shape", "")
                                           
1074:   Set CreateShapefile2 = pFeatClass


  Exit Function
ErrorHandler:
  HandleError True, "CreateShapefile2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function TempPathLocation() As String
  On Error GoTo ErrorHandler


  Dim sBuffer As String
1088:   sBuffer = Space(strMAXPATH)
1089:   If GetTempPath(strMAXPATH, sBuffer) <> 0 Then
1090:     TempPathLocation = Left$(sBuffer, _
      InStr(sBuffer, vbNullChar) - 1)
1092:   Else
1093:     TempPathLocation = ""
1094:   End If


  Exit Function
ErrorHandler:
  HandleError True, "TempPathLocation " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function GetTheUserName() As String
  On Error GoTo ErrorHandler


  Dim sBuffer As String
  Dim sUName As String
  Dim lSize As Long
1110:   sBuffer = Space$(255)
1111:   lSize = Len(sBuffer)
1112:   Call GetUserName(sBuffer, lSize)
1113:   If lSize > 0 Then
1114:     sUName = Left$(sBuffer, lSize)
1115:   Else
1116:     sUName = vbNullString
1117:   End If
1118:   GetTheUserName = BasicTrimAvenue(sUName, "", Chr(0))     ' NEED TO PEEL OFF THAT LAST ODD CHARACTER


  Exit Function
ErrorHandler:
  HandleError True, "GetTheUserName " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function BasicTrimAvenue(aString As String, aTrimLeft As String, aTrimRight As String) As String
  On Error GoTo ErrorHandler

  
1130:   Do While (aString <> "") And (InStr(1, aTrimRight, Right(aString, 1), vbTextCompare) > 0)
1131:     aString = Left(aString, Len(aString) - 1)
1132:   Loop
1133:   Do While (aString <> "") And (InStr(1, aTrimLeft, Left(aString, 1), vbTextCompare) > 0)
1134:     aString = Right(aString, Len(aString) - 1)
1135:   Loop
  
1137:   BasicTrimAvenue = aString


  Exit Function
ErrorHandler:
  HandleError True, "BasicTrimAvenue " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function InsertCommas(InputValue As Variant) As String
  On Error GoTo ErrorHandler

  
  Dim theString As String
1150:   theString = CStr(InputValue)
  
  Dim theDecLocation As Long
1153:   theDecLocation = InStr(theString, ".")
  
  Dim HasDecimal As Boolean
1156:   HasDecimal = theDecLocation > 0
  
  Dim theLength As Long
1159:   theLength = Len(theString)
  
  Dim theBaseNumber As String
  Dim theRemainder As String
  
1164:   If HasDecimal Then
1165:     theRemainder = Right(theString, theLength - theDecLocation)
1166:     theBaseNumber = Left(theString, theDecLocation - 1)
1167:   Else
1168:     theRemainder = ""
1169:     theBaseNumber = theString
1170:   End If
  
  Dim theCount As Long
1173:   theCount = Len(theBaseNumber)
  
  Dim theCommaString As String
  
1177:   If theCount > 3 Then
    Dim anIndex As Long
1179:     For anIndex = (theCount - 2) To 1 Step -3
1180:       theCommaString = Mid(theBaseNumber, anIndex, 3) & "," & theCommaString
1181:       If anIndex < 4 Then
1182:         theCommaString = Left(theBaseNumber, anIndex - 1) & "," & theCommaString
1183:       End If
1184:     Next anIndex
    
1186:     Do While Right(theCommaString, 1) = ","
1187:       theCommaString = Left(theCommaString, Len(theCommaString) - 1)
1188:     Loop
1189:     Do While Left(theCommaString, 1) = ","
1190:       theCommaString = Right(theCommaString, Len(theCommaString) - 1)
1191:     Loop
1192:   Else
1193:     theCommaString = theBaseNumber
1194:   End If
  
1196:   If HasDecimal Then
1197:     theCommaString = theCommaString & "." & theRemainder
1198:   End If
  
1200:   InsertCommas = theCommaString


  Exit Function
ErrorHandler:
  HandleError True, "InsertCommas " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ClipExtension(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1215:   aml_func_mod.ParseString strPathName, strDirTokens, "."
1216:   strDirPath = strDirTokens(0)
  
1218:   If (UBound(strDirTokens) = 0) Then
1219:     ClipExtension = strDirPath
1220:   Else
    Dim anIndex As Long
1222:     For anIndex = 1 To (UBound(strDirTokens) - 1)
1223:       strDirPath = strDirPath & "." & strDirTokens(anIndex)
1224:     Next anIndex
1225:     ClipExtension = strDirPath
1226:   End If


  Exit Function
ErrorHandler:
  HandleError True, "ClipExtension " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function FieldIsNumeric(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1239:   theFieldType = pTheField.Type
  
1241:   FieldIsNumeric = _
    (theFieldType = esriFieldTypeSmallInteger) Or (theFieldType = esriFieldTypeDouble) Or (theFieldType = esriFieldTypeInteger) Or _
          (theFieldType = esriFieldTypeSingle)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsNumeric " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function FieldIsString(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1255:   theFieldType = pTheField.Type
  
1257:   FieldIsString = (theFieldType = esriFieldTypeString)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function FieldIsDate(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1269:   theFieldType = pTheField.Type
  
1271:   FieldIsDate = (theFieldType = esriFieldTypeDate)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsDate " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function FieldIsShape(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1283:   theFieldType = pTheField.Type
  
1285:   FieldIsShape = (theFieldType = esriFieldTypeGeometry)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsShape " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SetExtension(strPathName As String, strExtension As String) As String
  On Error GoTo ErrorHandler

  
  Dim theClippedPath As String
1298:   SetExtension = ClipExtension(strPathName) & "." & strExtension


  Exit Function
ErrorHandler:
  HandleError True, "SetExtension " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function GetExtensionText(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1313:   aml_func_mod.ParseString strPathName, strDirTokens, "."
1314:   If UBound(strDirTokens) = 0 Then
1315:     GetExtensionText = ""
1316:   Else
1317:     GetExtensionText = strDirTokens(UBound(strDirTokens))
1318:   End If


  Exit Function
ErrorHandler:
  HandleError True, "GetExtensionText " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function GetFullFileString(str83Type As String) As String
  On Error GoTo ErrorHandler

  
  ' ADAPTED FROM BRETT MERONEY'S POST ABOVE
  
  Dim lLen As Long
  Dim sBuffer As String
  
1335:   sBuffer = String$(strMAXPATH, 0)
1336:   lLen = GetLongPathName(str83Type, sBuffer, Len(sBuffer))
1337:   If lLen > 0 And err.Number = 0 Then
1338:     GetFullFileString = Left$(sBuffer, lLen)
1339:   Else
1340:     GetFullFileString = str83Type
1341:   End If


  Exit Function
ErrorHandler:
  HandleError True, "GetFullFileString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnDir(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1356:   aml_func_mod.ParseString strPathName, strDirTokens, "\"
1357:   strDirPath = strDirTokens(0)
  
1359:   If (UBound(strDirTokens) = 0) Then
1360:     ReturnDir = strDirPath
1361:   Else
    Dim anIndex As Long
1363:     For anIndex = 1 To (UBound(strDirTokens) - 1)
1364:       strDirPath = strDirPath & "\" & strDirTokens(anIndex)
1365:     Next anIndex
1366:     ReturnDir = strDirPath
1367:   End If
  
1369:   ReturnDir = ReturnDir & "\"


  Exit Function
ErrorHandler:
  HandleError True, "ReturnDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function ReturnFilename(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1385:   aml_func_mod.ParseString strPathName, strDirTokens, "\"
1386:   ReturnFilename = strDirTokens(UBound(strDirTokens))


  Exit Function
ErrorHandler:
  HandleError True, "ReturnFilename " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Sub ParseString(str As String, strArray() As String, Delim As String)
  On Error GoTo ErrorHandler


' Populates a named string array with elements in a string. Each array element
' contains one word. Multiple words ' within single quotes ' are treated as
' one word. NOTE: Use parseStringR if you want both commas and blanks to be
' treated as delimiting characters.
'
' Before calling this Sub you must declare your array in the calling program
' As String, with no bounds.
'
' Dim myarray() As String
' mystring = "ARC YES, POLY NO, TICS YES"
' parseString (mystring), myarray, ","
' Returns:
' array(0) = ARC YES
' array(1) = POLY NO
' array(2) = TICS YES

' parseString(mystring),myarray," "
' Returns:
' array(0) = ARC
' array(1) = YES,
' array(2) = POLY
' array(3) = NO,
' array(4) = TICS
' array(5) = YES,

' mystring = "'Universe,Medium','Helvetica,Bold','Times,Medium'"
' parsestring(mystring),myarray,","
' Returns:
' array(0) = Universe,Medium
' array(1) = Helvetica,Bold
' array(2) = Times,Medium

' Dim counters

Dim i As Long
Dim tokenlen As Long
Dim tmpstr As String
Dim position As Long
Dim length As Long

'Dim variables to keep track of embedded quotes

Dim switch As Long
Dim position1 As Long
Dim position2 As Long
Dim pair As Long

On Error Resume Next

' If string contains no elements raise error
1447:   If Trim(Subst(str, Delim)) = "" Then
1448:     err.Raise vbObjectError + 1, "aml_func.ParseString", _
    "StringPassed"
    Exit Sub
1451:   End If

' intialize array. Warning: This will overwrite any data elements currently
' stored in this named array

  ReDim strArray(0)

'intializer counters and tracking variables

1460:   pair = False
1461:   switch = 0
1462:   length = Len(str)
1463:   position = 1
1464:   i = 0
1465:   tmpstr = str

'check each character in the array. If it is a quote, store if it is first or last
' 0 = havent read one yet
' 1 = read first single quote
' 2 = read second single quote

1472:   Do While position < length
1473:     If Mid(tmpstr, position, 1) = "'" Then
1474:       If Not (switch = 1) Then
1475:         switch = 1
1476:         position1 = position
1477:         pair = False
1478:       Else
1479:         switch = 2
1480:         position2 = position
1481:         pair = True
1482:       End If
1483:     End If

' if last char read was last in a pair of quotes, store contents between first and last
' in current array element and reset tracking variables

1488:     If pair = True Then
1489:       Mid(tmpstr, position1, 1) = " "
1490:       Mid(tmpstr, position2, 1) = " "
1491:       strArray(i) = Mid(tmpstr, position1, position2 - position1)
1492:       strArray(i) = Trim(strArray(i))
1493:       pair = False
1494:       switch = 0
      
' check to see if we are reading till the next single quote. If switch = 0, we are not
' if not check if the next character is a delimiter. if it is store everything to the left
' replace everything to the left of original str with blanks so we can safely use LEFT
' function then trim the blanks

1501:     Else
1502:       If switch = 0 Then
1503:         If Mid(tmpstr, position, 1) = Delim Then
1504:           strArray(i) = Left(tmpstr, position)
1505:           tokenlen = Len(strArray(i))
1506:           strArray(i) = Trim(strArray(i))
1507:           If Not (Len(strArray(i)) = 0) Then
1508:             Mid(tmpstr, 1, tokenlen) = String(tokenlen, " ")
            ReDim Preserve strArray(LBound(strArray) To UBound(strArray) + 1)
1510:             i = i + 1
1511:           End If
1512:         End If
1513:       End If
1514:     End If
1515:     position = position + 1
1516:   Loop
1517:   strArray(i) = Trim(tmpstr)

' we have populated our array, now remove the delimiters from each element

1521:   position = 1
1522:   For i = 0 To UBound(strArray)
1523:     position = Len(strArray(i))
1524:     If Mid(strArray(i), position, 1) = Delim Then
1525:       Mid(strArray(i), position, 1) = " "
1526:       strArray(i) = Trim(strArray(i))
1527:     End If
1528:   Next i


  Exit Sub
ErrorHandler:
  HandleError True, "ParseString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function After(str As String, SearchStr As String) As String
  On Error GoTo ErrorHandler


' Returns the substring of Str to the right of the leftmost
' occurrence of the searchStr.
Dim position As Long
Dim length As Long

1545:   position = InStr(str, SearchStr)
1546:   length = Len(SearchStr)
1547:   If Not (position = 0) Then
1548:    After = Mid(str, position + length)
1549:   End If


  Exit Function
ErrorHandler:
  HandleError True, "After " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Before(str As String, SearchStr As String) As String
  On Error GoTo ErrorHandler


' Returns the substring of Str to the left of the leftmost
' occurrence of the searchStr.
Dim position As Long
Dim length As Long

1566:   position = InStr(str, SearchStr)
1567:   length = Len(SearchStr)
1568:   If Not (position = 0) Then
1569:    Before = Mid(str, 1, position - 1)
1570:   End If


  Exit Function
ErrorHandler:
  HandleError True, "Before " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Function ExistFileDir(sTest As String) As Boolean
  On Error GoTo ErrorHandler


'Checks for the existance of a File or Directory
  Dim af As Long
1584:   af = -1
  On Error Resume Next
1586:   af = GetAttr(sTest)
1587:   ExistFileDir = (af <> -1)


  Exit Function
ErrorHandler:
  HandleError True, "ExistFileDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function



Public Function MakeUniqueFilename(strFilename As String) As String
  On Error GoTo ErrorHandler


1601:   If Not ExistFileDir(strFilename) Then
1602:     MakeUniqueFilename = strFilename
    Exit Function
1604:   Else
    
    Dim theCounter As Long
1607:     theCounter = 1
    
    Dim theFilename As String
    Dim theBaseName As String
    Dim thePointPos As Long
    Dim theExtension As String
    
1614:     If InStr(1, Right(strFilename, 5), ".", vbTextCompare) > 0 Then
1615:       thePointPos = InStrRev(strFilename, ".", -1, vbTextCompare)
1616:       theExtension = Right(strFilename, Len(strFilename) - (thePointPos - 1))
1617:       theFilename = Left(strFilename, thePointPos - 1)
1618:     Else
1619:       theExtension = ""
1620:       theFilename = strFilename
1621:     End If
    
1623:     theBaseName = theFilename
    
1625:     Do While ExistFileDir(theFilename & theExtension)
1626:       theCounter = theCounter + 1
1627:       theFilename = theBaseName & "_" & CStr(theCounter)
1628:     Loop
    
1630:     MakeUniqueFilename = theFilename & theExtension
    
1632:   End If


  Exit Function
ErrorHandler:
  HandleError True, "MakeUniqueFilename " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Extract(ElemNum As Long, ElemList As String) As String
  On Error GoTo ErrorHandler


' extracts an element from a list of elements

Dim strArray() As String

1648:   ParseStringR (ElemList), strArray
1649:   If ElemNum > UBound(strArray) + 1 Then
   Exit Function
1651:   End If
  
1653:   If ElemNum = 0 Then
   Exit Function
1655:   End If
1656:   Extract = strArray(ElemNum - 1)


  Exit Function
ErrorHandler:
  HandleError True, "Extract " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Index(str As String, SearchStr As String) As Long
  On Error GoTo ErrorHandler


' Returns the position of the leftmost occurrence of searcStr in str.

1670:   Index = InStr(str, SearchStr)


  Exit Function
ErrorHandler:
  HandleError True, "Index " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub ParseStringR(str As String, strArray() As String, Optional ReturnQuoted)
  On Error GoTo ErrorHandler


' Populates a named string arrary with elements in a string. Each array element
' contains one word. Multiple words ' within single quotes ' are treated as
' one word. Treats both blanks and commas as delimters NOTE: Use parseString
' to specify a specific delimiting character.

' ReturnQuoted - indicates if elements are to be returned quoted.
' FALSE - DEFAULT return elements unquoted
' TRUE - return elements quoted

' Before calling this function you must declare your array in the calling program
' As String, with no bounds.
'
' Dim myarray() As String
' mystring = "ARC YES, POLY NO, TICS YES"
' parseStringR(mystring),myarray

' Returns:
' array(0) = ARC
' array(1) = YES
' array(2) = POLY
' array(3) = NO
' array(4) = TICS
' array(5) = YES

' mystring = "'Universe,Medium','Helvetica,Bold','Times,Medium'"

' parsestringR(mystring),myarray
' Returns:
' array(0) = Universe,Medium
' array(1) = Helvetica,Bold
' array(2) = Times,Medium

' parsestring(mystring,myarray,TRUE)
' Returns:
' array(0) = 'Universe,Medium'
' array(1) = 'Helvetica,Bold'
' array(2) = 'Times,Medium'

' Dim counters

Dim i As Long
Dim tokenlen As Long
Dim switch As Long
Dim position1 As Long
Dim position2 As Long
Dim pair As Long
Dim parseAgain As Boolean
Dim tmpstr As String
Dim position As Long
Dim length As Long

On Error Resume Next

' If string contains no elements raise error
1736:   If Trim(Subst(str, ",")) = "" Then
1737:     err.Raise vbObjectError + 1, "aml_func.ParseStringR", _
    "StringPassed"
    Exit Sub
1740:   End If

' intialize counters and tracking variables
  ReDim strArray(0)
1744:   pair = False
1745:   switch = 0
1746:   length = Len(str)
1747:   position = 1
1748:   i = 0
1749:   tmpstr = str

1751:   If IsMissing(ReturnQuoted) Then
1752:     ReturnQuoted = False
1753:   End If
1754:   If Not (ReturnQuoted = False) Then
1755:     ReturnQuoted = True
1756:   End If

' check each character in the array. If it is a quote, store if it is first or last
' 0 = havent read one yet
' 1 = read first single quote
' 2 = just read second single quote

1763:   Do While position < length
1764:     If Mid(tmpstr, position, 1) = "'" Then
1765:       If Not (switch = 1) Then
1766:       switch = 1
1767:       position1 = position
1768:       pair = False
1769:     Else
1770:       switch = 2
1771:       position2 = position
1772:       pair = True
1773:       End If
1774:     End If
     
' if last char read was last in a pair of single quotes, store contents between first
' and last in current array element and reset tracking variables
 
1779:     If pair = True Then
1780:       Mid(tmpstr, position1, 1) = " "
1781:       Mid(tmpstr, position2, 1) = " "
1782:       strArray(i) = Mid(tmpstr, position1, position2 - position1)
1783:       strArray(i) = Trim(strArray(i))
1784:       pair = False
1785:       switch = 0
    
' check to see if we are reading till the next single quote. If switch = 0, we are not
' if not check if the next character is a delimiter. if it is store everything to the left
' replace everything to the left of original str with blanks so we can safely use LEFT
' function then trim the blanks
  
1792:     Else
1793:       If switch = 0 Then
1794:         If Mid(tmpstr, position, 1) = "," Or Mid(tmpstr, position, 1) = " " Then
1795:           strArray(i) = Left(tmpstr, position)
1796:           tokenlen = Len(strArray(i))
1797:           strArray(i) = Trim(strArray(i))
1798:           If Not (Len(strArray(i)) = 0) Then
1799:             Mid(tmpstr, 1, tokenlen) = String(tokenlen, " ")
            ReDim Preserve strArray(LBound(strArray) To UBound(strArray) + 1)
1801:             i = i + 1
1802:           End If
1803:         End If
1804:       End If
1805:     End If
1806:     position = position + 1
1807:   Loop
1808:     strArray(i) = Trim(tmpstr)

' we have populated our array, now remove the delimiters from each element
' set parseAgain flag if there are any blank elements

1813:   parseAgain = False
1814:   position = 1

1816:   For i = 0 To UBound(strArray)
1817:     position = Len(strArray(i))
1818:     If Mid(strArray(i), position, 1) = "," Then
1819:       Mid(strArray(i), position, 1) = " "
1820:       strArray(i) = Trim(strArray(i))
1821:       If Len(strArray(i)) = 0 Then
1822:         parseAgain = True
1823:       End If
1824:     End If
1825:   Next i
  
' now remove any blank elements
  
1829:   If parseAgain = True Then
1830:     tmpstr = ""
1831:     For i = 0 To UBound(strArray)
1832:       If Not (Len(strArray(i))) = 0 Then
1833:         tmpstr = tmpstr & "'" & strArray(i) & "'" & ","
1834:       End If
1835:     Next i
1836:     ParseString (tmpstr), strArray, ","
1837:   End If
      
1839:   If ReturnQuoted = True Then
1840:     For i = 0 To UBound(strArray)
1841:      strArray(i) = "'" & strArray(i) & "'"
1842:     Next i
1843:   End If


  Exit Sub
ErrorHandler:
  HandleError True, "ParseStringR " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function Keyword(str As String, SearchStr As String) As Long
  On Error GoTo ErrorHandler


' Returns the position of a string within a list of keywords.
' converts Str and searchStr to upper case before comparing
'  0  if keyword not found
' -1  if string is ambiguous - mutiple occurances of same keyword
'  n  position of keyword in string

Dim strArray() As String
Dim i As Long
Dim keywordCnt As Long

1865:   ParseStringR (str), strArray
1866:   keywordCnt = 0
1867:     For i = 0 To UBound(strArray)
1868:     If UCase(SearchStr) = UCase(strArray(i)) Then
1869:       Keyword = i + 1
1870:       keywordCnt = keywordCnt + 1
1871:     End If
1872:     Next i
  
1874:   If keywordCnt > 1 Then
1875:     Keyword = -1
1876:   End If


  Exit Function
ErrorHandler:
  HandleError True, "Keyword " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Search(str, SearchStr) As Long
  On Error GoTo ErrorHandler


' Returns the position of the first character in Str
' which occurs in searchStr.

Dim strArray() As String
Dim i As Long
Dim Index As Long
Dim firstchar
Dim InString As Boolean

1897:   Index = 1
  ReDim strArray(Len(str))

1900:   For i = 0 To Len(SearchStr) - 1
1901:     firstchar = Mid(SearchStr, i + 1, 1)
1902:     Index = InStr(str, firstchar)
1903:     strArray(Index) = i + 1
1904:   Next i
  
1906:   InString = False
  
1908:   For i = 1 To UBound(strArray)
1909:     If Not (strArray(i)) = "" Then
1910:     InString = True
1911:     Exit For
1912:     Else
1913:     End If
1914:   Next i
  
1916:   If InString = False Then
1917:     Search = 0
1918:   Else
1919:     Search = i
1920:   End If


  Exit Function
ErrorHandler:
  HandleError True, "Search " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Sort(str As String, Optional SortOption, Optional SortType) As String
  On Error GoTo ErrorHandler


' Returns a string of sorted elements

1934:   If IsMissing(SortOption) Then
1935:     SortOption = "-ASCEND"
1936:   ElseIf Not (UCase(SortOption) = "-DESCEND") Then
1937:     SortOption = "-ASCEND"
1938:   End If
  
1940:   If IsMissing(SortType) Then
1941:     SortType = "-CHARACTER"
1942:   ElseIf Not (UCase(SortType) = "-NUMERIC") Then
1943:     SortType = "-CHARACTER"
1944:   End If
  
1946:   If (UCase(SortType)) = "-NUMERIC" Then
1947:     Call Sort_Num(str, SortOption)
1948:   Else
1949:     Call Sort_Char(str, SortOption, True)
1950:   End If
1951:     Sort = str


  Exit Function
ErrorHandler:
  HandleError True, "Sort " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Function Sort_Num(str As String, SortOption) As String
  On Error GoTo ErrorHandler


' Sort function - performs a numerical sort
' Ref: Selectionsort Chapter8 of VB Algorithms; Rod Stephens

Dim i As Long
Dim J As Long
Dim min As Long
Dim max As Long
Dim best_value As String
Dim best_j As Long
Dim sortArray() As String
Dim sorted As String

1975:   ParseStringR (str), sortArray
  
1977:   min = LBound(sortArray)
1978:   max = UBound(sortArray)
  
1980:   For i = min To max - 1
1981:     best_value = sortArray(i)
1982:     best_j = i
    
1984:     For J = i + 1 To max
1985:       If Val(sortArray(J)) < Val(best_value) Then
1986:         best_value = sortArray(J)
1987:         best_j = J
1988:       End If
1989:     Next J
      
1991:     sortArray(best_j) = sortArray(i)
1992:     sortArray(i) = best_value
1993:   Next i
    
1995:   If UCase(SortOption) = "-DESCEND" Then
1996:     For i = max To min Step -1
1997:       sorted = sorted & sortArray(i) & ","
1998:     Next i
1999:   Else
2000:     For i = min To max
2001:       sorted = sorted & sortArray(i) & ","
2002:     Next i
2003:   End If
  
2005:   Mid(sorted, Len(sorted), 1) = " "
2006:   str = sorted
2007:   Sort_Num = sorted


  Exit Function
ErrorHandler:
  HandleError False, "Sort_Num " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Function Sort_Char(str As String, SortOption, Optional ReturnQuoted) As String
  On Error GoTo ErrorHandler


' Sort function - performs a character sort
' Ref: Selectionsort Chapter8 of VB Algorithms; Rod Stephens

Dim i As Long
Dim J As Long
Dim min As Long
Dim max As Long
Dim best_value As String
Dim best_j As Long
Dim sortArray() As String
Dim sorted As String

2031: If IsMissing(ReturnQuoted) Then
2032:   ReturnQuoted = False
2033: End If
2034: If Not (ReturnQuoted = False) Then
2035:   ReturnQuoted = True
2036: End If

2038: ParseStringR (str), sortArray, ReturnQuoted

2040: min = LBound(sortArray)
2041: max = UBound(sortArray)

2043:   For i = min To max - 1
2044:     best_value = sortArray(i)
2045:     best_j = i
    
2047:     For J = i + 1 To max
2048:       If sortArray(J) < best_value Then
2049:       best_value = sortArray(J)
2050:       best_j = J
2051:       End If
2052:     Next J
    
2054:     sortArray(best_j) = sortArray(i)
2055:     sortArray(i) = best_value
2056:   Next i
  
2058:   If UCase(SortOption) = "-DESCEND" Then
2059:     For i = max To min Step -1
2060:       sorted = sorted & sortArray(i) & " "
2061:     Next i
2062:   Else
2063:     For i = min To max
2064:       sorted = sorted & sortArray(i) & " "
2065:     Next i
2066:   End If
2067:   Mid(sorted, Len(sorted), 1) = " "
  
2069:   str = sorted
2070:   Sort_Char = sorted


  Exit Function
ErrorHandler:
  HandleError False, "Sort_Char " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Subst(str As String, SearchChar As String, Optional ReplaceChar) As String
  On Error GoTo ErrorHandler


' Replaces all occurances of specified char in string.

Dim complete As Boolean
Dim i As Long
Dim first As String
Dim last As String
Dim tmpstr As String
Dim position As Long

2091: tmpstr = str

2093: position = 1
2094: complete = False

2096: If IsMissing(ReplaceChar) Then
2097:   Do Until complete = True
2098:     position = InStr(position, tmpstr, SearchChar)
2099:     If position = 0 Or position > Len(tmpstr) Then
2100:       complete = True
2101:     Else
2102:       first = Before(tmpstr, SearchChar)
2103:       last = After(tmpstr, SearchChar)
2104:       tmpstr = first & last
2105:     End If
2106:   Loop
2107: End If

2109: Do Until complete = True
2110:   position = InStr(position, tmpstr, SearchChar)
2111:   If position = 0 Or position > Len(tmpstr) Then
2112:     complete = True
2113:   Else
2114:     Mid(tmpstr, position, Len(ReplaceChar)) = ReplaceChar
2115:     position = position + Len(ReplaceChar)
2116:   End If
2117: Loop

Subst = tmpstr


  Exit Function
ErrorHandler:
  HandleError True, "Subst " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Substr(str As String, position As Long, Optional NumChars) As String
  On Error GoTo ErrorHandler


'extracts a substring starting at a specified character position.

2133: If IsMissing(NumChars) Then
2134:   If position = 0 Or position > Len(str) Then
2135:     Substr = ""
2136:   Else
2137:     Substr = Mid(str, position)
2138:   End If
2139: Else
2140:   If position = 0 Or position > Len(str) Then
2141:     Substr = ""
2142:   Else
2143:    Substr = Mid(str, position, NumChars)
2144:   End If
2145: End If


  Exit Function
ErrorHandler:
  HandleError True, "Substr " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Token(ElemList As String, Arg As String, ParamArray OtherArgs()) As Variant
  On Error GoTo ErrorHandler


' Performs various functions for string manipulation

Dim strArray() As String
Dim i As Long
Dim temp As String
Dim from_elem  As Long
Dim to_elem As Long
Dim start_elem As Long
Dim insertStr As String
Dim DELETE As Long
Dim SearchStr As String

' Parse ElemList out to strarray
' Select TOKEN argument and perform function

2172:   ParseStringR (ElemList), strArray
2173:   Arg = Subst(Arg, "-")
  
  Select Case UCase(Arg)

  ' Count - returns the number of tokens in a list
    Case "COUNT"
2179:       Token = UBound(strArray) + 1
    
  ' Find <token> - returns the position of a token in a list
    Case "FIND"
2183:       SearchStr = OtherArgs(0)
2184:       Token = 0
2185:       For i = 0 To UBound(strArray)
2186:         If UCase(SearchStr) = UCase(strArray(i)) Then
2187:           Token = i + 1
2188:         End If
2189:       Next i
    
  ' Move <from_position> <to_position> - moves a token in the list
    Case "MOVE"
2193:       from_elem = OtherArgs(0) - 1
2194:       to_elem = OtherArgs(1)
2195:       temp = strArray(to_elem)
2196:       strArray(to_elem) = strArray(from_elem)
2197:       For i = from_elem To to_elem - 1
2198:         strArray(i) = strArray(i + 1)
2199:         Next i
2200:          strArray(i) = temp
2201:       For i = 0 To UBound(strArray) - 1
2202:         Token = Token & strArray(i) & ","
2203:       Next i
    
  ' Insert <position> - inserts a new token at <position> in the list
    Case "INSERT"
      ReDim Preserve strArray(UBound(strArray) + 1)
2208:       start_elem = OtherArgs(0) - 1
2209:       insertStr = OtherArgs(1)
2210:       For i = UBound(strArray) To start_elem Step -1
2211:         strArray(i) = strArray(i - 1)
2212:         Next i
2213:         strArray(start_elem) = insertStr
2214:         For i = 0 To UBound(strArray) - 1
2215:         Token = Token & strArray(i) & ","
2216:       Next i
    
  ' Delete <position> - removes the token at <position> from the list.
    Case "DELETE"
2220:       DELETE = OtherArgs(0) - 1
2221:       For i = DELETE To UBound(strArray) - 1
2222:         strArray(i) = strArray(i + 1)
2223:       Next i
2224:       For i = 0 To UBound(strArray) - 1
2225:         Token = Token & strArray(i) & ","
2226:       Next i
    
  ' Replace <position> <new_string> - replaces the token at <position> with the
  ' <new_string>.
    Case "REPLACE"
2231:       strArray(OtherArgs(1) - 1) = OtherArgs(0)
2232:       For i = 0 To UBound(strArray)
2233:         Token = Token & strArray(i) & ","
2234:       Next i
    
  ' Switch <position_1> <position_2> - moves token at <position_1> to <position_2> and moves
  ' token at <position_2> to <position_1>.
    Case "SWITCH"
2239:       from_elem = OtherArgs(0) - 1
2240:       to_elem = OtherArgs(1) - 1
2241:       temp = strArray(to_elem)
2242:       strArray(to_elem) = strArray(from_elem)
2243:       strArray(from_elem) = temp
2244:       For i = 0 To UBound(strArray)
2245:         Token = Token & strArray(i) & ","
2246:       Next i
    
    Case Else
2249:   End Select


  Exit Function
ErrorHandler:
  HandleError True, "Token " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function IsVersion10() As Boolean
  On Error GoTo ErrorHandler


  Dim strVersion As String
2262:   strVersion = ReturnArcGISVersion()
  
2264:   If InStr(1, strVersion, ".") Then
    Dim strTokens() As String
2266:     strTokens = Split(strVersion, ".")
2267:     strVersion = strTokens(0)
2268:   End If
  
2270:   strVersion = Trim(strVersion)
2271:   IsVersion10 = CBool(strVersion = "10")


  Exit Function
ErrorHandler:
  HandleError True, "IsVersion10 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function



