Attribute VB_Name = "MyGeometricOperations"
' MyGeometricOperations
'----------------------------------------------------------------------------------
'  Jeff Jenness
'  Jenness Enterprises
'  3020 N. Schevene Blvd.
'  Flagstaff, AZ  86004
'  http://www.jennessent.com
'  jeffj@jennessent.com
'  phone:  1-928-607-4638
'----------------------------------------------------------------------------------
'                 ArcCosJen - Returns ArcCos (Inverse Cosine)
'                 ArcSinJen - Returns ArcSin (Inverse Sine)
'                 AsDegrees - CONVERTS RADIANS TO DEGREES
'                 AsRadians - CONVERTS DEGREES TO RADIANS
'                     atan2 - Given DeltaY and DeltaX, returns ArcTangent that is sensitive to quadrant.
'          AzimuthHaversine - GIVEN TWO GEOGRAPHIC POINTS, RETURNS STARTING AZIMUTH POINTING IN GREAT CIRCLE OVER SPHERE.
'               CalcBearing - GIVEN TWO POINTS, CALCULATES THE CARTESIAN BEARING, WHERE 0 = NORTH, 360 DEGREES GOING CLOCKWISE
'              CalcBearing2 - SAME AS CalcBearing, EXCEPT IT RETURNS -999 IF THE TWO POINTS ARE COINCIDENT.
'        CalcCheckClockwise - CHECKS IF 3 CONSECUTIVE POINTS ARE ARRANGED COUNTERCLOCKWISE
'         CalcClosestPoints - GIVEN TWO GEOMETRIES AND OPTIONAL NUMBER OF TIMES TO GO BACK AND FORTH BETWEEN CURVES, RETURNS
'                             IArray CONTAINING:
'                             IStringArray containing either "Intersecting Shapes" or
'                                "Empty Shapes" + two booleans indicating which geometry is empty.
'                             -- OR --
'                             3 OBJECTS:
'                             0) Connector Line AS IPOLYLINE
'                             1) Closest Point on Geometry #1         AS IPOINT
'                             2) Closest Point on Geometry #2         AS IPOINT
'            CalcDistMatrix - GIVEN AN IARRAY OF SHAPES, RETURNS A COLLECTION WHERE:
'                             INDEX = IArrayIndex1 & "_" & IArrayIndex1, and Object =
'                             IArray of {Distance, Optional Line, Optional Azimuth}
'             CalcPointLine - GIVEN POINT, DISTANCE, AZIMUTH, EMPTY ENDPOINT AND EMPTY POLYLINE, REPLACES EMPTY
'                             ENDPOINT WITH ACTUAL ENDPOINT AND OPTIONALLY RETURNS A POLYLINE CONNECTOR
'            CurveToPolygon - SIMILAR TO EllipticArcToPolygon2 EXCEPT THAT IT RETURNS AN IPolygon.  DOESN'T INSERT
'                             POINTS IF SEGMENT IS A LINE
'           CurveToPolyline - SIMILAR TO EllipticArcToPolygon2 EXCEPT THAT IT RETURNS AN IPolyline.  DOESN'T INSERT
'                             POINTS IF SEGMENT IS A LINE
'                  DegToRad - Given Degrees (double), returns Radians (double) (ACCIDENTALLY DUPLICATED THIS FUNCTION!  SEE AsRadians)
'         DistanceHaversine - GIVEN Point A, Point B and optional earth radius, returns distance between points in meters.
'                             Less accurate but faster than using Vincenty's functions.
'    DistanceVincentyPoints - Given 2 points and empty double variables for azimuths, returns distance in meters (using WGS84)
'                             and start and end azimuths of geodesic curve.
'   DistanceVincentyPoints2 - MODIFICATION OF DistanceVincentyPoints, TO ALLOW FOR ANY ELLIPSOID
'   DistanceVincentyNumbers - Given numeric values for latitude and longitude for 2 points, plus empty double variables
'                             for azimuths, returns distance in meters (using WGS84) and start and end azimuths of geodesic curve.
'  DistanceVincentyNumbers2 - MODIFICATION OF DistanceVincentyNumbers, TO ALLOW FOR ANY ELLIPSOID
'      EllipticArcToPolygon - Given a segment collection and number of vertices, returns a polygon4 simulating the ellipse
'                             by generating points along the arc and then calculating a convex hull around the points.
'     EllipticArcToPolygon2 - GIVEN A SEG COLLECTION AND NUMBER OF VERTICES, RETURNS A MULTIPOINT WITH APPROXIMATELY THE
'                             REQUESTED NUMBER OF POINTS DISTRIBUTED ALONG THE ARC.
'         EnvelopeToPolygon - GIVEN AN ENVELOPE, RETURNS A POLYGON
'FeaturePlanetOCentricToPlanetOGraphic - RETURNS A NEW GEOMETRY (POINT, POLYLINE, POLYGON OR MULTIPOINT) IN WHICH EACH VERTEX
'                             HAS BEEN SHIFTED FROM AN OCENTRIC TO AN OGRAPHIC LOCATION.
'FeaturePlanetOGraphicToPlanetOCentric - RETURNS A NEW GEOMETRY (POINT, POLYLINE, POLYGON OR MULTIPOINT) IN WHICH EACH VERTEX
'                             HAS BEEN SHIFTED FROM AN OGRAPHIC TO AN OCENTRIC LOCATION.
'  Graphic_MakeFromGeometry - GIVEN A MAP DOCUMENT, GEOMETRY AND OPTIONAL NAME AND SYMBOLOGY, ADDS GRAPHIC TO MAP.
'Graphic_ReturnElementFromGeometry - GIVEN MAP DOC, GEOMETRY, OPTIONAL NAME AND OPTIONAL ADD-TO-VIEW, RETURNS THE GRAPHIC
'                                    ELEMENT
'                   HArcCos - GIVEN A VALUE, RETURNS INVERSE HYPERBOLIC COSINE
'                   HArcSin - GIVEN A VALUE, RETURNS INVERSE HYPERBOLIC SINE
'                   HArcTan - GIVEN A VALUE, RETURNS INVERSE HYPERBOLIC TANGENT
'                      HCos - GIVEN A VALUE IN RADIANS, RETURNS HYPERBOLIC COSINE
'                      HSin - GIVEN A VALUE IN RADIANS, RETURNS HYPERBOLIC SINE
'                      HTan - GIVEN A VALUE IN RADIANS, RETURNS HYPERBOLIC TANGENT
'        MultipointCentroid - GIVEN AN IMULTIPOINT, RETURNS AN IPOINT REPRESENTING AVERAGE OF POINTS
'  MultipointCentroidSphere - GIVEN A GEOGRAPHIC IMULTIPOINT, RETURNS AN IPOINT REPRESENTING AVERAGE ON SPHERE
'MultipointCentroidSpheriod - GIVEN A GEOGRAPHIC IMULTIPOINT, RETURNS AN IPOINT REPRESENTING AVERAGE ON SPHEROID
'                  PointAdd - ADDS TWO POINTS
'         PointLineVincenty - Given Point, Distance and Azimuth, plus empty NewPoint and End Azimuth, plus optional
'                             number of vertices and empty Polyline, returns a new point the specified distance from the
'                             origin along geodesic, plus optional polyline with specified number of vertices.
'        PointLineVincenty2 - MODIFICATION OF PointLineVincenty, TO ALLOW FOR ANY ELLIPSOID
' PointLineVincentyPerPoint - Given pPoint, dblLength, dblAzimuth, empty NewPoint and empty End Azimuth,
'                             fills empty point and azimuth with correct values.
'PointLineVincentyPerPoint2 - MODIFICATION OF PointLineVincentyPerPoint, TO ALLOW FOR ANY ELLIPSOID
'             PointSubtract - SUBTRACTS POINT B FROM POINT A
'         PolygonToPolyline - GIVEN A POLYGON, RETURNS A POLYLINE
'                  RadToDeg - Given Radians (double), returns Degrees (double) (ACCIDENTALLY DUPLICATED THIS FUNCTION!  SEE AsDegrees)
'              ShowVertices - GIVEN A MAP DOC, GEOMETRY AND OPTIONAL NAME, ADDS POINT GRAPHICS TO SCREEN SHOWING WHERE
'                             VERTICES ARE
'    SphericalLatLongToCart - SUBROUTINE:  GIVEN LATITUDE AND LONGITUDE, AND OPTIONAL RADIUS, FILLS X, Y, Z VALUES
'    SphericalCartToLatLong - SUBROUTINE:  GIVEN X, Y, Z, FILLS LATITUDE AND LONGITUDE VALUES
'      SphericalPolygonArea - GIVEN A POLYGON, CALCULATES AREA USING SERIES OF SphericalTriangleArea CALLS
'     SphericalPolygonArea2 - MODIFICATION OF SphericalPolygonArea TO ALLOW USER TO SET CUSTOM ELLIPSOID MAJOR AND MINOR AXES
'   SpheroidalCartToLatLong - GIVEN X, Y, Z, OPTIONAL SPHEROID RADII AND HEIGHT ABOVE ELLPISOID, FILLS LATITUDE AND LONGITUDE VALUES
'   SpheroidalLatLongToCart - GIVEN LATITUDE, LONGITUDE, OPTIONAL SPHEROID RADII AND HEIGHT ABOVE ELLPISOID, FILLS X, Y, Z VALUES
'  SpheroidalPolylineLength - GIVEN A GEOGRAPHIC POLYLINE, RETURNS LENGTH IN METERS BASED ON VINCENTY'S EQUATIONS
' SpheroidalPolylineLength2 - MODIFICATION OF SpheroidalPolylineLength, TO ALLOW FOR ANY ELLIPSOID
'SpheroidalPolylineMidpoint - GIVEN A GEOGRAPHIC POLYLINE, DISTANCE VALUE, booAsRatio, RETURNS POINT AND OPTIONAL POLYLINE DISTANCE
'SpheroidalPolylineMidpoint2 - MODIFICATION OF SpheroidalPolylineMidpoint, TO ALLOW FOR ANY ELLIPSOID
'     SphericalTriangleArea - GIVEN 3 GEOGRAPHIC POINTS, CALCULATES SPHERICAL AREA
'    SphericalTriangleArea2 - MODIFICATION OF SphericalTriangleArea, TO ALLOW USER TO OPTIONALLY SEND CUSTOM MAJOR AND MINOR ELLIPSOID AXES
'        TriangleAreaPoints - GIVEN THREE POINTS, RETURNS AREA IN LOCAL UNITS
'      TriangleAreaPoints3D - GIVEN THREE 3-DIMENSIONAL POINTS, RETURNS AREA IN LOCAL UNITS
'        TriangleCentroid3D - GIVEN X,Y,Z COORDINATES FOR THREE 3D POINTS, RETURNS 3D CENTROID
'     TriangleCentroidPlane - GIVEN X,Y COORDINATES FOR THREE POINTS, RETURNS CENTROID
'      XYOCentricToOGraphic - GIVEN LONGITUDE, LATITUDE, ELLIPSOID MAJOR AND MINOR AXES, AND OPTIONAL LONGITUDE SHIFT, SETS NEW
'                             LATITUDE AND LONGITUDE VALUES BY CONVERTING FROM OCENTRIC TO OGRAPHIC.
'      XYOGraphicToOCentric - GIVEN LONGITUDE, LATITUDE, ELLIPSOID MAJOR AND MINOR AXES, AND OPTIONAL LONGITUDE SHIFT, SETS NEW
'                             LATITUDE AND LONGITUDE VALUES BY CONVERTING FROM OGRAPHIC TO OCENTRIC.

Option Explicit

Const dblPi As Double = 3.14159265358979
Const dblE As Double = 2.71828182845905
' Variables used by the Error handler function - DO NOT REMOVE
Const c_sModuleFileName As String = "D:\arcGIS_stuff\general_tools\Registration\MyGeometricOperations.bas"


Public Function DistanceHaversine(pPointA As IPoint, pPointB As IPoint, Optional dblRadius As Double = 6371000.79000915) As Double
  On Error GoTo ErrorHandler

  
  Dim dblLat1 As Double
  Dim dblLat2 As Double
  Dim dblLat As Double
  Dim dblLong As Double
  Dim dblTemp As Double
  
120:   dblLat1 = DegToRad(pPointA.Y)
121:   dblLat2 = DegToRad(pPointB.Y)
122:   dblLat = dblLat1 - dblLat2
123:   dblLong = DegToRad(pPointA.X - pPointB.X)
124:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
125:   DistanceHaversine = (2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))) * dblRadius


  Exit Function
ErrorHandler:
  HandleError True, "DistanceHaversine " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function MultipointCentroidSpheroid(pMultipoint As IMultipoint, Optional dblEquatorialRadius As Double = 6378137, _
    Optional dblPolarRadius As Double = 6356752.31424518, Optional dblHeightAboveEllipsoid As Double = 0) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim dblX As Double
  Dim dblY As Double
  Dim dblZ As Double
  Dim dblRunningX As Double
  Dim dblRunningY As Double
  Dim dblRunningZ As Double
  Dim pPointCollection As IPointCollection
146:   Set pPointCollection = pMultipoint
  Dim lngIndex As Long
148:   dblRunningX = 0
149:   dblRunningY = 0
150:   dblRunningZ = 0
151:   Set pPoint = New Point
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  
  Dim lngCounter As Long
156:   lngCounter = pPointCollection.PointCount
157:   For lngIndex = 0 To lngCounter - 1
158:     pPointCollection.QueryPoint lngIndex, pPoint
159:     SpheroidalLatLongToCart pPoint.X, pPoint.Y, dblX, dblY, dblZ, dblEquatorialRadius, dblPolarRadius, dblHeightAboveEllipsoid
160:     dblRunningX = dblRunningX + dblX
161:     dblRunningY = dblRunningY + dblY
162:     dblRunningZ = dblRunningZ + dblZ
163:   Next lngIndex
  
165:   dblX = dblRunningX / lngCounter
166:   dblY = dblRunningY / lngCounter
167:   dblZ = dblRunningZ / lngCounter
  
  ' CONVERT BACK TO GEOGRAPHIC COORDINATES
170:   SpheroidalCartToLatLong dblLongitude, dblLatitude, dblX, dblY, dblZ, dblEquatorialRadius, dblPolarRadius, dblHeightAboveEllipsoid
171:   Set MultipointCentroidSpheroid = New Point
172:   MultipointCentroidSpheroid.PutCoords dblLongitude, dblLatitude


  Exit Function
ErrorHandler:
  HandleError True, "MultipointCentroidSpheroid " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function MultipointCentroidSphere(pMultipoint As IMultipoint) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim dblX As Double
  Dim dblY As Double
  Dim dblZ As Double
  Dim dblRunningX As Double
  Dim dblRunningY As Double
  Dim dblRunningZ As Double
  Dim pPointCollection As IPointCollection
192:   Set pPointCollection = pMultipoint
  Dim lngIndex As Long
194:   dblRunningX = 0
195:   dblRunningY = 0
196:   dblRunningZ = 0
197:   Set pPoint = New Point
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  
  Dim lngCounter As Long
202:   lngCounter = pPointCollection.PointCount
203:   For lngIndex = 0 To lngCounter - 1
204:     pPointCollection.QueryPoint lngIndex, pPoint
205:     SphericalLatLongToCart pPoint.X, pPoint.Y, dblX, dblY, dblZ
206:     dblRunningX = dblRunningX + dblX
207:     dblRunningY = dblRunningY + dblY
208:     dblRunningZ = dblRunningZ + dblZ
209:   Next lngIndex
  
211:   dblX = dblRunningX / lngCounter
212:   dblY = dblRunningY / lngCounter
213:   dblZ = dblRunningZ / lngCounter
  
  ' CONVERT BACK TO GEOGRAPHIC COORDINATES
216:   SphericalCartToLatLong dblLongitude, dblLatitude, dblX, dblY, dblZ
217:   Set MultipointCentroidSphere = New Point
218:   MultipointCentroidSphere.PutCoords dblLongitude, dblLatitude


  Exit Function
ErrorHandler:
  HandleError True, "MultipointCentroidSphere " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function MultipointCentroid(pMultipoint As IMultipoint) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim dblX As Double
  Dim dblY As Double
  Dim pPointCollection As IPointCollection
233:   Set pPointCollection = pMultipoint
  
  Dim lngIndex As Long
236:   dblX = 0
237:   dblY = 0
238:   Set pPoint = New Point
239:   For lngIndex = 0 To pPointCollection.PointCount - 1
240:     pPointCollection.QueryPoint lngIndex, pPoint
241:     dblX = dblX + pPoint.X
242:     dblY = dblY + pPoint.Y
243:   Next lngIndex
  
245:   Set MultipointCentroid = New Point
246:   MultipointCentroid.PutCoords dblX, dblY


  Exit Function
ErrorHandler:
  HandleError True, "MultipointCentroid " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function PolygonToPolyline(pPolygon As IPolygon2) As IPolyline
  On Error GoTo ErrorHandler

  
  Dim pNewGeometryCollection As IGeometryCollection
259:   Set pNewGeometryCollection = New Polyline
  Dim lngNumParts As Long
  Dim pPolyComponents() As IPolygon 'Declare an array of polygon
  Dim lngIndex As Long
  Dim pSubPolygon As IPolygon4
  Dim pTopoOp As ITopologicalOperator
  Dim pGeometryBag As IGeometryCollection
  Dim pNewSegCollection As ISegmentCollection
  Dim pOrigSegcollection As ISegmentCollection
  Dim pRing As IRing
  Dim pPath As IPath
  Dim pSpRef As ISpatialReference
  Dim lngIndex2 As Long
  Dim pPolyline As IPolyline
  
274:   Set pSpRef = pPolygon.SpatialReference
275:   Set pNewGeometryCollection = New Polyline
  
  ' GET CONNECTED COMPONENTS OF POLYGON
278:   lngNumParts = pPolygon.ExteriorRingCount
  ReDim pPolyComponents(lngNumParts - 1) 'Redimension the array of polygons with number of exterior rings
280:   pPolygon.GetConnectedComponents lngNumParts, pPolyComponents(0) 'Pass the first element of the array
  
282:   For lngIndex = 0 To lngNumParts - 1
283:     Set pSubPolygon = pPolyComponents(lngIndex)
284:     Set pTopoOp = pSubPolygon
285:     pTopoOp.Simplify
    
    ' GET SINGLE EXTERNAL RING AND ALL INTERNAL RINGS
288:     Set pGeometryBag = pSubPolygon.ExteriorRingBag
289:     Set pRing = pGeometryBag.Geometry(0)
290:     Set pOrigSegcollection = pRing
291:     Set pNewSegCollection = New Path
292:     pNewSegCollection.AddSegmentCollection pOrigSegcollection
293:     Set pPath = pNewSegCollection
294:     pNewGeometryCollection.AddGeometry pPath
    
    ' ADD ANY INTERNAL RINGS TO POLYLINE
297:     If pSubPolygon.InteriorRingCount(pRing) > 0 Then
298:       Set pGeometryBag = pSubPolygon.InteriorRingBag(pRing)
299:       For lngIndex2 = 0 To pGeometryBag.GeometryCount - 1
300:         Set pRing = pGeometryBag.Geometry(lngIndex2)
301:         Set pOrigSegcollection = pRing
302:         Set pNewSegCollection = New Path
303:         pNewSegCollection.AddSegmentCollection pOrigSegcollection
304:         Set pPath = pNewSegCollection
305:         pNewGeometryCollection.AddGeometry pPath
306:       Next lngIndex2
307:     End If
308:   Next lngIndex
  
  ' CLEAN NEW POLYLINE
311:   Set pPolyline = pNewGeometryCollection
312:   Set pTopoOp = pPolyline
313:   pTopoOp.Simplify
314:   Set pPolyline.SpatialReference = pSpRef

316:   Set PolygonToPolyline = pPolyline


  Exit Function
ErrorHandler:
  HandleError True, "PolygonToPolyline " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub TriangleCentroid3D(dblPX As Double, dblPY As Double, dblPZ As Double, _
                                           dblQX As Double, dblQY As Double, dblQZ As Double, _
                                           dblRX As Double, dblRY As Double, dblRZ As Double, _
                                           dblCentX As Double, dblCentY As Double, dblCentZ As Double)
  On Error GoTo ErrorHandler


331:   dblCentX = (dblPX + dblQX + dblRX) / 3
332:   dblCentY = (dblPY + dblQY + dblRY) / 3
333:   dblCentZ = (dblPZ + dblQZ + dblRZ) / 3


  Exit Sub
ErrorHandler:
  HandleError True, "TriangleCentroid3D " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub TriangleCentroidPlane(dblPX As Double, dblPY As Double, dblQX As Double, dblQY As Double, _
                                           dblRX As Double, dblRY As Double, dblCentX As Double, dblCentY As Double)
  On Error GoTo ErrorHandler


346:   dblCentX = (dblPX + dblQX + dblRX) / 3
347:   dblCentY = (dblPY + dblQY + dblRY) / 3


  Exit Sub
ErrorHandler:
  HandleError True, "TriangleCentroidPlane " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub


Public Sub SphericalCartToLatLong(dblLongitude As Double, dblLatitude As Double, X As Double, _
      Y As Double, Z As Double)
  On Error GoTo ErrorHandler


  ' Phi is angle from north pole down to Latitude
  ' Theta is angle from Greenwich
  
  Dim dblPhi As Double
  Dim dblTheta As Double
  
367:   dblPhi = atan2(Sqr(X ^ 2 + Y ^ 2), Z)
368:   dblTheta = atan2(Y, X)
  
370:   dblLongitude = RadToDeg(dblTheta)
371:   dblLatitude = 90 - RadToDeg(dblPhi)


  Exit Sub
ErrorHandler:
  HandleError True, "SphericalCartToLatLong " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub


Public Sub SpheroidalCartToLatLong(dblLongitude As Double, dblLatitude As Double, X As Double, _
      Y As Double, Z As Double, Optional dblEquatorialRadius As Double = 6378137, _
      Optional dblPolarRadius As Double = 6356752.31424518, Optional dblHeightAboveEllipsoid As Double = 0)
  On Error GoTo ErrorHandler


  ' IF SPHEROID PARAMETERS NOT INCLUDED, DEFAULTS TO WGS84
  ' Phi is angle from north pole down to Latitude
  ' Theta is angle from Greenwich
  
  ' MODIFIED FROM J.C. ILIFFE, CHAPTER 2
  ' NOTE:  ILIFFE USES PHI FOR LATITUDE DIRECTLY, RATHER THAN AS DISTANCE FROM POLES
  
  Dim dblPhi As Double
  Dim dblTheta As Double
  
  
  Dim dblP As Double
398:   dblP = Sqr(X ^ 2 + Y ^ 2)
  
  Dim dblU As Double
401:   dblU = atan2((Z * dblEquatorialRadius), (dblP * dblPolarRadius))
  
  Dim dblEccentSquared As Double
404:   dblEccentSquared = ((dblEquatorialRadius ^ 2) - (dblPolarRadius ^ 2)) / (dblEquatorialRadius ^ 2)
  
  Dim dblEpsilon As Double
407:   dblEpsilon = dblEccentSquared / (1 - dblEccentSquared)
  
409:   dblPhi = atan2(dblP - (dblEccentSquared * dblEquatorialRadius * (Cos(dblU) ^ 3)), _
                  Z + (dblEpsilon * dblPolarRadius * (Sin(dblU) ^ 3)))
411:   dblTheta = atan2(Y, X)
  
413:   dblLongitude = RadToDeg(dblTheta)
414:   dblLatitude = 90 - RadToDeg(dblPhi)


  Exit Sub
ErrorHandler:
  HandleError True, "SpheroidalCartToLatLong " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub SphericalLatLongToCart(dblLongitude As Double, dblLatitude As Double, X As Double, _
      Y As Double, Z As Double, Optional dblRadius = 1)
  On Error GoTo ErrorHandler


  ' Phi is angle from north pole down to Latitude
  ' Theta is angle from Greenwich
  
  Dim dblPhi As Double
  Dim dblTheta As Double
  
433:   dblPhi = DegToRad(90 - dblLatitude)
434:   dblTheta = DegToRad(dblLongitude)
  
436:   X = dblRadius * Sin(dblPhi) * Cos(dblTheta)
437:   Y = dblRadius * Sin(dblPhi) * Sin(dblTheta)
438:   Z = dblRadius * Cos(dblPhi)


  Exit Sub
ErrorHandler:
  HandleError True, "SphericalLatLongToCart " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub


Public Sub SpheroidalLatLongToCart(dblLongitude As Double, dblLatitude As Double, X As Double, _
      Y As Double, Z As Double, Optional dblEquatorialRadius As Double = 6378137, _
      Optional dblPolarRadius As Double = 6356752.31424518, Optional dblHeightAboveEllipsoid As Double = 0)
  On Error GoTo ErrorHandler

  
  ' IF SPHEROID PARAMETERS NOT INCLUDED, DEFAULTS TO WGS84

  ' Phi is angle from north pole down to Latitude
  ' Theta is angle from Greenwich
  
  ' MODIFIED FROM J.C. ILIFFE, CHAPTER 2
  ' NOTE:  ILIFFE USES PHI FOR LATITUDE DIRECTLY, RATHER THAN AS DISTANCE FROM POLES
  
  Dim dblPhi As Double
  Dim dblTheta As Double
  
464:   dblPhi = DegToRad(90 - dblLatitude)
465:   dblTheta = DegToRad(dblLongitude)
  
  Dim dblNu As Double
  Dim dblEccentSquared As Double
469:   dblEccentSquared = ((dblEquatorialRadius ^ 2) - (dblPolarRadius ^ 2)) / (dblEquatorialRadius ^ 2)
470:   dblNu = (dblEquatorialRadius) / Sqr(1 - (dblEccentSquared * (Cos(dblPhi) ^ 2)))
  
472:   X = (dblNu + dblHeightAboveEllipsoid) * Sin(dblPhi) * Cos(dblTheta)
473:   Y = (dblNu + dblHeightAboveEllipsoid) * Sin(dblPhi) * Sin(dblTheta)
474:   Z = (((1 - dblEccentSquared) * dblNu) + dblHeightAboveEllipsoid) * Cos(dblPhi)


  Exit Sub
ErrorHandler:
  HandleError True, "SpheroidalLatLongToCart " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function SpheroidalPolylineMidpoint(pPolyline As IPolyline, dblDistance As Double, _
    booIsRatio As Boolean, Optional dblPolylineLength As Double) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim pVarArray As esriSystem.IVariantArray
  Dim pDblArray As esriSystem.IDoubleArray
490:   Set pVarArray = New esriSystem.VarArray
  
  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  Dim dblX1 As Double
  Dim dblY1 As Double
  
504:   dblTotalLength = 0
505:   Set pSegCollection = pPolyline
  
  ' GET TOTAL LENGTH AND FILL ARRAY WITH SEGMENT STATISTICS
508:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
509:     Set pSeg = pSegCollection.Segment(lngIndex)
510:     Set pPoint1 = pSeg.FromPoint
511:     Set pPoint2 = pSeg.ToPoint
512:     dblLength = DistanceVincentyPoints(pPoint1, pPoint2, dblAz1, dblAz2)
513:     dblTotalLength = dblTotalLength + dblLength
514:     Set pDblArray = New esriSystem.DoubleArray
515:     pDblArray.Add dblLength
516:     pDblArray.Add dblTotalLength
517:     pDblArray.Add dblAz1
518:     pDblArray.Add pPoint1.X
519:     pDblArray.Add pPoint1.Y
520:     pVarArray.Add pDblArray
521:   Next lngIndex
  
523:   dblPolylineLength = dblTotalLength
  
  Dim dblHalfLength As Double
526:   If booIsRatio Then
527:     If dblDistance > 1 Then dblDistance = 1
528:     If dblDistance < 0 Then dblDistance = 0
529:     dblHalfLength = dblPolylineLength * dblDistance
530:   Else
531:     dblHalfLength = dblDistance
532:   End If
    
534:   For lngIndex = 0 To pVarArray.Count - 1
535:     Set pDblArray = pVarArray.Element(lngIndex)
536:     dblTotalLength = pDblArray.Element(1)
537:     If dblTotalLength > dblHalfLength Then
538:       dblLength = pDblArray.Element(0)
539:       dblAz1 = pDblArray.Element(2)
540:       dblX1 = pDblArray.Element(3)
541:       dblY1 = pDblArray.Element(4)
542:       Exit For
543:     End If
544:   Next lngIndex
  
  Dim dblProperDistance As Double
547:   dblProperDistance = dblLength - (dblTotalLength - dblHalfLength)
  
549:   Set pPoint1 = New Point
550:   pPoint1.PutCoords dblX1, dblY1
551:   Set pPoint2 = New Point
    
553:   PointLineVincenty pPoint1, dblProperDistance, dblAz1, pPoint2, dblAz2
  
555:   Set SpheroidalPolylineMidpoint = pPoint2


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineMidpoint " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function SpheroidalPolylineMidpoint2(pPolyline As IPolyline, dblDistance As Double, _
    booIsRatio As Boolean, Optional dblPolylineLength As Double, _
    Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As IPoint
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF SpheroidalPolylineMidpoint, TO ALLOW FOR ANY ELLIPSOID
  
  Dim pPoint As IPoint
  Dim pVarArray As esriSystem.IVariantArray
  Dim pDblArray As esriSystem.IDoubleArray
573:   Set pVarArray = New esriSystem.VarArray
  
  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  Dim dblX1 As Double
  Dim dblY1 As Double
  
587:   dblTotalLength = 0
588:   Set pSegCollection = pPolyline
  
  ' GET TOTAL LENGTH AND FILL ARRAY WITH SEGMENT STATISTICS
591:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
592:     Set pSeg = pSegCollection.Segment(lngIndex)
593:     Set pPoint1 = pSeg.FromPoint
594:     Set pPoint2 = pSeg.ToPoint
595:     dblLength = DistanceVincentyPoints2(pPoint1, pPoint2, dblAz1, dblAz2, dblEquatorialRadius, dblPolarRadius)
596:     dblTotalLength = dblTotalLength + dblLength
597:     Set pDblArray = New esriSystem.DoubleArray
598:     pDblArray.Add dblLength
599:     pDblArray.Add dblTotalLength
600:     pDblArray.Add dblAz1
601:     pDblArray.Add pPoint1.X
602:     pDblArray.Add pPoint1.Y
603:     pVarArray.Add pDblArray
604:   Next lngIndex
  
606:   dblPolylineLength = dblTotalLength
  
  Dim dblHalfLength As Double
609:   If booIsRatio Then
610:     If dblDistance > 1 Then dblDistance = 1
611:     If dblDistance < 0 Then dblDistance = 0
612:     dblHalfLength = dblPolylineLength * dblDistance
613:   Else
614:     dblHalfLength = dblDistance
615:   End If
    
617:   For lngIndex = 0 To pVarArray.Count - 1
618:     Set pDblArray = pVarArray.Element(lngIndex)
619:     dblTotalLength = pDblArray.Element(1)
620:     If dblTotalLength > dblHalfLength Then
621:       dblLength = pDblArray.Element(0)
622:       dblAz1 = pDblArray.Element(2)
623:       dblX1 = pDblArray.Element(3)
624:       dblY1 = pDblArray.Element(4)
625:       Exit For
626:     End If
627:   Next lngIndex
  
  Dim dblProperDistance As Double
630:   dblProperDistance = dblLength - (dblTotalLength - dblHalfLength)
  
632:   Set pPoint1 = New Point
633:   pPoint1.PutCoords dblX1, dblY1
634:   Set pPoint1.SpatialReference = pPolyline.SpatialReference
  
636:   Set pPoint2 = New Point
  
638:   PointLineVincenty2 pPoint1, dblProperDistance, dblAz1, pPoint2, dblAz2
  
640:   Set SpheroidalPolylineMidpoint2 = pPoint2


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineMidpoint2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function SpheroidalPolylineLength(pPolyline As IPolyline) As Double
  On Error GoTo ErrorHandler


  ' ASSUMES POLYLINE IS IN GEOGRAPHIC COORDINATES

  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  
663:   dblTotalLength = 0
664:   Set pSegCollection = pPolyline
  
666:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
667:     Set pSeg = pSegCollection.Segment(lngIndex)
668:     Set pPoint1 = pSeg.FromPoint
669:     Set pPoint2 = pSeg.ToPoint
670:     dblLength = DistanceVincentyPoints(pPoint1, pPoint2, dblAz1, dblAz2)
671:     dblTotalLength = dblTotalLength + dblLength
672:   Next lngIndex
  
674:   SpheroidalPolylineLength = dblTotalLength


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineLength " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SpheroidalPolylineLength2(pPolyline As IPolyline, _
    Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler


  ' ASSUMES POLYLINE IS IN GEOGRAPHIC COORDINATES

  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  
699:   dblTotalLength = 0
700:   Set pSegCollection = pPolyline
  
702:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
703:     Set pSeg = pSegCollection.Segment(lngIndex)
704:     Set pPoint1 = pSeg.FromPoint
705:     Set pPoint2 = pSeg.ToPoint
706:     dblLength = DistanceVincentyPoints2(pPoint1, pPoint2, dblAz1, dblAz2, dblEquatorialRadius, dblPolarRadius)
707:     dblTotalLength = dblTotalLength + dblLength
708:   Next lngIndex
  
710:   SpheroidalPolylineLength2 = dblTotalLength


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineLength2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SphericalPolygonArea2(pPolygon As IPolygon, Optional booCalcCentroid As Boolean = False, _
      Optional dblCentX As Double, Optional dblCentY As Double, _
      Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler

  
  ' REMEMBER THAT THIS STILL CALCULATES ON THE SPHERE BECAUSE I HAVE NOT FIGURED OUT A WAY TO CALCULATE POLYGON AREAS
  ' ON AN ELLISOID.  THE SPHERE IS DEFINED AS THAT SPHERE WITH THE SAME VOLUME AS THE ELLIPSOID WITH THE SPECIFIED
  ' MAJOR AND MINOR AXES.
  '
  ' MODIFICATION OF SphericalPolygonArea TO ALLOW USER TO SET CUSTOM ELLIPSOID MAJOR AND MINOR AXES
  '
  ' ASSUMES POLYGON IS IN GEOGRAPHIC COORDINATES
  ' BREAK UP POLYGON INTO CONNECTED COMPONENTS
  
  Dim pPoly4 As IPolygon4
734:   Set pPoly4 = pPolygon
  Dim pConnected As IGeometryCollection
  Dim pRingBag As IGeometryCollection
  Dim pExtRing As IRing
738:   Set pConnected = pPoly4.ConnectedComponentBag
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  Dim pPointCollection As IPointCollection
  
  Dim pArea As IArea
  Dim pCentroid As IPoint
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  
748:   If booCalcCentroid Then
'    Dim dbl1Long As Double
'    Dim dbl1Lat As Double
    Dim dbl1X As Double
    Dim dbl1Y As Double
    Dim dbl1Z As Double
'    Dim dbl2Long As Double
'    Dim dbl2Lat As Double
    Dim dbl2X As Double
    Dim dbl2Y As Double
    Dim dbl2Z As Double
    Dim dbl3X As Double
    Dim dbl3Y As Double
    Dim dbl3Z As Double
    Dim dblTempCentX As Double
    Dim dblTempCentY As Double
    Dim dblTempCentZ As Double
    Dim dblRunningX As Double
    Dim dblRunningY As Double
    Dim dblRunningZ As Double
768:     dblRunningX = 0
769:     dblRunningY = 0
770:     dblRunningZ = 0
    Dim dblVectLength As Double
772:   End If
  
'  Dim dbl3Long As Double
'  Dim dbl3Lat As Double
  
  Dim pSubPolygon As IPolygon4
  Dim dblArea As Double
  Dim dblTriangleArea As Double
780:   dblArea = 0
  Dim dblMultiplier As Double
  Dim lngCountPos As Long
  Dim lngCountNeg As Long
  
  ' FOR TESTING CENTROID
'  Dim dblTestArea As Double
'  Dim dblTestRunningArea As Double
'  Dim dblTestX As Double
'  Dim dblTestY As Double
'  Dim dblTestRunningX As Double
'  Dim dblTestRunningY As Double
  
  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
795:   For lngIndex = 0 To (pConnected.GeometryCount - 1)
796:     Set pSubPolygon = pConnected.Geometry(lngIndex)
797:     Set pArea = pSubPolygon
798:     Set pCentroid = pArea.Centroid
799:     Set pSegCollection = pSubPolygon
    
'    dbl3Long = pCentroid.X
'    dbl3Lat = pCentroid.Y
    
804:     For lngIndex2 = 0 To (pSegCollection.SegmentCount - 1)
805:       Set pSeg = pSegCollection.Segment(lngIndex2)
806:       Set pPoint1 = pSeg.FromPoint
807:       Set pPoint2 = pSeg.ToPoint
      
'      ThisDocument.Graphic_MakeFromGeometry Document, pPoint1, "delete_corridors"
'      ThisDocument.Graphic_MakeFromGeometry Document, pPoint2, "delete_corridors"
811:       dblTriangleArea = SphericalTriangleArea2(pPoint1, pPoint2, pCentroid, dblMultiplier, dblEquatorialRadius, dblPolarRadius)
812:       dblArea = dblArea + dblTriangleArea
      
      ' FOR TESTING CENTROID
'      dblTestArea = TriangleAreaPoints3DValues(pPoint1.X, pPoint1.Y, 1, pPoint2.X, pPoint2.Y, _
'                1, pCentroid.X, pCentroid.Y, 1)
'      dblTestRunningArea = dblTestRunningArea + dblTestArea
'      TriangleCentroidPlane pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, pCentroid.X, pCentroid.Y, dblTestX, dblTestY
'      dblTestRunningX = dblTestRunningX + (dblTestX * dblTestArea)
'      dblTestRunningY = dblTestRunningY + (dblTestY * dblTestArea)
      
822:       If booCalcCentroid Then
823:         SphericalLatLongToCart pPoint1.X, pPoint1.Y, dbl1X, dbl1Y, dbl1Z
824:         SphericalLatLongToCart pPoint2.X, pPoint2.Y, dbl2X, dbl2Y, dbl2Z
825:         SphericalLatLongToCart pCentroid.X, pCentroid.Y, dbl3X, dbl3Y, dbl3Z
826:         TriangleCentroid3D dbl1X, dbl1Y, dbl1Z, dbl2X, dbl2Y, dbl2Z, dbl3X, dbl3Y, dbl3Z, _
                dblTempCentX, dblTempCentY, dblTempCentZ
    
        ' NORMALIZE VECTOR
830:         dblVectLength = Sqr(dblTempCentX ^ 2 + dblTempCentY ^ 2 + dblTempCentZ ^ 2)
831:         dblTempCentX = dblTempCentX / dblVectLength
832:         dblTempCentY = dblTempCentY / dblVectLength
833:         dblTempCentZ = dblTempCentZ / dblVectLength
        
835:         dblRunningX = dblRunningX + (dblTempCentX * dblTriangleArea)
836:         dblRunningY = dblRunningY + (dblTempCentY * dblTriangleArea)
837:         dblRunningZ = dblRunningZ + (dblTempCentZ * dblTriangleArea)
838:       End If
839:     Next lngIndex2
840:   Next lngIndex
  
842:   If booCalcCentroid Then
843:     If dblArea > 0 Then
844:       dblRunningX = dblRunningX / dblArea
845:       dblRunningY = dblRunningY / dblArea
846:       dblRunningZ = dblRunningZ / dblArea
      
848:       SphericalCartToLatLong dblCentX, dblCentY, dblRunningX, dblRunningY, dblRunningZ
849:     Else
      
      ' IF AREA = 0 BUT HAS VERTICES, THEN CALCULATE CENTROID AS MULTIPOINT CENTROID?  NO; MIGHT HAVE OVERLAPPING VERTICES THAT WOULD SKEW
      ' IF HAS NO MASS, THEN CANNOT HAVE CENTER OF MASS
      
854:       dblCentX = -9999
855:       dblCentY = -9999
856:     End If
857:   End If
  
  ' FOR TESTING CENTROID
'  dblTestRunningX = dblTestRunningX / dblTestRunningArea
'  dblTestRunningY = dblTestRunningY / dblTestRunningArea
'  Debug.Print "Test Centroid:  X = " & dblTestRunningX & ",  Y = " & dblTestRunningY
  
864:   SphericalPolygonArea2 = dblArea


  Exit Function
ErrorHandler:
  HandleError True, "SphericalPolygonArea2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function SphericalPolygonArea(pPolygon As IPolygon, Optional booCalcCentroid As Boolean = False, _
      Optional dblCentX As Double, Optional dblCentY As Double) As Double
  On Error GoTo ErrorHandler


  ' ASSUMES POLYGON IS IN GEOGRAPHIC COORDINATES
  ' BREAK UP POLYGON INTO CONNECTED COMPONENTS
  Dim pPoly4 As IPolygon4
879:   Set pPoly4 = pPolygon
  Dim pConnected As IGeometryCollection
  Dim pRingBag As IGeometryCollection
  Dim pExtRing As IRing
883:   Set pConnected = pPoly4.ConnectedComponentBag
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  Dim pPointCollection As IPointCollection
  
  Dim pArea As IArea
  Dim pCentroid As IPoint
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  
893:   If booCalcCentroid Then
'    Dim dbl1Long As Double
'    Dim dbl1Lat As Double
    Dim dbl1X As Double
    Dim dbl1Y As Double
    Dim dbl1Z As Double
'    Dim dbl2Long As Double
'    Dim dbl2Lat As Double
    Dim dbl2X As Double
    Dim dbl2Y As Double
    Dim dbl2Z As Double
    Dim dbl3X As Double
    Dim dbl3Y As Double
    Dim dbl3Z As Double
    Dim dblTempCentX As Double
    Dim dblTempCentY As Double
    Dim dblTempCentZ As Double
    Dim dblRunningX As Double
    Dim dblRunningY As Double
    Dim dblRunningZ As Double
913:     dblRunningX = 0
914:     dblRunningY = 0
915:     dblRunningZ = 0
    Dim dblVectLength As Double
917:   End If
  
'  Dim dbl3Long As Double
'  Dim dbl3Lat As Double
  
  Dim pSubPolygon As IPolygon4
  Dim dblArea As Double
  Dim dblTriangleArea As Double
925:   dblArea = 0
  Dim dblMultiplier As Double
  Dim lngCountPos As Long
  Dim lngCountNeg As Long
  
  ' FOR TESTING CENTROID
'  Dim dblTestArea As Double
'  Dim dblTestRunningArea As Double
'  Dim dblTestX As Double
'  Dim dblTestY As Double
'  Dim dblTestRunningX As Double
'  Dim dblTestRunningY As Double
  
  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
940:   For lngIndex = 0 To (pConnected.GeometryCount - 1)
941:     Set pSubPolygon = pConnected.Geometry(lngIndex)
942:     Set pArea = pSubPolygon
943:     Set pCentroid = pArea.Centroid
944:     Set pSegCollection = pSubPolygon
    
'    dbl3Long = pCentroid.X
'    dbl3Lat = pCentroid.Y
    
949:     For lngIndex2 = 0 To (pSegCollection.SegmentCount - 1)
950:       Set pSeg = pSegCollection.Segment(lngIndex2)
951:       Set pPoint1 = pSeg.FromPoint
952:       Set pPoint2 = pSeg.ToPoint
      
'      ThisDocument.Graphic_MakeFromGeometry Document, pPoint1, "delete_corridors"
'      ThisDocument.Graphic_MakeFromGeometry Document, pPoint2, "delete_corridors"
956:       dblTriangleArea = SphericalTriangleArea(pPoint1, pPoint2, pCentroid, dblMultiplier)
957:       dblArea = dblArea + dblTriangleArea
      
      ' FOR TESTING CENTROID
'      dblTestArea = TriangleAreaPoints3DValues(pPoint1.X, pPoint1.Y, 1, pPoint2.X, pPoint2.Y, _
'                1, pCentroid.X, pCentroid.Y, 1)
'      dblTestRunningArea = dblTestRunningArea + dblTestArea
'      TriangleCentroidPlane pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, pCentroid.X, pCentroid.Y, dblTestX, dblTestY
'      dblTestRunningX = dblTestRunningX + (dblTestX * dblTestArea)
'      dblTestRunningY = dblTestRunningY + (dblTestY * dblTestArea)
      
967:       If booCalcCentroid Then
968:         SphericalLatLongToCart pPoint1.X, pPoint1.Y, dbl1X, dbl1Y, dbl1Z
969:         SphericalLatLongToCart pPoint2.X, pPoint2.Y, dbl2X, dbl2Y, dbl2Z
970:         SphericalLatLongToCart pCentroid.X, pCentroid.Y, dbl3X, dbl3Y, dbl3Z
971:         TriangleCentroid3D dbl1X, dbl1Y, dbl1Z, dbl2X, dbl2Y, dbl2Z, dbl3X, dbl3Y, dbl3Z, _
                dblTempCentX, dblTempCentY, dblTempCentZ
    
        ' NORMALIZE VECTOR
975:         dblVectLength = Sqr(dblTempCentX ^ 2 + dblTempCentY ^ 2 + dblTempCentZ ^ 2)
976:         dblTempCentX = dblTempCentX / dblVectLength
977:         dblTempCentY = dblTempCentY / dblVectLength
978:         dblTempCentZ = dblTempCentZ / dblVectLength
        
980:         dblRunningX = dblRunningX + (dblTempCentX * dblTriangleArea)
981:         dblRunningY = dblRunningY + (dblTempCentY * dblTriangleArea)
982:         dblRunningZ = dblRunningZ + (dblTempCentZ * dblTriangleArea)
983:       End If
984:     Next lngIndex2
985:   Next lngIndex
  
987:   If booCalcCentroid Then
988:     If dblArea > 0 Then
989:       dblRunningX = dblRunningX / dblArea
990:       dblRunningY = dblRunningY / dblArea
991:       dblRunningZ = dblRunningZ / dblArea
      
993:       SphericalCartToLatLong dblCentX, dblCentY, dblRunningX, dblRunningY, dblRunningZ
994:     Else
      
      ' IF AREA = 0 BUT HAS VERTICES, THEN CALCULATE CENTROID AS MULTIPOINT CENTROID?  NO; MIGHT HAVE OVERLAPPING VERTICES THAT WOULD SKEW
      ' IF HAS NO MASS, THEN CANNOT HAVE CENTER OF MASS
      
999:       dblCentX = -9999
1000:       dblCentY = -9999
1001:     End If
1002:   End If
  
  ' FOR TESTING CENTROID
'  dblTestRunningX = dblTestRunningX / dblTestRunningArea
'  dblTestRunningY = dblTestRunningY / dblTestRunningArea
'  Debug.Print "Test Centroid:  X = " & dblTestRunningX & ",  Y = " & dblTestRunningY
  
1009:   SphericalPolygonArea = dblArea


  Exit Function
ErrorHandler:
  HandleError True, "SphericalPolygonArea " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function AzimuthHaversine(pPointA As IPoint, pPointB As IPoint) As Double
  On Error GoTo ErrorHandler


  ' WITH HELP FROM http://www.movable-type.co.uk/scripts/latlong.html
  ' Formula:    Theta = atan2( sin(Deltalong)*cos(lat2), cos(lat1)*sin(lat2) - sin(lat1)*cos(lat2)*cos(DeltaLong) )
  
  Dim DeltaLong As Double
  Dim PX As Double
  Dim PY As Double
  Dim QX As Double
  Dim QY As Double
  
1030:   PX = DegToRad(pPointA.X)
1031:   PY = DegToRad(pPointA.Y)
1032:   QX = DegToRad(pPointB.X)
1033:   QY = DegToRad(pPointB.Y)
  
1035:   DeltaLong = QX - PX
  Dim dblTheta As Double
1037:   dblTheta = atan2(Sin(DeltaLong) * Cos(QY), Cos(PY) * Sin(QY) - Sin(PY) * Cos(QY) * Cos(DeltaLong))
1038:   AzimuthHaversine = RadToDeg(dblTheta)
1039:   If AzimuthHaversine < 360 Then AzimuthHaversine = AzimuthHaversine + 360


  Exit Function
ErrorHandler:
  HandleError True, "AzimuthHaversine " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SphericalTriangleArea2(pPointA As IPoint, pPointB As IPoint, pPointC As IPoint, Optional dblMult As Double, _
    Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler


  ' MODIFICATION OF SphericalTriangleArea, TO ALLOW USER TO OPTIONALLY SEND CUSTOM MAJOR AND MINOR ELLIPSOID AXES
  
  ' BASED ON GIRARD'S FORMULA:  Area = R^2 * (A + B + C - Pi)
  '                          Where A = Angle 1
  '                                B = Angle 2
  '                                C = Angle 3
  '                   A + B + C - Pi = Spherical Excess
  '                                R = Sphere Radius
  ' Trick is to get Angles A, B and C from points.
  '
  ' ANOTHER FORMULATION, BASED ON DISTANCES:
  '                       Tan(E / 4) = sqrt(Tan(S / 2) * Tan((S - A) / 2) * Tan((S - B) / 2) * Tan((S - C) / 2))
  '                 Spherical Excess = E
  '                   where  a, b, c = sides of spherical triangle
  '                                S = (A + B + C) / 2
  ' INITAL AZIMUTH = atn( sin (Lo2 - Lo1) / (cos (Lo2 - Lo1) sin L1 - cos L1 tan L2)
  '         http://fer3.com/arc/m2.aspx?i=1688&y=200111
  
  ' FOR DEBUGGING
'  Static dblMax As Double
'  Static dblMin As Double
  
1074:   If Abs(pPointA.X - pPointB.X) < 0.000000000001 And Abs(pPointA.X - pPointC.X) < 0.000000000001 Then
1075:     SphericalTriangleArea2 = 0
    Exit Function
1077:   End If
  
  Dim dblAB As Double
  Dim dblBC As Double
  Dim dblCA As Double
  
  Dim dblR As Double         ' RADIUS
1084:   dblR = (dblEquatorialRadius ^ 2 * dblPolarRadius) ^ (1 / 3)   ' PROPER 3-AXIS GEOMETRIC MEAN; (a^2 * b) ^ (1/3)
  
  Dim dblLat As Double
  Dim dblLong As Double
  Dim dblTemp As Double
  Dim dblLat1 As Double
  Dim dblLat2 As Double
  Dim dblC As Double
  Dim dblAzimuthAB As Double
  Dim dblAzimuthBC As Double
  Dim dblAzimuthAC As Double
  Dim dblAzimuthAA As Double
  Dim dblAzRev As Double
  Dim dblMultiplier As Double
  Dim dblLong2 As Double
  
  ' CALCULATE LENGTH OF GEOCURVE AB USING HAVERSINE FORMULA
1101:   dblLat1 = DegToRad(pPointA.Y)
1102:   dblLat2 = DegToRad(pPointB.Y)
1103:   dblLat = dblLat1 - dblLat2
1104:   dblLong = DegToRad(pPointA.X - pPointB.X)
1105:   dblLong2 = -dblLong
1106:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1107:   dblAB = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
  ' AZIMUTH FROM A TO B
'  DistanceVincentyNumbers pPointA.X, pPointA.Y, pPointB.X, pPointB.Y, dblAzimuthAB, dblAzRev
'  dblAB = DistanceVincentyNumbers(pPointA.x, pPointA.y, pPointB.x, pPointB.y, dblAzimuthAB, dblAzRev) / dblR
'  Debug.Print "A to B:  Vincenty = " & dblAzimuthAB
1112:   dblAzimuthAB = atan2(Sin(-dblLong) * Cos(dblLat2), _
        Cos(dblLat1) * Sin(dblLat2) - Sin(dblLat1) * Cos(dblLat2) * Cos(-dblLong))
'  Debug.Print "A to B:  Simpler = " & dblAzimuthAB
'  dblAB = dblR * dblC
  
  ' CALCULATE LENGTH OF GEOCURVE BC USING HAVERSINE FORMULA
1118:   dblLat1 = DegToRad(pPointB.Y)
1119:   dblLat2 = DegToRad(pPointC.Y)
1120:   dblLat = dblLat1 - dblLat2
1121:   dblLong = DegToRad(pPointB.X - pPointC.X)
1122:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1123:   dblBC = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
'  dblBC = DistanceVincentyNumbers(pPointB.x, pPointB.y, pPointC.x, pPointC.y, dblAzimuthBC, dblAzRev) / dblR
'  dblBC = DistanceVincentyNumbers(pPointB.x, pPointB.y, pPointC.x, pPointC.y, dblAzimuthBC, dblAzRev) / dblR
'  dblBC = dblR * dblC
  
  ' CALCULATE LENGTH OF GEOCURVE AB USING HAVERSINE FORMULA
1129:   dblLat1 = DegToRad(pPointC.Y)
1130:   dblLat2 = DegToRad(pPointA.Y)
1131:   dblLat = dblLat1 - dblLat2
1132:   dblLong = DegToRad(pPointC.X - pPointA.X)
  
1134:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1135:   dblCA = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
  ' AZIMUTH FROM A TO C
'  DistanceVincentyNumbers pPointA.x, pPointA.y, pPointC.x, pPointC.y, dblAzimuthAC, dblAzRev
'  dblCA = DistanceVincentyNumbers(pPointA.x, pPointA.y, pPointC.x, pPointC.y, dblAzimuthAC, dblAzRev) / dblR
'  Debug.Print "A to C:  Vincenty = " & dblAzimuth
'  dblAzimuth2 = RadToDeg(atan2(Sin(pPointC.x - pPointA.x), _
                    (Cos(pPointC.x - pPointA.x) * Sin(pPointA.y) - Cos(pPointA.y) * Tan(pPointC.y))))
'  dblCA = dblR * dblC
'  Debug.Print "Az1 = " & CStr(dblAzimuth1) & ",       Az2 = " & CStr(dblAzimuth2)

  ' NOTE:  Lat1 and Lat2 flipped in equation below because variables originally defined for Line CA, not Line AC
1146:   dblAzimuthAC = atan2(Sin(dblLong) * Cos(dblLat1), _
        Cos(dblLat2) * Sin(dblLat1) - Sin(dblLat2) * Cos(dblLat1) * Cos(dblLong))
'  Debug.Print "A to C:  Simpler = " & dblAzimuthAC
  
'  If dblAzimuthAB > dblMax Then dblMax = dblAzimuthAB
'  If dblAzimuthAB < dblMin Then dblMin = dblAzimuthAB
'  Debug.Print "Current Max = " & dblMax & ",   Current Min = " & dblMin
  
1154:   If dblAzimuthAB < 0 Then dblAzimuthAB = dblAzimuthAB + 2 * dblPi
1155:   If dblAzimuthAC < 0 Then dblAzimuthAC = dblAzimuthAC + 2 * dblPi
  
  Dim dblDiff As Double
1158:   dblDiff = dblAzimuthAC - dblAzimuthAB
1159:   If dblDiff > 0 Then              ' EITHER AC > AB or AC IS TO THE LEFT OF NORTH
1160:     If dblDiff > dblPi Then         ' THEN AC IS TO THE LEFT OF NORTH
1161:       dblMultiplier = -1           ' COUNTERCLOCKWISE
1162:     Else                           ' THEN AC > AB
1163:       dblMultiplier = 1            ' CLOCKWISE
1164:     End If
1165:   Else                             ' EITHER AC < AB or AB IS TO THE LEFT OF NORTH
1166:     If Abs(dblDiff) > dblPi Then   ' THEN AB IS TO THE LEFT OF NORTH
1167:       dblMultiplier = 1            ' CLOCKWISE
1168:     Else                           ' THEN AC < AB
1169:       dblMultiplier = -1           ' COUNTERCLOCKWISE
1170:     End If
1171:   End If
    
  
'  If ((dblDiff > dblPi) And (dblAzimuthAB > dblAzimuthAC)) Or ((dblDiff < dblPi) And (dblAzimuthAB < dblAzimuthAC)) Then
'    ' IS CLOCKWISE
'    dblMultiplier = 1
'  Else
'    ' IS COUNTERCLOCKWISE
'    dblMultiplier = -1
'  End If

'  If Abs(dblAzimuthAC - dblAzimuthAB) > dblPi Then dblAzimuthAB = dblAzimuth
'  dblAzimuthAB = dblAzimuthAB + 360
'  dblAzimuthAC = dblAzimuthAC + 360
'  If dblAzimuthAB > dblAzimuthAC Then
'    dblMultiplier = -1
'  Else
'    dblMultiplier = 1
'  End If
  
'  Dim booIsClockwise As Boolean
'  booIsClockwise = CalcCheckClockwise(pPointA, pPointB, pPointC)
'  If booIsClockwise Then
'    If dblMultiplier = -1 Then
'      Debug.Print "-1, 1:  AB = " & CStr(RadToDeg(dblAzimuthAB)) & ",   AC = " & CStr(RadToDeg(dblAzimuthAC)) & vbCrLf & _
'                  "  --> Point A:  X = " & pPointA.x & ",    Y = " & pPointA.y & vbCrLf & _
'                  "  --> Point B:  X = " & pPointB.x & ",    Y = " & pPointB.y & vbCrLf & _
'                  "  --> Point C:  X = " & pPointC.x & ",    Y = " & pPointC.y
'    End If
'  Else
'    If dblMultiplier = 1 Then
'      Debug.Print "1, -1:  AB = " & CStr(RadToDeg(dblAzimuthAB)) & ",  AC = " & CStr(RadToDeg(dblAzimuthAC)) & vbCrLf & _
'                  "  --> Point A:  X = " & pPointA.x & ",    Y = " & pPointA.y & vbCrLf & _
'                  "  --> Point B:  X = " & pPointB.x & ",    Y = " & pPointB.y & vbCrLf & _
'                  "  --> Point C:  X = " & pPointC.x & ",    Y = " & pPointC.y
'    End If
'  End If
  
  Dim dblS As Double
1210:   dblS = (dblAB + dblBC + dblCA) / 2
  
  Dim dblTanEOver4 As Double
1213:   dblTanEOver4 = Sqr(Tan(dblS / 2) * Tan((dblS - dblAB) / 2) * Tan((dblS - dblBC) / 2) * Tan((dblS - dblCA) / 2))
  Dim dblE As Double
1215:   dblE = Atn(dblTanEOver4) * 4
  
1217:   dblMult = dblMultiplier
1218:   SphericalTriangleArea2 = dblR ^ 2 * dblE * dblMultiplier


  Exit Function
ErrorHandler:
  HandleError True, "SphericalTriangleArea2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function SphericalTriangleArea(pPointA As IPoint, pPointB As IPoint, pPointC As IPoint, Optional dblMult As Double) As Double
  On Error GoTo ErrorHandler

  
  
  ' BASED ON GIRARD'S FORMULA:  Area = R^2 * (A + B + C - Pi)
  '                          Where A = Angle 1
  '                                B = Angle 2
  '                                C = Angle 3
  '                   A + B + C - Pi = Spherical Excess
  '                                R = Sphere Radius
  ' Trick is to get Angles A, B and C from points.
  '
  ' ANOTHER FORMULATION, BASED ON DISTANCES:
  '                       Tan(E / 4) = sqrt(Tan(S / 2) * Tan((S - A) / 2) * Tan((S - B) / 2) * Tan((S - C) / 2))
  '                 Spherical Excess = E
  '                   where  a, b, c = sides of spherical triangle
  '                                S = (A + B + C) / 2
  ' INITAL AZIMUTH = atn( sin (Lo2 - Lo1) / (cos (Lo2 - Lo1) sin L1 - cos L1 tan L2)
  '         http://fer3.com/arc/m2.aspx?i=1688&y=200111
  
  ' FOR DEBUGGING
'  Static dblMax As Double
'  Static dblMin As Double
  
1252:   If Abs(pPointA.X - pPointB.X) < 0.000000000001 And Abs(pPointA.X - pPointC.X) < 0.000000000001 Then
1253:     SphericalTriangleArea = 0
    Exit Function
1255:   End If
  
  Dim dblAB As Double
  Dim dblBC As Double
  Dim dblCA As Double
  
  Dim dblR As Double         ' RADIUS
'  dblR = (6378137 + 6356752.31424518) / 2 ' AVERAGE OF WGS ELLIPSOID MAJOR AND MINOR AXES
'  dblR = Sqr(6378137 * 6356752.31424518)  ' GEOMETRIC MEAN OF WGS ELLIPSOID MAJOR AND MINOR AXES
1264:   dblR = (6378137 ^ 2 * 6356752.31424518) ^ (1 / 3)   ' PROPER 3-AXIS GEOMETRIC MEAN; (a^2 * b) ^ (1/3)
  
  Dim dblLat As Double
  Dim dblLong As Double
  Dim dblTemp As Double
  Dim dblLat1 As Double
  Dim dblLat2 As Double
  Dim dblC As Double
  Dim dblAzimuthAB As Double
  Dim dblAzimuthBC As Double
  Dim dblAzimuthAC As Double
  Dim dblAzimuthAA As Double
  Dim dblAzRev As Double
  Dim dblMultiplier As Double
  Dim dblLong2 As Double
  
  ' CALCULATE LENGTH OF GEOCURVE AB USING HAVERSINE FORMULA
1281:   dblLat1 = DegToRad(pPointA.Y)
1282:   dblLat2 = DegToRad(pPointB.Y)
1283:   dblLat = dblLat1 - dblLat2
1284:   dblLong = DegToRad(pPointA.X - pPointB.X)
1285:   dblLong2 = -dblLong
1286:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1287:   dblAB = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
  ' AZIMUTH FROM A TO B
'  DistanceVincentyNumbers pPointA.X, pPointA.Y, pPointB.X, pPointB.Y, dblAzimuthAB, dblAzRev
'  dblAB = DistanceVincentyNumbers(pPointA.x, pPointA.y, pPointB.x, pPointB.y, dblAzimuthAB, dblAzRev) / dblR
'  Debug.Print "A to B:  Vincenty = " & dblAzimuthAB
1292:   dblAzimuthAB = atan2(Sin(-dblLong) * Cos(dblLat2), _
        Cos(dblLat1) * Sin(dblLat2) - Sin(dblLat1) * Cos(dblLat2) * Cos(-dblLong))
'  Debug.Print "A to B:  Simpler = " & dblAzimuthAB
'  dblAB = dblR * dblC
  
  ' CALCULATE LENGTH OF GEOCURVE BC USING HAVERSINE FORMULA
1298:   dblLat1 = DegToRad(pPointB.Y)
1299:   dblLat2 = DegToRad(pPointC.Y)
1300:   dblLat = dblLat1 - dblLat2
1301:   dblLong = DegToRad(pPointB.X - pPointC.X)
1302:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1303:   dblBC = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
'  dblBC = DistanceVincentyNumbers(pPointB.x, pPointB.y, pPointC.x, pPointC.y, dblAzimuthBC, dblAzRev) / dblR
'  dblBC = DistanceVincentyNumbers(pPointB.x, pPointB.y, pPointC.x, pPointC.y, dblAzimuthBC, dblAzRev) / dblR
'  dblBC = dblR * dblC
  
  ' CALCULATE LENGTH OF GEOCURVE AB USING HAVERSINE FORMULA
1309:   dblLat1 = DegToRad(pPointC.Y)
1310:   dblLat2 = DegToRad(pPointA.Y)
1311:   dblLat = dblLat1 - dblLat2
1312:   dblLong = DegToRad(pPointC.X - pPointA.X)
  
1314:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1315:   dblCA = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
  ' AZIMUTH FROM A TO C
'  DistanceVincentyNumbers pPointA.x, pPointA.y, pPointC.x, pPointC.y, dblAzimuthAC, dblAzRev
'  dblCA = DistanceVincentyNumbers(pPointA.x, pPointA.y, pPointC.x, pPointC.y, dblAzimuthAC, dblAzRev) / dblR
'  Debug.Print "A to C:  Vincenty = " & dblAzimuth
'  dblAzimuth2 = RadToDeg(atan2(Sin(pPointC.x - pPointA.x), _
                    (Cos(pPointC.x - pPointA.x) * Sin(pPointA.y) - Cos(pPointA.y) * Tan(pPointC.y))))
'  dblCA = dblR * dblC
'  Debug.Print "Az1 = " & CStr(dblAzimuth1) & ",       Az2 = " & CStr(dblAzimuth2)

  ' NOTE:  Lat1 and Lat2 flipped in equation below because variables originally defined for Line CA, not Line AC
1326:   dblAzimuthAC = atan2(Sin(dblLong) * Cos(dblLat1), _
        Cos(dblLat2) * Sin(dblLat1) - Sin(dblLat2) * Cos(dblLat1) * Cos(dblLong))
'  Debug.Print "A to C:  Simpler = " & dblAzimuthAC
  
'  If dblAzimuthAB > dblMax Then dblMax = dblAzimuthAB
'  If dblAzimuthAB < dblMin Then dblMin = dblAzimuthAB
'  Debug.Print "Current Max = " & dblMax & ",   Current Min = " & dblMin
  
1334:   If dblAzimuthAB < 0 Then dblAzimuthAB = dblAzimuthAB + 2 * dblPi
1335:   If dblAzimuthAC < 0 Then dblAzimuthAC = dblAzimuthAC + 2 * dblPi
  
  Dim dblDiff As Double
1338:   dblDiff = dblAzimuthAC - dblAzimuthAB
1339:   If dblDiff > 0 Then              ' EITHER AC > AB or AC IS TO THE LEFT OF NORTH
1340:     If dblDiff > dblPi Then         ' THEN AC IS TO THE LEFT OF NORTH
1341:       dblMultiplier = -1           ' COUNTERCLOCKWISE
1342:     Else                           ' THEN AC > AB
1343:       dblMultiplier = 1            ' CLOCKWISE
1344:     End If
1345:   Else                             ' EITHER AC < AB or AB IS TO THE LEFT OF NORTH
1346:     If Abs(dblDiff) > dblPi Then   ' THEN AB IS TO THE LEFT OF NORTH
1347:       dblMultiplier = 1            ' CLOCKWISE
1348:     Else                           ' THEN AC < AB
1349:       dblMultiplier = -1           ' COUNTERCLOCKWISE
1350:     End If
1351:   End If
    
  
'  If ((dblDiff > dblPi) And (dblAzimuthAB > dblAzimuthAC)) Or ((dblDiff < dblPi) And (dblAzimuthAB < dblAzimuthAC)) Then
'    ' IS CLOCKWISE
'    dblMultiplier = 1
'  Else
'    ' IS COUNTERCLOCKWISE
'    dblMultiplier = -1
'  End If

'  If Abs(dblAzimuthAC - dblAzimuthAB) > dblPi Then dblAzimuthAB = dblAzimuth
'  dblAzimuthAB = dblAzimuthAB + 360
'  dblAzimuthAC = dblAzimuthAC + 360
'  If dblAzimuthAB > dblAzimuthAC Then
'    dblMultiplier = -1
'  Else
'    dblMultiplier = 1
'  End If
  
'  Dim booIsClockwise As Boolean
'  booIsClockwise = CalcCheckClockwise(pPointA, pPointB, pPointC)
'  If booIsClockwise Then
'    If dblMultiplier = -1 Then
'      Debug.Print "-1, 1:  AB = " & CStr(RadToDeg(dblAzimuthAB)) & ",   AC = " & CStr(RadToDeg(dblAzimuthAC)) & vbCrLf & _
'                  "  --> Point A:  X = " & pPointA.x & ",    Y = " & pPointA.y & vbCrLf & _
'                  "  --> Point B:  X = " & pPointB.x & ",    Y = " & pPointB.y & vbCrLf & _
'                  "  --> Point C:  X = " & pPointC.x & ",    Y = " & pPointC.y
'    End If
'  Else
'    If dblMultiplier = 1 Then
'      Debug.Print "1, -1:  AB = " & CStr(RadToDeg(dblAzimuthAB)) & ",  AC = " & CStr(RadToDeg(dblAzimuthAC)) & vbCrLf & _
'                  "  --> Point A:  X = " & pPointA.x & ",    Y = " & pPointA.y & vbCrLf & _
'                  "  --> Point B:  X = " & pPointB.x & ",    Y = " & pPointB.y & vbCrLf & _
'                  "  --> Point C:  X = " & pPointC.x & ",    Y = " & pPointC.y
'    End If
'  End If
  
  Dim dblS As Double
1390:   dblS = (dblAB + dblBC + dblCA) / 2
  
  Dim dblTanEOver4 As Double
1393:   dblTanEOver4 = Sqr(Tan(dblS / 2) * Tan((dblS - dblAB) / 2) * Tan((dblS - dblBC) / 2) * Tan((dblS - dblCA) / 2))
  Dim dblE As Double
1395:   dblE = Atn(dblTanEOver4) * 4
  
1397:   dblMult = dblMultiplier
1398:   SphericalTriangleArea = dblR ^ 2 * dblE * dblMultiplier


  Exit Function
ErrorHandler:
  HandleError True, "SphericalTriangleArea " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ArcSinJen(dblValue As Double) As Double
  On Error GoTo ErrorHandler


1410:   ArcSinJen = Atn(dblValue / Sqr(-dblValue * dblValue + 1))


  Exit Function
ErrorHandler:
  HandleError True, "ArcSinJen " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function ArcCosJen(dblValue As Double) As Double
  On Error GoTo ErrorHandler


1421:   ArcCosJen = Atn(-dblValue / Sqr(-dblValue * dblValue + 1)) + 2 * Atn(1)


  Exit Function
ErrorHandler:
  HandleError True, "ArcCosJen " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function DistanceVincentyPoints(pPoint1 As IPoint, pPoint2 As IPoint, dblAz1 As Double, dblAz2 As Double) As Double
  On Error GoTo ErrorHandler


1433:   DistanceVincentyPoints = DistanceVincentyNumbers(pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, dblAz1, dblAz2)


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyPoints " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function DistanceVincentyPoints2(pPoint1 As IPoint, pPoint2 As IPoint, dblAz1 As Double, dblAz2 As Double, _
  Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF DistanceVincentyPoints, TO ALLOW FOR ANY ELLIPSOID
  
1448:   DistanceVincentyPoints2 = DistanceVincentyNumbers2(pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, dblAz1, dblAz2, _
        dblEquatorialRadius, dblPolarRadius)


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyPoints2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub PointLineVincenty(pPoint As IPoint, dblLength As Double, dblAzimuth As Double, pNewPoint As IPoint, _
      dblAz2 As Double, Optional lngNumVertices As Long, Optional pPolyline As IPolyline)
  On Error GoTo ErrorHandler

      
  Dim pWGS84 As IGeographicCoordinateSystem
  Dim pSpatRefFact As ISpatialReferenceFactory
  'Set the spatial reference factory to a new spatial reference environment
1465:   Set pSpatRefFact = New SpatialReferenceEnvironment
1466:   Set pWGS84 = pSpatRefFact.CreateGeographicCoordinateSystem(esriSRGeoCS_WGS1984)
  
1468:   If lngNumVertices > 0 Then
1469:     If lngNumVertices = 1 Then lngNumVertices = 2
    
    Dim dblShort As Double
1472:     dblShort = dblLength / (lngNumVertices - 1)
    
    Dim pPointCollection As IPointCollection
1475:     If pPolyline Is Nothing Then
1476:       Set pPolyline = New Polyline
1477:     End If
1478:     Set pPointCollection = pPolyline
    
    ' ADD FIRST VERTEX
1481:     pPointCollection.AddPoint pPoint
    
    ' ADD INTERNAL VERTICES
1484:     If lngNumVertices > 2 Then
      Dim lngCounter As Long
      Dim dblCurrentDistance As Double
      Dim pTempPoint As IPoint
1488:       For lngCounter = 1 To (lngNumVertices - 2)
1489:         Set pTempPoint = New Point
1490:         PointLineVincentyPerPoint pPoint, lngCounter * dblShort, dblAzimuth, pTempPoint, dblAz2
1491:         pPointCollection.AddPoint pTempPoint
1492:       Next lngCounter
1493:     End If
    
    ' ADD LAST VERTEX AND SET FINAL POINT AND AZIMUTH VALUES
1496:     PointLineVincentyPerPoint pPoint, dblLength, dblAzimuth, pNewPoint, dblAz2
1497:     pPointCollection.AddPoint pNewPoint
1498:     Set pPolyline.SpatialReference = pWGS84
1499:     Set pNewPoint.SpatialReference = pWGS84
1500:   Else
1501:     PointLineVincentyPerPoint pPoint, dblLength, dblAzimuth, pNewPoint, dblAz2
1502:     Set pNewPoint.SpatialReference = pWGS84
1503:   End If


  Exit Sub
ErrorHandler:
  HandleError True, "PointLineVincenty " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub PointLineVincenty2(pPoint As IPoint, dblLength As Double, dblAzimuth As Double, pNewPoint As IPoint, _
      dblAz2 As Double, Optional lngNumVertices As Long, Optional pPolyline As IPolyline)
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF PointLineVincenty, TO ALLOW FOR ANY ELLIPSOID
  ' ASSUMES INCOMING POINT IS IN GEOGRAPHIC COORDINATE SYSTEM
  
  Dim pGCS As IGeographicCoordinateSystem
  Dim pSpRef As ISpatialReference
  
1522:   Set pSpRef = pPoint.SpatialReference
1523:   If TypeOf pSpRef Is IGeographicCoordinateSystem Then
1524:     Set pGCS = pSpRef
1525:   Else
    Dim pPrjCS As IProjectedCoordinateSystem
1527:     Set pPrjCS = pSpRef
1528:     Set pGCS = pPrjCS.GeographicCoordinateSystem
1529:   End If
  
  Dim dblEquatorialRadius As Double
  Dim dblPolarRadius As Double
  Dim pEllipsoid As ISpheroid
  
1535:   Set pEllipsoid = pGCS.Datum.Spheroid
1536:   dblEquatorialRadius = pEllipsoid.SemiMajorAxis
1537:   dblPolarRadius = pEllipsoid.SemiMinorAxis
  
1539:   If lngNumVertices > 0 Then
1540:     If lngNumVertices = 1 Then lngNumVertices = 2
    
    Dim dblShort As Double
1543:     dblShort = dblLength / (lngNumVertices - 1)
    
    Dim pPointCollection As IPointCollection
1546:     If pPolyline Is Nothing Then
1547:       Set pPolyline = New Polyline
1548:     End If
1549:     Set pPointCollection = pPolyline
    
    ' ADD FIRST VERTEX
1552:     pPointCollection.AddPoint pPoint
    
    ' ADD INTERNAL VERTICES
1555:     If lngNumVertices > 2 Then
      Dim lngCounter As Long
      Dim dblCurrentDistance As Double
      Dim pTempPoint As IPoint
1559:       For lngCounter = 1 To (lngNumVertices - 2)
1560:         Set pTempPoint = New Point
1561:         PointLineVincentyPerPoint2 pPoint, lngCounter * dblShort, dblAzimuth, pTempPoint, dblAz2, dblEquatorialRadius, dblPolarRadius
1562:         pPointCollection.AddPoint pTempPoint
1563:       Next lngCounter
1564:     End If
    
    ' ADD LAST VERTEX AND SET FINAL POINT AND AZIMUTH VALUES
1567:     PointLineVincentyPerPoint2 pPoint, dblLength, dblAzimuth, pNewPoint, dblAz2, dblEquatorialRadius, dblPolarRadius
1568:     pPointCollection.AddPoint pNewPoint
1569:     Set pPolyline.SpatialReference = pGCS
1570:     Set pNewPoint.SpatialReference = pGCS
1571:   Else
1572:     PointLineVincentyPerPoint2 pPoint, dblLength, dblAzimuth, pNewPoint, dblAz2, dblEquatorialRadius, dblPolarRadius
1573:     Set pNewPoint.SpatialReference = pGCS
1574:   End If


  Exit Sub
ErrorHandler:
  HandleError True, "PointLineVincenty2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Sub PointLineVincentyPerPoint2(pPoint As IPoint, dblLength As Double, dblAzimuth As Double, _
      pNewPoint As IPoint, dblAz2 As Double, _
      Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518)
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF PointLineVincentyPerPoint, TO ALLOW FOR ANY ELLIPSOID
  
  ' ASSUMES pPoint IS GEOGRAPHIC

  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  Dim dblPX As Double
1599:   dblPX = pPoint.X
  Dim dblPY As Double
1601:   dblPY = pPoint.Y
  
1603:   If dblLength = 0 Then    ' SAME POINT
1604:     pNewPoint.X = dblPX
1605:     pNewPoint.Y = dblPY
1606:     dblAz2 = dblAzimuth
    Exit Sub
1608:   End If
  
  ' WGS84 PARAMETERS ----------------------------------------
  Dim A As Double
  Dim B As Double
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim f As Double
  Dim dblA1 As Double
  Dim dblB1 As Double
  
  Dim dblTanU1 As Double
  Dim dblSinU1 As Double
  Dim dblCosU1 As Double
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
'  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
'  U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
1628:   A = dblEquatorialRadius   ' SPHEROID; EQUATORIAL RADIUS
1629:   B = dblPolarRadius        ' SPHEROID; POLAR RADIUS
1630:   f = (A - B) / B           ' FLATTENING
  
1632:   dblTanU1 = (1 - f) * (Tan(DegToRad(dblPY)))
1633:   U1 = Atn(dblTanU1)
1634:   dblCosU1 = Cos(U1)
1635:   dblSinU1 = Sin(U1)
  
  Dim S As Double
1638:   S = dblLength
  
  Dim Sigma1 As Double
  Dim tanSigma1 As Double
  Dim cosAlpha1 As Double
  Dim sinAlpha1 As Double
'  Dim dblAlpha As Double                      ' AZIMUTH AT EQUATOR
  Dim cosAlpha As Double
  Dim cosSqAlpha As Double
  
1648:   cosAlpha1 = Cos(DegToRad(dblAzimuth))
1649:   sinAlpha1 = Sin(DegToRad(dblAzimuth))
1650:   tanSigma1 = dblTanU1 / cosAlpha1                                                                    ' [1]
1651:   Sigma1 = atan2(dblTanU1, cosAlpha1)
  Dim sinAlpha As Double
1653:   sinAlpha = dblCosU1 * sinAlpha1                                                                  ' [2]
1654:   cosSqAlpha = 1 - (sinAlpha ^ 2)                                                                  ' TRIG IDENTITY
1655:   cosAlpha = Sqr(cosSqAlpha)
'  dblAlpha = ArcSinJen(sinAlpha)
  
1658:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
1659:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
1660:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                                      ' [3]
1661:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
1662:   dblB = (uSq / 1024) * (256 + dblB1)                                                              ' [4]
  
  Dim Sigma As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim DeltaSigma As Double
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  Dim l As Double
  
  Dim lngIterations As Long
1676:   lngIterations = 40
  
  Dim SigmaCompare As Double
1679:   SigmaCompare = 2 * dblPi
1680:   Sigma = S / (B * dblA)                  ' FIRST ESTIMATION
  
1682:   Do While (Abs(Sigma - SigmaCompare) > 0.000000000001) And (lngIterations > 0)
1683:     cos2SigmaM = Cos(2 * Sigma1 + Sigma)                                                             ' [5]
1684:     sinSigma = Sin(Sigma)
1685:     cosSigma = Cos(Sigma)
1686:     DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
1687:     DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
1688:     DeltaSigma3 = cos2SigmaM + DeltaSigma2
1689:     DeltaSigma = dblB * sinSigma * DeltaSigma3                                                       ' [6]
1690:     SigmaCompare = Sigma
1691:     Sigma = (S / (B * dblA)) + DeltaSigma                                                            ' [7]
    
1693:     If lngIterations = 0 Then
1694:       MsgBox "Vincenty Formula failed to converge!"
      Exit Sub
1696:     End If
1697:     lngIterations = lngIterations - 1
1698:   Loop
  
1700:   cos2SigmaM = Cos(2 * Sigma1 + Sigma)
1701:   sinSigma = Sin(Sigma)
1702:   cosSigma = Cos(Sigma)
  Dim dblLat2Denom As Double
  Dim dblLat2Temp As Double
1705:   dblLat2Temp = dblSinU1 * sinSigma - dblCosU1 * cosSigma * cosAlpha1
1706:   dblLat2Denom = (1 - f) * (Sqr(sinAlpha ^ 2 + dblLat2Temp ^ 2))
  
  ' CALCULATE LATITUDE FOR NEW POINT
  Dim dblLat2 As Double
1710:   dblLat2 = atan2(dblSinU1 * cosSigma + dblCosU1 * sinSigma * cosAlpha1, dblLat2Denom)                ' [8]
  
  ' CALCULATE LONGITUDE FOR NEW POINT
  Dim dblLambda As Double
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
1716:   dblLambda = atan2(sinSigma * sinAlpha1, dblCosU1 * cosSigma - dblSinU1 * sinSigma * cosAlpha1)      ' [9]
1717:   C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                                      ' [10]
1718:   dblLambda1 = cos2SigmaM + C * cosSigma * (-1# + 2# * cos2SigmaM ^ 2#)
1719:   dblLambda1a = C * sinSigma * dblLambda1
  Dim dblLambda2 As Double
1721:   dblLambda2 = Sigma + dblLambda1a
1722:   l = dblLambda - ((1 - C) * f * sinAlpha * dblLambda2)                                               ' [11]
  
1724:   pNewPoint.X = dblPX + RadToDeg(l)
1725:   pNewPoint.Y = RadToDeg(dblLat2)
  
1727:   dblAz2 = RadToDeg(atan2(sinAlpha, -dblLat2Temp))
1728:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2


  Exit Sub
ErrorHandler:
  HandleError True, "PointLineVincentyPerPoint2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub PointLineVincentyPerPoint(pPoint As IPoint, dblLength As Double, dblAzimuth As Double, _
      pNewPoint As IPoint, dblAz2 As Double)
  On Error GoTo ErrorHandler

      
  ' ASSUMES pPoint IS GEOGRAPHIC

  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  Dim dblPX As Double
1751:   dblPX = pPoint.X
  Dim dblPY As Double
1753:   dblPY = pPoint.Y
  
1755:   If dblLength = 0 Then    ' SAME POINT
1756:     pNewPoint.X = dblPX
1757:     pNewPoint.Y = dblPY
1758:     dblAz2 = dblAzimuth
    Exit Sub
1760:   End If
  
  ' WGS84 PARAMETERS ----------------------------------------
  Dim A As Double
  Dim B As Double
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim f As Double
  Dim dblA1 As Double
  Dim dblB1 As Double
  
  Dim dblTanU1 As Double
  Dim dblSinU1 As Double
  Dim dblCosU1 As Double
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
'  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
'  U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
1780:   A = 6378137               ' WGS84 SPHEROID; EQUATORIAL RADIUS
1781:   B = 6356752.31424518      ' WGS84 SPHEROID; POLAR RADIUS
1782:   f = 1 / 298.257223563     ' WGS84 SPHEROID; FLATTENING
  
1784:   dblTanU1 = (1 - f) * (Tan(DegToRad(dblPY)))
1785:   U1 = Atn(dblTanU1)
1786:   dblCosU1 = Cos(U1)
1787:   dblSinU1 = Sin(U1)
  
  Dim S As Double
1790:   S = dblLength
  
  Dim Sigma1 As Double
  Dim tanSigma1 As Double
  Dim cosAlpha1 As Double
  Dim sinAlpha1 As Double
'  Dim dblAlpha As Double                      ' AZIMUTH AT EQUATOR
  Dim cosAlpha As Double
  Dim cosSqAlpha As Double
  
1800:   cosAlpha1 = Cos(DegToRad(dblAzimuth))
1801:   sinAlpha1 = Sin(DegToRad(dblAzimuth))
1802:   tanSigma1 = dblTanU1 / cosAlpha1                                                                    ' [1]
1803:   Sigma1 = atan2(dblTanU1, cosAlpha1)
  Dim sinAlpha As Double
1805:   sinAlpha = dblCosU1 * sinAlpha1                                                                  ' [2]
1806:   cosSqAlpha = 1 - (sinAlpha ^ 2)                                                                  ' TRIG IDENTITY
1807:   cosAlpha = Sqr(cosSqAlpha)
'  dblAlpha = ArcSinJen(sinAlpha)
  
1810:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
1811:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
1812:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                                      ' [3]
1813:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
1814:   dblB = (uSq / 1024) * (256 + dblB1)                                                              ' [4]
  
  Dim Sigma As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim DeltaSigma As Double
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  Dim l As Double
  
  Dim lngIterations As Long
1828:   lngIterations = 40
  
  Dim SigmaCompare As Double
1831:   SigmaCompare = 2 * dblPi
1832:   Sigma = S / (B * dblA)                  ' FIRST ESTIMATION
  
1834:   Do While (Abs(Sigma - SigmaCompare) > 0.000000000001) And (lngIterations > 0)
1835:     cos2SigmaM = Cos(2 * Sigma1 + Sigma)                                                             ' [5]
1836:     sinSigma = Sin(Sigma)
1837:     cosSigma = Cos(Sigma)
1838:     DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
1839:     DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
1840:     DeltaSigma3 = cos2SigmaM + DeltaSigma2
1841:     DeltaSigma = dblB * sinSigma * DeltaSigma3                                                       ' [6]
1842:     SigmaCompare = Sigma
1843:     Sigma = (S / (B * dblA)) + DeltaSigma                                                            ' [7]
    
1845:     If lngIterations = 0 Then
1846:       MsgBox "Vincenty Formula failed to converge!"
      Exit Sub
1848:     End If
1849:     lngIterations = lngIterations - 1
1850:   Loop
  
1852:   cos2SigmaM = Cos(2 * Sigma1 + Sigma)
1853:   sinSigma = Sin(Sigma)
1854:   cosSigma = Cos(Sigma)
  Dim dblLat2Denom As Double
  Dim dblLat2Temp As Double
1857:   dblLat2Temp = dblSinU1 * sinSigma - dblCosU1 * cosSigma * cosAlpha1
1858:   dblLat2Denom = (1 - f) * (Sqr(sinAlpha ^ 2 + dblLat2Temp ^ 2))
  
  ' CALCULATE LATITUDE FOR NEW POINT
  Dim dblLat2 As Double
1862:   dblLat2 = atan2(dblSinU1 * cosSigma + dblCosU1 * sinSigma * cosAlpha1, dblLat2Denom)                ' [8]
  
  ' CALCULATE LONGITUDE FOR NEW POINT
  Dim dblLambda As Double
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
1868:   dblLambda = atan2(sinSigma * sinAlpha1, dblCosU1 * cosSigma - dblSinU1 * sinSigma * cosAlpha1)      ' [9]
1869:   C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                                      ' [10]
1870:   dblLambda1 = cos2SigmaM + C * cosSigma * (-1# + 2# * cos2SigmaM ^ 2#)
1871:   dblLambda1a = C * sinSigma * dblLambda1
  Dim dblLambda2 As Double
1873:   dblLambda2 = Sigma + dblLambda1a
1874:   l = dblLambda - ((1 - C) * f * sinAlpha * dblLambda2)                                               ' [11]
  
1876:   pNewPoint.X = dblPX + RadToDeg(l)
1877:   pNewPoint.Y = RadToDeg(dblLat2)
  
1879:   dblAz2 = RadToDeg(atan2(sinAlpha, -dblLat2Temp))
1880:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2


  Exit Sub
ErrorHandler:
  HandleError True, "PointLineVincentyPerPoint " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function DistanceVincentyNumbers(dblPX As Double, dblPY As Double, dblQX As Double, dblQY As Double, _
        dblAz1 As Double, dblAz2 As Double) As Double
  On Error GoTo ErrorHandler

  
  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty-direct.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  
1901:   If dblPX = dblQX And dblPY = dblQY Then      ' SAME POINT
1902:     DistanceVincentyNumbers = 0
1903:     dblAz1 = 0
1904:     dblAz2 = 0
    Exit Function
1906:   End If
  
  
  Dim A As Double
  Dim B As Double
1911:   A = 6378137               ' WGS84 SPHEROID; EQUATORIAL RADIUS
1912:   B = 6356752.31424518      ' WGS84 SPHEROID; POLAR RADIUS
  
  Dim f As Double
1915:   f = 1 / 298.257223563     ' WGS84 SPHEROID; FLATTENING
  
  Dim l As Double           ' DIFFERENCE IN LONGITUDE
1918:   l = DegToRad(dblQX - dblPX)
  
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
1923:   U1 = Atn((1 - f) * (Tan(DegToRad(dblPY))))
1924:   U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
  Dim dblSinU1 As Double
  Dim dblSinU2 As Double
  Dim dblCosU1 As Double
  Dim dblCosU2 As Double
  
1931:   dblSinU1 = Sin(U1)
1932:   dblCosU1 = Cos(U1)
1933:   dblSinU2 = Sin(U2)
1934:   dblCosU2 = Cos(U2)
  
  Dim dblLambda As Double, dblLambdaComp As Double
1937:   dblLambda = l                     ' FIRST ESTIMATE OF LAMBDA
1938:   dblLambdaComp = 2 * dblPi
  Dim lngIterations As Long
1940:   lngIterations = 40
  
  Dim sinLambda As Double
  Dim cosLambda As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim Sigma As Double
  Dim sinAlpha As Double
  Dim cosSqAlpha As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
  
1955:   Do While (Abs(dblLambda - dblLambdaComp) > 0.000000000001) And (lngIterations > 0)       ' VINCENTY EQUATION NUMBERS
1956:     sinLambda = Sin(dblLambda)                                                          '  |
1957:     cosLambda = Cos(dblLambda)                                                          '  |
1958:     sinSigma = Sqr((dblCosU2 * sinLambda) ^ 2 + ((dblCosU1 * dblSinU2) - _
            (dblSinU1 * dblCosU2 * cosLambda)) ^ 2)                                     ' [14]
1960:     cosSigma = (dblSinU1 * dblSinU2) + (dblCosU1 * dblCosU2 * cosLambda)                ' [15]
1961:     Sigma = atan2(sinSigma, cosSigma)                                                   ' [16]
1962:     sinAlpha = (dblCosU1 * dblCosU2 * sinLambda) / sinSigma                             ' [17]
1963:     cosSqAlpha = 1 - (sinAlpha ^ 2)                                                     ' TRIG IDENTITY
1964:     If cosSqAlpha = 0 Then
1965:       cos2SigmaM = 0                                                                    ' ADAPTED FROM VENESS
1966:     Else
1967:       cos2SigmaM = cosSigma - ((2 * dblSinU1 * dblSinU2) / cosSqAlpha)                  ' [18]
1968:     End If
    
1970:     C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                      ' [10]
1971:     dblLambdaComp = dblLambda
1972:     dblLambda1 = cos2SigmaM + C * cosSigma * (-1 + (2 * cos2SigmaM * cos2SigmaM))
1973:     dblLambda1a = C * sinSigma * dblLambda1
    ' VINCENTY WRITES EQUATION AS "L = dblLambda - ((1 - C)...
1975:     dblLambda = l + ((1 - C) * f * sinAlpha * (Sigma + dblLambda1a))                    ' [11]
    
1977:     If lngIterations = 0 Then
1978:       MsgBox "Vincenty Formula failed to converge!"
1979:       DistanceVincentyNumbers = -9999
      Exit Function
1981:     End If
1982:     lngIterations = lngIterations - 1
1983:   Loop
  
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim DeltaSigma As Double
  Dim S As Double
  
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  
1995:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
  
  Dim dblA1 As Double
  Dim dblB1 As Double
  
2000:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
2001:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                           ' [3]
2002:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
2003:   dblB = (uSq / 1024) * (256 + dblB1)              ' [4]
  
2005:   DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
2006:   DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
2007:   DeltaSigma3 = cos2SigmaM + DeltaSigma2
2008:   DeltaSigma = dblB * sinSigma * DeltaSigma3                                                                 ' [6]
2009:   S = B * dblA * (Sigma - DeltaSigma)
  
'  var uSq = cosSqAlpha * (a*a - b*b) / (b*b);
'  var A = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
'  var B = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
'  var deltaSigma = B*sinSigma*(cos2SigmaM+B/4*(cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)-
'    B/6*cos2SigmaM*(-3+4*sinSigma*sinSigma)*(-3+4*cos2SigmaM*cos2SigmaM)));
'  var s = b*A*(sigma-deltaSigma);
'
'  s = s.toFixed(3); // round to 1mm precision
'  return s;
'}
  
  
2023:   dblAz1 = RadToDeg(atan2(dblCosU2 * sinLambda, (dblCosU1 * dblSinU2) - (dblSinU1 * dblCosU2 * cosLambda)))
2024:   dblAz2 = RadToDeg(atan2(dblCosU1 * sinLambda, -(dblSinU1 * dblCosU2) + (dblCosU1 * dblSinU2 * cosLambda)))
  
2026:   If dblAz1 < 0 Then dblAz1 = 360 + dblAz1
2027:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2
2028:   DistanceVincentyNumbers = S


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyNumbers " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function DistanceVincentyNumbers2(dblPX As Double, dblPY As Double, dblQX As Double, dblQY As Double, _
        dblAz1 As Double, dblAz2 As Double, _
        Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF DistanceVincentyNumbers TO ALLOW FOR ANY ELLIPSOID
  
  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty-direct.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  
2052:   If dblPX = dblQX And dblPY = dblQY Then      ' SAME POINT
2053:     DistanceVincentyNumbers2 = 0
2054:     dblAz1 = 0
2055:     dblAz2 = 0
    Exit Function
2057:   End If
  
  
  Dim A As Double
  Dim B As Double
2062:   A = dblEquatorialRadius   ' SPHEROID; EQUATORIAL RADIUS
2063:   B = dblPolarRadius        ' SPHEROID; POLAR RADIUS
  
  Dim f As Double
2066:   f = (A - B) / B           ' FLATTENING
  
  Dim l As Double           ' DIFFERENCE IN LONGITUDE
2069:   l = DegToRad(dblQX - dblPX)
  
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
2074:   U1 = Atn((1 - f) * (Tan(DegToRad(dblPY))))
2075:   U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
  Dim dblSinU1 As Double
  Dim dblSinU2 As Double
  Dim dblCosU1 As Double
  Dim dblCosU2 As Double
  
2082:   dblSinU1 = Sin(U1)
2083:   dblCosU1 = Cos(U1)
2084:   dblSinU2 = Sin(U2)
2085:   dblCosU2 = Cos(U2)
  
  Dim dblLambda As Double, dblLambdaComp As Double
2088:   dblLambda = l                     ' FIRST ESTIMATE OF LAMBDA
2089:   dblLambdaComp = 2 * dblPi
  Dim lngIterations As Long
2091:   lngIterations = 40
  
  Dim sinLambda As Double
  Dim cosLambda As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim Sigma As Double
  Dim sinAlpha As Double
  Dim cosSqAlpha As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
  
2106:   Do While (Abs(dblLambda - dblLambdaComp) > 0.000000000001) And (lngIterations > 0)       ' VINCENTY EQUATION NUMBERS
2107:     sinLambda = Sin(dblLambda)                                                          '  |
2108:     cosLambda = Cos(dblLambda)                                                          '  |
2109:     sinSigma = Sqr((dblCosU2 * sinLambda) ^ 2 + ((dblCosU1 * dblSinU2) - _
            (dblSinU1 * dblCosU2 * cosLambda)) ^ 2)                                     ' [14]
2111:     cosSigma = (dblSinU1 * dblSinU2) + (dblCosU1 * dblCosU2 * cosLambda)                ' [15]
2112:     Sigma = atan2(sinSigma, cosSigma)                                                   ' [16]
2113:     sinAlpha = (dblCosU1 * dblCosU2 * sinLambda) / sinSigma                             ' [17]
2114:     cosSqAlpha = 1 - (sinAlpha ^ 2)                                                     ' TRIG IDENTITY
2115:     If cosSqAlpha = 0 Then
2116:       cos2SigmaM = 0                                                                    ' ADAPTED FROM VENESS
2117:     Else
2118:       cos2SigmaM = cosSigma - ((2 * dblSinU1 * dblSinU2) / cosSqAlpha)                  ' [18]
2119:     End If
    
2121:     C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                      ' [10]
2122:     dblLambdaComp = dblLambda
2123:     dblLambda1 = cos2SigmaM + C * cosSigma * (-1 + (2 * cos2SigmaM * cos2SigmaM))
2124:     dblLambda1a = C * sinSigma * dblLambda1
    ' VINCENTY WRITES EQUATION AS "L = dblLambda - ((1 - C)...
2126:     dblLambda = l + ((1 - C) * f * sinAlpha * (Sigma + dblLambda1a))                    ' [11]
    
2128:     If lngIterations = 0 Then
2129:       MsgBox "Vincenty Formula failed to converge!"
2130:       DistanceVincentyNumbers2 = -9999
      Exit Function
2132:     End If
2133:     lngIterations = lngIterations - 1
2134:   Loop
  
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim DeltaSigma As Double
  Dim S As Double
  
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  
2146:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
  
  Dim dblA1 As Double
  Dim dblB1 As Double
  
2151:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
2152:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                           ' [3]
2153:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
2154:   dblB = (uSq / 1024) * (256 + dblB1)              ' [4]
  
2156:   DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
2157:   DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
2158:   DeltaSigma3 = cos2SigmaM + DeltaSigma2
2159:   DeltaSigma = dblB * sinSigma * DeltaSigma3                                                                 ' [6]
2160:   S = B * dblA * (Sigma - DeltaSigma)
  
'  var uSq = cosSqAlpha * (a*a - b*b) / (b*b);
'  var A = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
'  var B = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
'  var deltaSigma = B*sinSigma*(cos2SigmaM+B/4*(cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)-
'    B/6*cos2SigmaM*(-3+4*sinSigma*sinSigma)*(-3+4*cos2SigmaM*cos2SigmaM)));
'  var s = b*A*(sigma-deltaSigma);
'
'  s = s.toFixed(3); // round to 1mm precision
'  return s;
'}
  
  
2174:   dblAz1 = RadToDeg(atan2(dblCosU2 * sinLambda, (dblCosU1 * dblSinU2) - (dblSinU1 * dblCosU2 * cosLambda)))
2175:   dblAz2 = RadToDeg(atan2(dblCosU1 * sinLambda, -(dblSinU1 * dblCosU2) + (dblCosU1 * dblSinU2 * cosLambda)))
  
2177:   If dblAz1 < 0 Then dblAz1 = 360 + dblAz1
2178:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2
2179:   DistanceVincentyNumbers2 = S


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyNumbers2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function RadToDeg(dblRad As Double) As Double
  On Error GoTo ErrorHandler


2191:   RadToDeg = dblRad * 180 / dblPi


  Exit Function
ErrorHandler:
  HandleError True, "RadToDeg " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function DegToRad(dblDeg As Double) As Double
  On Error GoTo ErrorHandler


2202:   DegToRad = dblDeg * dblPi / 180


  Exit Function
ErrorHandler:
  HandleError True, "DegToRad " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function atan2(Y As Double, X As Double) As Double
  On Error GoTo ErrorHandler


2214:   If X > 0 Then
2215:     atan2 = Atn(Y / X)
2216:   ElseIf X < 0 Then
2217:     If Y = 0 Then
2218:       atan2 = (dblPi - Atn(Abs(Y / X)))
2219:     Else
2220:       atan2 = Sgn(Y) * (dblPi - Atn(Abs(Y / X)))
2221:     End If
2222:   Else    ' IF X = 0
2223:     If Y = 0 Then
2224:       atan2 = 0
2225:     Else
2226:       atan2 = Sgn(Y) * dblPi / 2
2227:     End If
2228:   End If


  Exit Function
ErrorHandler:
  HandleError True, "atan2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function TriangleAreaLegs(dblLeg1 As Double, dblLeg2 As Double, dblLeg3 As Double) As Double
  On Error GoTo ErrorHandler


  Dim dblS As Double
2240:   dblS = (dblLeg1 + dblLeg2 + dblLeg3) / 2
2241:   TriangleAreaLegs = Sqr(dblS * (dblS - dblLeg1) * (dblS - dblLeg2) * (dblS - dblLeg3))


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaLegs " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function TriangleAreaPoints(pPoint1 As IPoint, pPoint2 As IPoint, pPoint3 As IPoint) As Double
  On Error GoTo ErrorHandler


2252:   TriangleAreaPoints = Abs(((((pPoint2.X - pPoint1.X) * (pPoint3.Y - pPoint1.Y)) - ((pPoint3.X - pPoint1.X) * (pPoint2.Y - pPoint1.Y))) / 2))


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaPoints " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function TriangleAreaPoints3D(pPoint1 As IPoint, pPoint2 As IPoint, pPoint3 As IPoint) As Double
  On Error GoTo ErrorHandler

  
2263:   TriangleAreaPoints3D = TriangleAreaPoints3DValues(pPoint1.X, pPoint1.Y, pPoint1.Z, pPoint2.X, pPoint2.Y, pPoint2.Z, _
                                                  pPoint3.X, pPoint3.Y, pPoint3.Z)


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaPoints3D " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function



Public Function TriangleAreaPoints3DValues(dblPX As Double, dblPY As Double, dblPZ As Double, _
                                           dblQX As Double, dblQY As Double, dblQZ As Double, _
                                           dblRX As Double, dblRY As Double, dblRZ As Double) As Double
  On Error GoTo ErrorHandler


  Dim dblI As Double
  Dim dblJ As Double
  Dim dblK As Double
  
2284:   dblI = (((dblQY - dblPY) * (dblRZ - dblPZ)) - ((dblRY - dblPY) * (dblQZ - dblPZ))) ^ 2
2285:   dblJ = (((dblQX - dblPX) * (dblRZ - dblPZ)) - ((dblRX - dblPX) * (dblQZ - dblPZ))) ^ 2
2286:   dblK = (((dblQX - dblPX) * (dblRY - dblPY)) - ((dblRX - dblPX) * (dblQY - dblPY))) ^ 2

2288:   TriangleAreaPoints3DValues = (Sqr(dblI + dblJ + dblK)) / 2


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaPoints3DValues " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
                                           

Public Function EnvelopeToPolygon(pEnv As IEnvelope) As IPolygon
  On Error GoTo ErrorHandler


  Dim pPtColl As IPointCollection

2303:   Set pPtColl = New Polygon
2304:   With pPtColl
2305:       .AddPoint pEnv.LowerLeft
2306:       .AddPoint pEnv.UpperLeft
2307:       .AddPoint pEnv.UpperRight
2308:       .AddPoint pEnv.LowerRight
      'Close the polygon
2310:       .AddPoint pEnv.LowerLeft
2311:   End With
  
  Dim pPolygon As IPolygon
2314:   Set pPolygon = pPtColl
2315:   Set pPolygon.SpatialReference = pEnv.SpatialReference
  Dim pTopoOp As ITopologicalOperator
2317:   Set pTopoOp = pPolygon
2318:   pTopoOp.Simplify
    
2320:   Set EnvelopeToPolygon = pPtColl


  Exit Function
ErrorHandler:
  HandleError True, "EnvelopeToPolygon " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function EllipticArcToPolygon2(SegCollection As ISegmentCollection, NumVertices As Long) As IMultipoint
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  
'  Dim pEllArc As IEllipticArc
      
On Error GoTo erh

  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  
  Dim anIndex As Long
  Dim lngSegCount As Long
2342:   lngSegCount = SegCollection.SegmentCount - 1
  Dim theLength As Double
2344:   theLength = 0
  Dim theTestLength As Double
  Dim lngLengths() As Long
  ReDim lngLengths(lngSegCount)
2348:   For anIndex = 0 To lngSegCount
2349:     theTestLength = SegCollection.Segment(anIndex).length
2350:     theLength = theLength + theTestLength
2351:     lngLengths(anIndex) = theTestLength
2352:   Next anIndex
  
  Dim pProportion As Double
  Dim lngVertices() As Long
  Dim lngNumVertices As Long
  ReDim lngVertices(lngSegCount)
2358:   For anIndex = 0 To lngSegCount
2359:     lngNumVertices = Int((lngLengths(anIndex) / theLength) * NumVertices)
2360:     If lngNumVertices < 8 Then lngNumVertices = 8
2361:     lngVertices(anIndex) = lngNumVertices
2362:   Next anIndex
  
  Dim pMpt As IPointCollection
2365:   Set pMpt = New Multipoint
  Dim pPoint As IPoint
2367:   Set pPoint = New Point
  Dim pClone As IClone
  
  Dim pRatio As Double
  Dim anIndex2 As Long
  
2373:   For anIndex = 0 To lngSegCount
2374:     lngNumVertices = lngVertices(anIndex)
2375:     pRatio = 1 / lngNumVertices
2376:     Set pCurve = SegCollection.Segment(anIndex)
    
2378:     For anIndex2 = 0 To lngNumVertices
'      If pGeometry.GeometryType = esriGeometryEllipticArc Then
2380:       pCurve.QueryPoint 0, (pRatio * anIndex2), True, pPoint
2381:       Set pClone = pPoint
        
 '   Graphic_MakeFromGeometry pMxDoc, pPoint, "DeleteMe"
    
2385:       pMpt.AddPoint pClone.Clone
2386:     Next anIndex2
2387:   Next anIndex
  
2389:   Set EllipticArcToPolygon2 = pMpt
    Exit Function
  
erh:
2393:     MsgBox "Failed in EllipticArcToPolygon2: " & err.Description
End Function

Public Function CurveToPolygon(pOrigGeometry As IGeometry, NumVertices As Long) As IPolygon
On Error GoTo erh
  
  Dim pGeometryCollection As IGeometryCollection
2400:   Set pGeometryCollection = pOrigGeometry
  Dim pSpRef As ISpatialReference
2402:   Set pSpRef = pOrigGeometry.SpatialReference
  
  Dim pOrigPolygon As IPolycurve
2405:   Set pOrigPolygon = pOrigGeometry
  
  Dim dblFullLength As Double
2408:   dblFullLength = pOrigPolygon.length
  
  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  Dim pPolygon As IPointCollection
  Dim pRing As IRing
  Dim pSegment As ISegment
  Dim pStartPoint As IPoint
2416:   Set pStartPoint = New Point
  Dim pEndPoint As IPoint
2418:   Set pEndPoint = New Point
  Dim pClone As IClone
  Dim booFoundCurve As Boolean
  
  Dim lngRingCount As Long
  Dim lngNumVertices As Long
  Dim pRatio As Double
  Dim anIndex As Long
  Dim anIndex2 As Long
  Dim anIndex3 As Long
  Dim lngSegCount As Long
  
  Dim SegCollection As ISegmentCollection
  Dim pNewSegCollection As ISegmentCollection
  
  Dim pPathSegColl As ISegmentCollection
  Dim pNewSegment As ISegment
  Dim pNewLine As esriGeometry.ILine
  
  Dim pNewPolyGeoColl As IGeometryCollection
2438:   Set pNewPolyGeoColl = New Polygon
  Dim pNewRingGeometry As IGeometry
  Dim pPath As IPath
  Dim pSegmentCollection As ISegmentCollection
  Dim pNewSegCol As ISegmentCollection
  
2444:   lngRingCount = pGeometryCollection.GeometryCount - 1
2445:   For anIndex = 0 To lngRingCount
2446:     If TypeOf pOrigGeometry Is IPolyline Then
2447:       Set pPath = pGeometryCollection.Geometry(anIndex)
2448:       Set pSegmentCollection = pPath
2449:       Set pNewSegCol = New Ring
2450:       pNewSegCol.AddSegmentCollection pSegmentCollection
2451:       Set pRing = pNewSegCol
2452:       pRing.Close
2453:     Else
2454:       Set pRing = pGeometryCollection.Geometry(anIndex)
2455:     End If
2456:     Set SegCollection = pRing
2457:     Set pNewSegCollection = New Ring
2458:     lngSegCount = SegCollection.SegmentCount - 1
2459:     For anIndex2 = 0 To lngSegCount
2460:       Set pSegment = SegCollection.Segment(anIndex2)
2461:       Set pGeometry = pSegment
2462:       If pGeometry.GeometryType <> esriGeometryLine Then ' IF SEGMENT IS CURVE
2463:         booFoundCurve = True
2464:         lngNumVertices = Int((pSegment.length / dblFullLength) * NumVertices)
2465:         If lngNumVertices < 8 Then lngNumVertices = 8
2466:         pRatio = 1 / lngNumVertices
       
2468:         Set pCurve = pSegment
2469:         Set pPathSegColl = New Path
2470:         Set pNewSegment = New esriGeometry.Line
2471:         Set pStartPoint = pCurve.FromPoint
2472:         For anIndex3 = 1 To lngNumVertices
2473:           pCurve.QueryPoint 0, (pRatio * anIndex3), True, pEndPoint
2474:           pNewSegment.FromPoint = pStartPoint
2475:           pNewSegment.ToPoint = pEndPoint
          
2477:           Set pClone = pNewSegment
2478:           pPathSegColl.AddSegment pClone.Clone
          
'          If anIndex3 < 4 Then
'            MsgBox "Start Point:  X = " & CStr(pStartPoint.X) & ", Y = " & CStr(pStartPoint.Y) & vbCrLf & _
'              "End Point:  X = " & CStr(pEndPoint.X) & ", Y = " & CStr(pEndPoint.Y) & vbCrLf & _
'              "Segment Length = " & CStr(pNewSegment.length) & vbCrLf & _
'              "Segment Collection Count = " & CStr(pPathSegColl.SegmentCount)
'          End If
          
2487:           Set pClone = pEndPoint
2488:           Set pStartPoint = pClone.Clone
          
2490:         Next anIndex3
2491:         pNewSegCollection.AddSegmentCollection pPathSegColl

2493:       Else      ' IF SEGMENT IS ACTUALLY LINE, DON'T ADD MIDPOINTS
2494:         Set pClone = pSegment
2495:         pNewSegCollection.AddSegment pClone.Clone
2496:       End If
2497:     Next anIndex2
2498:     Set pNewRingGeometry = pNewSegCollection
2499:     pNewPolyGeoColl.AddGeometry pNewRingGeometry

2501:   Next anIndex
  
  Dim pNewPolygon As IPolygon
  
2505:   If booFoundCurve Or (TypeOf pOrigGeometry Is IPolyline) Then
  
2507:     Set pNewPolygon = pNewPolyGeoColl
    Dim pTopoOp As ITopologicalOperator
2509:     Set pTopoOp = pNewPolygon
2510:     pTopoOp.Simplify
2511:     Set pNewPolygon.SpatialReference = pSpRef

2513:   Else
2514:     Set pNewPolygon = pOrigGeometry
2515:     Set pNewPolygon.SpatialReference = pSpRef
2516:   End If
    
2518:   Set CurveToPolygon = pNewPolygon
  Exit Function
  
erh:
2522:     MsgBox "Failed in CurveToPolygon: " & vbCrLf & "Error = " & err.Description & vbCrLf & "Line Number = " & CStr(Erl)
End Function
Public Function CurveToPolyline(pOrigGeometry As IGeometry, NumVertices As Long) As IPolyline
On Error GoTo erh

  Dim pGeometryCollection As IGeometryCollection
2528:   Set pGeometryCollection = pOrigGeometry
  Dim pSpRef As ISpatialReference
2530:   Set pSpRef = pOrigGeometry.SpatialReference
  
  Dim pOrigPolyline As IPolycurve
  
2534:   Set pOrigPolyline = pOrigGeometry
  
  Dim dblFullLength As Double
2537:   dblFullLength = pOrigPolyline.length

  Dim pPath As IPath
  
  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  Dim pSegment As ISegment
  Dim pStartPoint As IPoint
2545:   Set pStartPoint = New Point
  Dim pEndPoint As IPoint
2547:   Set pEndPoint = New Point
  Dim pClone As IClone
  Dim booFoundCurve As Boolean
  
  Dim lngPathCount As Long
  Dim lngNumVertices As Long
  Dim pRatio As Double
  Dim anIndex As Long
  Dim anIndex2 As Long
  Dim anIndex3 As Long
  Dim lngSegCount As Long
  
  Dim SegCollection As ISegmentCollection
  Dim pNewSegCollection As ISegmentCollection
  
  Dim pPathSegColl As ISegmentCollection
  Dim pNewSegment As ISegment
  Dim pNewLine As esriGeometry.ILine
  
  Dim pNewPolyGeoColl As IGeometryCollection
2567:   Set pNewPolyGeoColl = New Polyline
  Dim pNewPathGeometry As IGeometry
  
  Dim pRing As IRing
  
2572:   lngPathCount = pGeometryCollection.GeometryCount - 1
2573:   For anIndex = 0 To lngPathCount
2574:     If TypeOf pOrigGeometry Is IPolygon Then
2575:       Set pRing = pGeometryCollection.Geometry(anIndex)
2576:       Set pPath = pRing
2577:     Else
2578:       Set pPath = pGeometryCollection.Geometry(anIndex)
2579:     End If
2580:     Set SegCollection = pPath
2581:     Set pNewSegCollection = New Path
2582:     lngSegCount = SegCollection.SegmentCount - 1
2583:     For anIndex2 = 0 To lngSegCount
2584:       Set pSegment = SegCollection.Segment(anIndex2)
2585:       Set pGeometry = pSegment
2586:       If pGeometry.GeometryType <> esriGeometryLine Then ' IF SEGMENT IS CURVE
2587:         booFoundCurve = True
2588:         lngNumVertices = Int((pSegment.length / dblFullLength) * NumVertices)
2589:         If lngNumVertices < 8 Then lngNumVertices = 8
2590:         pRatio = 1 / lngNumVertices
       
2592:         Set pCurve = pSegment
2593:         Set pPathSegColl = New Path
2594:         Set pNewSegment = New esriGeometry.Line
2595:         Set pStartPoint = pCurve.FromPoint
2596:         For anIndex3 = 1 To lngNumVertices
2597:           pCurve.QueryPoint 0, (pRatio * anIndex3), True, pEndPoint
2598:           pNewSegment.FromPoint = pStartPoint
2599:           pNewSegment.ToPoint = pEndPoint
          
2601:           Set pClone = pNewSegment
2602:           pPathSegColl.AddSegment pClone.Clone
2603:           Set pClone = pEndPoint
2604:           Set pStartPoint = pClone.Clone
2605:         Next anIndex3
2606:         pNewSegCollection.AddSegmentCollection pPathSegColl

2608:       Else      ' IF SEGMENT IS ACTUALLY LINE, DON'T ADD MIDPOINTS
2609:         Set pClone = pSegment
2610:         pNewSegCollection.AddSegment pClone.Clone
2611:       End If
2612:     Next anIndex2
2613:     Set pNewPathGeometry = pNewSegCollection
2614:     pNewPolyGeoColl.AddGeometry pNewPathGeometry

2616:   Next anIndex

  
  Dim pNewPolyline As IPolyline
  
2621:   If booFoundCurve Or (TypeOf pOrigGeometry Is IPolygon) Then
  
2623:     Set pNewPolyline = pNewPolyGeoColl
    Dim pTopoOp As ITopologicalOperator
2625:     Set pTopoOp = pNewPolyline
2626:     pTopoOp.Simplify
2627:     Set pNewPolyline.SpatialReference = pSpRef

2629:   Else
2630:     Set pNewPolyline = pOrigGeometry
2631:     Set pNewPolyline.SpatialReference = pSpRef
2632:   End If
    
2634:   Set CurveToPolyline = pNewPolyline
  Exit Function
  
erh:
2638:     MsgBox "Failed in CurveToPolyline: " & vbCrLf & "Error = " & err.Description & vbCrLf & "Line Number = " & CStr(Erl)
End Function
Public Sub Graphic_MakeFromGeometry(ByRef pMxDoc As IMxDocument, ByRef pGeometry As IGeometry, Optional strName As String)
  On Error GoTo ErrorHandler

  
  Dim pMxDocument As esriArcMapUI.IMxDocument
  Dim pActiveView As esriCarto.IActiveView
  
  Dim pGContainer As IGraphicsContainer
2648:   Set pGContainer = pMxDoc.FocusMap
  
  Dim pElement As IElement
  Dim pPolygonElement As IPolygonElement
  Dim pSpatialReference As ISpatialReference
  Dim pGraphicElement As IGraphicElement
  Dim pElementProperties As IElementProperties
  
  Dim pClone As IClone
2657:   Set pClone = pGeometry
  Dim pNewGeometry As IGeometry
2659:   Set pNewGeometry = pClone.Clone
  
  Dim pGeometryType As esriGeometryType
2662:   pGeometryType = pNewGeometry.GeometryType
  
  'ADD GEOMETRY, NAME AND SPATIAL REFERENCE TO GRAPHIC ELEMENT
  Select Case pGeometryType
    Case 0:
2667:       MsgBox "Null Geometry!  No graphic added..."
    Case 1:
2669:       Set pElement = New MarkerElement
    Case 3, 6, 13, 14, 15, 16:
2671:       Set pElement = New LineElement
    Case 4, 11:
2673:       Set pElement = New PolygonElement
    Case 5:
2675:       Set pElement = New RectangleElement
    Case Else:
2677:       MsgBox "Unexpected Shape Type:  Unable to add graphic..."
2678:   End Select
    
2680:   pElement.Geometry = pNewGeometry
2681:   Set pGraphicElement = pElement
2682:   Set pSpatialReference = pGeometry.SpatialReference
2683:   Set pGraphicElement.SpatialReference = pSpatialReference
2684:   Set pElementProperties = pElement
2685:   pElementProperties.Name = strName

  ' ADD GRAPHIC TO GRAPHICS CONTAINER
2688:   pGContainer.AddElement pElement, 0

  'Draw
2691:   pMxDoc.ActiveView.PartialRefresh esriViewGraphics, Nothing, Nothing


  Exit Sub
ErrorHandler:
  HandleError True, "Graphic_MakeFromGeometry " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Function Graphic_ReturnElementFromGeometry(ByRef pMxDoc As IMxDocument, ByRef pGeometry As IGeometry, _
    Optional strName As String, Optional AddToView As Boolean) As IElement
  On Error GoTo ErrorHandler

  
  Dim pMxDocument As esriArcMapUI.IMxDocument
  Dim pActiveView As esriCarto.IActiveView
  
  Dim pGContainer As IGraphicsContainer
2707:   Set pGContainer = pMxDoc.FocusMap
  
  Dim pElement As IElement
  Dim pPolygonElement As IPolygonElement
  Dim pSpatialReference As ISpatialReference
  Dim pGraphicElement As IGraphicElement
  Dim pElementProperties As IElementProperties
  
  Dim pClone As IClone
2716:   Set pClone = pGeometry
  Dim pNewGeometry As IGeometry
2718:   Set pNewGeometry = pClone.Clone
  
  Dim pGeometryType As esriGeometryType
2721:   pGeometryType = pNewGeometry.GeometryType
  
  'ADD GEOMETRY, NAME AND SPATIAL REFERENCE TO GRAPHIC ELEMENT
  Select Case pGeometryType
    Case 0:
2726:       MsgBox "Null Geometry!  No graphic added..."
    Case 1:
2728:       Set pElement = New MarkerElement
    Case 3, 6, 13, 14, 15, 16:
2730:       Set pElement = New LineElement
    Case 4, 11:
2732:       Set pElement = New PolygonElement
    Case 5:
2734:       Set pElement = New RectangleElement
    Case Else:
2736:       MsgBox "Unexpected Shape Type:  Unable to add graphic..."
      Exit Function
2738:   End Select
  
2740:   pElement.Geometry = pNewGeometry
2741:   Set pGraphicElement = pElement
2742:   Set pSpatialReference = pGeometry.SpatialReference
2743:   Set pGraphicElement.SpatialReference = pSpatialReference
2744:   Set pElementProperties = pElement
2745:   pElementProperties.Name = strName
  
2747:   If AddToView Then
    ' ADD GRAPHIC TO GRAPHICS CONTAINER
2749:     pGContainer.AddElement pElement, 0
    'Draw
2751:     pMxDoc.ActiveView.PartialRefresh esriViewGraphics, Nothing, Nothing
2752:   End If
  
2754:   Set Graphic_ReturnElementFromGeometry = pElement


  Exit Function
ErrorHandler:
  HandleError True, "Graphic_ReturnElementFromGeometry " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub ShowVertices(pMxDoc As IMxDocument, pGeometry As IGeometry, Optional strName As String, _
        Optional DeleteCurrentGraphicsWithName As Boolean)
  On Error GoTo ErrorHandler


  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim pPoly As IPolygon
  Dim pLine As IPolyline
  Dim pOutVertex As IPoint, lOutPart As Long, lOutVertex As Long
  
2773:   If DeleteCurrentGraphicsWithName And (strName <> "") Then
2774:     Call DeleteGraphicsByName(pMxDoc, "DeleteMe")
2775:   End If
  
  Dim pArray As IArray
2778:   Set pArray = New esriSystem.Array
  
  Dim pPointCollection As IPointCollection
2781:   Set pPointCollection = pGeometry
  
  Dim pPointEnum As IEnumVertex
2784:   Set pPointEnum = pPointCollection.EnumVertices
  
2786:   pPointEnum.Reset
  
  Dim pVertex As IPoint
2789:   Set pVertex = New Point
  'Query the next vertex - have to cocreate the point
  'QueryNext is faster than Next, because the method doesn't have
  'to create the point each time
2793:   pPointEnum.QueryNext pVertex, lOutPart, lOutVertex
  
2795:   Do While Not pVertex.IsEmpty
2796:     Graphic_MakeFromGeometry pMxDoc, pVertex, strName
2797:     pPointEnum.QueryNext pVertex, lOutPart, lOutVertex
'    Debug.Print lOutPart & ",    " & lOutVertex & ",  " & pVertex.IsEmpty
2799:   Loop


  Exit Sub
ErrorHandler:
  HandleError True, "ShowVertices " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Function CalcBearing(ByRef Point1 As IPoint, ByRef Point2 As IPoint) As Double
  On Error GoTo ErrorHandler


  Dim dblBearing As Double

  Dim xDist As Double
  Dim yDist As Double
  Dim xyTanDeg As Double
  
2816:   xDist = (Point1.X - Point2.X)
2817:   yDist = (Point1.Y - Point2.Y)
2818:   If yDist = 0 Then
2819:     If xDist < 0 Then
2820:       xyTanDeg = -90
2821:     ElseIf xDist = 0 Then
2822:       xyTanDeg = 0
2823:     Else
2824:       xyTanDeg = 90
2825:     End If
2826:   Else
2827:     xyTanDeg = AsDegrees(Atn(xDist / yDist))
2828:   End If

2830:   If (yDist >= 0) Then
2831:     dblBearing = 180 + xyTanDeg
2832:   Else
2833:     If (xDist <= 0) Then
2834:       dblBearing = xyTanDeg
2835:     Else
2836:       dblBearing = 360 + xyTanDeg
2837:     End If
2838:   End If ' END CALCULATING BEARING
  
2840:   dblBearing = Abs(dblBearing)
2841:   CalcBearing = dblBearing


  Exit Function
ErrorHandler:
  HandleError True, "CalcBearing " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CalcBearing2(ByRef Point1 As IPoint, ByRef Point2 As IPoint) As Double
  On Error GoTo ErrorHandler


  Dim dblBearing As Double

  Dim xDist As Double
  Dim yDist As Double
  Dim xyTanDeg As Double
  
2859:   xDist = (Point1.X - Point2.X)
2860:   yDist = (Point1.Y - Point2.Y)
  
2862:   If xDist = 0 And yDist = 0 Then
2863:     CalcBearing2 = -9999
2864:   Else
2865:     If yDist = 0 Then
2866:       If xDist < 0 Then
2867:         xyTanDeg = -90
2868:       ElseIf xDist = 0 Then
2869:         xyTanDeg = 0
2870:       Else
2871:         xyTanDeg = 90
2872:       End If
2873:     Else
2874:       xyTanDeg = AsDegrees(Atn(xDist / yDist))
2875:     End If
  
2877:     If (yDist >= 0) Then
2878:       dblBearing = 180 + xyTanDeg
2879:     Else
2880:       If (xDist <= 0) Then
2881:         dblBearing = xyTanDeg
2882:       Else
2883:         dblBearing = 360 + xyTanDeg
2884:       End If
2885:     End If ' END CALCULATING BEARING
    
2887:     dblBearing = Abs(dblBearing)
2888:     CalcBearing2 = dblBearing
2889:   End If


  Exit Function
ErrorHandler:
  HandleError True, "CalcBearing2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CalcDistMatrix(pArray As esriSystem.IArray, Optional IncludeLine As Boolean, _
    Optional IncludeBearing As Boolean, Optional pApp As IApplication) As Collection
  On Error GoTo ErrorHandler


2902:   Screen.MousePointer = vbHourglass
  
  ' RETURNS A COLLECTION OF IVariantArray OBJECTS
  ' EACH IVariantArray IDENTIFIED BY STRING; CONCATENATION OF [ORIGIN INDEX] & "_" & [DESTINATION INDEX]
  ' EACH IVariantArray OBJECT CONTAINS:
  '       0) ORIGIN SHAPE INDEX VALUE
  '       1) DESTINATION SHAPE INDEX VALUE
  '       2) DISTANCE
  '       3) CONNECTOR POLYLINE:  OPTIONAL; CONTAINS BOOLEAN "FALSE" IF NOT REQUESTED
  '       4) BEARING:             OPTIONAL; CONTAINS BOOLEAN "FALSE" IF NOT REQUESTED
  
  Dim pCollection As Collection
2914:   Set pCollection = New Collection
  
  Dim pProxOp As IProximityOperator
  
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  
  Dim lngArrayMaxIndex As Long
2922:   lngArrayMaxIndex = pArray.Count - 1
  
  Dim pGeometry1 As IGeometry
  Dim pGeometry2 As IGeometry
  
  Dim dblDistance As Double
  Dim dblBearing As Double
  Dim dblRevBearing As Double
  Dim strID As String
  Dim strRevID As String
  Dim pConnector As IPolyline
  Dim pRevConnector As IPolyline
  
  Dim pVarArray As IVariantArray
  Dim pRevVarArray As IVariantArray
  
2938:   If Not pApp Is Nothing Then
      ' PROGRESS BAR STUFF
    Dim psbar As IStatusBar
2941:     Set psbar = pApp.StatusBar
    Dim pPro As IStepProgressor
2943:     Set pPro = psbar.ProgressBar
    Dim lngCounter As Long
2945:     lngCounter = 0
    Dim lngTotalCount As Long
2947:     lngTotalCount = (((lngArrayMaxIndex + 1) * lngArrayMaxIndex) / 2)
    Dim strTotalCount As String
2949:     strTotalCount = CStr(lngTotalCount)
2950:     pPro.position = 1
2951:     psbar.ShowProgressBar "Building Preliminary Distance Matrix:  Step 1 of " & strTotalCount & "...", 1, _
            lngTotalCount, 1, True
2953:   End If
  
  Dim pOutputPointCollection As IPointCollection
  
2957:   For lngIndex = 0 To lngArrayMaxIndex
2958:     Set pGeometry1 = pArray.Element(lngIndex)
    
2960:     For lngIndex2 = lngIndex To lngArrayMaxIndex
      
2962:       strID = CStr(lngIndex) & "_" & CStr(lngIndex2)
2963:       strRevID = CStr(lngIndex2) & "_" & CStr(lngIndex)
      
2965:       Set pVarArray = New VarArray
2966:       Set pRevVarArray = New VarArray
      
      ' FIRST ELEMENT
2969:       pVarArray.Add lngIndex         ' FIRST VALUES IN THE ARRAY ARE ORIGIN NODES
2970:       pRevVarArray.Add lngIndex2
      
      ' SECOND ELEMENT
2973:       pVarArray.Add lngIndex2        ' SECOND VALUES IN THE ARRAY ARE "TO" NODES
2974:       pRevVarArray.Add lngIndex
      
2976:       If lngIndex = lngIndex2 Then   ' IF MEASURING DISTANCE TO ITSELF
2977:         Set pConnector = New Polyline
2978:         pConnector.SetEmpty
2979:         Set pRevConnector = New Polyline
2980:         pRevConnector.SetEmpty
        
        ' THIRD ELEMENT
2983:         pVarArray.Add 0              ' THIRD VALUE IS DISTANCE
        
        ' FOURTH ELEMENT
2986:         If IncludeLine Then
2987:           pVarArray.Add pConnector   ' FOURTH VALUE IS CONNECTION LINE
2988:         Else
2989:           pVarArray.Add False        ' FOURTH VALUE:  JUST ADDING SMALL PLACEHOLDER ELEMENT
2990:         End If
        
        ' FIFTH ELEMENT
2993:         If IncludeBearing Then
2994:           pVarArray.Add -999         ' FIFTH VALUE IS BEARING
2995:         Else
2996:           pVarArray.Add False
2997:         End If
        
        ' ADD VARARRAY TO ORIGINAL COLLECTION
3000:         pCollection.Add pVarArray, strID
      
3002:       Else
        
3004:         If Not pApp Is Nothing Then
3005:           lngCounter = lngCounter + 1
3006:           pPro.Message = "Building Preliminary Distance Matrix:  Step " & CStr(lngCounter) & " of " & strTotalCount & "..."
3007:           psbar.StepProgressBar
3008:         End If
        
3010:         Set pGeometry2 = pArray.Element(lngIndex2)
        
3012:         If IncludeLine Or IncludeBearing Then
          Dim pLineArray As IArray
3014:           Set pLineArray = CalcClosestPoints(pGeometry1, pGeometry2, 10)
          
3016:           If TypeOf pLineArray.Element(0) Is esriSystem.IStringArray Then      ' FUNCTION FAILED FOR SOME REASON
            Dim pStrArray As IStringArray
3018:             Set pStrArray = pLineArray.Element(0)
3019:             MsgBox "Failed to connect:" & vbCrLf & "Message = " & pStrArray.Element(0) & vbCrLf & _
                   "Index 1 = " & CStr(lngIndex) & " of " & CStr(lngArrayMaxIndex) & vbCrLf & _
                   "Index 2 = " & CStr(lngIndex2) & " of " & CStr(lngArrayMaxIndex)
3022:             Set pConnector = New Polyline
3023:             pConnector.SetEmpty
3024:             Set pRevConnector = New Polyline
3025:             pRevConnector.SetEmpty
            
            ' THIRD ELEMENT
3028:             pVarArray.Add 0              ' THIRD VALUE IS DISTANCE
            
            ' FOURTH ELEMENT
3031:             If IncludeLine Then
3032:               pVarArray.Add pConnector   ' FOURTH VALUE IS CONNECTION LINE
3033:             Else
3034:               pVarArray.Add False        ' FOURTH VALUE:  JUST ADDING SMALL PLACEHOLDER ELEMENT
3035:             End If
            
            ' FIFTH ELEMENT
3038:             If IncludeBearing Then
3039:               pVarArray.Add -999         ' FIFTH VALUE IS BEARING
3040:             Else
3041:               pVarArray.Add False
3042:             End If
3043:           Else
          
3045:             If IncludeLine Then
3046:               Set pConnector = pLineArray.Element(0)
3047:               Set pRevConnector = New Polyline
3048:               Set pOutputPointCollection = pRevConnector
3049:               pOutputPointCollection.AddPoint pLineArray.Element(2)
3050:               pOutputPointCollection.AddPoint pLineArray.Element(1)
              
              ' THIRD ELEMENT
3053:               pVarArray.Add pConnector.length
3054:               pRevVarArray.Add pConnector.length
              
              ' FOURTH ELEMENT
3057:               pVarArray.Add pConnector
3058:               pRevVarArray.Add pRevConnector
3059:             Else
              ' FOURTH ELEMENT
3061:               pVarArray.Add False        ' JUST ADDING SMALL PLACEHOLDER ELEMENT
3062:               pRevVarArray.Add False     ' JUST ADDING SMALL PLACEHOLDER ELEMENT
3063:             End If
3064:             If IncludeBearing Then
3065:               dblBearing = CalcBearing(pLineArray.Element(1), pLineArray.Element(2))
3066:               If dblBearing < 180 Then
3067:                 dblRevBearing = dblBearing + 180
3068:               Else
3069:                 dblRevBearing = dblBearing - 180
3070:               End If
              ' FIFTH ELEMENT
3072:               pVarArray.Add dblBearing
3073:               pRevVarArray.Add dblRevBearing
3074:             Else
              ' FIFTH ELEMENT
3076:               pVarArray.Add False
3077:               pRevVarArray.Add False
3078:             End If
3079:           End If
3080:         Else
          ' THIRD ELEMENT
3082:           Set pProxOp = pGeometry1
3083:           dblDistance = pProxOp.ReturnDistance(pGeometry2)
3084:           pVarArray.Add dblDistance
3085:           pRevVarArray.Add dblDistance
          ' FOURTH ELEMENT   (DISTANCE)
3087:           pVarArray.Add False
3088:           pRevVarArray.Add False
          ' FIFTH ELEMENT   (BEARING)
3090:           pVarArray.Add False
3091:           pRevVarArray.Add False
3092:         End If
        
3094:         pCollection.Add pVarArray, strID
3095:         pCollection.Add pRevVarArray, strRevID

3097:       End If
3098:     Next lngIndex2
3099:   Next lngIndex
  
3101:   Set CalcDistMatrix = pCollection

3103:   Screen.MousePointer = vbDefault
3104:   If Not pApp Is Nothing Then
3105:     pPro.position = 1
3106:     psbar.HideProgressBar
3107:   End If


  Exit Function
ErrorHandler:
  HandleError True, "CalcDistMatrix " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function CalcClosestPoints(ByVal Shape1 As IGeometry, ByVal shape2 As IGeometry, Optional intMaxCurveRepeat As Integer) As IArray
  On Error GoTo ErrorHandler


' CalcClosestPoints
' Jenness Enterprises (www.jennessent.com)
' Given two shapes, this script returns an IARRAY object containing the line connecting the closest points on each shape, plus the connection points
' CURRENTLY DOES NOT GUARANTEE SUCCESS WITH TRUE CURVES BECAUSE VERTICES ARE NOT GOOD QUERY POINTS; ATTEMPTS SEVERAL RUNS BACK AND FORTH

' Dim pRelationalOperator As IRelationalOperator
Dim pGeometryType1 As esriGeometryType
Dim pGeometryType2 As esriGeometryType
Dim pGeometry1 As IGeometry
Dim pGeometry2 As IGeometry

3129: Set pGeometry1 = Shape1
3130: Set pGeometry2 = shape2

3132: pGeometryType2 = shape2.GeometryType

' IF SHAPE #2 HAPPENS TO BE POINT, SET THAT ONE FIRST
Dim ShouldReverse As Boolean
3136: ShouldReverse = False
3137: If pGeometryType2 = esriGeometryPoint Then
3138:   Set pGeometry1 = shape2
3139:   Set pGeometry2 = Shape1
3140:   pGeometryType2 = pGeometry2.GeometryType
3141:   ShouldReverse = True
3142: End If
  
3144: pGeometryType1 = pGeometry1.GeometryType

Dim pArray As IArray
3147: Set pArray = New esriSystem.Array
Dim pOutputLine As IPolyline
3149: Set pOutputLine = New Polyline
Dim pOutputPointCollection As IPointCollection
3151: Set pOutputPointCollection = pOutputLine

Dim pStartPoint As IPoint
Dim pEndPoint As IPoint

Dim pPoint1 As IPoint
Dim pPoint2 As IPoint
3158: Set pPoint2 = New Point

Dim pProximityOp As IProximityOperator
Dim pStringArray As IStringArray

' CHECK FOR NULL SHAPES
' CHECK FOR INTERSECTING SHAPES
' NOT SURE IF THIS WILL WORK WITH MULTIPOINTS

3167: If pGeometry1.IsEmpty Or pGeometry2.IsEmpty Then
3168:   Set pStringArray = New strArray
3169:   pStringArray.Add "Empty Shapes"
3170:   pStringArray.Add CStr(pGeometry1.IsEmpty)
3171:   pStringArray.Add CStr(pGeometry2.IsEmpty)
3172:   pArray.Add pStringArray
3173:   Set CalcClosestPoints = pArray
  Exit Function
3175: Else
3176:   Set pProximityOp = pGeometry1
3177:   If pProximityOp.ReturnDistance(pGeometry2) = 0 Then
3178:     Set pStringArray = New strArray
3179:     pStringArray.Add "Intersecting Shapes"
3180:     pArray.Add pStringArray
3181:     Set CalcClosestPoints = pArray
    Exit Function
3183:   End If
3184: End If

3186: If pGeometryType1 = esriGeometryPoint Then
3187:   Set pPoint1 = pGeometry1
  
3189:   If pGeometryType2 = esriGeometryPoint Then
3190:     Set pPoint2 = pGeometry2
    
3192:     If pPoint1.X = pPoint2.X And pPoint1.Y = pPoint2.Y Then
3193:       If ShouldReverse Then
3194:         pArray.Add pOutputLine
3195:         pArray.Add pPoint2
3196:         pArray.Add pPoint1
3197:       Else
3198:         pArray.Add pOutputLine
3199:         pArray.Add pPoint1
3200:         pArray.Add pPoint2
3201:       End If
3202:     Else
3203:       If ShouldReverse Then
3204:         pOutputPointCollection.AddPoint pPoint2
3205:         pOutputPointCollection.AddPoint pPoint1
3206:         pArray.Add pOutputLine
3207:         pArray.Add pPoint2
3208:         pArray.Add pPoint1
3209:       Else
3210:         pOutputPointCollection.AddPoint pPoint1
3211:         pOutputPointCollection.AddPoint pPoint2
3212:         pArray.Add pOutputLine
3213:         pArray.Add pPoint1
3214:         pArray.Add pPoint2
3215:       End If
3216:     End If
3217:   Else
    
3219:     Set pProximityOp = pGeometry2
    
3221:     pProximityOp.QueryNearestPoint pPoint1, esriNoExtension, pPoint2
    
3223:     If ShouldReverse Then
3224:       pOutputPointCollection.AddPoint pPoint2
3225:       pOutputPointCollection.AddPoint pPoint1
3226:       pArray.Add pOutputLine
3227:       pArray.Add pPoint2
3228:       pArray.Add pPoint1
3229:     Else
3230:       pOutputPointCollection.AddPoint pPoint1
3231:       pOutputPointCollection.AddPoint pPoint2
3232:       pArray.Add pOutputLine
3233:       pArray.Add pPoint1
3234:       pArray.Add pPoint2
3235:     End If
    
    
3238:   End If
3239: Else
  Dim dblTestDistance As Double
  Dim pEnvelope As IEnvelope
  Dim pEnvelope2 As IEnvelope
3243:   Set pEnvelope = pGeometry1.Envelope
3244:   Set pEnvelope2 = pGeometry2.Envelope
3245:   pEnvelope.Union pEnvelope2
3246:   dblTestDistance = (pEnvelope.Height * pEnvelope.Width)
  Dim dblMaxDistance As Double
3248:   dblMaxDistance = dblTestDistance
  
  Dim pPointCollection1 As IPointCollection
  Dim pPointCollection2 As IPointCollection
  
3253:   If pGeometry1.GeometryType = esriGeometryEnvelope Then
    Dim pTempEnv As IEnvelope
3255:     Set pTempEnv = pGeometry1
    Dim pTempPoly1 As IPolygon
    Dim pTempPoint1 As IPoint
3258:     Set pTempPoly1 = New Polygon
3259:     Set pPointCollection1 = pTempPoly1
    Dim dXmin1 As Double
    Dim dYmin1 As Double
    Dim dXmax1 As Double
    Dim dYmax1 As Double
3264:     pTempEnv.QueryCoords dXmin1, dYmin1, dXmax1, dYmax1
3265:     Set pTempPoint1 = New Point
3266:     pTempPoint1.X = dXmin1
3267:     pTempPoint1.Y = dYmin1
3268:     pPointCollection1.AddPoint pTempPoint1
    
3270:     Set pTempPoint1 = New Point
3271:     pTempPoint1.X = dXmin1
3272:     pTempPoint1.Y = dYmax1
3273:     pPointCollection1.AddPoint pTempPoint1
    
3275:     Set pTempPoint1 = New Point
3276:     pTempPoint1.X = dXmax1
3277:     pTempPoint1.Y = dYmax1
3278:     pPointCollection1.AddPoint pTempPoint1
    
3280:     Set pTempPoint1 = New Point
3281:     pTempPoint1.X = dXmax1
3282:     pTempPoint1.Y = dYmin1
3283:     pPointCollection1.AddPoint pTempPoint1
3284:   Else
3285:     Set pPointCollection1 = pGeometry1
3286:   End If
  
3288:   If pGeometry2.GeometryType = esriGeometryEnvelope Then
    Dim pTempEnv2 As IEnvelope
3290:     Set pTempEnv2 = pGeometry2
    Dim pTempPoly2 As IPolygon
    Dim pTempPoint2 As IPoint
3293:     Set pTempPoly2 = New Polygon
3294:     Set pPointCollection2 = pTempPoly2
    Dim dXmin2 As Double
    Dim dYmin2 As Double
    Dim dXmax2 As Double
    Dim dYmax2 As Double
3299:     pTempEnv2.QueryCoords dXmin2, dYmin2, dXmax2, dYmax2
3300:     Set pTempPoint2 = New Point
3301:     pTempPoint2.X = dXmin2
3302:     pTempPoint2.Y = dYmin2
3303:     pPointCollection2.AddPoint pTempPoint2
    
3305:     Set pTempPoint2 = New Point
3306:     pTempPoint2.X = dXmin2
3307:     pTempPoint2.Y = dYmax2
3308:     pPointCollection2.AddPoint pTempPoint2
    
3310:     Set pTempPoint2 = New Point
3311:     pTempPoint2.X = dXmax2
3312:     pTempPoint2.Y = dYmax2
3313:     pPointCollection2.AddPoint pTempPoint2
    
3315:     Set pTempPoint2 = New Point
3316:     pTempPoint2.X = dXmax2
3317:     pTempPoint2.Y = dYmin2
3318:     pPointCollection2.AddPoint pTempPoint2
3319:   Else
3320:     Set pPointCollection2 = pGeometry2
3321:   End If
  
  Dim pClone As IClone
  
  Dim pVertex As IPoint
3326:   Set pVertex = New Point
  
  Dim pPointEnum As IEnumVertex
  Dim lngOutPart As Long
  Dim lngOutVertex As Long
  
3332:   Set pPointEnum = pPointCollection1.EnumVertices
3333:   pPointEnum.Reset
3334:   pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
  
  ' CHECK IF CURVES; THIS CODE JUST CHECKS FIRST SEGMENT FOR CURVATURE
  Dim booWorkingWithCurves As Boolean
  Dim pSegmentCollection1 As ISegmentCollection
3339:   Set pSegmentCollection1 = pGeometry1
  Dim pSegment1 As ISegment
3341:   Set pSegment1 = pSegmentCollection1.Segment(0)
  Dim pGeometryTypeA As esriGeometryType
3343:   pGeometryTypeA = pSegment1.GeometryType
  
  Dim pSegmentCollection2 As ISegmentCollection
3346:   Set pSegmentCollection2 = pGeometry2
  Dim pSegment2 As ISegment
3348:   Set pSegment2 = pSegmentCollection2.Segment(0)
  Dim pGeometryTypeB As esriGeometryType
3350:   pGeometryTypeB = pSegment2.GeometryType
  
3352:   booWorkingWithCurves = (pGeometryTypeA = esriGeometryBezier3Curve) Or _
    (pGeometryTypeA = esriGeometryCircularArc) Or _
    (pGeometryTypeA = esriGeometryEllipticArc) Or _
    (pGeometryTypeB = esriGeometryBezier3Curve) Or _
    (pGeometryTypeB = esriGeometryCircularArc) Or _
    (pGeometryTypeB = esriGeometryEllipticArc)

3359:   Do While Not pVertex.IsEmpty
3360:     Set pProximityOp = pGeometry2
3361:     dblTestDistance = pProximityOp.ReturnDistance(pVertex)
3362:     If dblTestDistance < dblMaxDistance Then
3363:       dblMaxDistance = dblTestDistance
3364:       Set pClone = pVertex
3365:       Set pPoint1 = pClone.Clone
3366:       pProximityOp.QueryNearestPoint pVertex, esriNoExtension, pPoint2
3367:     End If
3368:     pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
3369:   Loop
  
3371:   Set pPointEnum = pPointCollection2.EnumVertices
3372:   pPointEnum.Reset
3373:   pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
  
3375:   Do While Not pVertex.IsEmpty
3376:     Set pProximityOp = pGeometry1
3377:     dblTestDistance = pProximityOp.ReturnDistance(pVertex)
3378:     If dblTestDistance < dblMaxDistance Then
3379:       dblMaxDistance = dblTestDistance
3380:       Set pClone = pVertex
3381:       Set pPoint2 = pClone.Clone
3382:       pProximityOp.QueryNearestPoint pVertex, esriNoExtension, pPoint1
3383:     End If
3384:     pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
3385:   Loop
  
  ' FOR DEBUGGING
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  ' IF WORKING WITH CURVES, GO BACK AND FORTH A FEW TIMES
3391:   If booWorkingWithCurves Then
    Dim intRepeat As Integer
    Dim pPoint1Temp As IPoint, pPoint2Temp As IPoint
    
3395:     Do Until (intRepeat = intMaxCurveRepeat)
 '     Graphic_MakeFromGeometry pMxDoc, pPoint1, "DeleteMe"
 '     Graphic_MakeFromGeometry pMxDoc, pPoint2, "DeleteMe"
      
3399:       Set pProximityOp = pGeometry2
3400:       pProximityOp.QueryNearestPoint pPoint1, esriNoExtension, pPoint2
      
3402:       Set pProximityOp = pGeometry1
3403:       pProximityOp.QueryNearestPoint pPoint2, esriNoExtension, pPoint1
      
3405:       intRepeat = intRepeat + 1
    
3407:     Loop
  
3409:   End If
  
3411:   If ShouldReverse Then
3412:     pOutputPointCollection.AddPoint pPoint2
3413:     pOutputPointCollection.AddPoint pPoint1
3414:     pArray.Add pOutputLine
3415:     pArray.Add pPoint2
3416:     pArray.Add pPoint1
3417:   Else
3418:     pOutputPointCollection.AddPoint pPoint1
3419:     pOutputPointCollection.AddPoint pPoint2
3420:     pArray.Add pOutputLine
3421:     pArray.Add pPoint1
3422:     pArray.Add pPoint2
3423:   End If
  
3425: End If

3427: Set CalcClosestPoints = pArray


  Exit Function
ErrorHandler:
  HandleError True, "CalcClosestPoints " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CalcCheckClockwise(theP As IPoint, theQ As IPoint, theR As IPoint) As Boolean
 
On Error GoTo err
' CalcCheckClockwise
' Jenness Enterprises <www.jennessent.com)>
' Given 3 consecutive points, this scripts calculates whether the third point lies to the right
' (clockwise) or to the left (counter-clockwise) of the line connecting the first point to
' the second point.

' CLOCKWISE IF TRUE
3445: CalcCheckClockwise = ((theQ.X * (theR.Y - theP.Y)) + (theQ.Y * (theP.X - theR.X)) - ((theP.X) * (theR.Y)) _
      + ((theP.Y) * (theR.X)) < 0)
    Exit Function
err:
3449:   MsgBox "Messed up CalcCheckClockwise..."
End Function

Public Function PointAdd(pPointA As IPoint, pPointB As IPoint) As IPoint
  On Error GoTo ErrorHandler


3456:   Set PointAdd = New Point
3457:   PointAdd.PutCoords pPointA.X + pPointB.X, pPointA.Y + pPointB.Y


  Exit Function
ErrorHandler:
  HandleError True, "PointAdd " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function PointSubtract(pPointA As IPoint, pPointB As IPoint) As IPoint
  On Error GoTo ErrorHandler


3469:   Set PointSubtract = New Point
3470:   PointSubtract.PutCoords pPointA.X - pPointB.X, pPointA.Y - pPointB.Y


  Exit Function
ErrorHandler:
  HandleError True, "PointSubtract " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function AsRadians(theDegrees As Double) As Double
  On Error GoTo ErrorHandler


3482:   AsRadians = dblPi * (theDegrees / 180)


  Exit Function
ErrorHandler:
  HandleError True, "AsRadians " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function AsDegrees(theRadians As Double) As Double
  On Error GoTo ErrorHandler


3494:   AsDegrees = (theRadians * 180) / dblPi


  Exit Function
ErrorHandler:
  HandleError True, "AsDegrees " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub CalcPointLine(ptOrigin As IPoint, theLength As Double, dblAzimuth As Double, ptEndPoint As IPoint, _
    Optional pLine As IPolyline)
  On Error GoTo ErrorHandler


' Jenness Enterprises <www.jennessent.com>
' Given an origin point, distance and bearing, this script will return a new point at that distance and bearing, and a line
' connecting that new point to the origin point

'' MAKE SURE AZIMUTH IS BETWEEN 0 AND 360
Dim theAzimuth As Double
3513: theAzimuth = dblAzimuth

3515: Set ptEndPoint = New Point

3517: Do While theAzimuth < 0
3518:   theAzimuth = theAzimuth + 360
3519: Loop
3520: Do While theAzimuth > 360
3521:   theAzimuth = theAzimuth - 360
3522: Loop
'theAzimuth = theAzimuth Mod 360
'
'' NEW SEGMENT AND POINT DISTANCE NORTH/SOUTH AND EAST/WEST BASED ON DISTANCE AND BEARING FROM ORIGIN.
'' THERE ARE EIGHT DIFFERENT POSSIBILITIES:  THE BEARING COULD BE ONE OF THE FOUR CARDINAL DIRECTIONS OR IT
'' COULD BE IN ONE OF THE FOUR QUADRANTS.  THE BEARING IS TREATED DIFFERENTLY IN EACH OF THESE CIRCUMSTANCES.
'' THE CALCULATION TO DETERMINE THE NEW POINT LOCATION IS ESSENTIALLY A MATTER OF TAKING THE SINE OR THE
'' COSINE OF THE ANGLE (AFTER CONVERTING IT TO RADIANS), AND MULTIPLYING THAT SINE OR COSINE BY THE MEASURED
'' DISTANCE.  PLEASE CONTACT THE AUTHOR IF THIS DOESN'T MAKE SENSE, OR IF YOU WOULD LIKE FURTHER EXPLANATION.
Dim NorthSouthDistance As Double
Dim EastWestDistance As Double
Dim EastWest As Integer
Dim NorthSouth As Integer

3536: If theAzimuth = 0 Or theAzimuth = 360 Then
3537:   NorthSouthDistance = theLength
3538:   NorthSouth = 1
3539:   EastWestDistance = 0
3540:   EastWest = 1
3541: ElseIf (theAzimuth = 180) Then
3542:   NorthSouthDistance = theLength
3543:   NorthSouth = -1
3544:   EastWestDistance = 0
3545:   EastWest = 1
3546: ElseIf (theAzimuth = 90) Then
3547:   NorthSouthDistance = 0
3548:   NorthSouth = 1
3549:   EastWestDistance = theLength
3550:   EastWest = 1
3551: ElseIf (theAzimuth = 270) Then
3552:   NorthSouthDistance = 0
3553:   NorthSouth = 1
3554:   EastWestDistance = theLength
3555:   EastWest = -1
3556: ElseIf ((theAzimuth > 0) And (theAzimuth < 90)) Then
3557:   NorthSouthDistance = Cos(AsRadians(theAzimuth)) * theLength
3558:   NorthSouth = 1
3559:   EastWestDistance = Sin(AsRadians(theAzimuth)) * theLength
3560:   EastWest = 1
3561: ElseIf ((theAzimuth > 90) And (theAzimuth < 180)) Then
3562:   NorthSouthDistance = (Sin(AsRadians(theAzimuth - 90))) * theLength
3563:   NorthSouth = -1
3564:   EastWestDistance = (Cos(AsRadians(theAzimuth - 90))) * theLength
3565:   EastWest = 1
3566: ElseIf ((theAzimuth > 180) And (theAzimuth < 270)) Then
3567:   NorthSouthDistance = (Cos(AsRadians(theAzimuth - 180))) * theLength
3568:   NorthSouth = -1
3569:   EastWestDistance = (Sin(AsRadians(theAzimuth - 180))) * theLength
3570:   EastWest = -1
3571: ElseIf ((theAzimuth > 270) And (theAzimuth < 360)) Then
3572:   NorthSouthDistance = (Sin(AsRadians(theAzimuth - 270))) * theLength
3573:   NorthSouth = 1
3574:   EastWestDistance = (Cos(AsRadians(theAzimuth - 270))) * theLength
3575:   EastWest = -1
3576: End If

Dim theMovementNorth As Double
Dim theMovementWest As Double

3581: theMovementNorth = NorthSouthDistance * NorthSouth
3582: theMovementWest = EastWestDistance * EastWest

Dim startX As Double
Dim startY As Double

3587: ptOrigin.QueryCoords startX, startY
3588: ptEndPoint.PutCoords startX + theMovementWest, startY + theMovementNorth

3590: If Not pLine Is Nothing Then
  Dim pPointColl As IPointCollection
3592:   pLine.SetEmpty
3593:   Set pPointColl = pLine
3594:   pPointColl.AddPoint ptOrigin
3595:   pPointColl.AddPoint ptEndPoint
3596: End If


  Exit Sub
ErrorHandler:
  HandleError True, "CalcPointLine " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Function EllipticArcToPolygon(SegCollection As ISegmentCollection, NumVertices As Long) As IPolygon4
  On Error GoTo ErrorHandler


'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  
'  Dim pEllArc As IEllipticArc
  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  
  Dim anIndex As Long
  Dim lngSegCount As Long
3616:   lngSegCount = SegCollection.SegmentCount - 1
  Dim theLength As Double
3618:   theLength = 0
  Dim theTestLength As Double
  Dim lngLengths() As Long
  ReDim lngLengths(lngSegCount)
3622:   For anIndex = 0 To lngSegCount
3623:     theTestLength = SegCollection.Segment(anIndex).length
3624:     theLength = theLength + theTestLength
3625:     lngLengths(anIndex) = theTestLength
3626:   Next anIndex
  
  Dim pProportion As Double
  Dim lngVertices() As Long
  Dim lngNumVertices As Long
  ReDim lngVertices(lngSegCount)
3632:   For anIndex = 0 To lngSegCount
3633:     lngNumVertices = Int((lngLengths(anIndex) / theLength) * NumVertices)
3634:     If lngNumVertices < 8 Then lngNumVertices = 8
3635:     lngVertices(anIndex) = lngNumVertices
3636:   Next anIndex
  
  Dim pMpt As IPointCollection
3639:   Set pMpt = New Multipoint
  Dim pPoint As IPoint
3641:   Set pPoint = New Point
  Dim pClone As IClone
  
  Dim pRatio As Double
  Dim anIndex2 As Long
  
3647:   For anIndex = 0 To lngSegCount
3648:     lngNumVertices = lngVertices(anIndex)
3649:     pRatio = 1 / lngNumVertices
3650:     Set pCurve = SegCollection.Segment(anIndex)
    
3652:     For anIndex2 = 0 To lngNumVertices
'      If pGeometry.GeometryType = esriGeometryEllipticArc Then
3654:       pCurve.QueryPoint 0, (pRatio * anIndex2), True, pPoint
3655:       Set pClone = pPoint
        
 '   Graphic_MakeFromGeometry pMxDoc, pPoint, "DeleteMe"
    
3659:       pMpt.AddPoint pClone.Clone
3660:     Next anIndex2
3661:   Next anIndex
  
  Dim pPoly4 As IPolygon4
  Dim pTopoOp2 As ITopologicalOperator2
  Dim pTopoOp3 As ITopologicalOperator3
3666:   Set pTopoOp2 = pMpt
3667:   Set pPoly4 = pTopoOp2.ConvexHull
3668:   Set pTopoOp3 = pPoly4
3669:   pTopoOp3.IsKnownSimple = False
3670:   pTopoOp3.Simplify
  
3672:   Set EllipticArcToPolygon = pPoly4


  Exit Function
ErrorHandler:
  HandleError True, "EllipticArcToPolygon " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function FeaturePlanetOGraphicToPlanetOCentric(pGeometry As IGeometry, Optional dblMajorAxis As Double = -999, _
    Optional dblMinorAxis As Double = -999, Optional dblLongShift As Double = 0) As IGeometry
  On Error GoTo ErrorHandler

  


  ' ASSUMES pGeometry IS IN A GEOGRAPHIC PROJECTION AND IS EITHER A POINT, POLYLINE, POLYGON OR MULTIPOINT
  
3690:   If dblMajorAxis <= 0 Or dblMinorAxis <= 0 Then
3691:     If Not TypeOf pGeometry.SpatialReference Is IGeographicCoordinateSystem Then
3692:       MsgBox "Unexpected Spatial Reference:" & vbCrLf & _
          "The function 'FeaturePlanetOgraphicToPlanetOcentric' only accepts geometries with geographic projections." & vbCrLf & _
          "This geometry has spatial reference '" & pGeometry.SpatialReference.Name & "..."
3695:       Set FeaturePlanetOGraphicToPlanetOCentric = Nothing
3696:     End If
    Dim pGCS As IGeographicCoordinateSystem
3698:     Set pGCS = pGeometry.SpatialReference
    
    Dim pEllipsoid As ISpheroid
3701:     Set pEllipsoid = pGCS.Datum.Spheroid
    
  '  Dim dblFlattening As Double
3704:     dblMajorAxis = pEllipsoid.SemiMajorAxis
3705:     dblMinorAxis = pEllipsoid.SemiMinorAxis
  '  dblFlattening = pEllipsoid.Flattening
3707:   End If
  
  Dim pPointColl As IPointCollection
  
  Dim pOutput As IPointCollection
  Dim pPoint As IPoint
  Dim pClone As IClone
  
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  Dim dblNewLongitude As Double
  Dim dblNewLatitude As Double
  
  Dim pEnumVertex As IEnumVertex2
  Dim lngOutPart As Long
  Dim lngOutVertex As Long
  
  Dim lngIndex As Long
  
  ' IF A POINT, JUST CONVERT LATITUDE AND LONGITUDE AND RETURN NEW POINT
  ' IF POLYLINE, POLYGON OR MULTIPOINT, THEN CREATE NEW IPointCollection BY CLONING ORIGINAL SHAPE.  THEN JUST
  ' ADJUST EACH POINT IN POINT COLLECTION
  
3730:   If TypeOf pGeometry Is IPoint Then
    Dim pNewPoint As IPoint
3732:     Set pNewPoint = New Point
3733:     Set pNewPoint.SpatialReference = pGeometry.SpatialReference
    
3735:     Set pPoint = pGeometry
3736:     dblLongitude = pPoint.X
3737:     dblLatitude = pPoint.Y
    
3739:     XYOGraphicToOCentric dblLongitude, dblLatitude, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3740:     pNewPoint.PutCoords dblNewLongitude, dblNewLatitude
        
3742:     Set FeaturePlanetOGraphicToPlanetOCentric = pNewPoint
    
3744:   ElseIf TypeOf pGeometry Is IPolyline Then
    
3746:     Set pClone = pGeometry
    Dim pNewPolyline As IPolyline
3748:     Set pNewPolyline = pClone.Clone
3749:     Set pPointColl = pNewPolyline
3750:     Set pEnumVertex = pPointColl.EnumVertices
3751:     Set pPoint = New Point
    
3753:     pEnumVertex.Reset
3754:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3756:     Do While lngOutVertex > -1

3758:       XYOGraphicToOCentric pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3759:       pEnumVertex.put_Y dblNewLatitude
3760:       pEnumVertex.put_X dblNewLongitude
      
3762:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3764:     Loop
    
3766:     Set FeaturePlanetOGraphicToPlanetOCentric = pNewPolyline
    
3768:   ElseIf TypeOf pGeometry Is IPolygon Then
    
3770:     Set pClone = pGeometry
    Dim pNewPolygon As IPolygon
3772:     Set pNewPolygon = pClone.Clone
    
3774:     Set pPointColl = pNewPolygon
3775:     Set pEnumVertex = pPointColl.EnumVertices
3776:     Set pPoint = New Point
    
3778:     pEnumVertex.Reset
3779:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3781:     Do While lngOutVertex > -1

3783:       XYOGraphicToOCentric pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3784:       pEnumVertex.put_Y dblNewLatitude
3785:       pEnumVertex.put_X dblNewLongitude
      
3787:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3789:     Loop
    
3791:     Set FeaturePlanetOGraphicToPlanetOCentric = pNewPolygon
    
3793:   ElseIf TypeOf pGeometry Is IMultipoint Then
    
3795:     Set pClone = pGeometry
    Dim pNewMultipoint As IMultipoint
3797:     Set pNewMultipoint = pClone.Clone
3798:     Set pPointColl = pNewMultipoint
3799:     Set pEnumVertex = pPointColl.EnumVertices
3800:     Set pPoint = New Point
    
3802:     pEnumVertex.Reset
3803:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3805:     Do While lngOutVertex > -1

3807:       XYOGraphicToOCentric pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3808:       pEnumVertex.put_Y dblNewLatitude
3809:       pEnumVertex.put_X dblNewLongitude
      
3811:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3813:     Loop
    
3815:     Set FeaturePlanetOGraphicToPlanetOCentric = pNewMultipoint
  
3817:   Else
3818:     MsgBox "Unexpected Geometry Type:" & vbCrLf & _
        "The function 'PlanetOCentricToPlanetOGraphic' only accepts points, polylines, polygons or multipoints."
3820:     Set FeaturePlanetOGraphicToPlanetOCentric = Nothing
3821:   End If


  Exit Function
ErrorHandler:
  HandleError True, "FeaturePlanetOGraphicToPlanetOCentric " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function FeaturePlanetOCentricToPlanetOGraphic(pGeometry As IGeometry, Optional dblMajorAxis As Double = -999, _
    Optional dblMinorAxis As Double = -999, Optional dblLongShift As Double = 0) As IGeometry
  On Error GoTo ErrorHandler


  ' ASSUMES pGeometry IS IN A GEOGRAPHIC PROJECTION AND IS EITHER A POINT, POLYLINE, POLYGON OR MULTIPOINT
  
3837:   If dblMajorAxis <= 0 Or dblMinorAxis <= 0 Then
3838:     If Not TypeOf pGeometry.SpatialReference Is IGeographicCoordinateSystem Then
3839:       MsgBox "Unexpected Spatial Reference:" & vbCrLf & _
          "The function 'FeaturePlanetOCentricToPlanetOGraphic' only accepts geometries with geographic projections." & vbCrLf & _
          "This geometry has spatial reference '" & pGeometry.SpatialReference.Name & "..."
3842:       Set FeaturePlanetOCentricToPlanetOGraphic = Nothing
3843:     End If
    Dim pGCS As IGeographicCoordinateSystem
3845:     Set pGCS = pGeometry.SpatialReference
    
    Dim pEllipsoid As ISpheroid
3848:     Set pEllipsoid = pGCS.Datum.Spheroid
    
  '  Dim dblFlattening As Double
3851:     dblMajorAxis = pEllipsoid.SemiMajorAxis
3852:     dblMinorAxis = pEllipsoid.SemiMinorAxis
  '  dblFlattening = pEllipsoid.Flattening
3854:   End If
  
  Dim pPointColl As IPointCollection
  
  Dim pOutput As IPointCollection
  Dim pPoint As IPoint
  Dim pClone As IClone
  
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  Dim dblNewLongitude As Double
  Dim dblNewLatitude As Double
  
  Dim pEnumVertex As IEnumVertex2
  Dim lngOutPart As Long
  Dim lngOutVertex As Long
  
  Dim lngIndex As Long
  
  ' IF A POINT, JUST CONVERT LATITUDE AND LONGITUDE AND RETURN NEW POINT
  ' IF POLYLINE, POLYGON OR MULTIPOINT, THEN CREATE NEW IPointCollection BY CLONING ORIGINAL SHAPE.  THEN JUST
  ' ADJUST EACH POINT IN POINT COLLECTION
  
3877:   If TypeOf pGeometry Is IPoint Then
    Dim pNewPoint As IPoint
3879:     Set pNewPoint = New Point
3880:     Set pNewPoint.SpatialReference = pGeometry.SpatialReference
    
3882:     Set pPoint = pGeometry
3883:     dblLongitude = pPoint.X
3884:     dblLatitude = pPoint.Y
    
3886:     XYOCentricToOGraphic dblLongitude, dblLatitude, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3887:     pNewPoint.PutCoords dblNewLongitude, dblNewLatitude
        
3889:     Set FeaturePlanetOCentricToPlanetOGraphic = pNewPoint
    
3891:   ElseIf TypeOf pGeometry Is IPolyline Then
    
3893:     Set pClone = pGeometry
    Dim pNewPolyline As IPolyline
3895:     Set pNewPolyline = pClone.Clone
3896:     Set pPointColl = pNewPolyline
3897:     Set pEnumVertex = pPointColl.EnumVertices
3898:     Set pPoint = New Point
    
3900:     pEnumVertex.Reset
3901:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3903:     Do While lngOutVertex > -1

3905:       XYOCentricToOGraphic pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3906:       pEnumVertex.put_Y dblNewLatitude
3907:       pEnumVertex.put_X dblNewLongitude
      
3909:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3911:     Loop
    
3913:     Set FeaturePlanetOCentricToPlanetOGraphic = pNewPolyline
    
3915:   ElseIf TypeOf pGeometry Is IPolygon Then
    
3917:     Set pClone = pGeometry
    Dim pNewPolygon As IPolygon
3919:     Set pNewPolygon = pClone.Clone
    
3921:     Set pPointColl = pNewPolygon
3922:     Set pEnumVertex = pPointColl.EnumVertices
3923:     Set pPoint = New Point
    
3925:     pEnumVertex.Reset
3926:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3928:     Do While lngOutVertex > -1

3930:       XYOCentricToOGraphic pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3931:       pEnumVertex.put_Y dblNewLatitude
3932:       pEnumVertex.put_X dblNewLongitude
      
3934:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3936:     Loop
    
3938:     Set FeaturePlanetOCentricToPlanetOGraphic = pNewPolygon
    
3940:   ElseIf TypeOf pGeometry Is IMultipoint Then
    
3942:     Set pClone = pGeometry
    Dim pNewMultipoint As IMultipoint
3944:     Set pNewMultipoint = pClone.Clone
3945:     Set pPointColl = pNewMultipoint
3946:     Set pEnumVertex = pPointColl.EnumVertices
3947:     Set pPoint = New Point
    
3949:     pEnumVertex.Reset
3950:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3952:     Do While lngOutVertex > -1

3954:       XYOCentricToOGraphic pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3955:       pEnumVertex.put_Y dblNewLatitude
3956:       pEnumVertex.put_X dblNewLongitude
      
3958:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3960:     Loop
    
3962:     Set FeaturePlanetOCentricToPlanetOGraphic = pNewMultipoint
  
3964:   Else
3965:     MsgBox "Unexpected Geometry Type:" & vbCrLf & _
        "The function 'PlanetOCentricToPlanetOGraphic' only accepts points, polylines, polygons or multipoints."
3967:     Set FeaturePlanetOCentricToPlanetOGraphic = Nothing
3968:   End If


  Exit Function
ErrorHandler:
  HandleError True, "FeaturePlanetOCentricToPlanetOGraphic " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function FeatureLongitudeShift(pGeometry As IGeometry, dblLongShift As Double) As IGeometry
  On Error GoTo ErrorHandler

  
  Dim pPointColl As IPointCollection
  
  Dim pOutput As IPointCollection
  Dim pPoint As IPoint
  Dim pClone As IClone
  
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  Dim dblNewLongitude As Double
  Dim dblNewLatitude As Double
  
  Dim pEnumVertex As IEnumVertex2
  Dim lngOutPart As Long
  Dim lngOutVertex As Long
  
  Dim lngIndex As Long
  
  ' IF A POINT, JUST CONVERT LATITUDE AND LONGITUDE AND RETURN NEW POINT
  ' IF POLYLINE, POLYGON OR MULTIPOINT, THEN CREATE NEW IPointCollection BY CLONING ORIGINAL SHAPE.  THEN JUST
  ' ADJUST EACH POINT IN POINT COLLECTION
  
4001:   If TypeOf pGeometry Is IPoint Then
    Dim pNewPoint As IPoint
4003:     Set pNewPoint = New Point
4004:     Set pNewPoint.SpatialReference = pGeometry.SpatialReference
    
4006:     Set pPoint = pGeometry
4007:     dblLongitude = pPoint.X + dblLongShift
4008:     dblLatitude = pPoint.Y
    
4010:     pNewPoint.PutCoords dblLatitude, dblLongitude
        
4012:     Set FeatureLongitudeShift = pNewPoint
    
4014:   ElseIf TypeOf pGeometry Is IPolyline Then
    
4016:     Set pClone = pGeometry
    Dim pNewPolyline As IPolyline
4018:     Set pNewPolyline = pClone.Clone
4019:     Set pPointColl = pNewPolyline
4020:     Set pEnumVertex = pPointColl.EnumVertices
4021:     Set pPoint = New Point
    
4023:     pEnumVertex.Reset
4024:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4026:     Do While lngOutVertex > -1

4028:       pEnumVertex.put_Y pPoint.Y
4029:       pEnumVertex.put_X pPoint.X + dblLongShift
      
4031:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4033:     Loop
    
4035:     Set FeatureLongitudeShift = pNewPolyline
    
4037:   ElseIf TypeOf pGeometry Is IPolygon Then
    
4039:     Set pClone = pGeometry
    Dim pNewPolygon As IPolygon
4041:     Set pNewPolygon = pClone.Clone
    
4043:     Set pPointColl = pNewPolygon
4044:     Set pEnumVertex = pPointColl.EnumVertices
4045:     Set pPoint = New Point
    
4047:     pEnumVertex.Reset
4048:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4050:     Do While lngOutVertex > -1

4052:       pEnumVertex.put_Y pPoint.Y
4053:       pEnumVertex.put_X pPoint.X + dblLongShift
      
4055:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4057:     Loop
    
4059:     Set FeatureLongitudeShift = pNewPolygon
    
4061:   ElseIf TypeOf pGeometry Is IMultipoint Then
    
4063:     Set pClone = pGeometry
    Dim pNewMultipoint As IMultipoint
4065:     Set pNewMultipoint = pClone.Clone
4066:     Set pPointColl = pNewMultipoint
4067:     Set pEnumVertex = pPointColl.EnumVertices
4068:     Set pPoint = New Point
    
4070:     pEnumVertex.Reset
4071:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4073:     Do While lngOutVertex > -1

4075:       pEnumVertex.put_Y pPoint.Y
4076:       pEnumVertex.put_X pPoint.X + dblLongShift
      
4078:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4080:     Loop
    
4082:     Set FeatureLongitudeShift = pNewMultipoint
  
4084:   Else
4085:     MsgBox "Unexpected Geometry Type:" & vbCrLf & _
        "The function 'FeatureLongitudeShift' only accepts points, polylines, polygons or multipoints."
4087:     Set FeatureLongitudeShift = Nothing
4088:   End If


  Exit Function
ErrorHandler:
  HandleError True, "FeatureLongitudeShift " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub XYOCentricToOGraphic(dblLongitude As Double, dblLatitude As Double, dblMajorAxis As Double, dblMinorAxis As Double, _
    dblLongitudeShift As Double, dblNewLongitude As Double, dblNewLatitude As Double)
  On Error GoTo ErrorHandler

  
  ' ORIGINAL AVENUE CODE FROM View.Ocentric2Ographic
'  theLon = pt.GetX
'  theLon = theLon + theLonShift
'
'  theLat = pt.GetY
'  theLat = theLat * Number.GetPi / 180
'  theLat = ((((theMajorAx / theMinorAx)^2) * (theLat.tan))).atan
'  theLat = theLat * (180 / Number.GetPi)

4110:   dblNewLongitude = dblLongitude + dblLongitudeShift
4111:   dblNewLatitude = AsDegrees(Atn(((dblMajorAxis / dblMinorAxis) ^ 2) * (Tan(AsRadians(dblLatitude)))))


  Exit Sub
ErrorHandler:
  HandleError True, "XYOCentricToOGraphic " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub XYOGraphicToOCentric(dblLongitude As Double, dblLatitude As Double, dblMajorAxis As Double, dblMinorAxis As Double, _
    dblLongitudeShift As Double, dblNewLongitude As Double, dblNewLatitude As Double)
  On Error GoTo ErrorHandler

  
  ' ORIGINAL AVENUE CODE FROM View.Ographic2Ocentric
'  theLon = pt.GetX
'  theLon = theLon + theLonShift
'
'  theLat = pt.GetY
'  theLat = theLat * Number.GetPi / 180
'  theLat = (((theLat.tan) / ((theMajorAx / theMinorAx)^2))).atan
'  theLat = theLat * (180 / Number.GetPi)


4134:   dblNewLongitude = dblLongitude + dblLongitudeShift
4135:   dblNewLatitude = AsDegrees(Atn(Tan(AsRadians(dblLatitude)) / ((dblMajorAxis / dblMinorAxis) ^ 2)))


  Exit Sub
ErrorHandler:
  HandleError True, "XYOGraphicToOCentric " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function WrapToBoundary(pGeometry As IGeometry, dblXMin As Double, dblXMax As Double, dblYMin As Double, dblYMax As Double, _
      Optional pMxDoc As IMxDocument) As IGeometry
  On Error GoTo ErrorHandler

  
  Dim dblXRange As Double
  Dim dblYRange As Double
4150:   dblXRange = dblXMax - dblXMin
4151:   dblYRange = dblYMax - dblYMin
  
  Dim dblTestX As Double
  Dim dblTestY As Double
  Dim pPoint As IPoint
  Dim pNewPoint As IPoint
  
  Dim pTopoOp As ITopologicalOperator3
  Dim pSpRef As ISpatialReference
4160:   Set pSpRef = pGeometry.SpatialReference
    
  Dim lngIndex As Long
  
4164:   If TypeOf pGeometry Is IPoint Then
    
4166:     Set pPoint = pGeometry
4167:     dblTestX = pPoint.X
4168:     dblTestY = pPoint.Y
    
4170:     Do Until dblTestX <= dblXMax
4171:       dblTestX = dblTestX - dblXRange
4172:     Loop
4173:     Do Until dblTestX >= dblXMin
4174:       dblTestX = dblTestX + dblXRange
4175:     Loop
    
4177:     Do Until dblTestY <= dblYMax
4178:       dblTestY = dblTestY - dblYRange
4179:     Loop
4180:     Do Until dblTestY >= dblYMin
4181:       dblTestY = dblTestY + dblYRange
4182:     Loop
    
4184:     Set pNewPoint = New Point
4185:     pNewPoint.PutCoords dblTestX, dblTestY
4186:     Set pNewPoint.SpatialReference = pSpRef
    
4188:     Set WrapToBoundary = pNewPoint
4189:   Else
    
    ' START BY MAKING A SET OF CLIPPING ENVELOPES COVERING THE EXTENT OF THE SHAPE
    Dim pEnvelope As IEnvelope
4193:     Set pEnvelope = pGeometry.Envelope
    
    Dim dblEnvMaxX As Double
    Dim dblEnvMinX As Double
    Dim dblEnvMaxY As Double
    Dim dblEnvMinY As Double
    
4200:     dblEnvMaxX = pEnvelope.XMax
4201:     dblEnvMinX = pEnvelope.XMin
4202:     dblEnvMaxY = pEnvelope.YMax
4203:     dblEnvMinY = pEnvelope.YMin
    
    Dim dblRunningX As Double
    Dim dblRunningY As Double
    
4208:     dblRunningX = dblXMin
4209:     dblRunningY = dblYMin
    
    Dim dblShiftX As Double
    Dim dblShiftY As Double
    Dim dblShiftBaseX As Double
    Dim dblShiftBaseY As Double
    
4216:     dblShiftBaseX = 0
4217:     dblShiftBaseY = 0
    
    Dim dblRunningBaseY As Double
    
    ' IDENTIFY BOTTOM LEFT CORNER
4222:     Do Until dblRunningX <= dblEnvMinX
4223:       dblRunningX = dblRunningX - dblXRange
4224:       dblShiftBaseX = dblShiftBaseX - dblXRange
4225:     Loop
4226:     Do Until dblRunningY <= dblEnvMinY
4227:       dblRunningY = dblRunningY - dblYRange
4228:       dblShiftBaseY = dblShiftBaseY - dblYRange
4229:     Loop
4230:     dblRunningBaseY = dblRunningY
    
    ' MAKE SET OF RECTANGULAR POLYGONS, WHERE EACH RECTANGLE IS EQUAL TO SIZE OF BOUNDARY, AND SET ENTIRELY COVERS SHAPE
    Dim pRectArray As esriSystem.IArray
4234:     Set pRectArray = New esriSystem.Array
    
    Dim pShiftArray As esriSystem.IArray
4237:     Set pShiftArray = New esriSystem.Array
    Dim pShiftSubArray As esriSystem.IDoubleArray
    
    Dim pRect As IPointCollection
    
    Dim pRelOp As IRelationalOperator
4243:     Set pRelOp = pGeometry
    
4245:     dblShiftX = dblShiftBaseX
4246:     dblShiftY = dblShiftBaseY
    
4248:     Do Until dblRunningX >= dblEnvMaxX
4249:       Do Until dblRunningY >= dblEnvMaxY
4250:         Set pRect = New Polygon
        
4252:         Set pNewPoint = New Point
4253:         pNewPoint.PutCoords dblRunningX, dblRunningY
4254:         pRect.AddPoint pNewPoint
        
4256:         Set pNewPoint = New Point
4257:         pNewPoint.PutCoords dblRunningX, dblRunningY + dblYRange
4258:         pRect.AddPoint pNewPoint
        
4260:         Set pNewPoint = New Point
4261:         pNewPoint.PutCoords dblRunningX + dblXRange, dblRunningY + dblYRange
4262:         pRect.AddPoint pNewPoint
        
4264:         Set pNewPoint = New Point
4265:         pNewPoint.PutCoords dblRunningX + dblXRange, dblRunningY
4266:         pRect.AddPoint pNewPoint
        
4268:         Set pNewPoint = New Point
4269:         pNewPoint.PutCoords dblRunningX, dblRunningY
4270:         pRect.AddPoint pNewPoint
        
4272:         Set pTopoOp = pRect
4273:         pTopoOp.Simplify
        
        ' ONLY CONSIDER RECTANGLES THAT ACTUALLY INTERSECT THE ORIGINAL SHAPE
4276:         If Not pRelOp.Disjoint(pRect) Then
4277:           pRectArray.Add pRect
4278:           Set pShiftSubArray = New esriSystem.DoubleArray
4279:           pShiftSubArray.Add dblShiftX
4280:           pShiftSubArray.Add dblShiftY
4281:           pShiftArray.Add pShiftSubArray
          
4283:         End If
4284:         dblRunningY = dblRunningY + dblYRange
4285:         dblShiftY = dblShiftY + dblYRange
4286:       Loop
4287:       dblRunningY = dblRunningBaseY
4288:       dblShiftY = dblShiftBaseY
4289:       dblRunningX = dblRunningX + dblXRange
4290:       dblShiftX = dblShiftX + dblXRange
4291:     Loop
    
    Dim pOutputColl As IGeometryCollection
4294:     Set pOutputColl = New GeometryBag
    Dim pUnionTopoOp As ITopologicalOperator
    
    Dim pRectPoly As IPolygon
    Dim pClipPointColl As IPointCollection
    
    Dim pClone As IClone
    
4302:     If TypeOf pGeometry Is IPolyline Then
      Dim pPolyline As IPolyline
4304:       Set pPolyline = pGeometry
4305:       Set pUnionTopoOp = New Polyline
      
      Dim pClipPolyline As IPolyline
4308:       For lngIndex = 0 To pRectArray.Count - 1
4309:         Set pRectPoly = pRectArray.Element(lngIndex)
4310:         Set pClone = pPolyline
4311:         Set pClipPolyline = pClone.Clone
4312:         Set pTopoOp = pClipPolyline
4313:         pTopoOp.Simplify
        
4315:         pTopoOp.Clip pRectPoly.Envelope
4316:         Set pClipPointColl = pClipPolyline
        
4318:         ShiftPointsToWrapBoundary pClipPointColl, pShiftArray.Element(lngIndex)
        
'        If Not pMxDoc Is Nothing Then
'          InvoiceCreator.MyGeneralOperations.Graphic_MakeFromGeometry pMxDoc, pClipPointColl
'        End If
4323:         Set pTopoOp = pClipPolyline
4324:         pTopoOp.Simplify
        
4326:         pOutputColl.AddGeometry pClipPolyline
4327:       Next lngIndex
      
4329:     ElseIf TypeOf pGeometry Is IPolygon Then
      Dim pPolygon As IPolygon
4331:       Set pPolygon = pGeometry
4332:       Set pUnionTopoOp = New Polygon
      
      Dim pClipPolygon As IPolygon
4335:       For lngIndex = 0 To pRectArray.Count - 1
4336:         Set pRectPoly = pRectArray.Element(lngIndex)
4337:         Set pClone = pPolygon
4338:         Set pClipPolygon = pClone.Clone
4339:         Set pTopoOp = pClipPolygon
4340:         pTopoOp.Simplify
4341:         pTopoOp.ClipDense pRectPoly.Envelope, 0.1
4342:         Set pClipPointColl = pClipPolygon
        
4344:         ShiftPointsToWrapBoundary pClipPointColl, pShiftArray.Element(lngIndex)
4345:         Set pTopoOp = pClipPolygon
4346:         pTopoOp.Simplify
        
4348:         pOutputColl.AddGeometry pClipPolygon
4349:       Next lngIndex
    
4351:     ElseIf TypeOf pGeometry Is IMultipoint Then
      Dim pMultipoint As IMultipoint
4353:       Set pMultipoint = pGeometry
4354:       Set pUnionTopoOp = New Multipoint
      
      Dim pClipMultipoint As IMultipoint
4357:       For lngIndex = 0 To pRectArray.Count - 1
4358:         Set pRectPoly = pRectArray.Element(lngIndex)
4359:         Set pClone = pMultipoint
4360:         Set pClipMultipoint = pClone.Clone
4361:         Set pTopoOp = pClipMultipoint
4362:         pTopoOp.Clip pRectPoly.Envelope
4363:         Set pClipPointColl = pClipMultipoint
        
4365:         ShiftPointsToWrapBoundary pClipPointColl, pShiftArray.Element(lngIndex)
4366:         Set pTopoOp = pClipMultipoint
4367:         pTopoOp.Simplify
        
4369:         pOutputColl.AddGeometry pClipMultipoint
4370:       Next lngIndex
      
4372:     End If
4373:     pUnionTopoOp.ConstructUnion pOutputColl
4374:     pUnionTopoOp.Simplify
4375:     Set WrapToBoundary = pUnionTopoOp
4376:   End If


  Exit Function
ErrorHandler:
  HandleError True, "WrapToBoundary " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Sub ShiftPointsToWrapBoundary(pPointColl As IPointCollection, pShiftSubArray As esriSystem.IDoubleArray)
  On Error GoTo ErrorHandler

  
  ' THIS FUNCTION ASSUMES THAT SHAPE HAS ALREADY BEEN CLIPPED TO RECTANGLE REPRESENTING WRAP BOUNDARY INCREMENTS
  
  Dim pTransform As ITransform2D
4391:   Set pTransform = pPointColl
4392:   pTransform.Move -pShiftSubArray.Element(0), -pShiftSubArray.Element(1)


  Exit Sub
ErrorHandler:
  HandleError False, "ShiftPointsToWrapBoundary " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function HSin(dblRadians As Double) As Double
  On Error GoTo ErrorHandler


4404:   HSin = (Exp(dblRadians) - Exp(-dblRadians)) / 2


  Exit Function
ErrorHandler:
  HandleError True, "HSin " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function HCos(dblRadians As Double) As Double
  On Error GoTo ErrorHandler


4416:   HCos = (Exp(dblRadians) + Exp(-dblRadians)) / 2


  Exit Function
ErrorHandler:
  HandleError True, "HCos " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function HTan(dblRadians As Double) As Double
  On Error GoTo ErrorHandler


4428:   HTan = (Exp(dblRadians) - Exp(-dblRadians)) / (Exp(dblRadians) + Exp(-dblRadians))


  Exit Function
ErrorHandler:
  HandleError True, "HTan " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function HArcSin(dblValue As Double) As Double
  On Error GoTo ErrorHandler


4440:   HArcSin = Log(dblValue + Sqr(dblValue * dblValue + 1))


  Exit Function
ErrorHandler:
  HandleError True, "HArcSin " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function HArcCos(dblValue As Double) As Double
  On Error GoTo ErrorHandler


4452:   HArcCos = Log(dblValue + Sqr(dblValue * dblValue - 1))


  Exit Function
ErrorHandler:
  HandleError True, "HArcCos " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function HArcTan(dblValue As Double) As Double
  On Error GoTo ErrorHandler


4464:   HArcTan = Log((1 + dblValue) / (1 - dblValue)) / 2


  Exit Function
ErrorHandler:
  HandleError True, "HArcTan " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function BufferGeographic(pGeom As IGeometry, dblMeters As Double, _
      Optional booFoundProblems As Boolean = False, Optional strProblemReason As String) As IPolygon
  On Error GoTo ErrorHandler


  Dim pOrigSpRef As ISpatialReference
4478:   Set pOrigSpRef = pGeom.SpatialReference
4479:   If Not TypeOf pOrigSpRef Is IGeographicCoordinateSystem Then
4480:     booFoundProblems = True
4481:     strProblemReason = "Spatial Reference Not Geographic"
    Exit Function
4483:   End If
  
  Dim dblXMin As Double
  Dim dblXMax As Double
  Dim dblYMin As Double
  Dim dblYMax As Double
  
  Dim pPoint As IPoint
  Dim pEnv As IEnvelope
  
4493:   If TypeOf pGeom Is IPoint Then
4494:     Set pPoint = pGeom
4495:     dblXMin = pPoint.X - (dblMeters * 10)
4496:     dblXMax = pPoint.X + (dblMeters * 10)
4497:     dblYMin = pPoint.Y - (dblMeters * 10)
4498:     dblYMax = pPoint.Y + (dblMeters * 10)
4499:   Else
4500:     Set pPoint = New Point
4501:     Set pEnv = pGeom.Envelope
4502:     pPoint.PutCoords (pEnv.XMax - pEnv.XMin) / 2 + pEnv.XMin, (pEnv.YMax - pEnv.YMin) / 2 + pEnv.YMin
4503:     dblXMin = pEnv.XMin - (dblMeters * 10)
4504:     dblXMax = pEnv.XMax + (dblMeters * 10)
4505:     dblYMin = pEnv.YMin - (dblMeters * 10)
4506:     dblYMax = pEnv.YMax + (dblMeters * 10)
4507:   End If
    
  ' PROJECT INTO AZIMUTHAL EQUIDISTANT
  Dim pSpRefFact As ISpatialReferenceFactory3
4511:   Set pSpRefFact = New SpatialReferenceEnvironment
  Dim pPrjCS As IProjectedCoordinateSystem3
4513:   Set pPrjCS = pSpRefFact.CreateProjectedCoordinateSystem(esriSRProjCS_World_AzimuthalEquidistant)
  Dim pSpRef As ISpatialReference
4515:   Set pSpRef = pPrjCS
4516:   pPrjCS.CentralMeridian(True) = pPoint.X
4517:   pPrjCS.LatitudeOfOrigin = pPoint.Y
  
4519:   If Not MyGeomCheckSpRefDomain(pSpRef) Then
    Dim pSpRefRes As ISpatialReferenceResolution
4521:     Set pSpRefRes = pSpRef
4522:     pSpRefRes.ConstructFromHorizon
4523:   End If
  
4525:   pGeom.Project pSpRef
  Dim pTopoOp As ITopologicalOperator
4527:   Set pTopoOp = pGeom
4528:   pTopoOp.Simplify
  
  Dim pBuffer As IPolygon
4531:   Set pBuffer = pTopoOp.Buffer(dblMeters)
4532:   Set pTopoOp = pBuffer
4533:   pTopoOp.Simplify
  
4535:   pBuffer.Project pOrigSpRef
4536:   pTopoOp.Simplify
  
4538:   Set BufferGeographic = pBuffer
  
4540:   Set pOrigSpRef = Nothing
4541:   Set pPoint = Nothing
4542:   Set pEnv = Nothing
4543:   Set pSpRefFact = Nothing
4544:   Set pPrjCS = Nothing
4545:   Set pSpRef = Nothing
4546:   Set pSpRefRes = Nothing
4547:   Set pTopoOp = Nothing
4548:   Set pBuffer = Nothing


  Exit Function
ErrorHandler:
  HandleError True, "BufferGeographic " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Private Function MyGeomCheckSpRefDomain(pSpRef As ISpatialReference) As Boolean
  On Error GoTo ErrHand

  
  Dim dXmax As Double
  Dim dYmax As Double
  Dim dXmin As Double
  Dim dYmin As Double
  
  'get the xy domain extent of the dataset
  
4566:   pSpRef.GetDomain dXmin, dXmax, dYmin, dYmax
4567:   MyGeomCheckSpRefDomain = True
  
ErrHand:
4570:   MyGeomCheckSpRefDomain = False

End Function
